searchData={"items":[{"type":"module","title":"Phoenix.Component","doc":"Define reusable function components with HEEx templates.\n\nA function component is any function that receives an assigns\nmap as an argument and returns a rendered struct built with\n[the `~H` sigil](`sigil_H/2`):\n\n    defmodule MyComponent do\n      # In Phoenix apps, the line is typically: use MyAppWeb, :html\n      use Phoenix.Component\n\n      def greet(assigns) do\n        ~H\"\"\"\n         Hello, <%= @name %>! \n        \"\"\"\n      end\n    end\n\nThis function uses the `~H` sigil to return a rendered template.\n`~H` stands for HEEx (HTML + EEx). HEEx is a template language for\nwriting HTML mixed with Elixir interpolation. We can write Elixir\ncode inside HEEx using `<%= ... %>` tags and we use `@name` to\naccess the key `name` defined inside `assigns`.\n\nWhen invoked within a `~H` sigil or HEEx template file:\n\n```heex\n \n```\n\nThe following HTML is rendered:\n\n```html\n Hello, Jane! \n```\n\nIf the function component is defined locally, or its module is imported,\nthen the caller can invoke the function directly without specifying the module:\n\n```heex\n<.greet name=\"Jane\" />\n```\n\nFor dynamic values, you can interpolate Elixir expressions into a function component:\n\n```heex\n<.greet name={@user.name} />\n```\n\nFunction components can also accept blocks of HEEx content (more on this later):\n\n```heex\n<.card>\n   This is the body of my card! \n</.card>\n```\n\nIn this module we will learn how to build rich and composable components to\nuse in our applications.","ref":"Phoenix.Component.html"},{"type":"module","title":"Attributes - Phoenix.Component","doc":"`Phoenix.Component` provides the `attr/3` macro to declare what attributes the proceeding function\ncomponent expects to receive when invoked:\n\n    attr :name, :string, required: true\n\n    def greet(assigns) do\n      ~H\"\"\"\n       Hello, <%= @name %>! \n      \"\"\"\n    end\n\nBy calling `attr/3`, it is now clear that `greet/1` requires a string attribute called `name`\npresent in its assigns map to properly render. Failing to do so will result in a compilation\nwarning:\n\n```heex\n \n  <!-- warning: missing required attribute \"name\" for component MyAppWeb.MyComponent.greet/1\n           lib/app_web/my_component.ex:15 -->\n```\n\nAttributes can provide default values that are automatically merged into the assigns map:\n\n    attr :name, :string, default: \"Bob\"\n\nNow you can invoke the function component without providing a value for `name`:\n\n```heex\n<.greet />\n```\n\nRendering the following HTML:\n\n```html\n Hello, Bob! \n```\n\nAccessing an attribute which is required and does not have a default value will fail.\nYou must explicitly declare `default: nil` or assign a value programmatically with the\n`assign_new/3` function.\n\nMultiple attributes can be declared for the same function component:\n\n    attr :name, :string, required: true\n    attr :age, :integer, required: true\n\n    def celebrate(assigns) do\n      ~H\"\"\"\n       \n        Happy birthday <%= @name %>!\n        You are <%= @age %> years old.\n       \n      \"\"\"\n    end\n\nAllowing the caller to pass multiple values:\n\n```heex\n<.celebrate name={\"Genevieve\"} age={34} />\n```\n\nRendering the following HTML:\n\n```html\n \n  Happy birthday Genevieve!\n  You are 34 years old.\n \n```\n\nMultiple function components can be defined in the same module, with different attributes. In the\nfollowing example, ` ` requires a `name`, but *does not* require a `title`, and\n` ` requires a `title`, but *does not* require a `name`.\n\n    defmodule Components do\n      # In Phoenix apps, the line is typically: use MyAppWeb, :html\n      use Phoenix.Component\n\n      attr :title, :string, required: true\n\n      def heading(assigns) do\n        ~H\"\"\"\n         <%= @title %> \n        \"\"\"\n      end\n\n      attr :name, :string, required: true\n\n      def greet(assigns) do\n        ~H\"\"\"\n         Hello <%= @name %> \n        \"\"\"\n      end\n    end\n\nWith the `attr/3` macro you have the core ingredients to create reusable function components.\nBut what if you need your function components to support dynamic attributes, such as common HTML\nattributes to mix into a component's container?","ref":"Phoenix.Component.html#module-attributes"},{"type":"module","title":"Global attributes - Phoenix.Component","doc":"Global attributes are a set of attributes that a function component can accept when it\ndeclares an attribute of type `:global`. By default, the set of attributes accepted are those\nattributes common to all standard HTML tags.\nSee [Global attributes](https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes)\nfor a complete list of attributes.\n\nOnce a global attribute is declared, any number of attributes in the set can be passed by\nthe caller without having to modify the function component itself.\n\nBelow is an example of a function component that accepts a dynamic number of global attributes:\n\n    attr :message, :string, required: true\n    attr :rest, :global\n\n    def notification(assigns) do\n      ~H\"\"\"\n       <%= @message %> \n      \"\"\"\n    end\n\nThe caller can pass multiple global attributes (such as `phx-*` bindings or the `class` attribute):\n\n```heex\n<.notification message=\"You've got mail!\" class=\"bg-green-200\" phx-click=\"close\" />\n```\n\nRendering the following HTML:\n\n```html\n You've got mail! \n```\n\nNote that the function component did not have to explicitly declare a `class` or `phx-click`\nattribute in order to render.\n\nGlobal attributes can define defaults which are merged with attributes provided by the caller.\nFor example, you may declare a default `class` if the caller does not provide one:\n\n    attr :rest, :global, default: %{class: \"bg-blue-200\"}\n\nNow you can call the function component without a `class` attribute:\n\n```heex\n<.notification message=\"You've got mail!\" phx-click=\"close\" />\n```\n\nRendering the following HTML:\n\n```html\n You've got mail! \n```\n\nNote that the global attribute cannot be provided directly and doing so will emit\na warning. In other words, this is invalid:\n\n```heex\n<.notification message=\"You've got mail!\" rest={%{\"phx-click\" => \"close\"}} />\n```\n\n#","ref":"Phoenix.Component.html#module-global-attributes"},{"type":"module","title":"Included globals - Phoenix.Component","doc":"You may also specify which attributes are included in addition to the known globals\nwith the `:include` option. For example to support the `form` attribute on a button\ncomponent:\n\n```elixir\n# <.button form=\"my-form\"/>\nattr :rest, :global, include: ~w(form)\nslot :inner_block\ndef button(assigns) do\n  ~H\"\"\"\n   <%= render_slot(@inner_block) %> \n  \"\"\"\nend\n```\n\nThe `:include` option is useful to apply global additions on a case-by-case basis,\nbut sometimes you want to extend existing components with new global attributes,\nsuch as Alpine.js' `x-` prefixes, which we'll outline next.\n\n#","ref":"Phoenix.Component.html#module-included-globals"},{"type":"module","title":"Custom global attribute prefixes - Phoenix.Component","doc":"You can extend the set of global attributes by providing a list of attribute prefixes to\n`use Phoenix.Component`. Like the default attributes common to all HTML elements,\nany number of attributes that start with a global prefix will be accepted by function\ncomponents invoked by the current module. By default, the following prefixes are supported:\n`phx-`, `aria-`, and `data-`. For example, to support the `x-` prefix used by\n[Alpine.js](https://alpinejs.dev/), you can pass the `:global_prefixes` option to\n`use Phoenix.Component`:\n\n    use Phoenix.Component, global_prefixes: ~w(x-)\n\nIn your Phoenix application, this is typically done in your\n`lib/my_app_web.ex` file, inside the `def html` definition:\n\n    def html do\n      quote do\n        use Phoenix.Component, global_prefixes: ~w(x-)\n        # ...\n      end\n    end\n\nNow all function components invoked by this module will accept any number of attributes\nprefixed with `x-`, in addition to the default global prefixes.\n\nYou can learn more about attributes by reading the documentation for `attr/3`.","ref":"Phoenix.Component.html#module-custom-global-attribute-prefixes"},{"type":"module","title":"Slots - Phoenix.Component","doc":"In addition to attributes, function components can accept blocks of HEEx content, referred to\nas slots. Slots enable further customization of the rendered HTML, as the caller can pass the\nfunction component HEEx content they want the component to render. `Phoenix.Component` provides\nthe `slot/3` macro used to declare slots for function components:\n\n    slot :inner_block, required: true\n\n    def button(assigns) do\n      ~H\"\"\"\n       \n        <%= render_slot(@inner_block) %>\n       \n      \"\"\"\n    end\n\nThe expression `render_slot(@inner_block)` renders the HEEx content. You can invoke this function\ncomponent like so:\n\n```heex\n<.button>\n  This renders  inside  the button!\n</.button>\n```\n\nWhich renders the following HTML:\n\n```html\n \n  This renders  inside  the button!\n \n```\n\nLike the `attr/3` macro, using the `slot/3` macro will provide compile-time validations.\nFor example, invoking `button/1` without a slot of HEEx content will result in a compilation\nwarning being emitted:\n\n```heex\n<.button />\n  <!-- warning: missing required slot \"inner_block\" for component MyAppWeb.MyComponent.button/1\n           lib/app_web/my_component.ex:15 -->\n```\n\n#","ref":"Phoenix.Component.html#module-slots"},{"type":"module","title":"The default slot - Phoenix.Component","doc":"The example above uses the default slot, accessible as an assign named `@inner_block`, to render\nHEEx content via the `render_slot/1` function.\n\nIf the values rendered in the slot need to be dynamic, you can pass a second value back to the\nHEEx content by calling `render_slot/2`:\n\n    slot :inner_block, required: true\n\n    attr :entries, :list, default: []\n\n    def unordered_list(assigns) do\n      ~H\"\"\"\n       \n        <%= for entry <- @entries do %>\n           <%= render_slot(@inner_block, entry) %> \n        <% end %>\n       \n      \"\"\"\n    end\n\nWhen invoking the function component, you can use the special attribute `:let` to take the value\nthat the function component passes back and bind it to a variable:\n\n```heex\n<.unordered_list :let={fruit} entries={~w(apples bananas cherries)}>\n  I like  <%= fruit %> !\n</.unordered_list>\n```\n\nRendering the following HTML:\n\n```html\n \n   I like  apples ! \n   I like  bananas ! \n   I like  cherries ! \n \n```\n\nNow the separation of concerns is maintained: the caller can specify multiple values in a list\nattribute without having to specify the HEEx content that surrounds and separates them.\n\n#","ref":"Phoenix.Component.html#module-the-default-slot"},{"type":"module","title":"Named slots - Phoenix.Component","doc":"In addition to the default slot, function components can accept multiple, named slots of HEEx\ncontent. For example, imagine you want to create a modal that has a header, body, and footer:\n\n    slot :header\n    slot :inner_block, required: true\n    slot :footer, required: true\n\n    def modal(assigns) do\n      ~H\"\"\"\n       \n         \n          <%= render_slot(@header) || \"Modal\" %>\n         \n         \n          <%= render_slot(@inner_block) %>\n         \n         \n          <%= render_slot(@footer) %>\n         \n       \n      \"\"\"\n    end\n\nYou can invoke this function component using the named slot HEEx syntax:\n\n```heex\n<.modal>\n  This is the body, everything not in a named slot is rendered in the default slot.\n  <:footer>\n    This is the bottom of the modal.\n  </:footer>\n</.modal>\n```\n\nRendering the following HTML:\n\n```html\n \n   \n    Modal.\n   \n   \n    This is the body, everything not in a named slot is rendered in the default slot.\n   \n   \n    This is the bottom of the modal.\n   \n \n```\n\nAs shown in the example above, `render_slot/1` returns `nil` when an optional slot\nis declared and none is given. This can be used to attach default behaviour.\n\n#","ref":"Phoenix.Component.html#module-named-slots"},{"type":"module","title":"Slot attributes - Phoenix.Component","doc":"Unlike the default slot, it is possible to pass a named slot multiple pieces of HEEx content.\nNamed slots can also accept attributes, defined by passing a block to the `slot/3` macro.\nIf multiple pieces of content are passed, `render_slot/2` will merge and render all the values.\n\nBelow is a table component illustrating multiple named slots with attributes:\n\n    slot :column, doc: \"Columns with column labels\" do\n      attr :label, :string, required: true, doc: \"Column label\"\n    end\n\n    attr :rows, :list, default: []\n\n    def table(assigns) do\n      ~H\"\"\"\n       \n         \n          <%= for col <- @column do %>\n             <%= col.label %> \n          <% end %>\n         \n        <%= for row <- @rows do %>\n           \n            <%= for col <- @column do %>\n               <%= render_slot(col, row) %> \n            <% end %>\n           \n        <% end %>\n       \n      \"\"\"\n    end\n\nYou can invoke this function component like so:\n\n```heex\n<.table rows={[%{name: \"Jane\", age: \"34\"}, %{name: \"Bob\", age: \"51\"}]}>\n  <:column :let={user} label=\"Name\">\n    <%= user.name %>\n  </:column>\n  <:column :let={user} label=\"Age\">\n    <%= user.age %>\n  </:column>\n</.table>\n```\n\nRendering the following HTML:\n\n```html\n \n   \n     Name \n     Age \n   \n   \n     Jane \n     34 \n   \n   \n     Bob \n     51 \n   \n \n```\n\nYou can learn more about slots and the `slot/3` macro [in its documentation](`slot/3`).","ref":"Phoenix.Component.html#module-slot-attributes"},{"type":"module","title":"Embedding external template files - Phoenix.Component","doc":"The `embed_templates/1` macro can be used to embed `.html.heex` files\nas function components. The directory path is based on the current\nmodule (`__DIR__`), and a wildcard pattern may be used to select all\nfiles within a directory tree. For example, imagine a directory listing:\n\n    ├── components.ex\n    ├── cards\n    │   ├── pricing_card.html.heex\n    │   └── features_card.html.heex\n\nThen you can embed the page templates in your `components.ex` module\nand call them like any other function component:\n\n    defmodule MyAppWeb.Components do\n      use Phoenix.Component\n\n      embed_templates \"cards/*\"\n\n      def landing_hero(assigns) do\n        ~H\"\"\"\n        <.pricing_card />\n        <.features_card />\n        \"\"\"\n      end\n    end\n\nSee `embed_templates/1` for more information, including declarative\nassigns support for embedded templates.","ref":"Phoenix.Component.html#module-embedding-external-template-files"},{"type":"module","title":"Debug Annotations - Phoenix.Component","doc":"HEEx templates support debug annotations, which are special HTML comments\nthat wrap around rendered components to help you identify where markup\nin your HTML document is rendered within your function component tree.\n\nFor example, imagine the following HEEx template:\n\n```heex\n<.header>\n  <.button>Click</.button>\n</.header>\n```\n\nThe HTML document would receive the following comments when debug annotations\nare enabled:\n\n```html\n<!--   lib/app_web/core_components.ex:123 -->\n \n  <!--   lib/app_web/core_components.ex:456 -->\n   Click \n  <!--   -->\n \n<!--   -->\n```\n\nDebug annotations work across any `~H` or `.html.heex` template.\nThey can be enabled globally with the following configuration in your\n`config/dev.exs` file:\n\n    config :phoenix_live_view, debug_heex_annotations: true\n\nChanging this configuration will require `mix clean` and a full recompile.","ref":"Phoenix.Component.html#module-debug-annotations"},{"type":"function","title":"Phoenix.Component.assign/2","doc":"Adds key-value pairs to assigns.\n\nThe first argument is either a LiveView `socket` or an `assigns` map from function components.\n\nA keyword list or a map of assigns must be given as argument to be merged into existing assigns.","ref":"Phoenix.Component.html#assign/2"},{"type":"function","title":"Examples - Phoenix.Component.assign/2","doc":"iex> assign(socket, name: \"Elixir\", logo: \"💧\")\n    iex> assign(socket, %{name: \"Elixir\"})","ref":"Phoenix.Component.html#assign/2-examples"},{"type":"function","title":"Phoenix.Component.assign/3","doc":"Adds a `key`-`value` pair to `socket_or_assigns`.\n\nThe first argument is either a LiveView `socket` or an `assigns` map from function components.","ref":"Phoenix.Component.html#assign/3"},{"type":"function","title":"Examples - Phoenix.Component.assign/3","doc":"iex> assign(socket, :name, \"Elixir\")","ref":"Phoenix.Component.html#assign/3-examples"},{"type":"function","title":"Phoenix.Component.assign_new/3","doc":"Assigns the given `key` with value from `fun` into `socket_or_assigns` if one does not yet exist.\n\nThe first argument is either a LiveView `socket` or an `assigns` map from function components.\n\nThis function is useful for lazily assigning values and sharing assigns.\nWe will cover both use cases next.","ref":"Phoenix.Component.html#assign_new/3"},{"type":"function","title":"Lazy assigns - Phoenix.Component.assign_new/3","doc":"Imagine you have a function component that accepts a color:\n\n```heex\n<.my_component bg_color=\"red\" />\n```\n\nThe color is also optional, so you can skip it:\n\n```heex\n<.my_component />\n```\n\nIn such cases, the implementation can use `assign_new` to lazily\nassign a color if none is given. Let's make it so it picks a random one\nwhen none is given:\n\n    def my_component(assigns) do\n      assigns = assign_new(assigns, :bg_color, fn -> Enum.random(~w(bg-red-200 bg-green-200 bg-blue-200)) end)\n\n      ~H\"\"\"\n       \n        Example\n       \n      \"\"\"\n    end","ref":"Phoenix.Component.html#assign_new/3-lazy-assigns"},{"type":"function","title":"Sharing assigns - Phoenix.Component.assign_new/3","doc":"It is possible to share assigns between the Plug pipeline and LiveView on disconnected render\nand between LiveViews when connected.\n\n#","ref":"Phoenix.Component.html#assign_new/3-sharing-assigns"},{"type":"function","title":"When disconnected - Phoenix.Component.assign_new/3","doc":"When a user first accesses an application using LiveView, the LiveView is first rendered in its\ndisconnected state, as part of a regular HTML response. By using `assign_new` in the mount\ncallback of your LiveView, you can instruct LiveView to re-use any assigns already set in `conn`\nduring disconnected state.\n\nImagine you have a Plug that does:\n\n    # A plug\n    def authenticate(conn, _opts) do\n      if user_id = get_session(conn, :user_id) do\n        assign(conn, :current_user, Accounts.get_user!(user_id))\n      else\n        send_resp(conn, :forbidden)\n      end\n    end\n\nYou can re-use the `:current_user` assign in your LiveView during the initial render:\n\n    def mount(_params, %{\"user_id\" => user_id}, socket) do\n      {:ok, assign_new(socket, :current_user, fn -> Accounts.get_user!(user_id) end)}\n    end\n\nIn such case `conn.assigns.current_user` will be used if present. If there is no such\n`:current_user` assign or the LiveView was mounted as part of the live navigation, where no Plug\npipelines are invoked, then the anonymous function is invoked to execute the query instead.\n\n#","ref":"Phoenix.Component.html#assign_new/3-when-disconnected"},{"type":"function","title":"When connected - Phoenix.Component.assign_new/3","doc":"LiveView is also able to share assigns via `assign_new` within nested LiveView. If the parent\nLiveView defines a `:current_user` assign and the child LiveView also uses `assign_new/3` to\nfetch the `:current_user` in its `mount/3` callback, as above, the assign will be fetched from\nthe parent LiveView, once again avoiding additional database queries.\n\nNote that `fun` also provides access to the previously assigned values:\n\n    assigns =\n        assigns\n        |> assign_new(:foo, fn -> \"foo\" end)\n        |> assign_new(:bar, fn %{foo: foo} -> foo <> \"bar\" end)","ref":"Phoenix.Component.html#assign_new/3-when-connected"},{"type":"function","title":"Phoenix.Component.assigns_to_attributes/2","doc":"Filters the assigns as a list of keywords for use in dynamic tag attributes.\n\nOne should prefer to use declarative assigns and `:global` attributes\nover this function.","ref":"Phoenix.Component.html#assigns_to_attributes/2"},{"type":"function","title":"Examples - Phoenix.Component.assigns_to_attributes/2","doc":"Imagine the following `my_link` component which allows a caller\nto pass a `new_window` assign, along with any other attributes they\nwould like to add to the element, such as class, data attributes, etc:\n\n```heex\n<.my_link to=\"/\" id={@id} new_window={true} class=\"my-class\">Home</.my_link>\n```\n\nWe could support the dynamic attributes with the following component:\n\n    def my_link(assigns) do\n      target = if assigns[:new_window], do: \"_blank\", else: false\n      extra = assigns_to_attributes(assigns, [:new_window, :to])\n\n      assigns =\n        assigns\n        |> assign(:target, target)\n        |> assign(:extra, extra)\n\n      ~H\"\"\"\n       \n        <%= render_slot(@inner_block) %>\n       \n      \"\"\"\n    end\n\nThe above would result in the following rendered HTML:\n\n```heex\n Home \n```\n\nThe second argument (optional) to `assigns_to_attributes` is a list of keys to\nexclude. It typically includes reserved keys by the component itself, which either\ndo not belong in the markup, or are already handled explicitly by the component.","ref":"Phoenix.Component.html#assigns_to_attributes/2-examples"},{"type":"function","title":"Phoenix.Component.async_result/1","doc":"Renders an async assign with slots for the different loading states.\nThe result state takes precedence over subsequent loading and failed\nstates.\n\n*Note*: The inner block receives the result of the async assign as a :let.\nThe let is only accessible to the inner block and is not in scope to the\nother slots.","ref":"Phoenix.Component.html#async_result/1"},{"type":"function","title":"Examples - Phoenix.Component.async_result/1","doc":"```heex\n<.async_result :let={org} assign={@org}>\n  <:loading>Loading organization...</:loading>\n  <:failed :let={_failure}>there was an error loading the organization</:failed>\n  <%= if org do %>\n    <%= org.name %>\n  <% else %>\n    You don't have an organization yet.\n  <% end %>\n</.async_result>\n```\n\nTo display loading and failed states again on subsequent `assign_async` calls,\nreset the assign to a result-free `%AsyncResult{}`:\n\n```elixir\n{:noreply,\n  socket\n  |> assign_async(:page, :data, &reload_data/0)\n  |> assign(:page, AsyncResult.loading())}\n```","ref":"Phoenix.Component.html#async_result/1-examples"},{"type":"function","title":"Attributes - Phoenix.Component.async_result/1","doc":"* `assign` (`Phoenix.LiveView.AsyncResult`) (required)","ref":"Phoenix.Component.html#async_result/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.async_result/1","doc":"* `loading` - rendered while the assign is loading for the first time.\n* `failed` - rendered when an error or exit is caught or assign_async returns `{:error, reason}` for the first time. Receives the error as a `:let`.\n* `inner_block` - rendered when the assign is loaded successfully via `AsyncResult.ok/2`. Receives the result as a `:let`.","ref":"Phoenix.Component.html#async_result/1-slots"},{"type":"macro","title":"Phoenix.Component.attr/3","doc":"Declares attributes for a HEEx function components.","ref":"Phoenix.Component.html#attr/3"},{"type":"macro","title":"Arguments - Phoenix.Component.attr/3","doc":"* `name` - an atom defining the name of the attribute. Note that attributes cannot define the\nsame name as any other attributes or slots declared for the same component.\n\n* `type` - an atom defining the type of the attribute.\n\n* `opts` - a keyword list of options. Defaults to `[]`.\n\n#","ref":"Phoenix.Component.html#attr/3-arguments"},{"type":"macro","title":"Types - Phoenix.Component.attr/3","doc":"An attribute is declared by its name, type, and options. The following types are supported:\n\n| Name            | Description                                                          |\n|-----------------|----------------------------------------------------------------------|\n| `:any`          | any term                                                             |\n| `:string`       | any binary string                                                    |\n| `:atom`         | any atom (including `true`, `false`, and `nil`)                      |\n| `:boolean`      | any boolean                                                          |\n| `:integer`      | any integer                                                          |\n| `:float`        | any float                                                            |\n| `:list`         | any list of any arbitrary types                                      |\n| `:map`          | any map of any arbitrary types                                       |\n| `:global`       | any common HTML attributes, plus those defined by `:global_prefixes` |\n| A struct module | any module that defines a struct with `defstruct/1`                  |\n\n#","ref":"Phoenix.Component.html#attr/3-types"},{"type":"macro","title":"Options - Phoenix.Component.attr/3","doc":"* `:required` - marks an attribute as required. If a caller does not pass the given attribute,\na compile warning is issued.\n\n* `:default` - the default value for the attribute if not provided. If this option is\n  not set and the attribute is not given, accessing the attribute will fail unless a\n  value is explicitly set with `assign_new/3`.\n\n* `:examples` - a non-exhaustive list of values accepted by the attribute, used for documentation\n  purposes.\n\n* `:values` - an exhaustive list of values accepted by the attributes. If a caller passes a literal\n  not contained in this list, a compile warning is issued.\n\n* `:doc` - documentation for the attribute.","ref":"Phoenix.Component.html#attr/3-options"},{"type":"macro","title":"Compile-Time Validations - Phoenix.Component.attr/3","doc":"LiveView performs some validation of attributes via the `:phoenix_live_view` compiler.\nWhen attributes are defined, LiveView will warn at compilation time on the caller if:\n\n* A required attribute of a component is missing.\n\n* An unknown attribute is given.\n\n* You specify a literal attribute (such as `value=\"string\"` or `value`, but not `value={expr}`)\nand the type does not match. The following types currently support literal validation:\n`:string`, `:atom`, `:boolean`, `:integer`, `:float`, `:map` and `:list`.\n\n* You specify a literal attribute and it is not a member of the `:values` list.\n\nLiveView does not perform any validation at runtime. This means the type information is mostly\nused for documentation and reflection purposes.\n\nOn the side of the LiveView component itself, defining attributes provides the following quality\nof life improvements:\n\n* The default value of all attributes will be added to the `assigns` map upfront.\n\n* Attribute documentation is generated for the component.\n\n* Required struct types are annotated and emit compilation warnings. For example, if you specify\n`attr :user, User, required: true` and then you write `@user.non_valid_field` in your template,\na warning will be emitted.\n\n* Calls made to the component are tracked for reflection and validation purposes.","ref":"Phoenix.Component.html#attr/3-compile-time-validations"},{"type":"macro","title":"Documentation Generation - Phoenix.Component.attr/3","doc":"Public function components that define attributes will have their attribute\ntypes and docs injected into the function's documentation, depending on the\nvalue of the `@doc` module attribute:\n\n* if `@doc` is a string, the attribute docs are injected into that string. The optional\nplaceholder `[INSERT LVATTRDOCS]` can be used to specify where in the string the docs are\ninjected. Otherwise, the docs are appended to the end of the `@doc` string.\n\n* if `@doc` is unspecified, the attribute docs are used as the default `@doc` string.\n\n* if `@doc` is `false`, the attribute docs are omitted entirely.\n\nThe injected attribute docs are formatted as a markdown list:\n\n  * `name` (`:type`) (required) - attr docs. Defaults to `:default`.\n\nBy default, all attributes will have their types and docs injected into the function `@doc`\nstring. To hide a specific attribute, you can set the value of `:doc` to `false`.","ref":"Phoenix.Component.html#attr/3-documentation-generation"},{"type":"macro","title":"Example - Phoenix.Component.attr/3","doc":"attr :name, :string, required: true\n    attr :age, :integer, required: true\n\n    def celebrate(assigns) do\n      ~H\"\"\"\n       \n        Happy birthday <%= @name %>!\n        You are <%= @age %> years old.\n       \n      \"\"\"\n    end","ref":"Phoenix.Component.html#attr/3-example"},{"type":"function","title":"Phoenix.Component.changed?/2","doc":"Checks if the given key changed in `socket_or_assigns`.\n\nThe first argument is either a LiveView `socket` or an `assigns` map from function components.","ref":"Phoenix.Component.html#changed?/2"},{"type":"function","title":"Examples - Phoenix.Component.changed?/2","doc":"iex> changed?(socket, :count)","ref":"Phoenix.Component.html#changed?/2-examples"},{"type":"function","title":"Phoenix.Component.dynamic_tag/1","doc":"Generates a dynamically named HTML tag.\n\nRaises an `ArgumentError` if the tag name is found to be unsafe HTML.","ref":"Phoenix.Component.html#dynamic_tag/1"},{"type":"function","title":"Attributes - Phoenix.Component.dynamic_tag/1","doc":"* `name` (`:string`) (required) - The name of the tag, such as `div`.\n* Global attributes are accepted. Additional HTML attributes to add to the tag, ensuring proper escaping.","ref":"Phoenix.Component.html#dynamic_tag/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.dynamic_tag/1","doc":"* `inner_block`","ref":"Phoenix.Component.html#dynamic_tag/1-slots"},{"type":"function","title":"Examples - Phoenix.Component.dynamic_tag/1","doc":"```heex\n<.dynamic_tag name=\"input\" type=\"text\"/>\n```\n\n```html\n \n```\n\n```heex\n<.dynamic_tag name=\"p\">content</.dynamic_tag>\n```\n\n```html\n content \n```","ref":"Phoenix.Component.html#dynamic_tag/1-examples"},{"type":"macro","title":"Phoenix.Component.embed_templates/2","doc":"Embeds external template files into the module as function components.","ref":"Phoenix.Component.html#embed_templates/2"},{"type":"macro","title":"Options - Phoenix.Component.embed_templates/2","doc":"* `:root` - The root directory to embed files. Defaults to the current\n    module's directory (`__DIR__`)\n  * `:suffix` - The string value to append to embedded function names. By\n    default, function names will be the name of the template file excluding\n    the format and engine.\n\nA wildcard pattern may be used to select all files within a directory tree.\nFor example, imagine a directory listing:\n\n    ├── components.ex\n    ├── pages\n    │   ├── about_page.html.heex\n    │   └── welcome_page.html.heex\n\nThen to embed the page templates in your `components.ex` module:\n\n    defmodule MyAppWeb.Components do\n      use Phoenix.Component\n\n      embed_templates \"pages/*\"\n    end\n\nNow, your module will have an `about_page/1` and `welcome_page/1` function\ncomponent defined. Embedded templates also support declarative assigns\nvia bodyless function definitions, for example:\n\n    defmodule MyAppWeb.Components do\n      use Phoenix.Component\n\n      embed_templates \"pages/*\"\n\n      attr :name, :string, required: true\n      def welcome_page(assigns)\n\n      slot :header\n      def about_page(assigns)\n    end\n\nMultiple invocations of `embed_templates` is also supported, which can be\nuseful if you have more than one template format. For example:\n\n    defmodule MyAppWeb.Emails do\n      use Phoenix.Component\n\n      embed_templates \"emails/*.html\", suffix: \"_html\"\n      embed_templates \"emails/*.text\", suffix: \"_text\"\n    end\n\nNote: this function is the same as `Phoenix.Template.embed_templates/2`.\nIt is also provided here for convenience and documentation purposes.\nTherefore, if you want to embed templates for other formats, which are\nnot related to `Phoenix.Component`, prefer to\n`import Phoenix.Template, only: [embed_templates: 1]` than this module.","ref":"Phoenix.Component.html#embed_templates/2-options"},{"type":"function","title":"Phoenix.Component.focus_wrap/1","doc":"Wraps tab focus around a container for accessibility.\n\nThis is an essential accessibility feature for interfaces such as modals, dialogs, and menus.","ref":"Phoenix.Component.html#focus_wrap/1"},{"type":"function","title":"Attributes - Phoenix.Component.focus_wrap/1","doc":"* `id` (`:string`) (required) - The DOM identifier of the container tag.\n* Global attributes are accepted. Additional HTML attributes to add to the container tag.","ref":"Phoenix.Component.html#focus_wrap/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.focus_wrap/1","doc":"* `inner_block` (required) - The content rendered inside of the container tag.","ref":"Phoenix.Component.html#focus_wrap/1-slots"},{"type":"function","title":"Examples - Phoenix.Component.focus_wrap/1","doc":"Simply render your inner content within this component and focus will be wrapped around the\ncontainer as the user tabs through the containers content:\n\n```heex\n<.focus_wrap id=\"my-modal\" class=\"bg-white\">\n   \n    Are you sure?\n     Cancel \n     OK \n   \n</.focus_wrap>\n```","ref":"Phoenix.Component.html#focus_wrap/1-examples"},{"type":"function","title":"Phoenix.Component.form/1","doc":"Renders a form.\n\nThis function receives a `Phoenix.HTML.Form` struct, generally created with\n`to_form/2`, and generates the relevant form tags. It can be used either\ninside LiveView or outside.\n\n> To see how forms work in practice, you can run\n> `mix phx.gen.live Blog Post posts title body:text` inside your Phoenix\n> application, which will setup the necessary database tables and LiveViews\n> to manage your data.","ref":"Phoenix.Component.html#form/1"},{"type":"function","title":"Examples: inside LiveView - Phoenix.Component.form/1","doc":"Inside LiveViews, this function component is typically called with\nas `for={@form}`, where `@form` is the result of the `to_form/1` function.\n`to_form/1` expects either a map or an [`Ecto.Changeset`](https://hexdocs.pm/ecto/Ecto.Changeset.html)\nas the source of data and normalizes it into `Phoenix.HTML.Form` structure.\n\nFor example, you may use the parameters received in a\n`c:Phoenix.LiveView.handle_event/3` callback to create an Ecto changeset\nand then use `to_form/1` to convert it to a form. Then, in your templates,\nyou pass the `@form` as argument to `:for`:\n\n```heex\n<.form\n  for={@form}\n  phx-change=\"change_name\"\n>\n  <.input field={@form[:email]} />\n</.form>\n```\n\nThe `.input` component is generally defined as part of your own application\nand adds all styling necessary:\n\n```heex\ndef input(assigns) do\n  ~H\"\"\"\n   \n  \"\"\"\nend\n```\n\nA form accepts multiple options. For example, if you are doing file uploads\nand you want to capture submissions, you might write instead:\n\n```heex\n<.form\n  for={@form}\n  multipart\n  phx-change=\"change_user\"\n  phx-submit=\"save_user\"\n>\n  ...\n   \n</.form>\n```\n\nNotice how both examples use `phx-change`. The LiveView must implement the\n`phx-change` event and store the input values as they arrive on change.\nThis is important because, if an unrelated change happens on the page,\nLiveView should re-render the inputs with their updated values. Without `phx-change`,\nthe inputs would otherwise be cleared. Alternatively, you can use `phx-update=\"ignore\"`\non the form to discard any updates.\n\n#","ref":"Phoenix.Component.html#form/1-examples-inside-liveview"},{"type":"function","title":"Using the `for` attribute - Phoenix.Component.form/1","doc":"The `for` attribute can also be a map or an Ecto.Changeset. In such cases,\na form will be created on the fly, and you can capture it using `:let`:\n\n```heex\n<.form\n  :let={form}\n  for={@changeset}\n  phx-change=\"change_user\"\n>\n```\n\nHowever, such approach is discouraged in LiveView for two reasons:\n\n  * LiveView can better optimize your code if you access the form fields\n    using `@form[:field]` rather than through the let-variable `form`\n\n  * Ecto changesets are meant to be single use. By never storing the changeset\n    in the assign, you will be less tempted to use it across operations\n\n#","ref":"Phoenix.Component.html#form/1-using-the-for-attribute"},{"type":"function","title":"A note on `:errors` - Phoenix.Component.form/1","doc":"Even if `changeset.errors` is non-empty, errors will not be displayed in a\nform if [the changeset\n`:action`](https://hexdocs.pm/ecto/Ecto.Changeset.html#module-changeset-actions)\nis `nil` or `:ignore`.\n\nThis is useful for things like validation hints on form fields, e.g. an empty\nchangeset for a new form. That changeset isn't valid, but we don't want to\nshow errors until an actual user action has been performed.\n\nFor example, if the user submits and a `Repo.insert/1` is called and fails on\nchangeset validation, the action will be set to `:insert` to show that an\ninsert was attempted, and the presence of that action will cause errors to be\ndisplayed. The same is true for Repo.update/delete.\n\nIf you want to show errors manually you can also set the action yourself,\neither directly on the `Ecto.Changeset` struct field or by using\n`Ecto.Changeset.apply_action/2`. Since the action can be arbitrary, you can\nset it to `:validate` or anything else to avoid giving the impression that a\ndatabase operation has actually been attempted.","ref":"Phoenix.Component.html#form/1-a-note-on-errors"},{"type":"function","title":"Example: outside LiveView (regular HTTP requests) - Phoenix.Component.form/1","doc":"The `form` component can still be used to submit forms outside of LiveView.\nIn such cases, the `action` attribute MUST be given. Without said attribute,\nthe `form` method and csrf token are discarded.\n\n```heex\n<.form :let={f} for={@changeset} action={Routes.comment_path(:create, @comment)}>\n  <.input field={f[:body]} />\n</.form>\n```\n\nIn the example above, we passed a changeset to `for` and captured\nthe value using `:let={f}`. This approach is ok outside of LiveViews,\nas there are no change tracking optimizations to consider.\n\n#","ref":"Phoenix.Component.html#form/1-example-outside-liveview-regular-http-requests"},{"type":"function","title":"CSRF protection - Phoenix.Component.form/1","doc":"CSRF protection is a mechanism to ensure that the user who rendered\nthe form is the one actually submitting it. This module generates a\nCSRF token by default. Your application should check this token on\nthe server to avoid attackers from making requests on your server on\nbehalf of other users. Phoenix by default checks this token.\n\nWhen posting a form with a host in its address, such as \"//host.com/path\"\ninstead of only \"/path\", Phoenix will include the host signature in the\ntoken and validate the token only if the accessed host is the same as\nthe host in the token. This is to avoid tokens from leaking to third\nparty applications. If this behaviour is problematic, you can generate\na non-host specific token with `Plug.CSRFProtection.get_csrf_token/0` and\npass it to the form generator via the `:csrf_token` option.","ref":"Phoenix.Component.html#form/1-csrf-protection"},{"type":"function","title":"Attributes - Phoenix.Component.form/1","doc":"* `for` (`:any`) (required) - An existing form or the form source data.\n* `action` (`:string`) - The action to submit the form on.\n  This attribute must be given if you intend to submit the form to a URL without LiveView.\n\n* `as` (`:atom`) - The prefix to be used in names and IDs generated by the form.\n  For example, setting `as: :user_params` means the parameters\n  will be nested \"user_params\" in your `handle_event` or\n  `conn.params[\"user_params\"]` for regular HTTP requests.\n  If you set this option, you must capture the form with `:let`.\n\n* `csrf_token` (`:any`) - A token to authenticate the validity of requests.\n  One is automatically generated when an action is given and the method is not `get`.\n  When set to `false`, no token is generated.\n\n* `errors` (`:list`) - Use this to manually pass a keyword list of errors to the form.\n  This option is useful when a regular map is given as the form\n  source and it will make the errors available under `f.errors`.\n  If you set this option, you must capture the form with `:let`.\n\n* `method` (`:string`) - The HTTP method.\n  It is only used if an `:action` is given. If the method is not `get` nor `post`,\n  an input tag with name `_method` is generated alongside the form tag.\n  If an `:action` is given with no method, the method will default to `post`.\n\n* `multipart` (`:boolean`) - Sets `enctype` to `multipart/form-data`.\n  Required when uploading files.\n\n  Defaults to `false`.\n* Global attributes are accepted. Additional HTML attributes to add to the form tag. Supports all globals plus: `[\"autocomplete\", \"name\", \"rel\", \"enctype\", \"novalidate\", \"target\"]`.","ref":"Phoenix.Component.html#form/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.form/1","doc":"* `inner_block` (required) - The content rendered inside of the form tag.","ref":"Phoenix.Component.html#form/1-slots"},{"type":"function","title":"Phoenix.Component.inputs_for/1","doc":"Renders nested form inputs for associations or embeds.","ref":"Phoenix.Component.html#inputs_for/1"},{"type":"function","title":"Attributes - Phoenix.Component.inputs_for/1","doc":"* `field` (`Phoenix.HTML.FormField`) (required) - A %Phoenix.HTML.Form{}/field name tuple, for example: {@form[:email]}.\n* `id` (`:string`) - The id to be used in the form, defaults to the concatenation of the given\n  field to the parent form id.\n\n* `as` (`:atom`) - The name to be used in the form, defaults to the concatenation of the given\n  field to the parent form name.\n\n* `default` (`:any`) - The value to use if none is available.\n* `prepend` (`:list`) - The values to prepend when rendering. This only applies if the field value\n  is a list and no parameters were sent through the form.\n\n* `append` (`:list`) - The values to append when rendering. This only applies if the field value\n  is a list and no parameters were sent through the form.\n\n* `skip_hidden` (`:boolean`) - Skip the automatic rendering of hidden fields to allow for more tight control\n  over the generated markup.\n\n  Defaults to `false`.\n* `options` (`:list`) - Any additional options for the `Phoenix.HTML.FormData` protocol\n  implementation.\n\n  Defaults to `[]`.","ref":"Phoenix.Component.html#inputs_for/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.inputs_for/1","doc":"* `inner_block` (required) - The content rendered for each nested form.","ref":"Phoenix.Component.html#inputs_for/1-slots"},{"type":"function","title":"Examples - Phoenix.Component.inputs_for/1","doc":"```heex\n<.form\n  :let={f}\n  phx-change=\"change_name\"\n>\n  <.inputs_for :let={f_nested} field={f[:nested]}>\n    <.input type=\"text\" field={f_nested[:name]} />\n  </.inputs_for>\n</.form>\n```","ref":"Phoenix.Component.html#inputs_for/1-examples"},{"type":"function","title":"Dynamically adding and removing inputs - Phoenix.Component.inputs_for/1","doc":"Dynamically adding and removing inputs is supported by rendering named buttons for\ninserts and removals. Like inputs, buttons with name/value pairs are serialized with\nform data on change and submit events. Libraries such as Ecto, or custom param\nfiltering can then inspect the parameters and handle the added or removed fields.\nThis can be combined with `Ecto.Changeset.cast/3`'s `:sort_param` and `:drop_param`\noptions. For example, imagine a parent with an `:emails` `has_many` or `embeds_many`\nassociation. To cast the user input from a nested form, one simply needs to configure\nthe options:\n\n    schema \"mailing_lists\" do\n      field :title, :string\n\n      embeds_many :emails, EmailNotification, on_replace: :delete do\n        field :email, :string\n        field :name, :string\n      end\n    end\n\n    def changeset(list, attrs) do\n      list\n      |> cast(attrs, [:title])\n      |> cast_embed(:emails,\n        with: &email_changeset/2,\n        sort_param: :emails_sort,\n        drop_param: :emails_drop\n      )\n    end\n\nHere we see the `:sort_param` and `:drop_param` options in action.\n\n> Note: `on_replace: :delete` on the `has_many` and `embeds_many` is required\n> when using these options.\n\nWhen Ecto sees the specified sort or drop parameter from the form, it will sort\nthe children based on the order they appear in the form, add new children it hasn't\nseen, or drop children if the parameter instructs it to do so.\n\nThe markup for such a schema and association would look like this:\n\n```heex\n<.inputs_for :let={ef} field={@form[:emails]}>\n   \n  <.input type=\"text\" field={ef[:email]} placeholder=\"email\" />\n  <.input type=\"text\" field={ef[:name]} placeholder=\"name\" />\n   \n    <.icon name=\"hero-x-mark\" class=\"w-6 h-6 relative top-2\" />\n   \n</.inputs_for>\n\n \n\n \n  add more\n \n```\n\nWe used `inputs_for` to render inputs for the `:emails` association, which\ncontains an email address and name input for each child. Within the nested inputs,\nwe render a hidden `mailing_list[emails_sort][]` input, which is set to the index of the\ngiven child. This tells Ecto's cast operation how to sort existing children, or\nwhere to insert new children. Next, we render the email and name inputs as usual.\nThen we render a button containing the \"delete\" text with the name `mailing_list[emails_drop][]`,\ncontaining the index of the child as its value.\n\nLike before, this tells Ecto to delete the child at this index when the button is\nclicked. We use `phx-click={JS.dispatch(\"change\")}` on the button to tell LiveView\nto treat this button click as a change event, rather than a submit event on the form,\nwhich invokes our form's `phx-change` binding.\n\nOutside the `inputs_for`, we render an empty `mailing_list[emails_drop][]` input,\nto ensure that all children are deleted when saving a form where the user\ndropped all entries. This hidden input is required whenever dropping associations.\n\nFinally, we also render another button with the sort param name `mailing_list[emails_sort][]`\nand `value=\"new\"` name with accompanied \"add more\" text. Please note that this button must\nhave `type=\"button\"` to prevent it from submitting the form.\nEcto will treat unknown sort params as new children and build a new child.\nThis button is optional and only necessary if you want to dyamically add entries.\nYou can optionally add a similar button before the `<.inputs_for>`, in the case you want\nto prepend entries.","ref":"Phoenix.Component.html#inputs_for/1-dynamically-adding-and-removing-inputs"},{"type":"function","title":"Phoenix.Component.intersperse/1","doc":"Intersperses separator slot between an enumerable.\n\nUseful when you need to add a separator between items such as when\nrendering breadcrumbs for navigation. Provides each item to the\ninner block.","ref":"Phoenix.Component.html#intersperse/1"},{"type":"function","title":"Examples - Phoenix.Component.intersperse/1","doc":"```heex\n<.intersperse :let={item} enum={[\"home\", \"profile\", \"settings\"]}>\n  <:separator>\n     | \n  </:separator>\n  <%= item %>\n</.intersperse>\n```\n\nRenders the following markup:\n\n    home  |  profile  |  settings","ref":"Phoenix.Component.html#intersperse/1-examples"},{"type":"function","title":"Attributes - Phoenix.Component.intersperse/1","doc":"* `enum` (`:any`) (required) - the enumerable to intersperse with separators.","ref":"Phoenix.Component.html#intersperse/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.intersperse/1","doc":"* `inner_block` (required) - the inner_block to render for each item.\n* `separator` (required) - the slot for the separator.","ref":"Phoenix.Component.html#intersperse/1-slots"},{"type":"function","title":"Phoenix.Component.link/1","doc":"Generates a link to a given route.\n\nTo navigate across pages, using traditional browser navigation, use\nthe `href` attribute. To patch the current LiveView or navigate\nacross LiveViews, use `patch` and `navigate` respectively.","ref":"Phoenix.Component.html#link/1"},{"type":"function","title":"Attributes - Phoenix.Component.link/1","doc":"* `navigate` (`:string`) - Navigates from a LiveView to a new LiveView.\n  The browser page is kept, but a new LiveView process is mounted and its content on the page\n  is reloaded. It is only possible to navigate between LiveViews declared under the same router\n  `Phoenix.LiveView.Router.live_session/3`. Otherwise, a full browser redirect is used.\n\n* `patch` (`:string`) - Patches the current LiveView.\n  The `handle_params` callback of the current LiveView will be invoked and the minimum content\n  will be sent over the wire, as any other LiveView diff.\n\n* `href` (`:any`) - Uses traditional browser navigation to the new location.\n  This means the whole page is reloaded on the browser.\n\n* `replace` (`:boolean`) - When using `:patch` or `:navigate`,\n  should the browser's history be replaced with `pushState`?\n\n  Defaults to `false`.\n* `method` (`:string`) - The HTTP method to use with the link. This is intended for usage outside of LiveView\n  and therefore only works with the `href={...}` attribute. It has no effect on `patch`\n  and `navigate` instructions.\n\n  In case the method is not `get`, the link is generated inside the form which sets the proper\n  information. In order to submit the form, JavaScript must be enabled in the browser.\n\n  Defaults to `\"get\"`.\n* `csrf_token` (`:any`) - A boolean or custom token to use for links with an HTTP method other than `get`. Defaults to `true`.\n* Global attributes are accepted. Additional HTML attributes added to the `a` tag. Supports all globals plus: `[\"download\", \"hreflang\", \"referrerpolicy\", \"rel\", \"target\", \"type\"]`.","ref":"Phoenix.Component.html#link/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.link/1","doc":"* `inner_block` (required) - The content rendered inside of the `a` tag.","ref":"Phoenix.Component.html#link/1-slots"},{"type":"function","title":"Examples - Phoenix.Component.link/1","doc":"```heex\n<.link href=\"/\">Regular anchor link</.link>\n```\n\n```heex\n<.link navigate={~p\"/\"} class=\"underline\">home</.link>\n```\n\n```heex\n<.link navigate={~p\"/?sort=asc\"} replace={false}>\n  Sort By Price\n</.link>\n```\n\n```heex\n<.link patch={~p\"/details\"}>view details</.link>\n```\n\n```heex\n<.link href={URI.parse(\"https://elixir-lang.org\")}>hello</.link>\n```\n\n```heex\n<.link href=\"/the_world\" method=\"delete\" data-confirm=\"Really?\">delete</.link>\n```","ref":"Phoenix.Component.html#link/1-examples"},{"type":"function","title":"JavaScript dependency - Phoenix.Component.link/1","doc":"In order to support links where `:method` is not `\"get\"` or use the above data attributes,\n`Phoenix.HTML` relies on JavaScript. You can load `priv/static/phoenix_html.js` into your\nbuild tool.\n\n#","ref":"Phoenix.Component.html#link/1-javascript-dependency"},{"type":"function","title":"Data attributes - Phoenix.Component.link/1","doc":"Data attributes are added as a keyword list passed to the `data` key. The following data\nattributes are supported:\n\n* `data-confirm` - shows a confirmation prompt before generating and submitting the form when\n`:method` is not `\"get\"`.\n\n#","ref":"Phoenix.Component.html#link/1-data-attributes"},{"type":"function","title":"Overriding the default confirm behaviour - Phoenix.Component.link/1","doc":"`phoenix_html.js` does trigger a custom event `phoenix.link.click` on the clicked DOM element\nwhen a click happened. This allows you to intercept the event on its way bubbling up\nto `window` and do your own custom logic to enhance or replace how the `data-confirm`\nattribute is handled. You could for example replace the browsers `confirm()` behavior with\na custom javascript implementation:\n\n```javascript\n// listen on document.body, so it's executed before the default of\n// phoenix_html, which is listening on the window object\ndocument.body.addEventListener('phoenix.link.click', function (e) {\n  // Prevent default implementation\n  e.stopPropagation();\n  // Introduce alternative implementation\n  var message = e.target.getAttribute(\"data-confirm\");\n  if(!message){ return true; }\n  vex.dialog.confirm({\n    message: message,\n    callback: function (value) {\n      if (value == false) { e.preventDefault(); }\n    }\n  })\n}, false);\n```\n\nOr you could attach your own custom behavior.\n\n```javascript\nwindow.addEventListener('phoenix.link.click', function (e) {\n  // Introduce custom behaviour\n  var message = e.target.getAttribute(\"data-prompt\");\n  var answer = e.target.getAttribute(\"data-prompt-answer\");\n  if(message && answer && (answer != window.prompt(message))) {\n    e.preventDefault();\n  }\n}, false);\n```\n\nThe latter could also be bound to any `click` event, but this way you can be sure your custom\ncode is only executed when the code of `phoenix_html.js` is run.","ref":"Phoenix.Component.html#link/1-overriding-the-default-confirm-behaviour"},{"type":"function","title":"CSRF Protection - Phoenix.Component.link/1","doc":"By default, CSRF tokens are generated through `Plug.CSRFProtection`.","ref":"Phoenix.Component.html#link/1-csrf-protection"},{"type":"function","title":"Phoenix.Component.live_component/1","doc":"A function component for rendering `Phoenix.LiveComponent` within a parent LiveView.\n\nWhile `LiveView`s can be nested, each LiveView starts its own process. A `LiveComponent` provides\nsimilar functionality to `LiveView`, except they run in the same process as the `LiveView`,\nwith its own encapsulated state. That's why they are called stateful components.","ref":"Phoenix.Component.html#live_component/1"},{"type":"function","title":"Attributes - Phoenix.Component.live_component/1","doc":"* `id` (`:string`) (required) - A unique identifier for the LiveComponent. Note the `id` won't\nnecessarily be used as the DOM `id`. That is up to the component to decide.\n\n* `module` (`:atom`) (required) - The LiveComponent module to render.\n\nAny additional attributes provided will be passed to the LiveComponent as a map of assigns.\nSee `Phoenix.LiveComponent` for more information.","ref":"Phoenix.Component.html#live_component/1-attributes"},{"type":"function","title":"Examples - Phoenix.Component.live_component/1","doc":"```heex\n<.live_component module={MyApp.WeatherComponent} id=\"thermostat\" city=\"Kraków\" />\n```","ref":"Phoenix.Component.html#live_component/1-examples"},{"type":"function","title":"Phoenix.Component.live_file_input/1","doc":"Builds a file input tag for a LiveView upload.","ref":"Phoenix.Component.html#live_file_input/1"},{"type":"function","title":"Attributes - Phoenix.Component.live_file_input/1","doc":"* `upload` (`Phoenix.LiveView.UploadConfig`) (required) - The `Phoenix.LiveView.UploadConfig` struct.\n* `accept` (`:string`) - the optional override for the accept attribute. Defaults to :accept specified by allow_upload.\n* Global attributes are accepted. Supports all globals plus: `[\"webkitdirectory\", \"required\", \"disabled\"]`.","ref":"Phoenix.Component.html#live_file_input/1-attributes"},{"type":"function","title":"Drag and Drop - Phoenix.Component.live_file_input/1","doc":"Drag and drop is supported by annotating the droppable container with a `phx-drop-target`\nattribute pointing to the UploadConfig `ref`, so the following markup is all that is required\nfor drag and drop support:\n\n```heex\n \n  <!-- ... -->\n  <.live_file_input upload={@uploads.avatar} />\n \n```","ref":"Phoenix.Component.html#live_file_input/1-drag-and-drop"},{"type":"function","title":"Examples - Phoenix.Component.live_file_input/1","doc":"Rendering a file input:\n\n```heex\n<.live_file_input upload={@uploads.avatar} />\n```\n\nRendering a file input with a label:\n\n```heex\n Avatar \n<.live_file_input upload={@uploads.avatar} />\n```","ref":"Phoenix.Component.html#live_file_input/1-examples"},{"type":"function","title":"Phoenix.Component.live_flash/2","doc":"Returns the flash message from the LiveView flash assign.","ref":"Phoenix.Component.html#live_flash/2"},{"type":"function","title":"Examples - Phoenix.Component.live_flash/2","doc":"```heex\n <%= live_flash(@flash, :info) %> \n <%= live_flash(@flash, :error) %> \n```","ref":"Phoenix.Component.html#live_flash/2-examples"},{"type":"function","title":"Phoenix.Component.live_img_preview/1","doc":"Generates an image preview on the client for a selected file.","ref":"Phoenix.Component.html#live_img_preview/1"},{"type":"function","title":"Attributes - Phoenix.Component.live_img_preview/1","doc":"* `entry` (`Phoenix.LiveView.UploadEntry`) (required) - The `Phoenix.LiveView.UploadEntry` struct.\n* `id` (`:string`) - the id of the img tag. Derived by default from the entry ref, but can be overridden as needed if you need to render a preview of the same entry multiple times on the same page. Defaults to `nil`.\n* Global attributes are accepted.","ref":"Phoenix.Component.html#live_img_preview/1-attributes"},{"type":"function","title":"Examples - Phoenix.Component.live_img_preview/1","doc":"```heex\n<%= for entry <- @uploads.avatar.entries do %>\n  <.live_img_preview entry={entry} width=\"75\" />\n<% end %>\n```\n\nWhen you need to use it multiple times, make sure that they have distinct ids\n\n```heex\n<%= for entry <- @uploads.avatar.entries do %>\n  <.live_img_preview entry={entry} width=\"75\" />\n<% end %>\n\n<%= for entry <- @uploads.avatar.entries do %>\n  <.live_img_preview id={\"modal-#{entry.ref}\"} entry={entry} width=\"500\" />\n<% end %>\n```","ref":"Phoenix.Component.html#live_img_preview/1-examples"},{"type":"function","title":"Phoenix.Component.live_render/3","doc":"Renders a LiveView within a template.\n\nThis is useful in two situations:\n\n* When rendering a child LiveView inside a LiveView.\n\n* When rendering a LiveView inside a regular (non-live) controller/view.","ref":"Phoenix.Component.html#live_render/3"},{"type":"function","title":"Options - Phoenix.Component.live_render/3","doc":"* `:session` - a map of binary keys with extra session data to be serialized and sent\nto the client. All session data currently in the connection is automatically available\nin LiveViews. You can use this option to provide extra data. Remember all session data is\nserialized and sent to the client, so you should always keep the data in the session\nto a minimum. For example, instead of storing a User struct, you should store the \"user_id\"\nand load the User when the LiveView mounts.\n\n* `:container` - an optional tuple for the HTML tag and DOM attributes to be used for the\nLiveView container. For example: `{:li, style: \"color: blue;\"}`. By default it uses the module\ndefinition container. See the \"Containers\" section below for more information.\n\n* `:id` - both the DOM ID and the ID to uniquely identify a LiveView. An `:id` is\nautomatically generated when rendering root LiveViews but it is a required option when\nrendering a child LiveView.\n\n* `:sticky` - an optional flag to maintain the LiveView across live redirects, even if it is\nnested within another LiveView. If you are rendering the sticky view within your live layout,\nmake sure that the sticky view itself does not use the same layout. You can do so by returning\n`{:ok, socket, layout: false}` from mount.","ref":"Phoenix.Component.html#live_render/3-options"},{"type":"function","title":"Examples - Phoenix.Component.live_render/3","doc":"When rendering from a controller/view, you can call:\n\n```heex\n<%= live_render(@conn, MyApp.ThermostatLive) %>\n```\n\nOr:\n\n```heex\n<%= live_render(@conn, MyApp.ThermostatLive, session: %{\"home_id\" => @home.id}) %>\n```\n\nWithin another LiveView, you must pass the `:id` option:\n\n```heex\n<%= live_render(@socket, MyApp.ThermostatLive, id: \"thermostat\") %>\n```","ref":"Phoenix.Component.html#live_render/3-examples"},{"type":"function","title":"Containers - Phoenix.Component.live_render/3","doc":"When a `LiveView` is rendered, its contents are wrapped in a container. By default,\nthe container is a `div` tag with a handful of `LiveView` specific attributes.\n\nThe container can be customized in different ways:\n\n* You can change the default `container` on `use Phoenix.LiveView`:\n\n      use Phoenix.LiveView, container: {:tr, id: \"foo-bar\"}\n\n* You can override the container tag and pass extra attributes when calling `live_render`\n(as well as on your `live` call in your router):\n\n      live_render socket, MyLiveView, container: {:tr, class: \"highlight\"}\n\nIf you don't want the container to affect layout, you can use the CSS property\n`display: contents` or a class that applies it, like Tailwind's `.contents`.\n\nBeware if you set this to `:body`, as any content injected inside the body\n(such as `Phoenix.LiveReload` features) will be discarded once the LiveView\nconnects","ref":"Phoenix.Component.html#live_render/3-containers"},{"type":"function","title":"Phoenix.Component.live_title/1","doc":"Renders a title with automatic prefix/suffix on `@page_title` updates.","ref":"Phoenix.Component.html#live_title/1"},{"type":"function","title":"Attributes - Phoenix.Component.live_title/1","doc":"* `prefix` (`:string`) - A prefix added before the content of `inner_block`. Defaults to `nil`.\n* `suffix` (`:string`) - A suffix added after the content of `inner_block`. Defaults to `nil`.","ref":"Phoenix.Component.html#live_title/1-attributes"},{"type":"function","title":"Slots - Phoenix.Component.live_title/1","doc":"* `inner_block` (required) - Content rendered inside the `title` tag.","ref":"Phoenix.Component.html#live_title/1-slots"},{"type":"function","title":"Examples - Phoenix.Component.live_title/1","doc":"```heex\n<.live_title prefix=\"MyApp – \">\n  <%= assigns[:page_title] || \"Welcome\" %>\n</.live_title>\n```\n\n```heex\n<.live_title suffix=\"- MyApp\">\n  <%= assigns[:page_title] || \"Welcome\" %>\n</.live_title>\n```","ref":"Phoenix.Component.html#live_title/1-examples"},{"type":"macro","title":"Phoenix.Component.render_slot/2","doc":"Renders a slot entry with the given optional `argument`.\n\n```heex\n<%= render_slot(@inner_block, @form) %>\n```\n\nIf the slot has no entries, nil is returned.\n\nIf multiple slot entries are defined for the same slot,`render_slot/2` will automatically render\nall entries, merging their contents. In case you want to use the entries' attributes, you need\nto iterate over the list to access each slot individually.\n\nFor example, imagine a table component:\n\n```heex\n<.table rows={@users}>\n  <:col :let={user} label=\"Name\">\n    <%= user.name %>\n  </:col>\n\n  <:col :let={user} label=\"Address\">\n    <%= user.address %>\n  </:col>\n</.table>\n```\n\nAt the top level, we pass the rows as an assign and we define a `:col` slot for each column we\nwant in the table. Each column also has a `label`, which we are going to use in the table header.\n\nInside the component, you can render the table with headers, rows, and columns:\n\n    def table(assigns) do\n      ~H\"\"\"\n       \n         \n          <%= for col <- @col do %>\n             <%= col.label %> \n          <% end %>\n         \n        <%= for row <- @rows do %>\n           \n            <%= for col <- @col do %>\n               <%= render_slot(col, row) %> \n            <% end %>\n           \n        <% end %>\n       \n      \"\"\"\n    end","ref":"Phoenix.Component.html#render_slot/2"},{"type":"macro","title":"Phoenix.Component.sigil_H/2","doc":"The `~H` sigil for writing HEEx templates inside source files.\n\n`HEEx` is a HTML-aware and component-friendly extension of Elixir Embedded\nlanguage (`EEx`) that provides:\n\n  * Built-in handling of HTML attributes\n\n  * An HTML-like notation for injecting function components\n\n  * Compile-time validation of the structure of the template\n\n  * The ability to minimize the amount of data sent over the wire\n\n  * Out-of-the-box code formatting via `mix format`","ref":"Phoenix.Component.html#sigil_H/2"},{"type":"macro","title":"Example - Phoenix.Component.sigil_H/2","doc":"~H\"\"\"\n     \n       Hello <%= @name %> \n       \n     \n    \"\"\"","ref":"Phoenix.Component.html#sigil_H/2-example"},{"type":"macro","title":"Syntax - Phoenix.Component.sigil_H/2","doc":"`HEEx` is built on top of Embedded Elixir (`EEx`). In this section, we are going to\ncover the basic constructs in `HEEx` templates as well as its syntax extensions.\n\n#","ref":"Phoenix.Component.html#sigil_H/2-syntax"},{"type":"macro","title":"Interpolation - Phoenix.Component.sigil_H/2","doc":"Both `HEEx` and `EEx` templates use `<%= ... %>` for interpolating code inside the body\nof HTML tags:\n\n```heex\n Hello, <%= @name %> \n```\n\nSimilarly, conditionals and other block Elixir constructs are supported:\n\n```heex\n<%= if @show_greeting? do %>\n   Hello, <%= @name %> \n<% end %>\n```\n\nNote we don't include the equal sign `=` in the closing `<% end %>` tag\n(because the closing tag does not output anything).\n\nThere is one important difference between `HEEx` and Elixir's builtin `EEx`.\n`HEEx` uses a specific annotation for interpolating HTML tags and attributes.\nLet's check it out.\n\n#","ref":"Phoenix.Component.html#sigil_H/2-interpolation"},{"type":"macro","title":"HEEx extension: Defining attributes - Phoenix.Component.sigil_H/2","doc":"Since `HEEx` must parse and validate the HTML structure, code interpolation using\n`<%= ... %>` and `<% ... %>` are restricted to the body (inner content) of the\nHTML/component nodes and it cannot be applied within tags.\n\nFor instance, the following syntax is invalid:\n\n```heex\n \n  ...\n \n```\n\nInstead do:\n\n```heex\n \n  ...\n \n```\n\nYou can put any Elixir expression between `{ ... }`. For example, if you want\nto set classes, where some are static and others are dynamic, you can using\nstring interpolation:\n\n```heex\n \n  ...\n \n```\n\nThe following attribute values have special meaning:\n\n* `true` - if a value is `true`, the attribute is rendered with no value at all.\n  For example, ` ` is the same as ` `;\n\n* `false` or `nil` - if a value is `false` or `nil`, the attribute is omitted.\n  Some attributes may be rendered with an empty value, for optimization\n  purposes, if it has the same effect as omitting. For example,\n  ` ` renders to ` ` while,\n  ` ` renders to ` `;\n\n* `list` (only for the `class` attribute) - each element of the list is processed\n  as a different class. `nil` and `false` elements are discarded.\n\nFor multiple dynamic attributes, you can use the same notation but without\nassigning the expression to any specific attribute.\n\n```heex\n \n  ...\n \n```\n\nThe expression inside `{...}` must be either a keyword list or a map containing\nthe key-value pairs representing the dynamic attributes.\n\n#","ref":"Phoenix.Component.html#sigil_H/2-heex-extension-defining-attributes"},{"type":"macro","title":"HEEx extension: Defining function components - Phoenix.Component.sigil_H/2","doc":"Function components are stateless components implemented as pure functions\nwith the help of the `Phoenix.Component` module. They can be either local\n(same module) or remote (external module).\n\n`HEEx` allows invoking these function components directly in the template\nusing an HTML-like notation. For example, a remote function:\n\n```heex\n \n```\n\nA local function can be invoked with a leading dot:\n\n```heex\n<.city name=\"Kraków\"/>\n```\n\nwhere the component could be defined as follows:\n\n    defmodule MyApp.Weather do\n      use Phoenix.Component\n\n      def city(assigns) do\n        ~H\"\"\"\n        The chosen city is: <%= @name %>.\n        \"\"\"\n      end\n\n      def country(assigns) do\n        ~H\"\"\"\n        The chosen country is: <%= @name %>.\n        \"\"\"\n      end\n    end\n\nIt is typically best to group related functions into a single module, as\nopposed to having many modules with a single `render/1` function. Function\ncomponents support other important features, such as slots. You can learn\nmore about components in `Phoenix.Component`.\n\n#","ref":"Phoenix.Component.html#sigil_H/2-heex-extension-defining-function-components"},{"type":"macro","title":"HEEx extension: special attributes - Phoenix.Component.sigil_H/2","doc":"Apart from normal HTML attributes, HEEx also supports some special attributes\nsuch as `:let` and `:for`.\n\n#### :let\n\nThis is used by components and slots that want to yield a value back to the\ncaller. For an example, see how `form/1` works:\n\n```heex\n<.form :let={f} for={@form} phx-change=\"validate\" phx-submit=\"save\">\n  <.input field={f[:username]} type=\"text\" />\n  ...\n</.form>\n```\n\nNotice how the variable `f`, defined by `.form` is used by your `input` component.\nThe `Phoenix.Component` module has detailed documentation on how to use and\nimplement such functionality.\n\n#### :if and :for\n\nIt is a syntax sugar for `<%= if .. do %>` and `<%= for .. do %>` that can be\nused in regular HTML, function components, and slots.\n\nFor example in an HTML tag:\n\n```heex\n \n   \n     <%= user.name %> \n   \n \n```\n\nThe snippet above will only render the table if `@admin?` is true,\nand generate a `tr` per user as you would expect from the collection.\n\n`:for` can be used similarly in function components:\n\n```heex\n<.error :for={msg <- @errors} message={msg}/>\n```\n\nWhich is equivalent to writing:\n\n```heex\n<%= for msg <- @errors do %>\n  <.error message={msg} />\n<% end %>\n```\n\nAnd `:for` in slots behaves the same way:\n\n```heex\n<.table id=\"my-table\" rows={@users}>\n  <:col :for={header <- @headers} :let={user}>\n     <%= user[:header] %> \n  </:col>\n \n```\n\nYou can also combine `:for` and `:if` for tags, components, and slot to act as a filter:\n\n```heex\n<.error :for={msg <- @errors} :if={msg != nil} message={msg} />\n```\n\nNote that unlike Elixir's regular `for`, HEEx' `:for` does not support multiple\ngenerators in one expression.","ref":"Phoenix.Component.html#sigil_H/2-heex-extension-special-attributes"},{"type":"macro","title":"Code formatting - Phoenix.Component.sigil_H/2","doc":"You can automatically format HEEx templates (.heex) and `~H` sigils\nusing `Phoenix.LiveView.HTMLFormatter`. Please check that module\nfor more information.","ref":"Phoenix.Component.html#sigil_H/2-code-formatting"},{"type":"macro","title":"Phoenix.Component.slot/2","doc":"Declares a slot. See `slot/3` for more information.","ref":"Phoenix.Component.html#slot/2"},{"type":"macro","title":"Phoenix.Component.slot/3","doc":"Declares a function component slot.","ref":"Phoenix.Component.html#slot/3"},{"type":"macro","title":"Arguments - Phoenix.Component.slot/3","doc":"* `name` - an atom defining the name of the slot. Note that slots cannot define the same name\nas any other slots or attributes declared for the same component.\n\n* `opts` - a keyword list of options. Defaults to `[]`.\n\n* `block` - a code block containing calls to `attr/3`. Defaults to `nil`.\n\n#","ref":"Phoenix.Component.html#slot/3-arguments"},{"type":"macro","title":"Options - Phoenix.Component.slot/3","doc":"* `:required` - marks a slot as required. If a caller does not pass a value for a required slot,\na compilation warning is emitted. Otherwise, an omitted slot will default to `[]`.\n\n* `:validate_attrs` - when set to `false`, no warning is emitted when a caller passes attributes\nto a slot defined without a do block. If not set, defaults to `true`.\n\n* `:doc` - documentation for the slot. Any slot attributes declared\nwill have their documentation listed alongside the slot.\n\n#","ref":"Phoenix.Component.html#slot/3-options"},{"type":"macro","title":"Slot Attributes - Phoenix.Component.slot/3","doc":"A named slot may declare attributes by passing a block with calls to `attr/3`.\n\nUnlike attributes, slot attributes cannot accept the `:default` option. Passing one\nwill result in a compile warning being issued.\n\n#","ref":"Phoenix.Component.html#slot/3-slot-attributes"},{"type":"macro","title":"The Default Slot - Phoenix.Component.slot/3","doc":"The default slot can be declared by passing `:inner_block` as the `name` of the slot.\n\nNote that the `:inner_block` slot declaration cannot accept a block. Passing one will\nresult in a compilation error.","ref":"Phoenix.Component.html#slot/3-the-default-slot"},{"type":"macro","title":"Compile-Time Validations - Phoenix.Component.slot/3","doc":"LiveView performs some validation of slots via the `:phoenix_live_view` compiler.\nWhen slots are defined, LiveView will warn at compilation time on the caller if:\n\n* A required slot of a component is missing.\n\n* An unknown slot is given.\n\n* An unknown slot attribute is given.\n\nOn the side of the function component itself, defining attributes provides the following\nquality of life improvements:\n\n* Slot documentation is generated for the component.\n\n* Calls made to the component are tracked for reflection and validation purposes.","ref":"Phoenix.Component.html#slot/3-compile-time-validations"},{"type":"macro","title":"Documentation Generation - Phoenix.Component.slot/3","doc":"Public function components that define slots will have their docs injected into the function's\ndocumentation, depending on the value of the `@doc` module attribute:\n\n* if `@doc` is a string, the slot docs are injected into that string. The optional placeholder\n`[INSERT LVATTRDOCS]` can be used to specify where in the string the docs are injected.\nOtherwise, the docs are appended to the end of the `@doc` string.\n\n* if `@doc` is unspecified, the slot docs are used as the default `@doc` string.\n\n* if `@doc` is `false`, the slot docs are omitted entirely.\n\nThe injected slot docs are formatted as a markdown list:\n\n  * `name` (required) - slot docs. Accepts attributes:\n    * `name` (`:type`) (required) - attr docs. Defaults to `:default`.\n\nBy default, all slots will have their docs injected into the function `@doc` string.\nTo hide a specific slot, you can set the value of `:doc` to `false`.","ref":"Phoenix.Component.html#slot/3-documentation-generation"},{"type":"macro","title":"Example - Phoenix.Component.slot/3","doc":"slot :header\n    slot :inner_block, required: true\n    slot :footer\n\n    def modal(assigns) do\n      ~H\"\"\"\n       \n         \n          <%= render_slot(@header) || \"Modal\" %>\n         \n         \n          <%= render_slot(@inner_block) %>\n         \n         \n          <%= render_slot(@footer) || submit_button() %>\n         \n       \n      \"\"\"\n    end\n\nAs shown in the example above, `render_slot/1` returns `nil` when an optional slot is declared\nand none is given. This can be used to attach default behaviour.","ref":"Phoenix.Component.html#slot/3-example"},{"type":"function","title":"Phoenix.Component.to_form/2","doc":"Converts a given data structure to a `Phoenix.HTML.Form`.\n\nThis is commonly used to convert a map or an Ecto changeset\ninto a form to be given to the `form/1` component.","ref":"Phoenix.Component.html#to_form/2"},{"type":"function","title":"Creating a form from params - Phoenix.Component.to_form/2","doc":"If you want to create a form based on `handle_event` parameters,\nyou could do:\n\n    def handle_event(\"submitted\", params, socket) do\n      {:noreply, assign(socket, form: to_form(params))}\n    end\n\nWhen you pass a map to `to_form/1`, it assumes said map contains\nthe form parameters, which are expected to have string keys.\n\nYou can also specify a name to nest the parameters:\n\n    def handle_event(\"submitted\", %{\"user\" => user_params}, socket) do\n      {:noreply, assign(socket, form: to_form(user_params, as: :user))}\n    end","ref":"Phoenix.Component.html#to_form/2-creating-a-form-from-params"},{"type":"function","title":"Creating a form from changesets - Phoenix.Component.to_form/2","doc":"When using changesets, the underlying data, form parameters, and\nerrors are retrieved from it. The `:as` option is automatically\ncomputed too. For example, if you have a user schema:\n\n    defmodule MyApp.Users.User do\n      use Ecto.Schema\n\n      schema \"...\" do\n        ...\n      end\n    end\n\nAnd then you create a changeset that you pass to `to_form`:\n\n    %MyApp.Users.User{}\n    |> Ecto.Changeset.change()\n    |> to_form()\n\nIn this case, once the form is submitted, the parameters will\nbe available under `%{\"user\" => user_params}`.","ref":"Phoenix.Component.html#to_form/2-creating-a-form-from-changesets"},{"type":"function","title":"Options - Phoenix.Component.to_form/2","doc":"* `:as` - the `name` prefix to be used in form inputs\n  * `:id` - the `id` prefix to be used in form inputs\n  * `:errors` - keyword list of errors (used by maps exclusively)\n\nThe underlying data may accept additional options when\nconverted to forms. For example, a map accepts `:errors`\nto list errors, but such option is not accepted by\nchangesets. `:errors` is a keyword of tuples in the shape\nof `{error_message, options_list}`. Here is an example:\n\n    to_form(%{\"search\" => nil}, errors: [search: {\"Can't be blank\", []}])\n\nIf an existing `Phoenix.HTML.Form` struct is given, the\noptions above will override its existing values if given.\nThen the remaining options are merged with the existing\nform options.\n\nErrors in a form are only displayed if the changeset's `action`\nfield is set (and it is not set to `:ignore`). Refer to\n[a note on :errors for more information](#form/1-a-note-on-errors).","ref":"Phoenix.Component.html#to_form/2-options"},{"type":"function","title":"Phoenix.Component.update/3","doc":"Updates an existing `key` with `fun` in the given `socket_or_assigns`.\n\nThe first argument is either a LiveView `socket` or an `assigns` map from function components.\n\nThe update function receives the current key's value and returns the updated value.\nRaises if the key does not exist.\n\nThe update function may also be of arity 2, in which case it receives the current key's value\nas the first argument and the current assigns as the second argument.\nRaises if the key does not exist.","ref":"Phoenix.Component.html#update/3"},{"type":"function","title":"Examples - Phoenix.Component.update/3","doc":"iex> update(socket, :count, fn count -> count + 1 end)\n    iex> update(socket, :count, &(&1 + 1))\n    iex> update(socket, :max_users_this_session, fn current_max, %{users: users} ->\n    ...>   max(current_max, length(users))\n    ...> end)","ref":"Phoenix.Component.html#update/3-examples"},{"type":"function","title":"Phoenix.Component.upload_errors/1","doc":"Returns the errors for an upload.\n\nNote this function returns errors that apply to the allowed upload as a whole. For errors that\napply to a specific uploaded entry, use `upload_errors/2`.\n\nThe following error may be returned:\n\n* `:too_many_files` - The number of selected files exceeds the `:max_entries` constraint","ref":"Phoenix.Component.html#upload_errors/1"},{"type":"function","title":"Examples - Phoenix.Component.upload_errors/1","doc":"def upload_error_to_string(:too_many_files), do: \"You have selected too many files\"\n\n```heex\n \n  <%= upload_error_to_string(err) %>\n \n```","ref":"Phoenix.Component.html#upload_errors/1-examples"},{"type":"function","title":"Phoenix.Component.upload_errors/2","doc":"Returns the entry errors for an upload.\n\nThe following errors may be returned:\n\n* `:too_large` - The entry exceeds the `:max_file_size` constraint\n* `:not_accepted` - The entry does not match the `:accept` MIME types\n* `:external_client_failure` - When external upload fails\n* `{:writer_failure, reason}` - When the custom writer fails with `reason`","ref":"Phoenix.Component.html#upload_errors/2"},{"type":"function","title":"Examples - Phoenix.Component.upload_errors/2","doc":"```elixir\ndefp upload_error_to_string(:too_large), do: \"The file is too large\"\ndefp upload_error_to_string(:not_accepted), do: \"You have selected an unacceptable file type\"\ndefp upload_error_to_string(:external_client_failure), do: \"Something went terribly wrong\"\n```\n\n```heex\n<%= for entry <- @uploads.avatar.entries do %>\n   \n    <%= upload_error_to_string(err) %>\n   \n<% end %>\n```","ref":"Phoenix.Component.html#upload_errors/2-examples"},{"type":"behaviour","title":"Phoenix.LiveComponent","doc":"LiveComponents are a mechanism to compartmentalize state, markup, and\nevents in LiveView.\n\nLiveComponents are defined by using `Phoenix.LiveComponent` and are used\nby calling `Phoenix.Component.live_component/1` in a parent LiveView.\nThey run inside the LiveView process but have their own state and\nlife-cycle. For this reason, they are also often called \"stateful components\".\nThis is a contrast to `Phoenix.Component`, also known as \"function components\",\nwhich are stateless and can only compartmentalize markup.\n\nThe smallest LiveComponent only needs to define a `c:render/1` function:\n\n    defmodule HeroComponent do\n      # In Phoenix apps, the line is typically: use MyAppWeb, :live_component\n      use Phoenix.LiveComponent\n\n      def render(assigns) do\n        ~H\"\"\"\n         <%= @content %> \n        \"\"\"\n      end\n    end\n\nA LiveComponent is rendered as:\n\n    <.live_component module={HeroComponent} id=\"hero\" content={@content} />\n\nYou must always pass the `module` and `id` attributes. The `id` will be\navailable as an assign and it must be used to uniquely identify the\ncomponent. All other attributes will be available as assigns inside the\nLiveComponent.\n\n> ##","ref":"Phoenix.LiveComponent.html"},{"type":"behaviour","title":"Functional components or live components? {: .neutral} - Phoenix.LiveComponent","doc":">\n> Generally speaking, you should prefer functional components over live\n> components, as they are a simpler abstraction, with a smaller surface\n> area. The use case for live components only arises when there is a need\n> for encapsulating both event handling and additional state.","ref":"Phoenix.LiveComponent.html#module-functional-components-or-live-components-neutral"},{"type":"behaviour","title":"Life-cycle - Phoenix.LiveComponent","doc":"#","ref":"Phoenix.LiveComponent.html#module-life-cycle"},{"type":"behaviour","title":"Mount and update - Phoenix.LiveComponent","doc":"Live components are identified by the component module and their ID.\nTherefore, two live components with the same module and ID are treated\nas the same component. We often tie the component ID to some application based ID:\n\n    <.live_component module={UserComponent} id={@user.id} user={@user} />\n\nWhen [`live_component/1`](`Phoenix.Component.live_component/1`) is called,\n`c:mount/1` is called once, when the component is first added to the page. `c:mount/1`\nreceives the `socket` as argument. Then `c:update/2` is invoked with all of the\nassigns given to [`live_component/1`](`Phoenix.Component.live_component/1`).\nIf `c:update/2` is not defined all assigns are simply merged into the socket.\nThe assigns received as the first argument of the [`update/2`](`c:Phoenix.LiveComponent.update/2`)\ncallback will only include the _new_ assigns passed from this function.\nPre-existing assigns may be found in `socket.assigns`.\n\nAfter the component is updated, `c:render/1` is called with all assigns.\nOn first render, we get:\n\n    mount(socket) -> update(assigns, socket) -> render(assigns)\n\nOn further rendering:\n\n    update(assigns, socket) -> render(assigns)\n\nThe given `id` is not automatically used as the DOM ID. If you want to set\na DOM ID, it is your responsibility to do so when rendering:\n\n    defmodule UserComponent do\n      # In Phoenix apps, the line is typically: use MyAppWeb, :live_component\n      use Phoenix.LiveComponent\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n          <%= @user.name %>\n         \n        \"\"\"\n      end\n    end\n\n#","ref":"Phoenix.LiveComponent.html#module-mount-and-update"},{"type":"behaviour","title":"Events - Phoenix.LiveComponent","doc":"LiveComponents can also implement the `c:handle_event/3` callback\nthat works exactly the same as in LiveView. For a client event to\nreach a component, the tag must be annotated with a `phx-target`.\nIf you want to send the event to yourself, you can simply use the\n`@myself` assign, which is an *internal unique reference* to the\ncomponent instance:\n\n     \n      Say hello!\n     \n\nNote that `@myself` is not set for stateless components, as they cannot\nreceive events.\n\nIf you want to target another component, you can also pass an ID\nor a class selector to any element inside the targeted component.\nFor example, if there is a `UserComponent` with the DOM ID of `\"user-13\"`,\nusing a query selector, we can send an event to it with:\n\n     \n      Say hello!\n     \n\nIn both cases, `c:handle_event/3` will be called with the\n\"say_hello\" event. When `c:handle_event/3` is called for a component,\nonly the diff of the component is sent to the client, making them\nextremely efficient.\n\nAny valid query selector for `phx-target` is supported, provided that the\nmatched nodes are children of a LiveView or LiveComponent, for example\nto send the `close` event to multiple components:\n\n     \n      Dismiss\n     \n\n#","ref":"Phoenix.LiveComponent.html#module-events"},{"type":"behaviour","title":"Update many - Phoenix.LiveComponent","doc":"Live components also support an optional `c:update_many/1` callback\nas an alternative to `c:update/2`. While `c:update/2` is called for\neach component individually, `c:update_many/1` is called with all\nLiveComponents of the same module being currently rendered/updated.\nThe advantage is that you can preload data from the database using\na single query for all components, instead of running one query per\ncomponent.\n\nTo provide a more complete understanding of why both callbacks are necessary,\nlet's see an example. Imagine you are implementing a component and the component\nneeds to load some state from the database. For example:\n\n    <.live_component module={UserComponent} id={user_id} />\n\nA possible implementation would be to load the user on the `c:update/2`\ncallback:\n\n    def update(assigns, socket) do\n      user = Repo.get!(User, assigns.id)\n      {:ok, assign(socket, :user, user)}\n    end\n\nHowever, the issue with said approach is that, if you are rendering\nmultiple user components in the same page, you have a N+1 query problem.\nBy using `c:update_many/1` instead of `c:update/2` , we receive a list\nof all assigns and sockets, allowing us to update many at once:\n\n    def update_many(assigns_sockets) do\n      list_of_ids = Enum.map(assigns_sockets, fn {assigns, _sockets} -> assigns.id end)\n\n      users =\n        from(u in User, where: u.id in ^list_of_ids, select: {u.id, u})\n        |> Repo.all()\n        |> Map.new()\n\n      Enum.map(assigns_sockets, fn {assigns, socket} ->\n        assign(socket, :user, users[assigns.id])\n      end)\n    end\n\nNow only a single query to the database will be made. In fact, the\n`update_many/1` algorithm is a breadth-first tree traversal, which means\nthat even for nested components, the amount of queries are kept to\na minimum.\n\nFinally, note that `c:update_many/1` must return an updated list of\nsockets in the same order as they are given. If `c:update_many/1` is\ndefined, `c:update/2` is not invoked.\n\n#","ref":"Phoenix.LiveComponent.html#module-update-many"},{"type":"behaviour","title":"Summary - Phoenix.LiveComponent","doc":"All of the life-cycle events are summarized in the diagram below.\nThe bubble events in white are triggers that invoke the component.\nIn blue you have component callbacks, where the underlined names\nrepresent required callbacks:\n\n```mermaid\nflowchart LR\n    *((start)):::event-.->M\n    WE([wait for parent changes]):::event-.->M\n    W([wait for events]):::event-.->H\n\n    subgraph j__transparent[\" \"]\n\n      subgraph i[\" \"]\n        direction TB\n        M(mount/1  only once ):::callback\n        M-->U\n        M-->UM\n      end\n\n      U(update/2):::callback-->A\n      UM(update_many/1):::callback-->A\n\n      subgraph j[\" \"]\n        direction TB\n        A --> |yes| R\n        H(handle_event/3):::callback-->A{any changes?}:::diamond\n      end\n\n      A --> |no| W\n\n    end\n\n    R(render/1):::callback_req-->W\n\n    classDef event fill:#fff,color:#000,stroke:#000\n    classDef diamond fill:#FFC28C,color:#000,stroke:#000\n    classDef callback fill:#B7ADFF,color:#000,stroke-width:0\n    classDef callback_req fill:#B7ADFF,color:#000,stroke-width:0,text-decoration:underline\n```","ref":"Phoenix.LiveComponent.html#module-summary"},{"type":"behaviour","title":"Managing state - Phoenix.LiveComponent","doc":"Now that we have learned how to define and use components, as well as\nhow to use `c:update_many/1` as a data loading optimization, it is important\nto talk about how to manage state in components.\n\nGenerally speaking, you want to avoid both the parent LiveView and the\nLiveComponent working on two different copies of the state. Instead, you\nshould assume only one of them to be the source of truth. Let's discuss\nthe two different approaches in detail.\n\nImagine a scenario where a LiveView represents a board with each card\nin it as a separate LiveComponent. Each card has a form to\nallow update of the card title directly in the component, as follows:\n\n    defmodule CardComponent do\n      use Phoenix.LiveComponent\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n           <%= @card.title %> \n          ...\n         \n        \"\"\"\n      end\n\n      ...\n    end\n\nWe will see how to organize the data flow to keep either the board LiveView or\nthe card LiveComponents as the source of truth.\n\n#","ref":"Phoenix.LiveComponent.html#module-managing-state"},{"type":"behaviour","title":"LiveView as the source of truth - Phoenix.LiveComponent","doc":"If the board LiveView is the source of truth, it will be responsible\nfor fetching all of the cards in a board. Then it will call\n[`live_component/1`](`Phoenix.Component.live_component/1`)\nfor each card, passing the card struct as argument to `CardComponent`:\n\n    <%= for card <- @cards do %>\n      <.live_component module={CardComponent} card={card} id={card.id} board_id={@id} />\n    <% end %>\n\nNow, when the user submits the form, `CardComponent.handle_event/3`\nwill be triggered. However, if the update succeeds, you must not\nchange the card struct inside the component. If you do so, the card\nstruct in the component will get out of sync with the LiveView.  Since\nthe LiveView is the source of truth, you should instead tell the\nLiveView that the card was updated.\n\nLuckily, because the component and the view run in the same process,\nsending a message from the LiveComponent to the parent LiveView is as\nsimple as sending a message to `self()`:\n\n    defmodule CardComponent do\n      ...\n      def handle_event(\"update_title\", %{\"title\" => title}, socket) do\n        send self(), {:updated_card, %{socket.assigns.card | title: title}}\n        {:noreply, socket}\n      end\n    end\n\nThe LiveView then receives this event using `c:Phoenix.LiveView.handle_info/2`:\n\n    defmodule BoardView do\n      ...\n      def handle_info({:updated_card, card}, socket) do\n        # update the list of cards in the socket\n        {:noreply, updated_socket}\n      end\n    end\n\nBecause the list of cards in the parent socket was updated, the parent\nLiveView will be re-rendered, sending the updated card to the component.\nSo in the end, the component does get the updated card, but always\ndriven from the parent.\n\nAlternatively, instead of having the component send a message directly to the\nparent view, the component could broadcast the update using `Phoenix.PubSub`.\nSuch as:\n\n    defmodule CardComponent do\n      ...\n      def handle_event(\"update_title\", %{\"title\" => title}, socket) do\n        message = {:updated_card, %{socket.assigns.card | title: title}}\n        Phoenix.PubSub.broadcast(MyApp.PubSub, board_topic(socket), message)\n        {:noreply, socket}\n      end\n\n      defp board_topic(socket) do\n        \"board:\" <> socket.assigns.board_id\n      end\n    end\n\nAs long as the parent LiveView subscribes to the `board: ` topic,\nit will receive updates. The advantage of using PubSub is that we get\ndistributed updates out of the box. Now, if any user connected to the\nboard changes a card, all other users will see the change.\n\n#","ref":"Phoenix.LiveComponent.html#module-liveview-as-the-source-of-truth"},{"type":"behaviour","title":"LiveComponent as the source of truth - Phoenix.LiveComponent","doc":"If each card LiveComponent is the source of truth, then the board LiveView\nmust no longer fetch the card structs from the database. Instead, the board\nLiveView must only fetch the card ids, then render each component only by\npassing an ID:\n\n    <%= for card_id <- @card_ids do %>\n      <.live_component module={CardComponent} id={card_id} board_id={@id} />\n    <% end %>\n\nNow, each CardComponent will load its own card. Of course, doing so\nper card could be expensive and lead to N queries, where N is the\nnumber of cards, so we can use the `c:update_many/1` callback to make it\nefficient.\n\nOnce the card components are started, they can each manage their own\ncard, without concerning themselves with the parent LiveView.\n\nHowever, note that components do not have a `c:Phoenix.LiveView.handle_info/2`\ncallback. Therefore, if you want to track distributed changes on a card,\nyou must have the parent LiveView receive those events and redirect them\nto the appropriate card. For example, assuming card updates are sent\nto the \"board:ID\" topic, and that the board LiveView is subscribed to\nsaid topic, one could do:\n\n    def handle_info({:updated_card, card}, socket) do\n      send_update CardComponent, id: card.id, board_id: socket.assigns.id\n      {:noreply, socket}\n    end\n\nWith `Phoenix.LiveView.send_update/3`, the `CardComponent` given by `id`\nwill be invoked, triggering the update or update_many callback, which will\nload the most up to date data from the database.\n\n#","ref":"Phoenix.LiveComponent.html#module-livecomponent-as-the-source-of-truth"},{"type":"behaviour","title":"Unifying LiveView and LiveComponent communication - Phoenix.LiveComponent","doc":"In the examples above, we have used `send/2` to communicate with LiveView\nand `send_update/2` to communicate with components. This introduces a problem:\nwhat if you have a component that may be mounted both inside a LiveView\nor another component? Given each uses a different API for exchanging data,\nthis may seem tricky at first, but an elegant solution is to use anonymous\nfunctions as callbacks. Let's see an example.\n\nIn the sections above, we wrote the following code in our `CardComponent`:\n\n```elixir\ndef handle_event(\"update_title\", %{\"title\" => title}, socket) do\n  send self(), {:updated_card, %{socket.assigns.card | title: title}}\n  {:noreply, socket}\nend\n```\n\nThe issue with this code is that, if CardComponent is mounted inside another\ncomponent, it will still message the LiveView. Not only that, this code may\nbe hard to maintain because the message sent by the component is defined far\naway from the LiveView that will receive it.\n\nInstead let's define a callback that will be invoked by CardComponent:\n\n```elixir\ndef handle_event(\"update_title\", %{\"title\" => title}, socket) do\n  socket.assigns.on_card_update.(%{socket.assigns.card | title: title})\n  {:noreply, socket}\nend\n```\n\nAnd now when initializing the CardComponent from a LiveView, we may write:\n\n```heex\n<.live_component\n  module={CardComponent}\n  card={card}\n  id={card.id}\n  board_id={@id}\n  on_card_update={fn card -> send(self(), {:updated_card, card}) end} />\n```\n\nIf initializing it inside another component, one may write:\n\n```heex\n<.live_component\n  module={CardComponent}\n  card={card}\n  id={card.id}\n  board_id={@id}\n  on_card_update={fn card -> send_update(@myself, card: card) end} />\n```\n\nThe major benefit in both cases is that the parent has explicit control\nover the messages it will receive.","ref":"Phoenix.LiveComponent.html#module-unifying-liveview-and-livecomponent-communication"},{"type":"behaviour","title":"Slots - Phoenix.LiveComponent","doc":"LiveComponent can also receive slots, in the same way as a `Phoenix.Component`:\n\n    <.live_component module={MyComponent} id={@data.id} >\n       Inner content here \n    </.live_component>\n\nIf the LiveComponent defines an `c:update/2`, be sure that the socket it returns\nincludes the `:inner_block` assign it received.\n\nSee [the docs](Phoenix.Component.html#module-slots.md) for `Phoenix.Component` for more information.","ref":"Phoenix.LiveComponent.html#module-slots"},{"type":"behaviour","title":"Live patches and live redirects - Phoenix.LiveComponent","doc":"A template rendered inside a component can use `<.link patch={...}>` and\n`<.link navigate={...}>`. Patches are always handled by the parent `LiveView`,\nas components do not provide `handle_params`.","ref":"Phoenix.LiveComponent.html#module-live-patches-and-live-redirects"},{"type":"behaviour","title":"Cost of live components - Phoenix.LiveComponent","doc":"The internal infrastructure LiveView uses to keep track of live\ncomponents is very lightweight. However, be aware that in order to\nprovide change tracking and to send diffs over the wire, all of the\ncomponents' assigns are kept in memory - exactly as it is done in\nLiveViews themselves.\n\nTherefore it is your responsibility to keep only the assigns necessary\nin each component. For example, avoid passing all of LiveView's assigns\nwhen rendering a component:\n\n    <.live_component module={MyComponent} {assigns} />\n\nInstead pass only the keys that you need:\n\n    <.live_component module={MyComponent} user={@user} org={@org} />\n\nLuckily, because LiveViews and LiveComponents are in the same process,\nthey share the data structure representations in memory. For example,\nin the code above, the view and the component will share the same copies\nof the `@user` and `@org` assigns.\n\nYou should also avoid using live components to provide abstract DOM\ncomponents. As a guideline, a good LiveComponent encapsulates\napplication concerns and not DOM functionality. For example, if you\nhave a page that shows products for sale, you can encapsulate the\nrendering of each of those products in a component. This component\nmay have many buttons and events within it. On the opposite side,\ndo not write a component that is simply encapsulating generic DOM\ncomponents. For instance, do not do this:\n\n    defmodule MyButton do\n      use Phoenix.LiveComponent\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n          <%= @text %>\n         \n        \"\"\"\n      end\n\n      def handle_event(\"click\", _, socket) do\n        _ = socket.assigns.on_click.()\n        {:noreply, socket}\n      end\n    end\n\nInstead, it is much simpler to create a function component:\n\n    def my_button(%{text: _, click: _} = assigns) do\n      ~H\"\"\"\n       \n        <%= @text %>\n       \n      \"\"\"\n    end\n\nIf you keep components mostly as an application concern with\nonly the necessary assigns, it is unlikely you will run into\nissues related to live components.","ref":"Phoenix.LiveComponent.html#module-cost-of-live-components"},{"type":"behaviour","title":"Limitations - Phoenix.LiveComponent","doc":"Live Components require a single HTML tag at the root. It is not possible\nto have components that render only text or multiple tags.","ref":"Phoenix.LiveComponent.html#module-limitations"},{"type":"macro","title":"Phoenix.LiveComponent.__using__/1","doc":"Uses LiveComponent in the current module.\n\n    use Phoenix.LiveComponent","ref":"Phoenix.LiveComponent.html#__using__/1"},{"type":"macro","title":"Options - Phoenix.LiveComponent.__using__/1","doc":"* `:global_prefixes` - the global prefixes to use for components. See\n    `Global Attributes` in `Phoenix.Component` for more information.","ref":"Phoenix.LiveComponent.html#__using__/1-options"},{"type":"callback","title":"Phoenix.LiveComponent.handle_async/3","doc":"","ref":"Phoenix.LiveComponent.html#c:handle_async/3"},{"type":"callback","title":"Phoenix.LiveComponent.handle_event/3","doc":"","ref":"Phoenix.LiveComponent.html#c:handle_event/3"},{"type":"callback","title":"Phoenix.LiveComponent.mount/1","doc":"","ref":"Phoenix.LiveComponent.html#c:mount/1"},{"type":"callback","title":"Phoenix.LiveComponent.render/1","doc":"","ref":"Phoenix.LiveComponent.html#c:render/1"},{"type":"callback","title":"Phoenix.LiveComponent.update/2","doc":"","ref":"Phoenix.LiveComponent.html#c:update/2"},{"type":"callback","title":"Phoenix.LiveComponent.update_many/1","doc":"","ref":"Phoenix.LiveComponent.html#c:update_many/1"},{"type":"behaviour","title":"Phoenix.LiveView","doc":"A LiveView is a process that receives events, updates\nits state, and renders updates to a page as diffs.\n\nTo get started, see [the Welcome guide](welcome.md).\nThis module provides advanced documentation and features\nabout using LiveView.","ref":"Phoenix.LiveView.html"},{"type":"behaviour","title":"Life-cycle - Phoenix.LiveView","doc":"A LiveView begins as a regular HTTP request and HTML response,\nand then upgrades to a stateful view on client connect,\nguaranteeing a regular HTML page even if JavaScript is disabled.\nAny time a stateful view changes or updates its socket assigns, it is\nautomatically re-rendered and the updates are pushed to the client.\n\nSocket assigns are stateful values kept on the server side in\n`Phoenix.LiveView.Socket`. This is different from the common stateless\nHTTP pattern of sending the connection state to the client in the form\nof a token or cookie and rebuilding the state on the server to service\nevery request.\n\nYou begin by rendering a LiveView typically from your router.\nWhen LiveView is first rendered, the `c:mount/3` callback is invoked\nwith the current params, the current session and the LiveView socket.\nAs in a regular request, `params` contains public data that can be\nmodified by the user. The `session` always contains private data set\nby the application itself. The `c:mount/3` callback wires up socket\nassigns necessary for rendering the view. After mounting, `c:handle_params/3`\nis invoked so uri and query params are handled. Finally, `c:render/1`\nis invoked and the HTML is sent as a regular HTML response to the\nclient.\n\nAfter rendering the static page, LiveView connects from the client\nto the server where stateful views are spawned to push rendered updates\nto the browser, and receive client events via `phx-` bindings. Just like\nthe first rendering, `c:mount/3`, is invoked  with params, session,\nand socket state. However in the connected client case, a LiveView process\nis spawned on the server, runs `c:handle_params/3` again and then pushes\nthe result of `c:render/1` to the client and continues on for the duration\nof the connection. If at any point during the stateful life-cycle a crash\nis encountered, or the client connection drops, the client gracefully\nreconnects to the server, calling `c:mount/3` and `c:handle_params/3` again.\n\nLiveView also allows attaching hooks to specific life-cycle stages with\n`attach_hook/4`.","ref":"Phoenix.LiveView.html#module-life-cycle"},{"type":"behaviour","title":"Template collocation - Phoenix.LiveView","doc":"There are two possible ways of rendering content in a LiveView. The first\none is by explicitly defining a render function, which receives `assigns`\nand returns a `HEEx` template defined with [the `~H` sigil](`Phoenix.Component.sigil_H/2`).\n\n    defmodule MyAppWeb.DemoLive do\n      use Phoenix.LiveView\n\n      def render(assigns) do\n        ~H\"\"\"\n        Hello world!\n        \"\"\"\n      end\n    end\n\nFor larger templates, you can place them in a file in the same directory\nand same name as the LiveView. For example, if the file above is placed\nat `lib/my_app_web/live/demo_live.ex`, you can also remove the\n`render/1` function altogether and put the template code at\n`lib/my_app_web/live/demo_live.html.heex`.","ref":"Phoenix.LiveView.html#module-template-collocation"},{"type":"behaviour","title":"Async Operations - Phoenix.LiveView","doc":"Performing asynchronous work is common in LiveViews and LiveComponents.\nIt allows the user to get a working UI quickly while the system fetches some\ndata in the background or talks to an external service, without blocking the\nrender or event handling. For async work, you also typically need to handle\nthe different states of the async operation, such as loading, error, and the\nsuccessful result. You also want to catch any errors or exits and translate it\nto a meaningful update in the UI rather than crashing the user experience.\n\n#","ref":"Phoenix.LiveView.html#module-async-operations"},{"type":"behaviour","title":"Async assigns - Phoenix.LiveView","doc":"The `assign_async/3` function takes a name, a list of keys which will be assigned\nasynchronously, and a function. This function will be wrapped in a `task` by\n`assign_async`, making it easy for you to return the result. This function must\nreturn an `{:ok, assigns}` or `{:error, reason}` tuple, where `assigns` is a map\nof the keys passed to `assign_async`.\nIf the function returns anything else, an error is raised.\n\nThe task is only started when the socket is connected.\n\nFor example, let's say we want to async fetch a user's organization from the database,\nas well as their profile and rank:\n\n    def mount(%{\"slug\" => slug}, _, socket) do\n      {:ok,\n       socket\n       |> assign(:foo, \"bar\")\n       |> assign_async(:org, fn -> {:ok, %{org: fetch_org!(slug)}} end)\n       |> assign_async([:profile, :rank], fn -> {:ok, %{profile: ..., rank: ...}} end)}\n    end\n\nThe state of the async operation is stored in the socket assigns within an\n`Phoenix.LiveView.AsyncResult`. It carries the loading and failed states, as\nwell as the result. For example, if we wanted to show the loading states in\nthe UI for the `:org`, our template could conditionally render the states:\n\n```heex\n Loading organization... \n <%= org.name %> loaded! \n```\n\nThe `Phoenix.Component.async_result/1` function component can also be used to\ndeclaratively render the different states using slots:\n\n```heex\n<.async_result :let={org} assign={@org}>\n  <:loading>Loading organization...</:loading>\n  <:failed :let={_failure}>there was an error loading the organization</:failed>\n  <%= org.name %>\n</.async_result>\n```\n\n#","ref":"Phoenix.LiveView.html#module-async-assigns"},{"type":"behaviour","title":"Arbitrary async operations - Phoenix.LiveView","doc":"Sometimes you need lower level control of asynchronous operations, while\nstill receiving process isolation and error handling. For this, you can use\n`start_async/3` and the `Phoenix.LiveView.AsyncResult` module directly:\n\n    def mount(%{\"id\" => id}, _, socket) do\n      {:ok,\n       socket\n       |> assign(:org, AsyncResult.loading())\n       |> start_async(:my_task, fn -> fetch_org!(id) end)}\n    end\n\n    def handle_async(:my_task, {:ok, fetched_org}, socket) do\n      %{org: org} = socket.assigns\n      {:noreply, assign(socket, :org, AsyncResult.ok(org, fetched_org))}\n    end\n\n    def handle_async(:my_task, {:exit, reason}, socket) do\n      %{org: org} = socket.assigns\n      {:noreply, assign(socket, :org, AsyncResult.failed(org, {:exit, reason}))}\n    end\n\n`start_async/3` is used to fetch the organization asynchronously. The\n`c:handle_async/3` callback is called when the task completes or exits,\nwith the results wrapped in either `{:ok, result}` or `{:exit, reason}`.\nThe `AsyncResult` module is used to directly to update the state of the\nasync operation, but you can also assign any value directly to the socket\nif you want to handle the state yourself.","ref":"Phoenix.LiveView.html#module-arbitrary-async-operations"},{"type":"behaviour","title":"Endpoint configuration - Phoenix.LiveView","doc":"LiveView accepts the following configuration in your endpoint under\nthe `:live_view` key:\n\n  * `:signing_salt` (required) - the salt used to sign data sent\n    to the client\n\n  * `:hibernate_after` (optional) - the idle time in milliseconds allowed in\n  the LiveView before compressing its own memory and state.\n  Defaults to 15000ms (15 seconds)","ref":"Phoenix.LiveView.html#module-endpoint-configuration"},{"type":"function","title":"Phoenix.LiveView.__live__/1","doc":"Defines metadata for a LiveView.\n\nThis must be returned from the `__live__` callback.\n\nIt accepts:\n\n  * `:container` - an optional tuple for the HTML tag and DOM attributes to\n    be used for the LiveView container. For example: `{:li, style: \"color: blue;\"}`.\n\n  * `:layout` - configures the layout the `LiveView` will be rendered in.\n    This layout can be overridden by on `c:mount/3` or via the `:layout`\n    option in `Phoenix.LiveView.Router.live_session/2`\n\n  * `:log` - configures the log level for the `LiveView`, either false\n    or a log level\n\n  * `:on_mount` - a list of tuples with module names and argument to be invoked\n    as `on_mount` hooks","ref":"Phoenix.LiveView.html#__live__/1"},{"type":"macro","title":"Phoenix.LiveView.__using__/1","doc":"Uses LiveView in the current module to mark it a LiveView.\n\n    use Phoenix.LiveView,\n      container: {:tr, class: \"colorized\"},\n      layout: {MyAppWeb.LayoutView, :app},\n      log: :info","ref":"Phoenix.LiveView.html#__using__/1"},{"type":"macro","title":"Options - Phoenix.LiveView.__using__/1","doc":"* `:container` - an optional tuple for the HTML tag and DOM attributes to\n    be used for the LiveView container. For example: `{:li, style: \"color: blue;\"}`.\n    See `Phoenix.Component.live_render/3` for more information and examples.\n\n  * `:global_prefixes` - the global prefixes to use for components. See\n    `Global Attributes` in `Phoenix.Component` for more information.\n\n  * `:layout` - configures the layout the `LiveView` will be rendered in.\n    This layout can be overridden by on `c:mount/3` or via the `:layout`\n    option in `Phoenix.LiveView.Router.live_session/2`\n\n  * `:log` - configures the log level for the `LiveView`, either false\n    or a Log level","ref":"Phoenix.LiveView.html#__using__/1-options"},{"type":"function","title":"Phoenix.LiveView.allow_upload/3","doc":"Allows an upload for the provided name.","ref":"Phoenix.LiveView.html#allow_upload/3"},{"type":"function","title":"Options - Phoenix.LiveView.allow_upload/3","doc":"* `:accept` - Required. A list of unique file type specifiers or the\n    atom :any to allow any kind of file. For example, `[\".jpeg\"]`, `:any`, etc.\n\n  * `:max_entries` - The maximum number of selected files to allow per\n    file input. Defaults to 1.\n\n  * `:max_file_size` - The maximum file size in bytes to allow to be uploaded.\n    Defaults 8MB. For example, `12_000_000`.\n\n  * `:chunk_size` - The chunk size in bytes to send when uploading.\n    Defaults `64_000`.\n\n  * `:chunk_timeout` - The time in milliseconds to wait before closing the\n    upload channel when a new chunk has not been received. Defaults `10_000`.\n\n  * `:external` - The 2-arity function for generating metadata for external\n    client uploaders. This function must return either `{:ok, meta, socket}`\n    or `{:error, meta, socket}` where meta is a map. See the Uploads section\n    for example usage.\n\n  * `:progress` - The optional 3-arity function for receiving progress events\n\n  * `:auto_upload` - Instructs the client to upload the file automatically\n    on file selection instead of waiting for form submits. Default false.\n\n  * `:writer` - The `Phoenix.LiveView.UploadWriter` module to use for writing\n    the uploaded chunks. Defaults to writing to a temporary file for consumption.\n    See the `Phoenix.LiveView.UploadWriter` docs for custom usage.\n\nRaises when a previously allowed upload under the same name is still active.","ref":"Phoenix.LiveView.html#allow_upload/3-options"},{"type":"function","title":"Examples - Phoenix.LiveView.allow_upload/3","doc":"allow_upload(socket, :avatar, accept: ~w(.jpg .jpeg), max_entries: 2)\n    allow_upload(socket, :avatar, accept: :any)\n\nFor consuming files automatically as they are uploaded, you can pair `auto_upload: true` with\na custom progress function to consume the entries as they are completed. For example:\n\n    allow_upload(socket, :avatar, accept: :any, progress: &handle_progress/3, auto_upload: true)\n\n    defp handle_progress(:avatar, entry, socket) do\n      if entry.done? do\n        uploaded_file =\n          consume_uploaded_entry(socket, entry, fn %{} = meta ->\n            {:ok, ...}\n          end)\n\n        {:noreply, put_flash(socket, :info, \"file #{uploaded_file.name} uploaded\")}\n      else\n        {:noreply, socket}\n      end\n    end","ref":"Phoenix.LiveView.html#allow_upload/3-examples"},{"type":"function","title":"Phoenix.LiveView.assign_async/4","doc":"Assigns keys asynchronously.\n\nWraps your function in a task linked to the caller, errors are wrapped.\nEach key passed to `assign_async/3` will be assigned to\nan `%AsyncResult{}` struct holding the status of the operation\nand the result when the function completes.\n\nThe task is only started when the socket is connected.","ref":"Phoenix.LiveView.html#assign_async/4"},{"type":"function","title":"Options - Phoenix.LiveView.assign_async/4","doc":"* `:supervisor` - allows you to specify a `Task.Supervisor` to supervise the task.\n  * `:reset` - remove previous results during async operation when true. Defaults to false","ref":"Phoenix.LiveView.html#assign_async/4-options"},{"type":"function","title":"Examples - Phoenix.LiveView.assign_async/4","doc":"def mount(%{\"slug\" => slug}, _, socket) do\n      {:ok,\n       socket\n       |> assign(:foo, \"bar\")\n       |> assign_async(:org, fn -> {:ok, %{org: fetch_org!(slug)}} end)\n       |> assign_async([:profile, :rank], fn -> {:ok, %{profile: ..., rank: ...}} end)}\n    end\n\nSee the moduledoc for more information.\n\n## `assign_async/3` and `send_update/3`\n\nSince the code inside `assign_async/3` runs in a separate process,\n`send_update(Component, data)` does not work inside `assign_async/3`,\nsince `send_update/2` assumes it is running inside the LiveView process.\nThe solution is to explicitly send the update to the LiveView:\n\n    parent = self()\n    assign_async(socket, :org, fn ->\n      # ...\n      send_update(parent, Component, data)\n    end)","ref":"Phoenix.LiveView.html#assign_async/4-examples"},{"type":"function","title":"Phoenix.LiveView.attach_hook/4","doc":"Attaches the given `fun` by `name` for the lifecycle `stage` into `socket`.\n\n> Note: This function is for server-side lifecycle callbacks.\n> For client-side hooks, see the\n> [JS Interop guide](js-interop.html#client-hooks).\n\nHooks provide a mechanism to tap into key stages of the LiveView\nlifecycle in order to bind/update assigns, intercept events,\npatches, and regular messages when necessary, and to inject\ncommon functionality. Use `attach_hook/1` on any of the following\nlifecycle stages: `:handle_params`, `:handle_event`, `:handle_info`, `:handle_async`, and\n`:after_render`. To attach a hook to the `:mount` stage, use `on_mount/1`.\n\n> Note: only `:after_render` hooks are currently supported in LiveComponents.","ref":"Phoenix.LiveView.html#attach_hook/4"},{"type":"function","title":"Return Values - Phoenix.LiveView.attach_hook/4","doc":"Lifecycle hooks take place immediately before a given lifecycle\ncallback is invoked on the LiveView. With the exception of `:after_render`,\na hook may return `{:halt, socket}` to halt the reduction, otherwise\nit must return `{:cont, socket}` so the operation may continue until\nall hooks have been invoked for the current stage.\n\nFor `:after_render` hooks, the `socket` itself must be returned.\nAny updates to the socket assigns *will not* trigger a new render\nor diff calculation to the client.","ref":"Phoenix.LiveView.html#attach_hook/4-return-values"},{"type":"function","title":"Halting the lifecycle - Phoenix.LiveView.attach_hook/4","doc":"Note that halting from a hook _will halt the entire lifecycle stage_.\nThis means that when a hook returns `{:halt, socket}` then the\nLiveView callback will **not** be invoked. This has some\nimplications.\n\n#","ref":"Phoenix.LiveView.html#attach_hook/4-halting-the-lifecycle"},{"type":"function","title":"Implications for plugin authors - Phoenix.LiveView.attach_hook/4","doc":"When defining a plugin that matches on specific callbacks, you **must**\ndefine a catch-all clause, as your hook will be invoked even for events\nyou may not be interested on.\n\n#","ref":"Phoenix.LiveView.html#attach_hook/4-implications-for-plugin-authors"},{"type":"function","title":"Implications for end-users - Phoenix.LiveView.attach_hook/4","doc":"Allowing a hook to halt the invocation of the callback means that you can\nattach hooks to intercept specific events before detaching themselves,\nwhile allowing other events to continue normally.","ref":"Phoenix.LiveView.html#attach_hook/4-implications-for-end-users"},{"type":"function","title":"Replying to events - Phoenix.LiveView.attach_hook/4","doc":"Hooks attached to the `:handle_event` stage are able to reply to client events\nby returning `{:halt, reply, socket}`. This is useful especially for [JavaScript\ninteroperability](js-interop.html#client-hooks-via-phx-hook) because a client hook\ncan push an event and receive a reply.","ref":"Phoenix.LiveView.html#attach_hook/4-replying-to-events"},{"type":"function","title":"Examples - Phoenix.LiveView.attach_hook/4","doc":"Attaching and detaching a hook:\n\n    def mount(_params, _session, socket) do\n      socket =\n        attach_hook(socket, :my_hook, :handle_event, fn\n          \"very-special-event\", _params, socket ->\n            # Handle the very special event and then detach the hook\n            {:halt, detach_hook(socket, :my_hook, :handle_event)}\n\n          _event, _params, socket ->\n            {:cont, socket}\n        end)\n\n      {:ok, socket}\n    end\n\nReplying to a client event:\n\n    # JavaScript:\n    # let Hooks = {}\n    # Hooks.ClientHook = {\n    #   mounted() {\n    #     this.pushEvent(\"ClientHook:mounted\", {hello: \"world\"}, (reply) => {\n    #       console.log(\"received reply:\", reply)\n    #     })\n    #   }\n    # }\n    # let liveSocket = new LiveSocket(\"/live\", Socket, {hooks: Hooks, ...})\n\n    def render(assigns) do\n      ~H\"\"\"\n        \n      \"\"\"\n    end\n\n    def mount(_params, _session, socket) do\n      socket =\n        attach_hook(socket, :reply_on_client_hook_mounted, :handle_event, fn\n          \"ClientHook:mounted\", params, socket ->\n            {:halt, params, socket}\n\n          _, _, socket ->\n            {:cont, socket}\n        end)\n\n      {:ok, socket}\n    end","ref":"Phoenix.LiveView.html#attach_hook/4-examples"},{"type":"function","title":"Phoenix.LiveView.cancel_async/3","doc":"Cancels an async operation if one exists.\n\nAccepts either the `%AsyncResult{}` when using `assign_async/3` or\nthe key passed to `start_async/3`.\n\nThe underlying process will be killed with the provided reason, or\n`{:shutdown, :cancel}`. if no reason is passed. For `assign_async/3`\noperations, the `:failed` field will be set to `{:exit, reason}`.\nFor `start_async/3`, the `c:handle_async/3` callback will receive\n`{:exit, reason}` as the result.\n\nReturns the `%Phoenix.LiveView.Socket{}`.","ref":"Phoenix.LiveView.html#cancel_async/3"},{"type":"function","title":"Examples - Phoenix.LiveView.cancel_async/3","doc":"cancel_async(socket, :preview)\n    cancel_async(socket, :preview, :my_reason)\n    cancel_async(socket, socket.assigns.preview)","ref":"Phoenix.LiveView.html#cancel_async/3-examples"},{"type":"function","title":"Phoenix.LiveView.cancel_upload/3","doc":"Cancels an upload for the given entry.","ref":"Phoenix.LiveView.html#cancel_upload/3"},{"type":"function","title":"Examples - Phoenix.LiveView.cancel_upload/3","doc":"<%= for entry <- @uploads.avatar.entries do %>\n      ...\n       cancel \n    <% end %>\n\n    def handle_event(\"cancel-upload\", %{\"ref\" => ref}, socket) do\n      {:noreply, cancel_upload(socket, :avatar, ref)}\n    end","ref":"Phoenix.LiveView.html#cancel_upload/3-examples"},{"type":"function","title":"Phoenix.LiveView.clear_flash/1","doc":"Clears the flash.","ref":"Phoenix.LiveView.html#clear_flash/1"},{"type":"function","title":"Examples - Phoenix.LiveView.clear_flash/1","doc":"iex> clear_flash(socket)","ref":"Phoenix.LiveView.html#clear_flash/1-examples"},{"type":"function","title":"Phoenix.LiveView.clear_flash/2","doc":"Clears a key from the flash.","ref":"Phoenix.LiveView.html#clear_flash/2"},{"type":"function","title":"Examples - Phoenix.LiveView.clear_flash/2","doc":"iex> clear_flash(socket, :info)","ref":"Phoenix.LiveView.html#clear_flash/2-examples"},{"type":"function","title":"Phoenix.LiveView.connected?/1","doc":"Returns true if the socket is connected.\n\nUseful for checking the connectivity status when mounting the view.\nFor example, on initial page render, the view is mounted statically,\nrendered, and the HTML is sent to the client. Once the client\nconnects to the server, a LiveView is then spawned and mounted\nstatefully within a process. Use `connected?/1` to conditionally\nperform stateful work, such as subscribing to pubsub topics,\nsending messages, etc.","ref":"Phoenix.LiveView.html#connected?/1"},{"type":"function","title":"Examples - Phoenix.LiveView.connected?/1","doc":"defmodule DemoWeb.ClockLive do\n      use Phoenix.LiveView\n      ...\n      def mount(_params, _session, socket) do\n        if connected?(socket), do: :timer.send_interval(1000, self(), :tick)\n\n        {:ok, assign(socket, date: :calendar.local_time())}\n      end\n\n      def handle_info(:tick, socket) do\n        {:noreply, assign(socket, date: :calendar.local_time())}\n      end\n    end","ref":"Phoenix.LiveView.html#connected?/1-examples"},{"type":"function","title":"Phoenix.LiveView.consume_uploaded_entries/3","doc":"Consumes the uploaded entries.\n\nRaises when there are still entries in progress.\nTypically called when submitting a form to handle the\nuploaded entries alongside the form data. For form submissions,\nit is guaranteed that all entries have completed before the submit event\nis invoked. Once entries are consumed, they are removed from the upload.\n\nThe function passed to consume may return a tagged tuple of the form\n`{:ok, my_result}` to collect results about the consumed entries, or\n`{:postpone, my_result}` to collect results, but postpone the file\nconsumption to be performed later.","ref":"Phoenix.LiveView.html#consume_uploaded_entries/3"},{"type":"function","title":"Examples - Phoenix.LiveView.consume_uploaded_entries/3","doc":"def handle_event(\"save\", _params, socket) do\n      uploaded_files =\n        consume_uploaded_entries(socket, :avatar, fn %{path: path}, _entry ->\n          dest = Path.join(\"priv/static/uploads\", Path.basename(path))\n          File.cp!(path, dest)\n          {:ok, Routes.static_path(socket, \"/uploads/#{Path.basename(dest)}\")}\n        end)\n      {:noreply, update(socket, :uploaded_files, &(&1 ++ uploaded_files))}\n    end","ref":"Phoenix.LiveView.html#consume_uploaded_entries/3-examples"},{"type":"function","title":"Phoenix.LiveView.consume_uploaded_entry/3","doc":"Consumes an individual uploaded entry.\n\nRaises when the entry is still in progress.\nTypically called when submitting a form to handle the\nuploaded entries alongside the form data. Once entries are consumed,\nthey are removed from the upload.\n\nThis is a lower-level feature than `consume_uploaded_entries/3` and useful\nfor scenarios where you want to consume entries as they are individually completed.\n\nLike `consume_uploaded_entries/3`, the function passed to consume may return\na tagged tuple of the form `{:ok, my_result}` to collect results about the\nconsumed entries, or `{:postpone, my_result}` to collect results,\nbut postpone the file consumption to be performed later.","ref":"Phoenix.LiveView.html#consume_uploaded_entry/3"},{"type":"function","title":"Examples - Phoenix.LiveView.consume_uploaded_entry/3","doc":"def handle_event(\"save\", _params, socket) do\n      case uploaded_entries(socket, :avatar) do\n        {[_|_] = entries, []} ->\n          uploaded_files = for entry <- entries do\n            consume_uploaded_entry(socket, entry, fn %{path: path} ->\n              dest = Path.join(\"priv/static/uploads\", Path.basename(path))\n              File.cp!(path, dest)\n              {:ok, Routes.static_path(socket, \"/uploads/#{Path.basename(dest)}\")}\n            end)\n          end\n          {:noreply, update(socket, :uploaded_files, &(&1 ++ uploaded_files))}\n\n        _ ->\n          {:noreply, socket}\n      end\n    end","ref":"Phoenix.LiveView.html#consume_uploaded_entry/3-examples"},{"type":"function","title":"Phoenix.LiveView.detach_hook/3","doc":"Detaches a hook with the given `name` from the lifecycle `stage`.\n\n> Note: This function is for server-side lifecycle callbacks.\n> For client-side hooks, see the\n> [JS Interop guide](js-interop.html#client-hooks).\n\nIf no hook is found, this function is a no-op.","ref":"Phoenix.LiveView.html#detach_hook/3"},{"type":"function","title":"Examples - Phoenix.LiveView.detach_hook/3","doc":"def handle_event(_, socket) do\n      {:noreply, detach_hook(socket, :hook_that_was_attached, :handle_event)}\n    end","ref":"Phoenix.LiveView.html#detach_hook/3-examples"},{"type":"function","title":"Phoenix.LiveView.disallow_upload/2","doc":"Revokes a previously allowed upload from `allow_upload/3`.","ref":"Phoenix.LiveView.html#disallow_upload/2"},{"type":"function","title":"Examples - Phoenix.LiveView.disallow_upload/2","doc":"disallow_upload(socket, :avatar)","ref":"Phoenix.LiveView.html#disallow_upload/2-examples"},{"type":"function","title":"Phoenix.LiveView.get_connect_info/1","doc":"","ref":"Phoenix.LiveView.html#get_connect_info/1"},{"type":"function","title":"Phoenix.LiveView.get_connect_info/2","doc":"Accesses a given connect info key from the socket.\n\nThe following keys are supported: `:peer_data`, `:trace_context_headers`,\n`:x_headers`, `:uri`, and `:user_agent`.\n\nThe connect information is available only during mount. During disconnected\nrender, all keys are available. On connected render, only the keys explicitly\ndeclared in your socket are available. See `Phoenix.Endpoint.socket/3` for\na complete description of the keys.","ref":"Phoenix.LiveView.html#get_connect_info/2"},{"type":"function","title":"Examples - Phoenix.LiveView.get_connect_info/2","doc":"The first step is to declare the `connect_info` you want to receive.\nTypically, it includes at least the session, but you must include all\nother keys you want to access on connected mount, such as `:peer_data`:\n\n    socket \"/live\", Phoenix.LiveView.Socket,\n      websocket: [connect_info: [:peer_data, session: @session_options]]\n\nThose values can now be accessed on the connected mount as\n`get_connect_info/2`:\n\n    def mount(_params, _session, socket) do\n      peer_data = get_connect_info(socket, :peer_data)\n      {:ok, assign(socket, ip: peer_data.address)}\n    end\n\nIf the key is not available, usually because it was not specified\nin `connect_info`, it returns nil.","ref":"Phoenix.LiveView.html#get_connect_info/2-examples"},{"type":"function","title":"Phoenix.LiveView.get_connect_params/1","doc":"Accesses the connect params sent by the client for use on connected mount.\n\nConnect params are only sent when the client connects to the server and\nonly remain available during mount. `nil` is returned when called in a\ndisconnected state and a `RuntimeError` is raised if called after mount.","ref":"Phoenix.LiveView.html#get_connect_params/1"},{"type":"function","title":"Reserved params - Phoenix.LiveView.get_connect_params/1","doc":"The following params have special meaning in LiveView:\n\n  * `\"_csrf_token\"` - the CSRF Token which must be explicitly set by the user\n    when connecting\n  * `\"_mounts\"` - the number of times the current LiveView is mounted.\n    It is 0 on first mount, then increases on each reconnect. It resets\n    when navigating away from the current LiveView or on errors\n  * `\"_track_static\"` - set automatically with a list of all href/src from\n    tags with the `phx-track-static` annotation in them. If there are no\n    such tags, nothing is sent\n  * `\"_live_referer\"` - sent by the client as the referer URL when a\n    live navigation has occurred from `push_navigate` or client link navigate.","ref":"Phoenix.LiveView.html#get_connect_params/1-reserved-params"},{"type":"function","title":"Examples - Phoenix.LiveView.get_connect_params/1","doc":"def mount(_params, _session, socket) do\n      {:ok, assign(socket, width: get_connect_params(socket)[\"width\"] || @width)}\n    end","ref":"Phoenix.LiveView.html#get_connect_params/1-examples"},{"type":"callback","title":"Phoenix.LiveView.handle_async/3","doc":"Invoked when the result of an `start_async/3` operation is available.\n\nFor a deeper understanding of using this callback,\nrefer to the [\"Arbitrary async operations\"](#module-arbitrary-async-operations) section.","ref":"Phoenix.LiveView.html#c:handle_async/3"},{"type":"callback","title":"Phoenix.LiveView.handle_call/3","doc":"Invoked to handle calls from other Elixir processes.\n\nSee `GenServer.call/3` and `c:GenServer.handle_call/3`\nfor more information.","ref":"Phoenix.LiveView.html#c:handle_call/3"},{"type":"callback","title":"Phoenix.LiveView.handle_cast/2","doc":"Invoked to handle casts from other Elixir processes.\n\nSee `GenServer.cast/2` and `c:GenServer.handle_cast/2`\nfor more information. It must always return `{:noreply, socket}`,\nwhere `:noreply` means no additional information is sent\nto the process which cast the message.","ref":"Phoenix.LiveView.html#c:handle_cast/2"},{"type":"callback","title":"Phoenix.LiveView.handle_event/3","doc":"Invoked to handle events sent by the client.\n\nIt receives the `event` name, the event payload as a map,\nand the socket.\n\nIt must return `{:noreply, socket}`, where `:noreply` means\nno additional information is sent to the client, or\n`{:reply, map(), socket}`, where the given `map()` is encoded\nand sent as a reply to the client.","ref":"Phoenix.LiveView.html#c:handle_event/3"},{"type":"callback","title":"Phoenix.LiveView.handle_info/2","doc":"Invoked to handle messages from other Elixir processes.\n\nSee `Kernel.send/2` and `c:GenServer.handle_info/2`\nfor more information. It must always return `{:noreply, socket}`,\nwhere `:noreply` means no additional information is sent\nto the process which sent the message.","ref":"Phoenix.LiveView.html#c:handle_info/2"},{"type":"callback","title":"Phoenix.LiveView.handle_params/3","doc":"Invoked after mount and whenever there is a live patch event.\n\nIt receives the current `params`, including parameters from\nthe router, the current `uri` from the client and the `socket`.\nIt is invoked after mount or whenever there is a live navigation\nevent caused by `push_patch/2` or `<.link patch={...}>`.\n\nIt must always return `{:noreply, socket}`, where `:noreply`\nmeans no additional information is sent to the client.","ref":"Phoenix.LiveView.html#c:handle_params/3"},{"type":"callback","title":"Phoenix.LiveView.mount/3","doc":"The LiveView entry-point.\n\nFor each LiveView in the root of a template, `c:mount/3` is invoked twice:\nonce to do the initial page load and again to establish the live socket.\n\nIt expects three arguments:\n\n  * `params` - a map of string keys which contain public information that\n    can be set by the user. The map contains the query params as well as any\n    router path parameter. If the LiveView was not mounted at the router,\n    this argument is the atom `:not_mounted_at_router`\n  * `session` - the connection session\n  * `socket` - the LiveView socket\n\nIt must return either `{:ok, socket}` or `{:ok, socket, options}`, where\n`options` is one of:\n\n  * `:temporary_assigns` - a keyword list of assigns that are temporary\n    and must be reset to their value after every render. Note that once\n    the value is reset, it won't be re-rendered again until it is explicitly\n    assigned\n\n  * `:layout` - the optional layout to be used by the LiveView. Setting\n    this option will override any layout previously set via\n    `Phoenix.LiveView.Router.live_session/2` or on `use Phoenix.LiveView`","ref":"Phoenix.LiveView.html#c:mount/3"},{"type":"macro","title":"Phoenix.LiveView.on_mount/1","doc":"Declares a module callback to be invoked on the LiveView's mount.\n\nThe function within the given module, which must be named `on_mount`,\nwill be invoked before both disconnected and connected mounts. The hook\nhas the option to either halt or continue the mounting process as usual.\nIf you wish to redirect the LiveView, you **must** halt, otherwise an error\nwill be raised.\n\nTip: if you need to define multiple `on_mount` callbacks, avoid defining\nmultiple modules. Instead, pass a tuple and use pattern matching to handle\ndifferent cases:\n\n    def on_mount(:admin, _params, _session, socket) do\n      {:cont, socket}\n    end\n\n    def on_mount(:user, _params, _session, socket) do\n      {:cont, socket}\n    end\n\nAnd then invoke it as:\n\n    on_mount {MyAppWeb.SomeHook, :admin}\n    on_mount {MyAppWeb.SomeHook, :user}\n\nRegistering `on_mount` hooks can be useful to perform authentication\nas well as add custom behaviour to other callbacks via `attach_hook/4`.\n\nThe `on_mount` callback can return a keyword list of options as a third\nelement in the return tuple. These options are identical to what can\noptionally be returned in `mount/3`.","ref":"Phoenix.LiveView.html#on_mount/1"},{"type":"macro","title":"Examples - Phoenix.LiveView.on_mount/1","doc":"The following is an example of attaching a hook via\n`Phoenix.LiveView.Router.live_session/3`:\n\n    # lib/my_app_web/live/init_assigns.ex\n    defmodule MyAppWeb.InitAssigns do\n      @moduledoc \"\"\"\n      Ensures common `assigns` are applied to all LiveViews attaching this hook.\n      \"\"\"\n      import Phoenix.LiveView\n      import Phoenix.Component\n\n      def on_mount(:default, _params, _session, socket) do\n        {:cont, assign(socket, :page_title, \"DemoWeb\")}\n      end\n\n      def on_mount(:user, params, session, socket) do\n        # code\n      end\n\n      def on_mount(:admin, _params, _session, socket) do\n        {:cont, socket, layout: {DemoWeb.Layouts, :admin}}\n      end\n    end\n\n    # lib/my_app_web/router.ex\n    defmodule MyAppWeb.Router do\n      use MyAppWeb, :router\n\n      # pipelines, plugs, etc.\n\n      live_session :default, on_mount: MyAppWeb.InitAssigns do\n        scope \"/\", MyAppWeb do\n          pipe_through :browser\n          live \"/\", PageLive, :index\n        end\n      end\n\n      live_session :authenticated, on_mount: {MyAppWeb.InitAssigns, :user} do\n        scope \"/\", MyAppWeb do\n          pipe_through [:browser, :require_user]\n          live \"/profile\", UserLive.Profile, :index\n        end\n      end\n\n      live_session :admins, on_mount: {MyAppWeb.InitAssigns, :admin} do\n        scope \"/admin\", MyAppWeb.Admin do\n          pipe_through [:browser, :require_user, :require_admin]\n          live \"/\", AdminLive.Index, :index\n        end\n      end\n    end","ref":"Phoenix.LiveView.html#on_mount/1-examples"},{"type":"function","title":"Phoenix.LiveView.push_event/3","doc":"Pushes an event to the client.\n\nEvents can be handled in two ways:\n\n  1. They can be handled on `window` via `addEventListener`.\n     A \"phx:\" prefix will be added to the event name.\n\n  2. They can be handled inside a hook via `handleEvent`.\n\nEvents are dispatched to all active hooks on the client who are\nhandling the given `event`. If you need to scope events, then\nthis must be done by namespacing them.\n\nEvents pushed during `push_redirect` are currently discarded,\nas the LiveView is immediately dismounted.","ref":"Phoenix.LiveView.html#push_event/3"},{"type":"function","title":"Hook example - Phoenix.LiveView.push_event/3","doc":"If you push a \"scores\" event from your LiveView:\n\n    {:noreply, push_event(socket, \"scores\", %{points: 100, user: \"josé\"})}\n\nA hook declared via `phx-hook` can handle it via `handleEvent`:\n\n    this.handleEvent(\"scores\", data => ...)\n\n## `window` example\n\nAll events are also dispatched on the `window`. This means you can handle\nthem by adding listeners. For example, if you want to remove an element\nfrom the page, you can do this:\n\n    {:noreply, push_event(socket, \"remove-el\", %{id: \"foo-bar\"})}\n\nAnd now in your app.js you can register and handle it:\n\n    window.addEventListener(\n      \"phx:remove-el\",\n      e => document.getElementById(e.detail.id).remove()\n    )","ref":"Phoenix.LiveView.html#push_event/3-hook-example"},{"type":"function","title":"Phoenix.LiveView.push_navigate/2","doc":"Annotates the socket for navigation to another LiveView.\n\nThe current LiveView will be shutdown and a new one will be mounted\nin its place, without reloading the whole page. This can\nalso be used to remount the same LiveView, in case you want to start\nfresh. If you want to navigate to the same LiveView without remounting\nit, use `push_patch/2` instead.","ref":"Phoenix.LiveView.html#push_navigate/2"},{"type":"function","title":"Options - Phoenix.LiveView.push_navigate/2","doc":"* `:to` - the required path to link to. It must always be a local path\n  * `:replace` - the flag to replace the current history or push a new state.\n    Defaults `false`.","ref":"Phoenix.LiveView.html#push_navigate/2-options"},{"type":"function","title":"Examples - Phoenix.LiveView.push_navigate/2","doc":"{:noreply, push_navigate(socket, to: \"/\")}\n    {:noreply, push_navigate(socket, to: \"/\", replace: true)}","ref":"Phoenix.LiveView.html#push_navigate/2-examples"},{"type":"function","title":"Phoenix.LiveView.push_patch/2","doc":"Annotates the socket for navigation within the current LiveView.\n\nWhen navigating to the current LiveView, `c:handle_params/3` is\nimmediately invoked to handle the change of params and URL state.\nThen the new state is pushed to the client, without reloading the\nwhole page while also maintaining the current scroll position.\nFor live navigation to another LiveView, use `push_navigate/2`.","ref":"Phoenix.LiveView.html#push_patch/2"},{"type":"function","title":"Options - Phoenix.LiveView.push_patch/2","doc":"* `:to` - the required path to link to. It must always be a local path\n  * `:replace` - the flag to replace the current history or push a new state.\n    Defaults `false`.","ref":"Phoenix.LiveView.html#push_patch/2-options"},{"type":"function","title":"Examples - Phoenix.LiveView.push_patch/2","doc":"{:noreply, push_patch(socket, to: \"/\")}\n    {:noreply, push_patch(socket, to: \"/\", replace: true)}","ref":"Phoenix.LiveView.html#push_patch/2-examples"},{"type":"function","title":"Phoenix.LiveView.push_redirect/2","doc":"Annotates the socket for navigation to another LiveView.\n\nThe current LiveView will be shutdown and a new one will be mounted\nin its place, without reloading the whole page. This can\nalso be used to remount the same LiveView, in case you want to start\nfresh. If you want to navigate to the same LiveView without remounting\nit, use `push_patch/2` instead.","ref":"Phoenix.LiveView.html#push_redirect/2"},{"type":"function","title":"Options - Phoenix.LiveView.push_redirect/2","doc":"* `:to` - the required path to link to. It must always be a local path\n  * `:replace` - the flag to replace the current history or push a new state.\n    Defaults `false`.","ref":"Phoenix.LiveView.html#push_redirect/2-options"},{"type":"function","title":"Examples - Phoenix.LiveView.push_redirect/2","doc":"{:noreply, push_redirect(socket, to: \"/\")}\n    {:noreply, push_redirect(socket, to: \"/\", replace: true)}","ref":"Phoenix.LiveView.html#push_redirect/2-examples"},{"type":"function","title":"Phoenix.LiveView.put_flash/3","doc":"Adds a flash message to the socket to be displayed.\n\n*Note*: While you can use `put_flash/3` inside a `Phoenix.LiveComponent`,\ncomponents have their own `@flash` assigns. The `@flash` assign\nin a component is only copied to its parent LiveView if the component\ncalls `push_navigate/2` or `push_patch/2`.\n\n*Note*: You must also place the `Phoenix.LiveView.Router.fetch_live_flash/2`\nplug in your browser's pipeline in place of `fetch_flash` for LiveView flash\nmessages be supported, for example:\n\n    import Phoenix.LiveView.Router\n\n    pipeline :browser do\n      ...\n      plug :fetch_live_flash\n    end","ref":"Phoenix.LiveView.html#put_flash/3"},{"type":"function","title":"Examples - Phoenix.LiveView.put_flash/3","doc":"iex> put_flash(socket, :info, \"It worked!\")\n    iex> put_flash(socket, :error, \"You can't access that page\")","ref":"Phoenix.LiveView.html#put_flash/3-examples"},{"type":"function","title":"Phoenix.LiveView.put_private/3","doc":"Puts a new private key and value in the socket.\n\nPrivates are *not change tracked*. This storage is meant to be used by\nusers and libraries to hold state that doesn't require\nchange tracking. The keys should be prefixed with the app/library name.","ref":"Phoenix.LiveView.html#put_private/3"},{"type":"function","title":"Examples - Phoenix.LiveView.put_private/3","doc":"Key values can be placed in private:\n\n    put_private(socket, :myapp_meta, %{foo: \"bar\"})\n\nAnd then retrieved:\n\n    socket.private[:myapp_meta]","ref":"Phoenix.LiveView.html#put_private/3-examples"},{"type":"function","title":"Phoenix.LiveView.redirect/2","doc":"Annotates the socket for redirect to a destination path.\n\n*Note*: LiveView redirects rely on instructing client\nto perform a `window.location` update on the provided\nredirect location. The whole page will be reloaded and\nall state will be discarded.","ref":"Phoenix.LiveView.html#redirect/2"},{"type":"function","title":"Options - Phoenix.LiveView.redirect/2","doc":"* `:to` - the path to redirect to. It must always be a local path\n  * `:external` - an external path to redirect to. Either a string\n    or `{scheme, url}` to redirect to a custom scheme","ref":"Phoenix.LiveView.html#redirect/2-options"},{"type":"function","title":"Examples - Phoenix.LiveView.redirect/2","doc":"{:noreply, redirect(socket, to: \"/\")}\n    {:noreply, redirect(socket, external: \"https://example.com\")}","ref":"Phoenix.LiveView.html#redirect/2-examples"},{"type":"callback","title":"Phoenix.LiveView.render/1","doc":"Renders a template.\n\nThis callback is invoked whenever LiveView detects\nnew content must be rendered and sent to the client.\n\nIf you define this function, it must return a template\ndefined via the `Phoenix.Component.sigil_H/2`.\n\nIf you don't define this function, LiveView will attempt\nto render a template in the same directory as your LiveView.\nFor example, if you have a LiveView named `MyApp.MyCustomView`\ninside `lib/my_app/live_views/my_custom_view.ex`, Phoenix\nwill look for a template at `lib/my_app/live_views/my_custom_view.html.heex`.","ref":"Phoenix.LiveView.html#c:render/1"},{"type":"function","title":"Phoenix.LiveView.render_with/2","doc":"Configures which function to use to render a LiveView/LiveComponent.\n\nBy default, LiveView invokes the `render/1` function in the same module\nthe LiveView/LiveComponent is defined, passing `assigns` as its sole\nargument. This function allows you to set a different rendering function.\n\nOne possible use case for this function is to set a different template\non disconnected render. When the user first accesses a LiveView, we will\nperform a disconnected render to send to the browser. This is useful for\nseveral reasons, such as reducing the time to first paint and for search\nengine indexing.\n\nHowever, when LiveView is gated behind an authentication page, it may be\nuseful to render a placeholder on disconnected render and perform the\nfull render once the WebSocket connects. This can be achieved with\n`render_with/2` and is particularly useful on complex pages (such as\ndashboards and reports).\n\nTo do so, you must simply invoke `render_with(socket, &some_function_component/1)`,\nconfiguring your socket with a new rendering function.","ref":"Phoenix.LiveView.html#render_with/2"},{"type":"function","title":"Phoenix.LiveView.send_update/3","doc":"Asynchronously updates a `Phoenix.LiveComponent` with new assigns.\n\nThe `pid` argument is optional and it defaults to the current process,\nwhich means the update instruction will be sent to a component running\non the same LiveView. If the current process is not a LiveView or you\nwant to send updates to a live component running on another LiveView,\nyou should explicitly pass the LiveView's pid instead.\n\nThe second argument can be either the value of the `@myself` or the module of\nthe live component. If you pass the module, then the `:id` that identifies\nthe component must be passed as part of the assigns.\n\nWhen the component receives the update,\n[`update_many/1`](`c:Phoenix.LiveComponent.update_many/1`) will be invoked if\nit is defined, otherwise [`update/2`](`c:Phoenix.LiveComponent.update/2`) is\ninvoked with the new assigns.  If\n[`update/2`](`c:Phoenix.LiveComponent.update/2`) is not defined all assigns\nare simply merged into the socket. The assigns received as the first argument\nof the [`update/2`](`c:Phoenix.LiveComponent.update/2`) callback will only\ninclude the _new_ assigns passed from this function.  Pre-existing assigns may\nbe found in `socket.assigns`.\n\nWhile a component may always be updated from the parent by updating some\nparent assigns which will re-render the child, thus invoking\n[`update/2`](`c:Phoenix.LiveComponent.update/2`) on the child component,\n`send_update/3` is useful for updating a component that entirely manages its\nown state, as well as messaging between components mounted in the same\nLiveView.","ref":"Phoenix.LiveView.html#send_update/3"},{"type":"function","title":"Examples - Phoenix.LiveView.send_update/3","doc":"def handle_event(\"cancel-order\", _, socket) do\n      ...\n      send_update(Cart, id: \"cart\", status: \"cancelled\")\n      {:noreply, socket}\n    end\n\n    def handle_event(\"cancel-order-asynchronously\", _, socket) do\n      ...\n      pid = self()\n\n      Task.Supervisor.start_child(MyTaskSup, fn ->\n        # Do something asynchronously\n        send_update(pid, Cart, id: \"cart\", status: \"cancelled\")\n      end)\n\n      {:noreply, socket}\n    end\n\n    def render(assigns) do\n      ~H\"\"\"\n      <.some_component on_complete={&send_update(@myself, completed: &1)} />\n      \"\"\"\n    end","ref":"Phoenix.LiveView.html#send_update/3-examples"},{"type":"function","title":"Phoenix.LiveView.send_update_after/4","doc":"Similar to `send_update/3` but the update will be delayed according to the given `time_in_milliseconds`.","ref":"Phoenix.LiveView.html#send_update_after/4"},{"type":"function","title":"Examples - Phoenix.LiveView.send_update_after/4","doc":"def handle_event(\"cancel-order\", _, socket) do\n      ...\n      send_update_after(Cart, [id: \"cart\", status: \"cancelled\"], 3000)\n      {:noreply, socket}\n    end\n\n    def handle_event(\"cancel-order-asynchronously\", _, socket) do\n      ...\n      pid = self()\n\n      Task.start(fn ->\n        # Do something asynchronously\n        send_update_after(pid, Cart, [id: \"cart\", status: \"cancelled\"], 3000)\n      end)\n\n      {:noreply, socket}\n    end","ref":"Phoenix.LiveView.html#send_update_after/4-examples"},{"type":"function","title":"Phoenix.LiveView.start_async/3","doc":"Wraps your function in an asynchronous task and invokes a callback `name` to\nhandle the result.\n\nThe task is linked to the caller and errors/exits are wrapped.\nThe result of the task is sent to the `c:handle_async/3` callback\nof the caller LiveView or LiveComponent.\n\nThe task is only started when the socket is connected.","ref":"Phoenix.LiveView.html#start_async/3"},{"type":"function","title":"Examples - Phoenix.LiveView.start_async/3","doc":"def mount(%{\"id\" => id}, _, socket) do\n      {:ok,\n       socket\n       |> assign(:org, AsyncResult.loading())\n       |> start_async(:my_task, fn -> fetch_org!(id) end)}\n    end\n\n    def handle_async(:my_task, {:ok, fetched_org}, socket) do\n      %{org: org} = socket.assigns\n      {:noreply, assign(socket, :org, AsyncResult.ok(org, fetched_org))}\n    end\n\n    def handle_async(:my_task, {:exit, reason}, socket) do\n      %{org: org} = socket.assigns\n      {:noreply, assign(socket, :org, AsyncResult.failed(org, {:exit, reason}))}\n    end\n\nSee the moduledoc for more information.","ref":"Phoenix.LiveView.html#start_async/3-examples"},{"type":"function","title":"Phoenix.LiveView.static_changed?/1","doc":"Returns true if the socket is connected and the tracked static assets have changed.\n\nThis function is useful to detect if the client is running on an outdated\nversion of the marked static files. It works by comparing the static paths\nsent by the client with the one on the server.\n\n**Note:** this functionality requires Phoenix v1.5.2 or later.\n\nTo use this functionality, the first step is to annotate which static files\nyou want to be tracked by LiveView, with the `phx-track-static`. For example:\n\n     \"/>\n     \"> \n\nNow, whenever LiveView connects to the server, it will send a copy `src`\nor `href` attributes of all tracked statics and compare those values with\nthe latest entries computed by `mix phx.digest` in the server.\n\nThe tracked statics on the client will match the ones on the server the\nhuge majority of times. However, if there is a new deployment, those values\nmay differ. You can use this function to detect those cases and show a\nbanner to the user, asking them to reload the page. To do so, first set the\nassign on mount:\n\n    def mount(params, session, socket) do\n      {:ok, assign(socket, static_changed?: static_changed?(socket))}\n    end\n\nAnd then in your views:\n\n    <%= if @static_changed? do %>\n       \n        The app has been updated. Click here to  reload .\n       \n    <% end %>\n\nIf you prefer, you can also send a JavaScript script that immediately\nreloads the page.\n\n**Note:** only set `phx-track-static` on your own assets. For example, do\nnot set it in external JavaScript files:\n\n      \n\nBecause you don't actually serve the file above, LiveView will interpret\nthe static above as missing, and this function will return true.","ref":"Phoenix.LiveView.html#static_changed?/1"},{"type":"function","title":"Phoenix.LiveView.stream/4","doc":"Assigns a new stream to the socket or inserts items into an existing stream.\n\nStreams are a mechanism for managing large collections on the client without\nkeeping the resources on the server.\n\n  * `name` - The string or atom name of the key to place under the\n    `@streams` assign.\n  * `items` - The enumerable of items to insert.\n\nThe following options are supported:\n\n  * `:at` - the index to insert or update the items in the\n    collection on the client. By default `-1` is used, which appends the items\n    to the parent DOM container. A value of `0` prepends the items.\n\n    Note that this operation is equal to inserting the items one by one, each at\n    the given index. Therefore, when inserting multiple items at an index other than `-1`,\n    the UI will display the items in reverse order:\n\n        stream(socket, :songs, [song1, song2, song3], at: 0)\n\n    In this case the UI will prepend `song1`, then `song2` and then `song3`, so it will show\n    `song3`, `song2`, `song1` and then any previously inserted items.\n\n    To insert in the order of the list, use `Enum.reverse/1`:\n\n        stream(socket, :songs, Enum.reverse([song1, song2, song3]), at: 0)\n\n  * `:reset` - the boolean to reset the stream on the client or not. Defaults\n    to `false`.\n\n  * `:limit` - the optional positive or negative number of results to limit\n    on the UI on the client. As new items are streamed, the UI will remove existing\n    items to maintain the limit. For example, to limit the stream to the last 10 items\n    in the UI while appending new items, pass a negative value:\n\n        stream(socket, :songs, songs, at: -1, limit: -10)\n\n    Likewise, to limit the stream to the first 10 items, while prepending new items,\n    pass a positive value:\n\n        stream(socket, :songs, songs, at: 0, limit: 10)\n\nOnce a stream is defined, a new `@streams` assign is available containing\nthe name of the defined streams. For example, in the above definition, the\nstream may be referenced as `@streams.songs` in your template. Stream items\nare temporary and freed from socket state immediately after the `render/1`\nfunction is invoked (or a template is rendered from disk).\n\nBy default, calling `stream/4` on an existing stream will bulk insert the new items\non the client while leaving the existing items in place. Streams may also be reset\nwhen calling `stream/4`, which we discuss below.","ref":"Phoenix.LiveView.html#stream/4"},{"type":"function","title":"Resetting a stream - Phoenix.LiveView.stream/4","doc":"To empty a stream container on the client, you can pass `:reset` with an empty list:\n\n        stream(socket, :songs, [], reset: true)\n\nOr you can replace the entire stream on the client with a new collection:\n\n        stream(socket, :songs, new_songs, reset: true)","ref":"Phoenix.LiveView.html#stream/4-resetting-a-stream"},{"type":"function","title":"Limiting a stream - Phoenix.LiveView.stream/4","doc":"It is often useful to limit the number of items in the UI while allowing the\nserver to stream new items in a fire-and-forget fashion. This prevents\nthe server from overwhelming the client with new results while also opening up\npowerful features like virtualized infinite scrolling. See a complete\nbidirectional infinite scrolling example with stream limits in the\n[scroll events guide](bindings.md#scroll-events-and-infinite-stream-pagination)\n\nWhen a stream exceeds the limit on the client, the existing items will be pruned\nbased on the number of items in the stream container and the limit direction. A\npositive limit will prune items from the end of the container, while a negative\nlimit will prune items from the beginning of the container.\n\nNote that the limit is not enforced on the first `mount/3` render (when no websocket\nconnection was established yet), as it means more data than necessary has been\nloaded. In such cases, you should only load and pass the desired amount of items\nto the stream.","ref":"Phoenix.LiveView.html#stream/4-limiting-a-stream"},{"type":"function","title":"Required DOM attributes - Phoenix.LiveView.stream/4","doc":"For stream items to be trackable on the client, the following requirements\nmust be met:\n\n  1. The parent DOM container must include a `phx-update=\"stream\"` attribute,\n     along with a unique DOM id.\n  2. Each stream item must include its DOM id on the item's element.\n\n> **Note**: Failing to place `phx-update=\"stream\"` on the **immediate parent** for\n> **each stream** will result in broken behavior.\n\nWhen consuming a stream in a template, the DOM id and item is passed as a tuple,\nallowing convenient inclusion of the DOM id for each item. For example:\n\n```heex\n \n   \n     \n       <%= song.title %> \n       <%= song.duration %> \n     \n   \n \n```\nWe consume the stream in a for comprehension by referencing the\n`@streams.songs` assign. We used the computed DOM id to populate\nthe ` ` id, then we render the table row as usual.\n\nNow `stream_insert/3` and `stream_delete/3` may be issued and new rows will\nbe inserted or deleted from the client.","ref":"Phoenix.LiveView.html#stream/4-required-dom-attributes"},{"type":"function","title":"Phoenix.LiveView.stream_configure/3","doc":"Configures a stream.\n\nThe following options are supported:\n\n  * `:dom_id` - The optional function to generate each stream item's DOM id.\n    The function accepts each stream item and converts the item to a string id.\n    By default, the `:id` field of a map or struct will be used if the item has\n    such a field, and will be prefixed by the `name` hyphenated with the id.\n    For example, the following examples are equivalent:\n\n        stream(socket, :songs, songs)\n\n        socket\n        |> stream_configure(:songs, dom_id: &(\"songs-#{&1.id}\"))\n        |> stream(:songs, songs)\n\nA stream must be configured before items are inserted, and once configured,\na stream may not be re-configured. To ensure a stream is only configured a\nsingle time in a LiveComponent, use the `mount/1` callback. For example:\n\n    def mount(socket) do\n      {:ok, stream_configure(socket, :songs, dom_id: &(\"songs-#{&1.id}\"))}\n    end\n\n    def update(assigns, socket) do\n      {:ok, stream(socket, :songs, ...)}\n    end","ref":"Phoenix.LiveView.html#stream_configure/3"},{"type":"function","title":"Phoenix.LiveView.stream_delete/3","doc":"Deletes an item from the stream.\n\nThe item's DOM is computed from the `:dom_id` provided in the `stream/3` definition.\nDelete information for this DOM id is sent to the client and the item's element\nis removed from the DOM, following the same behavior of element removal, such as\ninvoking `phx-remove` commands and executing client hook `destroyed()` callbacks.","ref":"Phoenix.LiveView.html#stream_delete/3"},{"type":"function","title":"Examples - Phoenix.LiveView.stream_delete/3","doc":"def handle_event(\"delete\", %{\"id\" => id}, socket) do\n      song = get_song!(id)\n      {:noreply, stream_delete(socket, :songs, song)}\n    end\n\nSee `stream_delete_by_dom_id/3` to remove an item without requiring the\noriginal datastructure.","ref":"Phoenix.LiveView.html#stream_delete/3-examples"},{"type":"function","title":"Phoenix.LiveView.stream_delete_by_dom_id/3","doc":"Deletes an item from the stream given its computed DOM id.\n\nBehaves just like `stream_delete/3`, but accept the precomputed DOM id,\nwhich allows deleting from a stream without fetching or building the original\nstream datastructure.","ref":"Phoenix.LiveView.html#stream_delete_by_dom_id/3"},{"type":"function","title":"Examples - Phoenix.LiveView.stream_delete_by_dom_id/3","doc":"def render(assigns) do\n      ~H\"\"\"\n       \n         \n           \n             <%= song.title %> \n              delete  \n           \n         \n       \n      \"\"\"\n    end\n\n    def handle_event(\"delete\", %{\"id\" => dom_id}, socket) do\n      {:noreply, stream_delete_by_dom_id(socket, :songs, dom_id)}\n    end","ref":"Phoenix.LiveView.html#stream_delete_by_dom_id/3-examples"},{"type":"function","title":"Phoenix.LiveView.stream_insert/4","doc":"Inserts a new item or updates an existing item in the stream.\n\nBy default, the item is appended to the parent DOM container.\nThe `:at` option may be provided to insert or update an item\nto a particular index in the collection on the client.\n\nSee `stream/4` for inserting multiple items at once.","ref":"Phoenix.LiveView.html#stream_insert/4"},{"type":"function","title":"Examples - Phoenix.LiveView.stream_insert/4","doc":"Imagine you define a stream on mount with a single item:\n\n    stream(socket, :songs, [%Song{id: 1, title: \"Song 1\"}])\n\nThen, in a callback such as `handle_info` or `handle_event`, you\ncan append a new song:\n\n    stream_insert(socket, :songs, %Song{id: 2, title: \"Song 2\"})\n\nOr prepend a new song with `at: 0`:\n\n    stream_insert(socket, :songs, %Song{id: 2, title: \"Song 2\"}, at: 0)\n\nOr updating an existing song, while also moving it to the top of the collection:\n\n    stream_insert(socket, :songs, %Song{id: 1, title: \"Song 1 updated\"}, at: 0)","ref":"Phoenix.LiveView.html#stream_insert/4-examples"},{"type":"function","title":"Updating Items - Phoenix.LiveView.stream_insert/4","doc":"As shown, an existing item on the client can be updated by issuing a `stream_insert`\nfor the existing item. When the client updates an existing item, the item will remain\nin the same location as it was previously, and will not be moved to the end of the\nparent children. To both update an existing item and move it to another position,\nissue a `stream_delete`, followed by a `stream_insert`. For example:\n\n    song = get_song!(id)\n\n    socket\n    |> stream_delete(:songs, song)\n    |> stream_insert(:songs, song, at: -1)\n\nSee `stream_delete/3` for more information on deleting items.","ref":"Phoenix.LiveView.html#stream_insert/4-updating-items"},{"type":"callback","title":"Phoenix.LiveView.terminate/2","doc":"Invoked when the LiveView is terminating.\n\nIn case of errors, this callback is only invoked if the LiveView\nis trapping exits. See `c:GenServer.terminate/2` for more info.","ref":"Phoenix.LiveView.html#c:terminate/2"},{"type":"function","title":"Phoenix.LiveView.transport_pid/1","doc":"Returns the transport pid of the socket.\n\nRaises `ArgumentError` if the socket is not connected.","ref":"Phoenix.LiveView.html#transport_pid/1"},{"type":"function","title":"Examples - Phoenix.LiveView.transport_pid/1","doc":"iex> transport_pid(socket)\n    #PID<0.107.0>","ref":"Phoenix.LiveView.html#transport_pid/1-examples"},{"type":"function","title":"Phoenix.LiveView.uploaded_entries/2","doc":"Returns the completed and in progress entries for the upload.","ref":"Phoenix.LiveView.html#uploaded_entries/2"},{"type":"function","title":"Examples - Phoenix.LiveView.uploaded_entries/2","doc":"case uploaded_entries(socket, :photos) do\n      {[_ | _] = completed, []} ->\n        # all entries are completed\n\n      {[], [_ | _] = in_progress} ->\n        # all entries are still in progress\n    end","ref":"Phoenix.LiveView.html#uploaded_entries/2-examples"},{"type":"type","title":"Phoenix.LiveView.unsigned_params/0","doc":"","ref":"Phoenix.LiveView.html#t:unsigned_params/0"},{"type":"module","title":"Phoenix.LiveView.AsyncResult","doc":"Provides a datastructure for tracking the state of an async assign.\n\nSee the `Async Operations` section of the `Phoenix.LiveView` docs for more information.","ref":"Phoenix.LiveView.AsyncResult.html"},{"type":"module","title":"Fields - Phoenix.LiveView.AsyncResult","doc":"* `:ok?` - When true, indicates the `:result` has been set successfully at least once.\n  * `:loading` - The current loading state\n  * `:failed` - The current failed state\n  * `:result` - The successful result of the async task","ref":"Phoenix.LiveView.AsyncResult.html#module-fields"},{"type":"function","title":"Phoenix.LiveView.AsyncResult.failed/2","doc":"Updates the failed state.\n\nWhen failed, the loading state will be reset to `nil`.\nIf the result was previously `ok?`, both `result` and\n`failed` will be set.","ref":"Phoenix.LiveView.AsyncResult.html#failed/2"},{"type":"function","title":"Examples - Phoenix.LiveView.AsyncResult.failed/2","doc":"iex> result = AsyncResult.loading()\n    iex> result = AsyncResult.failed(result, {:exit, :boom})\n    iex> result.failed\n    {:exit, :boom}\n    iex> result.loading\n    nil","ref":"Phoenix.LiveView.AsyncResult.html#failed/2-examples"},{"type":"function","title":"Phoenix.LiveView.AsyncResult.loading/0","doc":"Creates an async result in loading state.","ref":"Phoenix.LiveView.AsyncResult.html#loading/0"},{"type":"function","title":"Examples - Phoenix.LiveView.AsyncResult.loading/0","doc":"iex> result = AsyncResult.loading()\n    iex> result.loading\n    true\n    iex> result.ok?\n    false","ref":"Phoenix.LiveView.AsyncResult.html#loading/0-examples"},{"type":"function","title":"Phoenix.LiveView.AsyncResult.loading/1","doc":"Updates the loading state.\n\nWhen loading, the failed state will be reset to `nil`.","ref":"Phoenix.LiveView.AsyncResult.html#loading/1"},{"type":"function","title":"Examples - Phoenix.LiveView.AsyncResult.loading/1","doc":"iex> result = AsyncResult.loading(%{my: :loading_state})\n    iex> result.loading\n    %{my: :loading_state}\n    iex> result = AsyncResult.loading(result)\n    iex> result.loading\n    true","ref":"Phoenix.LiveView.AsyncResult.html#loading/1-examples"},{"type":"function","title":"Phoenix.LiveView.AsyncResult.loading/2","doc":"Updates the loading state of an existing `async_result`.\n\nWhen loading, the failed state will be reset to `nil`.\nIf the result was previously `ok?`, both `result` and\n`loading` will be set.","ref":"Phoenix.LiveView.AsyncResult.html#loading/2"},{"type":"function","title":"Examples - Phoenix.LiveView.AsyncResult.loading/2","doc":"iex> result = AsyncResult.loading()\n    iex> result = AsyncResult.loading(result, %{my: :other_state})\n    iex> result.loading\n    %{my: :other_state}","ref":"Phoenix.LiveView.AsyncResult.html#loading/2-examples"},{"type":"function","title":"Phoenix.LiveView.AsyncResult.ok/1","doc":"Creates a successful result.\n\nThe `:ok?` field will also be set to `true` to indicate this result has\ncompleted successfully at least once, regardless of future state changes.\n\n#","ref":"Phoenix.LiveView.AsyncResult.html#ok/1"},{"type":"function","title":"Examples - Phoenix.LiveView.AsyncResult.ok/1","doc":"iex> result = AsyncResult.ok(\"initial result\")\n    iex> result.ok?\n    true\n    iex> result.result\n    \"initial result\"","ref":"Phoenix.LiveView.AsyncResult.html#ok/1-examples"},{"type":"function","title":"Phoenix.LiveView.AsyncResult.ok/2","doc":"Updates the successful result.\n\nThe `:ok?` field will also be set to `true` to indicate this result has\ncompleted successfully at least once, regardless of future state changes.\n\nWhen ok'd, the loading and failed state will be reset to `nil`.","ref":"Phoenix.LiveView.AsyncResult.html#ok/2"},{"type":"function","title":"Examples - Phoenix.LiveView.AsyncResult.ok/2","doc":"iex> result = AsyncResult.loading()\n    iex> result = AsyncResult.ok(result, \"completed\")\n    iex> result.ok?\n    true\n    iex> result.result\n    \"completed\"\n    iex> result.loading\n    nil","ref":"Phoenix.LiveView.AsyncResult.html#ok/2-examples"},{"type":"module","title":"Phoenix.LiveView.Controller","doc":"Helpers for rendering LiveViews from a controller.","ref":"Phoenix.LiveView.Controller.html"},{"type":"function","title":"Phoenix.LiveView.Controller.live_render/3","doc":"Renders a live view from a Plug request and sends an HTML response\nfrom within a controller.\n\nIt also automatically sets the `@live_module` assign with the value\nof the LiveView to be rendered.","ref":"Phoenix.LiveView.Controller.html#live_render/3"},{"type":"function","title":"Options - Phoenix.LiveView.Controller.live_render/3","doc":"See `Phoenix.Component.live_render/3` for all supported options.","ref":"Phoenix.LiveView.Controller.html#live_render/3-options"},{"type":"function","title":"Examples - Phoenix.LiveView.Controller.live_render/3","doc":"defmodule ThermostatController do\n      use MyAppWeb, :controller\n\n      # \"use MyAppWeb, :controller\" should import Phoenix.LiveView.Controller.\n      # If it does not, you can either import it there or uncomment the line below:\n      # import Phoenix.LiveView.Controller\n\n      def show(conn, %{\"id\" => thermostat_id}) do\n        live_render(conn, ThermostatLive, session: %{\n          \"thermostat_id\" => thermostat_id,\n          \"current_user_id\" => get_session(conn, :user_id)\n        })\n      end\n    end","ref":"Phoenix.LiveView.Controller.html#live_render/3-examples"},{"type":"module","title":"Phoenix.LiveView.JS","doc":"Provides commands for executing JavaScript utility operations on the client.\n\nJS commands support a variety of utility operations for common client-side\nneeds, such as adding or removing CSS classes, setting or removing tag attributes,\nshowing or hiding content, and transitioning in and out with animations.\nWhile these operations can be accomplished via client-side hooks,\nJS commands are DOM-patch aware, so operations applied\nby the JS APIs will stick to elements across patches from the server.\n\nIn addition to purely client-side utilities, the JS commands include a\nrich `push` API, for extending the default `phx-` binding pushes with\noptions to customize targets, loading states, and additional payload values.","ref":"Phoenix.LiveView.JS.html"},{"type":"module","title":"Client Utility Commands - Phoenix.LiveView.JS","doc":"The following utilities are included:\n\n  * `add_class` - Add classes to elements, with optional transitions\n  * `remove_class` - Remove classes from elements, with optional transitions\n  * `set_attribute` - Set an attribute on elements\n  * `remove_attribute` - Remove an attribute from elements\n  * `toggle_attribute` - Sets or removes element attribute based on attribute presence.\n  * `show` - Show elements, with optional transitions\n  * `hide` - Hide elements, with optional transitions\n  * `toggle` - Shows or hides elements based on visibility, with optional transitions\n  * `transition` - Apply a temporary transition to elements for animations\n  * `dispatch` - Dispatch a DOM event to elements\n\nFor example, the following modal component can be shown or hidden on the\nclient without a trip to the server:\n\n    alias Phoenix.LiveView.JS\n\n    def hide_modal(js \\\\ %JS{}) do\n      js\n      |> JS.hide(transition: \"fade-out\", to: \"#modal\")\n      |> JS.hide(transition: \"fade-out-scale\", to: \"#modal-content\")\n    end\n\n    def modal(assigns) do\n      ~H\"\"\"\n       \n         \n           ✖ \n           <%= @text %> \n         \n       \n      \"\"\"\n    end","ref":"Phoenix.LiveView.JS.html#module-client-utility-commands"},{"type":"module","title":"Enhanced push events - Phoenix.LiveView.JS","doc":"The `push/1` command allows you to extend the built-in pushed event handling\nwhen a `phx-` event is pushed to the server. For example, you may wish to\ntarget a specific component, specify additional payload values to include\nwith the event, apply loading states to external elements, etc. For example,\ngiven this basic `phx-click` event:\n\n     + \n\nImagine you need to target your current component, and apply a loading state\nto the parent container while the client awaits the server acknowledgement:\n\n    alias Phoenix.LiveView.JS\n\n     + \n\nPush commands also compose with all other utilities. For example,\nto add a class when pushing:\n\n      JS.add_class(\"warmer\", to: \".thermo\")\n    }>+ \n\nAny `phx-value-*` attributes will also be included in the payload, their\nvalues will be overwritten by values given directly to `push/1`. Any\n`phx-target` attribute will also be used, and overwritten.\n\n     +","ref":"Phoenix.LiveView.JS.html#module-enhanced-push-events"},{"type":"module","title":"Custom JS events with `JS.dispatch/1` and `window.addEventListener` - Phoenix.LiveView.JS","doc":"`dispatch/1` can be used to dispatch custom JavaScript events to\nelements. For example, you can use `JS.dispatch(\"click\", to: \"#foo\")`,\nto dispatch a click event to an element.\n\nThis also means you can augment your elements with custom events,\nby using JavaScript's `window.addEventListener` and invoking them\nwith `dispatch/1`. For example, imagine you want to provide\na copy-to-clipboard functionality in your application. You can\nadd a custom event for it:\n\n    window.addEventListener(\"my_app:clipcopy\", (event) => {\n      if (\"clipboard\" in navigator) {\n        const text = event.target.textContent;\n        navigator.clipboard.writeText(text);\n      } else {\n        alert(\"Sorry, your browser does not support clipboard copy.\");\n      }\n    });\n\nNow you can have a button like this:\n\n     \n      Copy content\n     \n\nThe combination of `dispatch/1` with `window.addEventListener` is\na powerful mechanism to increase the amount of actions you can trigger\nclient-side from your LiveView code.\n\nYou can also use `window.addEventListener` to listen to events pushed\nfrom the server. You can learn more in our [JS interoperability guide](js-interop.md).","ref":"Phoenix.LiveView.JS.html#module-custom-js-events-with-js-dispatch-1-and-window-addeventlistener"},{"type":"function","title":"Phoenix.LiveView.JS.add_class/1","doc":"Adds classes to elements.\n\n  * `names` - The string of classes to add.","ref":"Phoenix.LiveView.JS.html#add_class/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.add_class/1","doc":"* `:to` - The optional DOM selector to add classes to.\n    Defaults to the interacted element.\n  * `:transition` - The string of classes to apply before adding classes or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-0\", \"opacity-100\"}`\n  * `:time` - The time to apply the transition from `:transition`.\n    Defaults 200","ref":"Phoenix.LiveView.JS.html#add_class/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.add_class/1","doc":"My Item \n     \n      highlight!","ref":"Phoenix.LiveView.JS.html#add_class/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.add_class/2","doc":"See `add_class/1`.","ref":"Phoenix.LiveView.JS.html#add_class/2"},{"type":"function","title":"Phoenix.LiveView.JS.add_class/3","doc":"See `add_class/1`.","ref":"Phoenix.LiveView.JS.html#add_class/3"},{"type":"function","title":"Phoenix.LiveView.JS.dispatch/2","doc":"Dispatches an event to the DOM.\n\n  * `event` - The string event name to dispatch.\n\n*Note*: All events dispatched are of a type\n[CustomEvent](https://developer.mozilla.org/en-US/docs/Web/API/CustomEvent),\nwith the exception of `\"click\"`. For a `\"click\"`, a\n[MouseEvent](https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent)\nis dispatched to properly simulate a UI click.\n\nFor emitted `CustomEvent`'s, the event detail will contain a `dispatcher`,\nwhich references the DOM node that dispatched the JS event to the target\nelement.","ref":"Phoenix.LiveView.JS.html#dispatch/2"},{"type":"function","title":"Options - Phoenix.LiveView.JS.dispatch/2","doc":"* `:to` - The optional DOM selector to dispatch the event to.\n    Defaults to the interacted element.\n  * `:detail` - The optional detail map to dispatch along\n    with the client event. The details will be available in the\n    `event.detail` attribute for event listeners.\n  * `:bubbles` – The boolean flag to bubble the event or not. Default `true`.","ref":"Phoenix.LiveView.JS.html#dispatch/2-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.dispatch/2","doc":"window.addEventListener(\"click\", e => console.log(\"clicked!\", e.detail))\n\n     Click me!","ref":"Phoenix.LiveView.JS.html#dispatch/2-examples"},{"type":"function","title":"Phoenix.LiveView.JS.dispatch/3","doc":"See `dispatch/2`.","ref":"Phoenix.LiveView.JS.html#dispatch/3"},{"type":"function","title":"Phoenix.LiveView.JS.exec/1","doc":"Executes JS commands located in element attributes.\n\n  * `attr` - The string attribute where the JS command is specified","ref":"Phoenix.LiveView.JS.html#exec/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.exec/1","doc":"* `:to` - The optional DOM selector to fetch the attribute from.\n    Defaults to the current element.","ref":"Phoenix.LiveView.JS.html#exec/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.exec/1","doc":"... \n     close","ref":"Phoenix.LiveView.JS.html#exec/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.exec/2","doc":"See `exec/1`.","ref":"Phoenix.LiveView.JS.html#exec/2"},{"type":"function","title":"Phoenix.LiveView.JS.exec/3","doc":"See `exec/1`.","ref":"Phoenix.LiveView.JS.html#exec/3"},{"type":"function","title":"Phoenix.LiveView.JS.focus/1","doc":"Sends focus to a selector.","ref":"Phoenix.LiveView.JS.html#focus/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.focus/1","doc":"* `:to` - The optional DOM selector to send focus to.\n    Defaults to the current element.","ref":"Phoenix.LiveView.JS.html#focus/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.focus/1","doc":"JS.focus(to: \"main\")","ref":"Phoenix.LiveView.JS.html#focus/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.focus/2","doc":"See `focus/1`.","ref":"Phoenix.LiveView.JS.html#focus/2"},{"type":"function","title":"Phoenix.LiveView.JS.focus_first/1","doc":"Sends focus to the first focusable child in selector.","ref":"Phoenix.LiveView.JS.html#focus_first/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.focus_first/1","doc":"* `:to` - The optional DOM selector to focus.\n    Defaults to the current element.","ref":"Phoenix.LiveView.JS.html#focus_first/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.focus_first/1","doc":"JS.focus_first(to: \"#modal\")","ref":"Phoenix.LiveView.JS.html#focus_first/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.focus_first/2","doc":"See `focus_first/1`.","ref":"Phoenix.LiveView.JS.html#focus_first/2"},{"type":"function","title":"Phoenix.LiveView.JS.hide/1","doc":"Hides elements.","ref":"Phoenix.LiveView.JS.html#hide/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.hide/1","doc":"* `:to` - The optional DOM selector to hide.\n    Defaults to the interacted element.\n  * `:transition` - The string of classes to apply before hiding or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-100\", \"opacity-0\"}`\n  * `:time` - The time to apply the transition from `:transition`.\n    Defaults 200\n\nDuring the process, the following events will be dispatched to the hidden elements:\n\n  * When the action is triggered on the client, `phx:hide-start` is dispatched.\n  * After the time specified by `:time`, `phx:hide-end` is dispatched.","ref":"Phoenix.LiveView.JS.html#hide/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.hide/1","doc":"My Item \n\n     \n      hide!\n     \n\n     \n      hide fancy!","ref":"Phoenix.LiveView.JS.html#hide/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.hide/2","doc":"See `hide/1`.","ref":"Phoenix.LiveView.JS.html#hide/2"},{"type":"function","title":"Phoenix.LiveView.JS.navigate/1","doc":"Sends a navigation event to the server and updates the browser's pushState history.","ref":"Phoenix.LiveView.JS.html#navigate/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.navigate/1","doc":"* `:replace` - Whether to replace the browser's pushState history. Defaults false.","ref":"Phoenix.LiveView.JS.html#navigate/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.navigate/1","doc":"JS.navigate(\"/my-path\")","ref":"Phoenix.LiveView.JS.html#navigate/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.navigate/2","doc":"See `navigate/1`.","ref":"Phoenix.LiveView.JS.html#navigate/2"},{"type":"function","title":"Phoenix.LiveView.JS.navigate/3","doc":"See `navigate/1`.","ref":"Phoenix.LiveView.JS.html#navigate/3"},{"type":"function","title":"Phoenix.LiveView.JS.patch/1","doc":"Sends a patch event to the server and updates the browser's pushState history.","ref":"Phoenix.LiveView.JS.html#patch/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.patch/1","doc":"* `:replace` - Whether to replace the browser's pushState history. Defaults false.","ref":"Phoenix.LiveView.JS.html#patch/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.patch/1","doc":"JS.patch(\"/my-path\")","ref":"Phoenix.LiveView.JS.html#patch/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.patch/2","doc":"See `patch/1`.","ref":"Phoenix.LiveView.JS.html#patch/2"},{"type":"function","title":"Phoenix.LiveView.JS.patch/3","doc":"See `patch/1`.","ref":"Phoenix.LiveView.JS.html#patch/3"},{"type":"function","title":"Phoenix.LiveView.JS.pop_focus/1","doc":"Focuses the last pushed element.","ref":"Phoenix.LiveView.JS.html#pop_focus/1"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.pop_focus/1","doc":"JS.pop_focus()","ref":"Phoenix.LiveView.JS.html#pop_focus/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.push/1","doc":"Pushes an event to the server.\n\n  * `event` - The string event name to push.","ref":"Phoenix.LiveView.JS.html#push/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.push/1","doc":"* `:target` - The selector or component ID to push to. This value will\n    overwrite any `phx-target` attribute present on the element.\n  * `:loading` - The selector to apply the phx loading classes to.\n  * `:page_loading` - Boolean to trigger the phx:page-loading-start and\n    phx:page-loading-stop events for this push. Defaults to `false`.\n  * `:value` - The map of values to send to the server. These values will be\n    merged over any `phx-value-*` attributes that are present on the element.\n    All keys will be treated as strings when merging.","ref":"Phoenix.LiveView.JS.html#push/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.push/1","doc":"click me! \n     click me! \n     click me!","ref":"Phoenix.LiveView.JS.html#push/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.push/2","doc":"See `push/1`.","ref":"Phoenix.LiveView.JS.html#push/2"},{"type":"function","title":"Phoenix.LiveView.JS.push/3","doc":"See `push/1`.","ref":"Phoenix.LiveView.JS.html#push/3"},{"type":"function","title":"Phoenix.LiveView.JS.push_focus/1","doc":"Pushes focus from the source element to be later popped.","ref":"Phoenix.LiveView.JS.html#push_focus/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.push_focus/1","doc":"* `:to` - The optional DOM selector to push focus to.\n    Defaults to the current element.","ref":"Phoenix.LiveView.JS.html#push_focus/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.push_focus/1","doc":"JS.push_focus()\n    JS.push_focus(to: \"#my-button\")","ref":"Phoenix.LiveView.JS.html#push_focus/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.push_focus/2","doc":"See `push_focus/1`.","ref":"Phoenix.LiveView.JS.html#push_focus/2"},{"type":"function","title":"Phoenix.LiveView.JS.remove_attribute/1","doc":"Removes an attribute from elements.\n\n  * `attr` - The string attribute name to remove.","ref":"Phoenix.LiveView.JS.html#remove_attribute/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.remove_attribute/1","doc":"* `:to` - The optional DOM selector to remove attributes from.\n    Defaults to the interacted element.","ref":"Phoenix.LiveView.JS.html#remove_attribute/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.remove_attribute/1","doc":"hide","ref":"Phoenix.LiveView.JS.html#remove_attribute/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.remove_attribute/2","doc":"See `remove_attribute/1`.","ref":"Phoenix.LiveView.JS.html#remove_attribute/2"},{"type":"function","title":"Phoenix.LiveView.JS.remove_attribute/3","doc":"See `remove_attribute/1`.","ref":"Phoenix.LiveView.JS.html#remove_attribute/3"},{"type":"function","title":"Phoenix.LiveView.JS.remove_class/1","doc":"Removes classes from elements.\n\n  * `names` - The string of classes to remove.","ref":"Phoenix.LiveView.JS.html#remove_class/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.remove_class/1","doc":"* `:to` - The optional DOM selector to remove classes from.\n    Defaults to the interacted element.\n  * `:transition` - The string of classes to apply before removing classes or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-0\", \"opacity-100\"}`\n  * `:time` - The time to apply the transition from `:transition`.\n    Defaults 200","ref":"Phoenix.LiveView.JS.html#remove_class/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.remove_class/1","doc":"My Item \n     \n      remove highlight!","ref":"Phoenix.LiveView.JS.html#remove_class/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.remove_class/2","doc":"See `remove_class/1`.","ref":"Phoenix.LiveView.JS.html#remove_class/2"},{"type":"function","title":"Phoenix.LiveView.JS.remove_class/3","doc":"See `remove_class/1`.","ref":"Phoenix.LiveView.JS.html#remove_class/3"},{"type":"function","title":"Phoenix.LiveView.JS.set_attribute/1","doc":"Sets an attribute on elements.\n\nAccepts a tuple containing the string attribute name/value pair.","ref":"Phoenix.LiveView.JS.html#set_attribute/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.set_attribute/1","doc":"* `:to` - The optional DOM selector to add attributes to.\n    Defaults to the interacted element.","ref":"Phoenix.LiveView.JS.html#set_attribute/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.set_attribute/1","doc":"show","ref":"Phoenix.LiveView.JS.html#set_attribute/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.set_attribute/2","doc":"See `set_attribute/1`.","ref":"Phoenix.LiveView.JS.html#set_attribute/2"},{"type":"function","title":"Phoenix.LiveView.JS.set_attribute/3","doc":"See `set_attribute/1`.","ref":"Phoenix.LiveView.JS.html#set_attribute/3"},{"type":"function","title":"Phoenix.LiveView.JS.show/1","doc":"Shows elements.","ref":"Phoenix.LiveView.JS.html#show/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.show/1","doc":"* `:to` - The optional DOM selector to show.\n    Defaults to the interacted element.\n  * `:transition` - The string of classes to apply before showing or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-0\", \"opacity-100\"}`\n  * `:time` - The time to apply the transition from `:transition`.\n    Defaults 200\n  * `:display` - The optional display value to set when showing. Defaults `\"block\"`.\n\nDuring the process, the following events will be dispatched to the shown elements:\n\n  * When the action is triggered on the client, `phx:show-start` is dispatched.\n  * After the time specified by `:time`, `phx:show-end` is dispatched.","ref":"Phoenix.LiveView.JS.html#show/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.show/1","doc":"My Item \n\n     \n      show!\n     \n\n     \n      show fancy!","ref":"Phoenix.LiveView.JS.html#show/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.show/2","doc":"See `show/1`.","ref":"Phoenix.LiveView.JS.html#show/2"},{"type":"function","title":"Phoenix.LiveView.JS.toggle/1","doc":"Toggles element visibility.","ref":"Phoenix.LiveView.JS.html#toggle/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.toggle/1","doc":"* `:to` - The optional DOM selector to toggle.\n    Defaults to the interacted element.\n  * `:in` - The string of classes to apply when toggling in, or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-0\", \"opacity-100\"}`\n  * `:out` - The string of classes to apply when toggling out, or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-100\", \"opacity-0\"}`\n  * `:time` - The time to apply the transition `:in` and `:out` classes.\n    Defaults 200\n  * `:display` - The optional display value to set when toggling in. Defaults `\"block\"`.\n\nWhen the toggle is complete on the client, a `phx:show-start` or `phx:hide-start`, and\n`phx:show-end` or `phx:hide-end` event will be dispatched to the toggled elements.","ref":"Phoenix.LiveView.JS.html#toggle/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.toggle/1","doc":"My Item \n\n     \n      toggle item!\n     \n\n     \n      toggle fancy!","ref":"Phoenix.LiveView.JS.html#toggle/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.toggle/2","doc":"See `toggle/1`.","ref":"Phoenix.LiveView.JS.html#toggle/2"},{"type":"function","title":"Phoenix.LiveView.JS.toggle_attribute/1","doc":"Sets or removes element attribute based on attribute presence.\n\nAccepts a two or three-element tuple:\n\n* `{attr, val}` - Sets the attribute to the given value or removes it\n* `{attr, val1, val2}` - Toggles the attribute between `val1` and `val2`","ref":"Phoenix.LiveView.JS.html#toggle_attribute/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.toggle_attribute/1","doc":"* `:to` - The optional DOM selector to set or remove attributes from.\n    Defaults to the interacted element.","ref":"Phoenix.LiveView.JS.html#toggle_attribute/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.toggle_attribute/1","doc":"toggle\n     \n\n     \n      toggle","ref":"Phoenix.LiveView.JS.html#toggle_attribute/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.toggle_attribute/2","doc":"See `toggle_attribute/1`.","ref":"Phoenix.LiveView.JS.html#toggle_attribute/2"},{"type":"function","title":"Phoenix.LiveView.JS.toggle_attribute/3","doc":"See `toggle_attribute/1`.","ref":"Phoenix.LiveView.JS.html#toggle_attribute/3"},{"type":"function","title":"Phoenix.LiveView.JS.toggle_class/1","doc":"Toggles classes on an element.\n\n  * `names` - The string of classes to add.","ref":"Phoenix.LiveView.JS.html#toggle_class/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.toggle_class/1","doc":"* `:to` - The optional DOM selector to toggle classes to.\n    Defaults to the interacted element.\n  * `:transition` - The string of classes to apply before adding classes or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-0\", \"opacity-100\"}`\n  * `:time` - The time to apply the transition from `:transition`.\n    Defaults 200","ref":"Phoenix.LiveView.JS.html#toggle_class/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.toggle_class/1","doc":"My Item \n     \n      toggle active!","ref":"Phoenix.LiveView.JS.html#toggle_class/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.toggle_class/2","doc":"","ref":"Phoenix.LiveView.JS.html#toggle_class/2"},{"type":"function","title":"Phoenix.LiveView.JS.toggle_class/3","doc":"","ref":"Phoenix.LiveView.JS.html#toggle_class/3"},{"type":"function","title":"Phoenix.LiveView.JS.transition/1","doc":"Transitions elements.\n\n  * `transition` - The string of classes to apply before removing classes or\n    a 3-tuple containing the transition class, the class to apply\n    to start the transition, and the ending transition class, such as:\n    `{\"ease-out duration-300\", \"opacity-0\", \"opacity-100\"}`\n\nTransitions are useful for temporarily adding an animation class\nto element(s), such as for highlighting content changes.","ref":"Phoenix.LiveView.JS.html#transition/1"},{"type":"function","title":"Options - Phoenix.LiveView.JS.transition/1","doc":"* `:to` - The optional DOM selector to apply transitions to.\n    Defaults to the interacted element.\n  * `:time` - The time to apply the transition from `:transition`.\n    Defaults 200","ref":"Phoenix.LiveView.JS.html#transition/1-options"},{"type":"function","title":"Examples - Phoenix.LiveView.JS.transition/1","doc":"My Item \n     Shake!","ref":"Phoenix.LiveView.JS.html#transition/1-examples"},{"type":"function","title":"Phoenix.LiveView.JS.transition/2","doc":"See `transition/1`.","ref":"Phoenix.LiveView.JS.html#transition/2"},{"type":"function","title":"Phoenix.LiveView.JS.transition/3","doc":"See `transition/1`.","ref":"Phoenix.LiveView.JS.html#transition/3"},{"type":"opaque","title":"Phoenix.LiveView.JS.t/0","doc":"","ref":"Phoenix.LiveView.JS.html#t:t/0"},{"type":"module","title":"Phoenix.LiveView.Router","doc":"Provides LiveView routing for Phoenix routers.","ref":"Phoenix.LiveView.Router.html"},{"type":"function","title":"Phoenix.LiveView.Router.fetch_live_flash/2","doc":"Fetches the LiveView and merges with the controller flash.\n\nReplaces the default `:fetch_flash` plug used by `Phoenix.Router`.","ref":"Phoenix.LiveView.Router.html#fetch_live_flash/2"},{"type":"function","title":"Examples - Phoenix.LiveView.Router.fetch_live_flash/2","doc":"defmodule MyAppWeb.Router do\n      use LiveGenWeb, :router\n      import Phoenix.LiveView.Router\n\n      pipeline :browser do\n        ...\n        plug :fetch_live_flash\n      end\n      ...\n    end","ref":"Phoenix.LiveView.Router.html#fetch_live_flash/2-examples"},{"type":"macro","title":"Phoenix.LiveView.Router.live/4","doc":"Defines a LiveView route.\n\nA LiveView can be routed to by using the `live` macro with a path and\nthe name of the LiveView:\n\n    live \"/thermostat\", ThermostatLive\n\nBy default, you can generate a route to this LiveView by using the `live_path` helper:\n\n    live_path(@socket, ThermostatLive)\n\n> ##","ref":"Phoenix.LiveView.Router.html#live/4"},{"type":"macro","title":"HTTP requests {: .info} - Phoenix.LiveView.Router.live/4","doc":">\n> The HTTP request method that a route defined by the `live/4` macro\n> responds to is `GET`.","ref":"Phoenix.LiveView.Router.html#live/4-http-requests-info"},{"type":"macro","title":"Actions and live navigation - Phoenix.LiveView.Router.live/4","doc":"It is common for a LiveView to have multiple states and multiple URLs.\nFor example, you can have a single LiveView that lists all articles on\nyour web app. For each article there is an \"Edit\" button which, when\npressed, opens up a modal on the same page to edit the article. It is a\nbest practice to use live navigation in those cases, so when you click\nedit, the URL changes to \"/articles/1/edit\", even though you are still\nwithin the same LiveView. Similarly, you may also want to show a \"New\"\nbutton, which opens up the modal to create new entries, and you want\nthis to be reflected in the URL as \"/articles/new\".\n\nIn order to make it easier to recognize the current \"action\" your\nLiveView is on, you can pass the action option when defining LiveViews\ntoo:\n\n    live \"/articles\", ArticleLive.Index, :index\n    live \"/articles/new\", ArticleLive.Index, :new\n    live \"/articles/:id/edit\", ArticleLive.Index, :edit\n\nWhen an action is given, the generated route helpers are named after\nthe LiveView itself (in the same way as for a controller). For the example\nabove, we will have:\n\n    article_index_path(@socket, :index)\n    article_index_path(@socket, :new)\n    article_index_path(@socket, :edit, 123)\n\nThe current action will always be available inside the LiveView as\nthe `@live_action` assign, that can be used to render a LiveComponent:\n\n    <%= if @live_action == :new do %>\n      <.live_component module={MyAppWeb.ArticleLive.FormComponent} id=\"form\" />\n    <% end %>\n\nOr can be used to show or hide parts of the template:\n\n    <%= if @live_action == :edit do %>\n      <%= render(\"form.html\", user: @user) %>\n    <% end %>\n\nNote that `@live_action` will be `nil` if no action is given on the route definition.","ref":"Phoenix.LiveView.Router.html#live/4-actions-and-live-navigation"},{"type":"macro","title":"Options - Phoenix.LiveView.Router.live/4","doc":"* `:container` - an optional tuple for the HTML tag and DOM attributes to\n    be used for the LiveView container. For example: `{:li, style: \"color: blue;\"}`.\n    See `Phoenix.Component.live_render/3` for more information and examples.\n\n  * `:as` - optionally configures the named helper. Defaults to `:live` when\n    using a LiveView without actions or defaults to the LiveView name when using\n    actions.\n\n  * `:metadata` - a map to optional feed metadata used on telemetry events and route info,\n    for example: `%{route_name: :foo, access: :user}`. This data can be retrieved by\n    calling `Phoenix.Router.route_info/4` with the `uri` from the `handle_params`\n    callback. This can be used to customize a LiveView which may be invoked from\n    different routes.\n\n  * `:private` - an optional map of private data to put in the *plug connection*,\n    for example: `%{route_name: :foo, access: :user}`. The data will be available\n    inside `conn.private` in plug functions.","ref":"Phoenix.LiveView.Router.html#live/4-options"},{"type":"macro","title":"Examples - Phoenix.LiveView.Router.live/4","doc":"defmodule MyApp.Router\n      use Phoenix.Router\n      import Phoenix.LiveView.Router\n\n      scope \"/\", MyApp do\n        pipe_through [:browser]\n\n        live \"/thermostat\", ThermostatLive\n        live \"/clock\", ClockLive\n        live \"/dashboard\", DashboardLive, container: {:main, class: \"row\"}\n      end\n    end\n\n    iex> MyApp.Router.Helpers.live_path(MyApp.Endpoint, MyApp.ThermostatLive)\n    \"/thermostat\"","ref":"Phoenix.LiveView.Router.html#live/4-examples"},{"type":"macro","title":"Phoenix.LiveView.Router.live_session/3","doc":"Defines a live session for live redirects within a group of live routes.\n\n`live_session/3` allow routes defined with `live/4` to support\n`live_redirect` from the client with navigation purely over the existing\nwebsocket connection. This allows live routes defined in the router to\nmount a new root LiveView without additional HTTP requests to the server.\nFor backwards compatibility reasons, all live routes defined outside\nof any live session are considered part of a single unnamed live session.","ref":"Phoenix.LiveView.Router.html#live_session/3"},{"type":"macro","title":"Security Considerations - Phoenix.LiveView.Router.live_session/3","doc":"In a regular web application, we perform authentication and authorization\nchecks on every request. Given LiveViews start as a regular HTTP request,\nthey share the authentication logic with regular requests through plugs.\nOnce the user is authenticated, we typically validate the sessions on\nthe `mount` callback. Authorization rules generally happen on `mount`\n(for instance, is the user allowed to see this page?) and also on\n`handle_event` (is the user allowed to delete this item?). Performing\nauthorization on mount is important because `live_redirect`s *do not go\nthrough the plug pipeline*.\n\n`live_session` can be used to draw boundaries between groups of LiveViews.\nRedirecting between `live_session`s will always force a full page reload\nand establish a brand new LiveView connection. This is useful when LiveViews\nrequire different authentication strategies or simply when they use different\nroot layouts (as the root layout is not updated between live redirects).\n\nPlease [read our guide on the security model](security-model.md) for a\ndetailed description and general tips on authentication, authorization,\nand more.","ref":"Phoenix.LiveView.Router.html#live_session/3-security-considerations"},{"type":"macro","title":"Options - Phoenix.LiveView.Router.live_session/3","doc":"* `:session` - The optional extra session map or MFA tuple to be merged with\n    the LiveView session. For example, `%{\"admin\" => true}` or `{MyMod, :session, []}`.\n    For MFA, the function is invoked and the `Plug.Conn` struct is prepended\n    to the arguments list.\n\n  * `:root_layout` - The optional root layout tuple for the initial HTTP render to\n    override any existing root layout set in the router.\n\n  * `:on_mount` - The optional list of hooks to attach to the mount lifecycle _of\n    each LiveView in the session_. See `Phoenix.LiveView.on_mount/1`. Passing a\n    single value is also accepted.\n\n  * `:layout` - The optional layout the LiveView will be rendered in. Setting\n    this option overrides the layout via `use Phoenix.LiveView`. This option\n    may be overridden inside a LiveView by returning `{:ok, socket, layout: ...}`\n    from the mount callback","ref":"Phoenix.LiveView.Router.html#live_session/3-options"},{"type":"macro","title":"Examples - Phoenix.LiveView.Router.live_session/3","doc":"scope \"/\", MyAppWeb do\n      pipe_through :browser\n\n      live_session :default do\n        live \"/feed\", FeedLive, :index\n        live \"/status\", StatusLive, :index\n        live \"/status/:id\", StatusLive, :show\n      end\n\n      live_session :admin, on_mount: MyAppWeb.AdminLiveAuth do\n        live \"/admin\", AdminDashboardLive, :index\n        live \"/admin/posts\", AdminPostLive, :index\n      end\n    end\n\nIn the example above, we have two live sessions. Live navigation between live views\nin the different sessions is not possible and will always require a full page reload.\nThis is important in the example above because the `:admin` live session has authentication\nrequirements, defined by `on_mount: MyAppWeb.AdminLiveAuth`, that the other LiveViews\ndo not have.\n\nIf you have both regular HTTP routes (via get, post, etc) and `live` routes, then\nyou need to perform the same authentication and authorization rules in both.\nFor example, if you were to add a `get \"/admin/health\"` entry point inside the\n`:admin` live session above, then you must create your own plug that performs the\nsame authentication and authorization rules as `MyAppWeb.AdminLiveAuth`, and then\npipe through it:\n\n    live_session :admin, on_mount: MyAppWeb.AdminLiveAuth do\n      scope \"/\" do\n        # Regular routes\n        pipe_through [MyAppWeb.AdminPlugAuth]\n        get \"/admin/health\", AdminHealthController, :index\n\n        # Live routes\n        live \"/admin\", AdminDashboardLive, :index\n        live \"/admin/posts\", AdminPostLive, :index\n      end\n    end\n\nThe opposite is also true, if you have regular http routes and you want to\nadd your own `live` routes, the same authentication and authorization checks\nexecuted by the plugs listed in `pipe_through` must be ported to LiveViews\nand be executed via `on_mount` hooks.","ref":"Phoenix.LiveView.Router.html#live_session/3-examples"},{"type":"module","title":"Phoenix.LiveViewTest","doc":"Conveniences for testing function components as well as\nLiveViews and LiveComponents.","ref":"Phoenix.LiveViewTest.html"},{"type":"module","title":"Testing function components - Phoenix.LiveViewTest","doc":"There are two mechanisms for testing function components. Imagine the\nfollowing component:\n\n    def greet(assigns) do\n      ~H\"\"\"\n       Hello, <%= @name %>! \n      \"\"\"\n    end\n\nYou can test it by using `render_component/3`, passing the function\nreference to the component as first argument:\n\n    import Phoenix.LiveViewTest\n\n    test \"greets\" do\n      assert render_component(&MyComponents.greet/1, name: \"Mary\") ==\n               \" Hello, Mary! \"\n    end\n\nHowever, for complex components, often the simplest way to test them\nis by using the `~H` sigil itself:\n\n    import Phoenix.Component\n    import Phoenix.LiveViewTest\n\n    test \"greets\" do\n      assigns = %{}\n      assert rendered_to_string(~H\"\"\"\n              \n             \"\"\") ==\n               \" Hello, Mary! \"\n    end\n\nThe difference is that we use `rendered_to_string/1` to convert the rendered\ntemplate to a string for testing.","ref":"Phoenix.LiveViewTest.html#module-testing-function-components"},{"type":"module","title":"Testing LiveViews and LiveComponents - Phoenix.LiveViewTest","doc":"In LiveComponents and LiveView tests, we interact with views\nvia process communication in substitution of a browser.\nLike a browser, our test process receives messages about the\nrendered updates from the view which can be asserted against\nto test the life-cycle and behavior of LiveViews and their\nchildren.\n\n#","ref":"Phoenix.LiveViewTest.html#module-testing-liveviews-and-livecomponents"},{"type":"module","title":"Testing LiveViews - Phoenix.LiveViewTest","doc":"The life-cycle of a LiveView as outlined in the `Phoenix.LiveView`\ndocs details how a view starts as a stateless HTML render in a disconnected\nsocket state. Once the browser receives the HTML, it connects to the\nserver and a new LiveView process is started, remounted in a connected\nsocket state, and the view continues statefully. The LiveView test functions\nsupport testing both disconnected and connected mounts separately, for example:\n\n    import Plug.Conn\n    import Phoenix.ConnTest\n    import Phoenix.LiveViewTest\n    @endpoint MyEndpoint\n\n    test \"disconnected and connected mount\", %{conn: conn} do\n      conn = get(conn, \"/my-path\")\n      assert html_response(conn, 200) =~ \" My Disconnected View \"\n\n      {:ok, view, html} = live(conn)\n    end\n\n    test \"redirected mount\", %{conn: conn} do\n      assert {:error, {:redirect, %{to: \"/somewhere\"}}} = live(conn, \"my-path\")\n    end\n\nHere, we start by using the familiar `Phoenix.ConnTest` function, `get/2` to\ntest the regular HTTP GET request which invokes mount with a disconnected socket.\nNext, `live/1` is called with our sent connection to mount the view in a connected\nstate, which starts our stateful LiveView process.\n\nIn general, it's often more convenient to test the mounting of a view\nin a single step, provided you don't need the result of the stateless HTTP\nrender. This is done with a single call to `live/2`, which performs the\n`get` step for us:\n\n    test \"connected mount\", %{conn: conn} do\n      {:ok, _view, html} = live(conn, \"/my-path\")\n      assert html =~ \" My Connected View \"\n    end\n\n#","ref":"Phoenix.LiveViewTest.html#module-testing-liveviews"},{"type":"module","title":"Testing Events - Phoenix.LiveViewTest","doc":"The browser can send a variety of events to a LiveView via `phx-` bindings,\nwhich are sent to the `handle_event/3` callback. To test events sent by the\nbrowser and assert on the rendered side effect of the event, use the\n`render_*` functions:\n\n  * `render_click/1` - sends a phx-click event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\n  * `render_focus/2` - sends a phx-focus event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\n  * `render_blur/1` - sends a phx-blur event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\n  * `render_submit/1` - sends a form phx-submit event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\n  * `render_change/1` - sends a form phx-change event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\n  * `render_keydown/1` - sends a form phx-keydown event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\n  * `render_keyup/1` - sends a form phx-keyup event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\n  * `render_hook/3` - sends a hook event and value, returning\n    the rendered result of the `handle_event/3` callback.\n\nFor example:\n\n    {:ok, view, _html} = live(conn, \"/thermo\")\n\n    assert view\n           |> element(\"button#inc\")\n           |> render_click() =~ \"The temperature is: 31℉\"\n\nIn the example above, we are looking for a particular element on the page\nand triggering its phx-click event. LiveView takes care of making sure the\nelement has a phx-click and automatically sends its values to the server.\n\nYou can also bypass the element lookup and directly trigger the LiveView\nevent in most functions:\n\n    assert render_click(view, :inc, %{}) =~ \"The temperature is: 31℉\"\n\nThe `element` style is preferred as much as possible, as it helps LiveView\nperform validations and ensure the events in the HTML actually matches the\nevent names on the server.\n\n#","ref":"Phoenix.LiveViewTest.html#module-testing-events"},{"type":"module","title":"Testing regular messages - Phoenix.LiveViewTest","doc":"LiveViews are `GenServer`'s under the hood, and can send and receive messages\njust like any other server. To test the side effects of sending or receiving\nmessages, simply message the view and use the `render` function to test the\nresult:\n\n    send(view.pid, {:set_temp, 50})\n    assert render(view) =~ \"The temperature is: 50℉\"\n\n#","ref":"Phoenix.LiveViewTest.html#module-testing-regular-messages"},{"type":"module","title":"Testing LiveComponents - Phoenix.LiveViewTest","doc":"LiveComponents can be tested in two ways. One way is to use the same\n`render_component/2` function as function components. This will mount\nthe LiveComponent and render it once, without testing any of its events:\n\n    assert render_component(MyComponent, id: 123, user: %User{}) =~\n             \"some markup in component\"\n\nHowever, if you want to test how components are mounted by a LiveView\nand interact with DOM events, you must use the regular `live/2` macro\nto build the LiveView with the component and then scope events by\npassing the view and a **DOM selector** in a list:\n\n    {:ok, view, html} = live(conn, \"/users\")\n    html = view |> element(\"#user-13 a\", \"Delete\") |> render_click()\n    refute html =~ \"user-13\"\n    refute view |> element(\"#user-13\") |> has_element?()\n\nIn the example above, LiveView will lookup for an element with\nID=user-13 and retrieve its `phx-target`. If `phx-target` points\nto a component, that will be the component used, otherwise it will\nfallback to the view.","ref":"Phoenix.LiveViewTest.html#module-testing-livecomponents"},{"type":"function","title":"Phoenix.LiveViewTest.assert_patch/2","doc":"Asserts a live patch will happen within `timeout` milliseconds.\nThe default `timeout` is [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html#configure/1)'s\n`assert_receive_timeout` (100 ms).\n\nIt returns the new path.\n\nTo assert on the flash message, you can assert on the result of the\nrendered LiveView.","ref":"Phoenix.LiveViewTest.html#assert_patch/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.assert_patch/2","doc":"render_click(view, :event_that_triggers_patch)\n    assert_patch view\n\n    render_click(view, :event_that_triggers_patch)\n    assert_patch view, 30\n\n    render_click(view, :event_that_triggers_patch)\n    path = assert_patch view\n    assert path =~ ~r/path/+/","ref":"Phoenix.LiveViewTest.html#assert_patch/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.assert_patch/3","doc":"Asserts a live patch will happen to a given path within `timeout`\nmilliseconds.\n\nThe default `timeout` is [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html#configure/1)'s\n`assert_receive_timeout` (100 ms).\n\nIt always returns `:ok`.\n\nTo assert on the flash message, you can assert on the result of the\nrendered LiveView.","ref":"Phoenix.LiveViewTest.html#assert_patch/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.assert_patch/3","doc":"render_click(view, :event_that_triggers_patch)\n    assert_patch view, \"/path\"\n\n    render_click(view, :event_that_triggers_patch)\n    assert_patch view, \"/path\", 30","ref":"Phoenix.LiveViewTest.html#assert_patch/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.assert_patched/2","doc":"Asserts a live patch was performed, and returns the new path.\n\nTo assert on the flash message, you can assert on the result of\nthe rendered LiveView.","ref":"Phoenix.LiveViewTest.html#assert_patched/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.assert_patched/2","doc":"render_click(view, :event_that_triggers_redirect)\n    assert_patched view, \"/path\"","ref":"Phoenix.LiveViewTest.html#assert_patched/2-examples"},{"type":"macro","title":"Phoenix.LiveViewTest.assert_push_event/4","doc":"Asserts an event will be pushed within `timeout`.\nThe default `timeout` is [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html#configure/1)'s\n`assert_receive_timeout` (100 ms).","ref":"Phoenix.LiveViewTest.html#assert_push_event/4"},{"type":"macro","title":"Examples - Phoenix.LiveViewTest.assert_push_event/4","doc":"assert_push_event view, \"scores\", %{points: 100, user: \"josé\"}","ref":"Phoenix.LiveViewTest.html#assert_push_event/4-examples"},{"type":"function","title":"Phoenix.LiveViewTest.assert_redirect/2","doc":"Asserts a redirect will happen within `timeout` milliseconds.\nThe default `timeout` is [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html#configure/1)'s\n`assert_receive_timeout` (100 ms).\n\nIt returns a tuple containing the new path and the flash messages from said\nredirect, if any. Note the flash will contain string keys.","ref":"Phoenix.LiveViewTest.html#assert_redirect/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.assert_redirect/2","doc":"render_click(view, :event_that_triggers_redirect)\n    {path, flash} = assert_redirect view\n    assert flash[\"info\"] == \"Welcome\"\n    assert path =~ ~r/path\\/\\d+/\n\n    render_click(view, :event_that_triggers_redirect)\n    assert_redirect view, 30","ref":"Phoenix.LiveViewTest.html#assert_redirect/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.assert_redirect/3","doc":"Asserts a redirect will happen to a given path within `timeout` milliseconds.\n\nThe default `timeout` is [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html#configure/1)'s\n`assert_receive_timeout` (100 ms).\n\nIt returns the flash messages from said redirect, if any.\nNote the flash will contain string keys.","ref":"Phoenix.LiveViewTest.html#assert_redirect/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.assert_redirect/3","doc":"render_click(view, :event_that_triggers_redirect)\n    flash = assert_redirect view, \"/path\"\n    assert flash[\"info\"] == \"Welcome\"\n\n    render_click(view, :event_that_triggers_redirect)\n    assert_redirect view, \"/path\", 30","ref":"Phoenix.LiveViewTest.html#assert_redirect/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.assert_redirected/2","doc":"Asserts a redirect was performed.\n\nIt returns the flash messages from said redirect, if any. Note the\nflash will contain string keys.","ref":"Phoenix.LiveViewTest.html#assert_redirected/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.assert_redirected/2","doc":"render_click(view, :event_that_triggers_redirect)\n    flash = assert_redirected view, \"/path\"\n    assert flash[\"info\"] == \"Welcome\"","ref":"Phoenix.LiveViewTest.html#assert_redirected/2-examples"},{"type":"macro","title":"Phoenix.LiveViewTest.assert_reply/3","doc":"Asserts a hook reply was returned from a `handle_event` callback.\n\nThe default `timeout` is [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html#configure/1)'s\n`assert_receive_timeout` (100 ms).","ref":"Phoenix.LiveViewTest.html#assert_reply/3"},{"type":"macro","title":"Examples - Phoenix.LiveViewTest.assert_reply/3","doc":"assert_reply view, %{result: \"ok\", transaction_id: _}","ref":"Phoenix.LiveViewTest.html#assert_reply/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.element/3","doc":"Returns an element to scope a function to.\n\nIt expects the current LiveView, a query selector, and a text filter.\n\nAn optional text filter may be given to filter the results by the query\nselector. If the text filter is a string or a regex, it will match any\nelement that contains the string (including as a substring) or matches the\nregex.\n\nSo a link containing the text \"unopened\" will match `element(\"a\", \"opened\")`.\nTo prevent this, a regex could specify that \"opened\" appear without the prefix \"un\".\nFor example, `element(\"a\", ~r{(?<!un)opened})`.\nBut it may be clearer to add an HTML attribute to make the element easier to\nselect.\n\nAfter the text filter is applied, only one element must remain, otherwise an\nerror is raised.\n\nIf no text filter is given, then the query selector itself must return\na single element.\n\n    assert view\n          |> element(\"#term > :first-child\", \"Increment\")\n          |> render() =~ \"Increment \"\n\nAttribute selectors are also supported, and may be used on special cases\nlike ids which contain periods:\n\n    assert view\n           |> element(~s{[href=\"/foo\"][id=\"foo.bar.baz\"]})\n           |> render() =~ \"Increment \"","ref":"Phoenix.LiveViewTest.html#element/3"},{"type":"macro","title":"Phoenix.LiveViewTest.file_input/4","doc":"Builds a file input for testing uploads within a form.\n\nGiven the form DOM selector, the upload name, and a list of maps of client metadata\nfor the upload, the returned file input can be passed to `render_upload/2`.\n\nClient metadata takes the following form:\n\n  * `:last_modified` - the last modified timestamp\n  * `:name` - the name of the file\n  * `:content` - the binary content of the file\n  * `:size` - the byte size of the content\n  * `:type` - the MIME type of the file\n  * `:relative_path` - for simulating webkitdirectory metadata\n  * `:meta` - optional metadata sent by the client","ref":"Phoenix.LiveViewTest.html#file_input/4"},{"type":"macro","title":"Examples - Phoenix.LiveViewTest.file_input/4","doc":"avatar = file_input(lv, \"#my-form-id\", :avatar, [%{\n      last_modified: 1_594_171_879_000,\n      name: \"myfile.jpeg\",\n      content: File.read!(\"myfile.jpg\"),\n      size: 1_396_009,\n      type: \"image/jpeg\"\n    }])\n\n    assert render_upload(avatar, \"myfile.jpeg\") =~ \"100%\"","ref":"Phoenix.LiveViewTest.html#file_input/4-examples"},{"type":"function","title":"Phoenix.LiveViewTest.find_live_child/2","doc":"Gets the nested LiveView child by `child_id` from the `parent` LiveView.","ref":"Phoenix.LiveViewTest.html#find_live_child/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.find_live_child/2","doc":"{:ok, view, _html} = live(conn, \"/thermo\")\n    assert clock_view = find_live_child(view, \"clock\")\n    assert render_click(clock_view, :snooze) =~ \"snoozing\"","ref":"Phoenix.LiveViewTest.html#find_live_child/2-examples"},{"type":"macro","title":"Phoenix.LiveViewTest.follow_redirect/3","doc":"Follows the redirect from a `render_*` action or an `{:error, redirect}`\ntuple.\n\nImagine you have a LiveView that redirects on a `render_click`\nevent. You can make sure it immediately redirects after the\n`render_click` action by calling `follow_redirect/3`:\n\n    live_view\n    |> render_click(\"redirect\")\n    |> follow_redirect(conn)\n\nOr in the case of an error tuple:\n\n    assert {:error, {:redirect, %{to: \"/somewhere\"}}} = result = live(conn, \"my-path\")\n    {:ok, view, html} = follow_redirect(result, conn)\n\n`follow_redirect/3` expects a connection as second argument.\nThis is the connection that will be used to perform the underlying\nrequest.\n\nIf the LiveView redirects with a live redirect, this macro returns\n`{:ok, live_view, disconnected_html}` with the content of the new\nLiveView, the same as the `live/3` macro. If the LiveView redirects\nwith a regular redirect, this macro returns `{:ok, conn}` with the\nrendered redirected page. In any other case, this macro raises.\n\nFinally, note that you can optionally assert on the path you are\nbeing redirected to by passing a third argument:\n\n    live_view\n    |> render_click(\"redirect\")\n    |> follow_redirect(conn, \"/redirected/page\")","ref":"Phoenix.LiveViewTest.html#follow_redirect/3"},{"type":"macro","title":"Phoenix.LiveViewTest.follow_trigger_action/2","doc":"Receives a `form_element` and asserts that `phx-trigger-action` has been\nset to true, following up on that request.\n\nImagine you have a LiveView that sends an HTTP form submission. Say that it\nsets the `phx-trigger-action` to true, as a response to a submit event.\nYou can follow the trigger action like this:\n\n    form = form(live_view, selector, %{\"form\" => \"data\"})\n\n    # First we submit the form. Optionally verify that phx-trigger-action\n    # is now part of the form.\n    assert render_submit(form) =~ ~r/phx-trigger-action/\n\n    # Now follow the request made by the form\n    conn = follow_trigger_action(form, conn)\n    assert conn.method == \"POST\"\n    assert conn.params == %{\"form\" => \"data\"}","ref":"Phoenix.LiveViewTest.html#follow_trigger_action/2"},{"type":"function","title":"Phoenix.LiveViewTest.form/3","doc":"Returns a form element to scope a function to.\n\nIt expects the current LiveView, a query selector, and the form data.\nThe query selector must return a single element.\n\nThe form data will be validated directly against the form markup and\nmake sure the data you are changing/submitting actually exists, failing\notherwise.","ref":"Phoenix.LiveViewTest.html#form/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.form/3","doc":"assert view\n          |> form(\"#term\", user: %{name: \"hello\"})\n          |> render_submit() =~ \"Name updated\"\n\nThis function is meant to mimic what the user can actually do, so you cannot\n set hidden input values. However, hidden values can be given when calling\n `render_submit/2` or `render_change/2`, see their docs for examples.","ref":"Phoenix.LiveViewTest.html#form/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.has_element?/1","doc":"Checks if the given element exists on the page.","ref":"Phoenix.LiveViewTest.html#has_element?/1"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.has_element?/1","doc":"assert view |> element(\"#some-element\") |> has_element?()","ref":"Phoenix.LiveViewTest.html#has_element?/1-examples"},{"type":"function","title":"Phoenix.LiveViewTest.has_element?/3","doc":"Checks if the given `selector` with `text_filter` is on `view`.\n\nSee `element/3` for more information.","ref":"Phoenix.LiveViewTest.html#has_element?/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.has_element?/3","doc":"assert has_element?(view, \"#some-element\")","ref":"Phoenix.LiveViewTest.html#has_element?/3-examples"},{"type":"macro","title":"Phoenix.LiveViewTest.live/2","doc":"Spawns a connected LiveView process.\n\nIf a `path` is given, then a regular `get(conn, path)`\nis done and the page is upgraded to a `LiveView`. If\nno path is given, it assumes a previously rendered\n`%Plug.Conn{}` is given, which will be converted to\na `LiveView` immediately.","ref":"Phoenix.LiveViewTest.html#live/2"},{"type":"macro","title":"Examples - Phoenix.LiveViewTest.live/2","doc":"{:ok, view, html} = live(conn, \"/path\")\n    assert view.module == MyLive\n    assert html =~ \"the count is 3\"\n\n    assert {:error, {:redirect, %{to: \"/somewhere\"}}} = live(conn, \"/path\")","ref":"Phoenix.LiveViewTest.html#live/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.live_children/1","doc":"Returns the current list of LiveView children for the `parent` LiveView.\n\nChildren are returned in the order they appear in the rendered HTML.","ref":"Phoenix.LiveViewTest.html#live_children/1"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.live_children/1","doc":"{:ok, view, _html} = live(conn, \"/thermo\")\n    assert [clock_view] = live_children(view)\n    assert render_click(clock_view, :snooze) =~ \"snoozing\"","ref":"Phoenix.LiveViewTest.html#live_children/1-examples"},{"type":"macro","title":"Phoenix.LiveViewTest.live_isolated/3","doc":"Spawns a connected LiveView process mounted in isolation as the sole rendered element.\n\nUseful for testing LiveViews that are not directly routable, such as those\nbuilt as small components to be re-used in multiple parents. Testing routable\nLiveViews is still recommended whenever possible since features such as\nlive navigation require routable LiveViews.","ref":"Phoenix.LiveViewTest.html#live_isolated/3"},{"type":"macro","title":"Options - Phoenix.LiveViewTest.live_isolated/3","doc":"* `:session` - the session to be given to the LiveView\n\nAll other options are forwarded to the LiveView for rendering. Refer to\n`Phoenix.Component.live_render/3` for a list of supported render\noptions.","ref":"Phoenix.LiveViewTest.html#live_isolated/3-options"},{"type":"macro","title":"Examples - Phoenix.LiveViewTest.live_isolated/3","doc":"{:ok, view, html} =\n      live_isolated(conn, MyAppWeb.ClockLive, session: %{\"tz\" => \"EST\"})\n\nUse `put_connect_params/2` to put connect params for a call to\n`Phoenix.LiveView.get_connect_params/1` in `c:Phoenix.LiveView.mount/3`:\n\n    {:ok, view, html} =\n      conn\n      |> put_connect_params(%{\"param\" => \"value\"})\n      |> live_isolated(AppWeb.ClockLive, session: %{\"tz\" => \"EST\"})","ref":"Phoenix.LiveViewTest.html#live_isolated/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.live_redirect/2","doc":"Performs a live redirect from one LiveView to another.\n\nWhen redirecting between two LiveViews of the same `live_session`,\nmounts the new LiveView and shutsdown the previous one, which\nmimics general browser live navigation behaviour.\n\nWhen attempting to navigate from a LiveView of a different\n`live_session`, an error redirect condition is returned indicating\na failed `live_redirect` from the client.","ref":"Phoenix.LiveViewTest.html#live_redirect/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.live_redirect/2","doc":"assert {:ok, page_live, _html} = live(conn, \"/page/1\")\n    assert {:ok, page2_live, _html} = live(conn, \"/page/2\")\n\n    assert {:error, {:redirect, _}} = live_redirect(page2_live, to: \"/admin\")","ref":"Phoenix.LiveViewTest.html#live_redirect/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.open_browser/2","doc":"Open the default browser to display current HTML of `view_or_element`.","ref":"Phoenix.LiveViewTest.html#open_browser/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.open_browser/2","doc":"view\n    |> element(\"#term > :first-child\", \"Increment\")\n    |> open_browser()\n\n    assert view\n           |> form(\"#term\", user: %{name: \"hello\"})\n           |> open_browser()\n           |> render_submit() =~ \"Name updated\"","ref":"Phoenix.LiveViewTest.html#open_browser/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.page_title/1","doc":"Returns the most recent title that was updated via a `page_title` assign.","ref":"Phoenix.LiveViewTest.html#page_title/1"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.page_title/1","doc":"render_click(view, :event_that_triggers_page_title_update)\n    assert page_title(view) =~ \"my title\"","ref":"Phoenix.LiveViewTest.html#page_title/1-examples"},{"type":"function","title":"Phoenix.LiveViewTest.preflight_upload/1","doc":"Performs a preflight upload request.\n\nUseful for testing external uploaders to retrieve the `:external` entry metadata.","ref":"Phoenix.LiveViewTest.html#preflight_upload/1"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.preflight_upload/1","doc":"avatar = file_input(lv, \"#my-form-id\", :avatar, [%{name: ..., ...}, ...])\n    assert {:ok, %{ref: _ref, config: %{chunk_size: _}}} = preflight_upload(avatar)","ref":"Phoenix.LiveViewTest.html#preflight_upload/1-examples"},{"type":"function","title":"Phoenix.LiveViewTest.put_connect_info/2","doc":"","ref":"Phoenix.LiveViewTest.html#put_connect_info/2"},{"type":"function","title":"Phoenix.LiveViewTest.put_connect_params/2","doc":"Puts connect params to be used on LiveView connections.\n\nSee `Phoenix.LiveView.get_connect_params/1`.","ref":"Phoenix.LiveViewTest.html#put_connect_params/2"},{"type":"function","title":"Phoenix.LiveViewTest.put_submitter/2","doc":"Puts the submitter `element_or_selector` on the given `form` element.\n\nA submitter is an element that initiates the form's submit event on the client. When a submitter\nis put on an element created with `form/3` and then the form is submitted via `render_submit/2`,\nthe name/value pair of the submitter will be included in the submit event payload.\n\nThe given element or selector must exist within the form and match one of the following:\n\n- A `button` or `input` element with `type=\"submit\"`.\n\n- A `button` element without a `type` attribute.","ref":"Phoenix.LiveViewTest.html#put_submitter/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.put_submitter/2","doc":"form = view |> form(\"#my-form\")\n\n    assert form\n           |> put_submitter(\"button[name=example]\")\n           |> render_submit() =~ \"Submitted example\"","ref":"Phoenix.LiveViewTest.html#put_submitter/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.refute_redirected/2","doc":"Refutes a redirect to a given path was performed.\n\nIt returns :ok if the specified redirect isn't already in the mailbox.","ref":"Phoenix.LiveViewTest.html#refute_redirected/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.refute_redirected/2","doc":"render_click(view, :event_that_triggers_redirect_to_path)\n    :ok = refute_redirected view, \"/wrong_path\"","ref":"Phoenix.LiveViewTest.html#refute_redirected/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render/1","doc":"Returns the HTML string of the rendered view or element.\n\nIf a view is provided, the entire LiveView is rendered.\nIf a view after calling `with_target/2` or an element\nare given, only that particular context is returned.","ref":"Phoenix.LiveViewTest.html#render/1"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render/1","doc":"{:ok, view, _html} = live(conn, \"/thermo\")\n    assert render(view) =~ ~s| Snooze |\n\n    assert view\n           |> element(\"#alarm\")\n           |> render() == \"Snooze\"","ref":"Phoenix.LiveViewTest.html#render/1-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_async/2","doc":"Awaits all current `assign_async` and `start_async` for a given LiveView or element.\n\nIt renders the LiveView or Element once complete and returns the result.\nThe default `timeout` is [ExUnit](https://hexdocs.pm/ex_unit/ExUnit.html#configure/1)'s\n`assert_receive_timeout` (100 ms).","ref":"Phoenix.LiveViewTest.html#render_async/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_async/2","doc":"{:ok, lv, html} = live(conn, \"/path\")\n    assert html =~ \"loading data...\"\n    assert render_async(lv) =~ \"data loaded!\"","ref":"Phoenix.LiveViewTest.html#render_async/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_blur/2","doc":"Sends a blur event given by `element` and returns the rendered result.\n\nThe `element` is created with `element/3` and must point to a single\nelement on the page with a `phx-blur` attribute in it. The event name\ngiven set on `phx-blur` is then sent to the appropriate LiveView\n(or component if `phx-target` is set accordingly). All `phx-value-*`\nentries in the element are sent as values. Extra values can be given\nwith the `value` argument.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_blur/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_blur/2","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n\n    assert view\n           |> element(\"#inactive\")\n           |> render_blur() =~ \"Tap to wake\"","ref":"Phoenix.LiveViewTest.html#render_blur/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_blur/3","doc":"Sends a blur event to the view and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_blur/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_blur/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert render_blur(view, :inactive) =~ \"Tap to wake\"","ref":"Phoenix.LiveViewTest.html#render_blur/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_change/2","doc":"Sends a form change event given by `element` and returns the rendered result.\n\nThe `element` is created with `element/3` and must point to a single\nelement on the page with a `phx-change` attribute in it. The event name\ngiven set on `phx-change` is then sent to the appropriate LiveView\n(or component if `phx-target` is set accordingly). All `phx-value-*`\nentries in the element are sent as values.\n\nIf you need to pass any extra values or metadata, such as the \"_target\"\nparameter, you can do so by giving a map under the `value` argument.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_change/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_change/2","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n\n    assert view\n           |> element(\"form\")\n           |> render_change(%{deg: 123}) =~ \"123 exceeds limits\"\n\n    # Passing metadata\n    {:ok, view, html} = live(conn, \"/thermo\")\n\n    assert view\n           |> element(\"form\")\n           |> render_change(%{_target: [\"deg\"], deg: 123}) =~ \"123 exceeds limits\"\n\nAs with `render_submit/2`, hidden input field values can be provided like so:\n\n    refute view\n          |> form(\"#term\", user: %{name: \"hello\"})\n          |> render_change(%{user: %{\"hidden_field\" => \"example\"}}) =~ \"can't be blank\"","ref":"Phoenix.LiveViewTest.html#render_change/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_change/3","doc":"Sends a form change event to the view and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_change/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_change/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert render_change(view, :validate, %{deg: 123}) =~ \"123 exceeds limits\"","ref":"Phoenix.LiveViewTest.html#render_change/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_click/2","doc":"Sends a click event given by `element` and returns the rendered result.\n\nThe `element` is created with `element/3` and must point to a single\nelement on the page with a `phx-click` attribute in it. The event name\ngiven set on `phx-click` is then sent to the appropriate LiveView\n(or component if `phx-target` is set accordingly). All `phx-value-*`\nentries in the element are sent as values. Extra values can be given\nwith the `value` argument.\n\nIf the element does not have a `phx-click` attribute but it is\na link (the ` ` tag), the link will be followed accordingly:\n\n  * if the link is a `live_patch`, the current view will be patched\n  * if the link is a `live_redirect`, this function will return\n    `{:error, {:live_redirect, %{to: url}}}`, which can be followed\n    with `follow_redirect/2`\n  * if the link is a regular link, this function will return\n    `{:error, {:redirect, %{to: url}}}`, which can be followed\n    with `follow_redirect/2`\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_click/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_click/2","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n\n    assert view\n           |> element(\"button\", \"Increment\")\n           |> render_click() =~ \"The temperature is: 30℉\"","ref":"Phoenix.LiveViewTest.html#render_click/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_click/3","doc":"Sends a click `event` to the `view` with `value` and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_click/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_click/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temperature is: 30℉\"\n    assert render_click(view, :inc) =~ \"The temperature is: 31℉\"","ref":"Phoenix.LiveViewTest.html#render_click/3-examples"},{"type":"macro","title":"Phoenix.LiveViewTest.render_component/3","doc":"Renders a component.\n\nThe first argument may either be a function component, as an\nanonymous function:\n\n    assert render_component(&Weather.city/1, name: \"Kraków\") =~\n             \"some markup in component\"\n\nOr a stateful component as a module. In this case, this function\nwill mount, update, and render the component. The `:id` option is\na required argument:\n\n    assert render_component(MyComponent, id: 123, user: %User{}) =~\n             \"some markup in component\"\n\nIf your component is using the router, you can pass it as argument:\n\n    assert render_component(MyComponent, %{id: 123, user: %User{}}, router: SomeRouter) =~\n             \"some markup in component\"","ref":"Phoenix.LiveViewTest.html#render_component/3"},{"type":"function","title":"Phoenix.LiveViewTest.render_focus/2","doc":"Sends a focus event given by `element` and returns the rendered result.\n\nThe `element` is created with `element/3` and must point to a single\nelement on the page with a `phx-focus` attribute in it. The event name\ngiven set on `phx-focus` is then sent to the appropriate LiveView\n(or component if `phx-target` is set accordingly). All `phx-value-*`\nentries in the element are sent as values. Extra values can be given\nwith the `value` argument.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_focus/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_focus/2","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n\n    assert view\n           |> element(\"#inactive\")\n           |> render_focus() =~ \"Tap to wake\"","ref":"Phoenix.LiveViewTest.html#render_focus/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_focus/3","doc":"Sends a focus event to the view and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_focus/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_focus/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert render_focus(view, :inactive) =~ \"Tap to wake\"","ref":"Phoenix.LiveViewTest.html#render_focus/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_hook/3","doc":"Sends a hook event to the view or an element and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_hook/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_hook/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert render_hook(view, :refresh, %{deg: 32}) =~ \"The temp is: 32℉\"\n\nIf you are pushing events from a hook to a component, then you must pass\nan `element`, created with `element/3`, as first argument and it must point\nto a single element on the page with a `phx-target` attribute in it:\n\n    {:ok, view, _html} = live(conn, \"/thermo\")\n    assert view\n           |> element(\"#thermo-component\")\n           |> render_hook(:refresh, %{deg: 32}) =~ \"The temp is: 32℉\"","ref":"Phoenix.LiveViewTest.html#render_hook/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_keydown/2","doc":"Sends a keydown event given by `element` and returns the rendered result.\n\nThe `element` is created with `element/3` and must point to a single element\non the page with a `phx-keydown` or `phx-window-keydown` attribute in it.\nThe event name given set on `phx-keydown` is then sent to the appropriate\nLiveView (or component if `phx-target` is set accordingly). All `phx-value-*`\nentries in the element are sent as values. Extra values can be given with\nthe `value` argument.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_keydown/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_keydown/2","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert view |> element(\"#inc\") |> render_keydown() =~ \"The temp is: 31℉\"","ref":"Phoenix.LiveViewTest.html#render_keydown/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_keydown/3","doc":"Sends a keydown event to the view and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_keydown/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_keydown/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert render_keydown(view, :inc) =~ \"The temp is: 31℉\"","ref":"Phoenix.LiveViewTest.html#render_keydown/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_keyup/2","doc":"Sends a keyup event given by `element` and returns the rendered result.\n\nThe `element` is created with `element/3` and must point to a single\nelement on the page with a `phx-keyup` or `phx-window-keyup` attribute\nin it. The event name given set on `phx-keyup` is then sent to the\nappropriate LiveView (or component if `phx-target` is set accordingly).\nAll `phx-value-*` entries in the element are sent as values. Extra values\ncan be given with the `value` argument.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_keyup/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_keyup/2","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert view |> element(\"#inc\") |> render_keyup() =~ \"The temp is: 31℉\"","ref":"Phoenix.LiveViewTest.html#render_keyup/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_keyup/3","doc":"Sends a keyup event to the view and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_keyup/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_keyup/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert render_keyup(view, :inc) =~ \"The temp is: 31℉\"","ref":"Phoenix.LiveViewTest.html#render_keyup/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_patch/2","doc":"Simulates a `live_patch` to the given `path` and returns the rendered result.","ref":"Phoenix.LiveViewTest.html#render_patch/2"},{"type":"function","title":"Phoenix.LiveViewTest.render_submit/2","doc":"Sends a form submit event given by `element` and returns the rendered result.\n\nThe `element` is created with `element/3` and must point to a single\nelement on the page with a `phx-submit` attribute in it. The event name\ngiven set on `phx-submit` is then sent to the appropriate LiveView\n(or component if `phx-target` is set accordingly). All `phx-value-*`\nentries in the element are sent as values. Extra values, including hidden\ninput fields, can be given with the `value` argument.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_submit/2"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_submit/2","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n\n    assert view\n           |> element(\"form\")\n           |> render_submit(%{deg: 123, avatar: upload}) =~ \"123 exceeds limits\"\n\nTo submit a form along with some with hidden input values:\n\n    assert view\n           |> form(\"#term\", user: %{name: \"hello\"})\n           |> render_submit(%{user: %{\"hidden_field\" => \"example\"}}) =~ \"Name updated\"\n\nTo submit a form by a specific submit element via `put_submitter/2`:\n\n    assert view\n           |> form(\"#term\", user: %{name: \"hello\"})\n           |> put_submitter(\"button[name=example_action]\")\n           |> render_submit() =~ \"Action taken\"","ref":"Phoenix.LiveViewTest.html#render_submit/2-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_submit/3","doc":"Sends a form submit event to the view and returns the rendered result.\n\nIt returns the contents of the whole LiveView or an `{:error, redirect}`\ntuple.","ref":"Phoenix.LiveViewTest.html#render_submit/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_submit/3","doc":"{:ok, view, html} = live(conn, \"/thermo\")\n    assert html =~ \"The temp is: 30℉\"\n    assert render_submit(view, :refresh, %{deg: 32}) =~ \"The temp is: 32℉\"","ref":"Phoenix.LiveViewTest.html#render_submit/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.render_upload/3","doc":"Performs an upload of a file input and renders the result.\n\nSee `file_input/4` for details on building a file input.","ref":"Phoenix.LiveViewTest.html#render_upload/3"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.render_upload/3","doc":"Given the following LiveView template:\n\n    <%= for entry <- @uploads.avatar.entries do %>\n        <%= entry.name %>: <%= entry.progress %>%\n    <% end %>\n\nYour test case can assert the uploaded content:\n\n    avatar = file_input(lv, \"#my-form-id\", :avatar, [\n      %{\n        last_modified: 1_594_171_879_000,\n        name: \"myfile.jpeg\",\n        content: File.read!(\"myfile.jpg\"),\n        size: 1_396_009,\n        type: \"image/jpeg\"\n      }\n    ])\n\n    assert render_upload(avatar, \"myfile.jpeg\") =~ \"100%\"\n\nBy default, the entire file is chunked to the server, but an optional\npercentage to chunk can be passed to test chunk-by-chunk uploads:\n\n    assert render_upload(avatar, \"myfile.jpeg\", 49) =~ \"49%\"\n    assert render_upload(avatar, \"myfile.jpeg\", 51) =~ \"100%\"\n\nBefore making assertions about the how the upload is consumed server-side,\nyou will need to call `render_submit/1`.\n\nIn the case where an upload progress callback issues a navigate, patch, or\nredirect, the following will be returned:\n\n  * if the navigate is a `live_patch`, the current view will be patched\n    {:error, {:redirect, %{to: url}}}\n  * if the navigate is a `live_redirect`, this function will return\n    `{:error, {:live_redirect, %{to: url}}}`, which can be followed\n    with `follow_redirect/2`\n  * if the navigate is a regular redirect, this function will return\n    `{:error, {:redirect, %{to: url}}}`, which can be followed\n    with `follow_redirect/2`","ref":"Phoenix.LiveViewTest.html#render_upload/3-examples"},{"type":"function","title":"Phoenix.LiveViewTest.rendered_to_string/1","doc":"Converts a rendered template to a string.","ref":"Phoenix.LiveViewTest.html#rendered_to_string/1"},{"type":"function","title":"Examples - Phoenix.LiveViewTest.rendered_to_string/1","doc":"import Phoenix.Component\n    import Phoenix.LiveViewTest\n\n    test \"greets\" do\n      assigns = %{}\n      assert rendered_to_string(~H\"\"\"\n              \n             \"\"\") ==\n               \" Hello, Mary! \"\n    end","ref":"Phoenix.LiveViewTest.html#rendered_to_string/1-examples"},{"type":"macro","title":"Phoenix.LiveViewTest.submit_form/2","doc":"Receives a form element and submits the HTTP request through the plug pipeline.\n\nImagine you have a LiveView that validates form data, but submits the form to\na controller via the normal form `action` attribute. This is especially useful\nin scenarios where the result of a form submit needs to write to the plug session.\n\nYou can follow submit the form with the `%Plug.Conn{}`, like this:\n\n    form = form(live_view, selector, %{\"form\" => \"data\"})\n\n    # Now submit the LiveView form to the plug pipeline\n    conn = submit_form(form, conn)\n    assert conn.method == \"POST\"\n    assert conn.params == %{\"form\" => \"data\"}","ref":"Phoenix.LiveViewTest.html#submit_form/2"},{"type":"function","title":"Phoenix.LiveViewTest.with_target/2","doc":"Sets the target of the view for events.\n\nThis emulates `phx-target` directly in tests, without\nhaving to dispatch the event to a specific element.\nThis can be useful for invoking events to one or\nmultiple components at the same time:\n\n    view\n    |> with_target(\"#user-1,#user-2\")\n    |> render_click(\"Hide\", %{})","ref":"Phoenix.LiveViewTest.html#with_target/2"},{"type":"module","title":"Phoenix.LiveView.HTMLFormatter","doc":"Format HEEx templates from `.heex` files or `~H` sigils.\n\nThis is a `mix format` [plugin](https://hexdocs.pm/mix/main/Mix.Tasks.Format.html#module-plugins).\n\n> Note: The HEEx HTML Formatter requires Elixir v1.13.4 or later.","ref":"Phoenix.LiveView.HTMLFormatter.html"},{"type":"module","title":"Setup - Phoenix.LiveView.HTMLFormatter","doc":"Add it as plugin to your `.formatter.exs` file and make sure to put\nthe`heex` extension in the `inputs` option.\n\n```elixir\n[\n  plugins: [Phoenix.LiveView.HTMLFormatter],\n  inputs: [\"*.{heex,ex,exs}\", \"priv/*/seeds.exs\", \"{config,lib,test}/**/*.{heex,ex,exs}\"],\n  # ...\n]\n```\n\n> #","ref":"Phoenix.LiveView.HTMLFormatter.html#module-setup"},{"type":"module","title":"For umbrella projects {: .info} - Phoenix.LiveView.HTMLFormatter","doc":">\n> In umbrella projects you must also change two files at the umbrella root,\n> add `:phoenix_live_view` to your `deps` in the `mix.exs` file\n> and add `plugins: [Phoenix.LiveView.HTMLFormatter]` in the `.formatter.exs` file.\n> This is because the formatter does not attempt to load the dependencies of\n> all children applications.\n\n#","ref":"Phoenix.LiveView.HTMLFormatter.html#module-for-umbrella-projects-info"},{"type":"module","title":"Editor support - Phoenix.LiveView.HTMLFormatter","doc":"Most editors that support `mix format` integration should automatically format\n`.heex` and `~H` templates. Other editors may require custom integration or\neven provide additional functionality. Here are some reference posts:\n\n  * [Formatting HEEx templates in VS Code](https://pragmaticstudio.com/tutorials/formatting-heex-templates-in-vscode)","ref":"Phoenix.LiveView.HTMLFormatter.html#module-editor-support"},{"type":"module","title":"Options - Phoenix.LiveView.HTMLFormatter","doc":"* `:line_length` - The Elixir formatter defaults to a maximum line length\n    of 98 characters, which can be overwritten with the `:line_length` option\n    in your `.formatter.exs` file.\n\n  * `:heex_line_length` - change the line length only for the HEEx formatter.\n\n    ```elixir\n    [\n      # ...omitted\n      heex_line_length: 300\n    ]\n    ```","ref":"Phoenix.LiveView.HTMLFormatter.html#module-options"},{"type":"module","title":"Formatting - Phoenix.LiveView.HTMLFormatter","doc":"This formatter tries to be as consistent as possible with the Elixir formatter.\n\nGiven HTML like this:\n\n```eex\n        <%= @user.name %>   \n```\n\nIt will be formatted as:\n\n```eex\n \n    <%= @user.name %>  \n \n```\n\nA block element will go to the next line, while inline elements will be kept in the current line\nas long as they fit within the configured line length.\n\nThe following links list all block and inline elements.\n\n* https://developer.mozilla.org/en-US/docs/Web/HTML/Block-level_elements#elements\n* https://developer.mozilla.org/en-US/docs/Web/HTML/Inline_elements#list_of_inline_elements\n\nIt will also keep inline elements in their own lines if you intentionally write them this way:\n\n```eex\n \n   \n     <%= @user.name %> \n   \n \n```\n\nThis formatter will place all attributes on their own lines when they do not all fit in the\ncurrent line. Therefore this:\n\n```eex\n \n   Hi \n \n```\n\nWill be formatted to:\n\n```eex\n \n   Hi \n \n```\n\nThis formatter **does not** format Elixir expressions with `do...end`.\nThe content within it will be formatted accordingly though. Therefore, the given\ninput:\n\n```eex\n<%= live_redirect(\n       to: \"/my/path\",\n  class: \"my class\"\n) do %>\n        My Link\n<% end %>\n```\n\nWill be formatted to\n\n```eex\n<%= live_redirect(\n       to: \"/my/path\",\n  class: \"my class\"\n) do %>\n  My Link\n<% end %>\n```\n\nNote that only the text `My Link` has been formatted.\n\n#","ref":"Phoenix.LiveView.HTMLFormatter.html#module-formatting"},{"type":"module","title":"Intentional new lines - Phoenix.LiveView.HTMLFormatter","doc":"The formatter will keep intentional new lines. However, the formatter will\nalways keep a maximum of one line break in case you have multiple ones:\n\n```eex\n \n  text\n\n\n  text\n \n```\n\nWill be formatted to:\n\n```eex\n \n  text\n\n  text\n \n```\n\n#","ref":"Phoenix.LiveView.HTMLFormatter.html#module-intentional-new-lines"},{"type":"module","title":"Inline elements - Phoenix.LiveView.HTMLFormatter","doc":"We don't format inline elements when there is a text without whitespace before\nor after the element. Otherwise it would compromise what is rendered adding\nan extra whitespace.\n\nThis is the list of inline elements:\n\nhttps://developer.mozilla.org/en-US/docs/Web/HTML/Inline_elements#list_of_inline_elements","ref":"Phoenix.LiveView.HTMLFormatter.html#module-inline-elements"},{"type":"module","title":"Skip formatting - Phoenix.LiveView.HTMLFormatter","doc":"In case you don't want part of your HTML to be automatically formatted.\nYou can use the special `phx-no-format` attribute so that the formatter will\nskip the element block. Note that this attribute will not be rendered.\n\nTherefore:\n\n```eex\n<.textarea phx-no-format>My content</.textarea>\n```\n\nWill be kept as is your code editor, but rendered as:\n\n```html\n My content \n```","ref":"Phoenix.LiveView.HTMLFormatter.html#module-skip-formatting"},{"type":"module","title":"Comments - Phoenix.LiveView.HTMLFormatter","doc":"Inline comments `<%# comment %>` are deprecated and the formatter will discard them\nsilently from templates. You must change them to the multi-line comment\n`<%!-- comment --%>` on Elixir v1.14+ or introduce a space between `<%` and `#`,\nsuch as `<% # comment %>`.","ref":"Phoenix.LiveView.HTMLFormatter.html#module-comments"},{"type":"macro","title":"Phoenix.LiveView.HTMLFormatter.is_tag_open/1","doc":"","ref":"Phoenix.LiveView.HTMLFormatter.html#is_tag_open/1"},{"type":"module","title":"Phoenix.LiveView.Logger","doc":"Instrumenter to handle logging of `Phoenix.LiveView` and `Phoenix.LiveComponent` life-cycle events.","ref":"Phoenix.LiveView.Logger.html"},{"type":"module","title":"Installation - Phoenix.LiveView.Logger","doc":"The logger is installed automatically when Live View starts.\nBy default, the log level is set to `:debug`.","ref":"Phoenix.LiveView.Logger.html#module-installation"},{"type":"module","title":"Module configuration - Phoenix.LiveView.Logger","doc":"The log level can be overridden for an individual Live View module:\n\n    use Phoenix.LiveView, log: :debug\n\nTo disable logging for an individual Live View module:\n\n    use Phoenix.LiveView, log: false","ref":"Phoenix.LiveView.Logger.html#module-module-configuration"},{"type":"module","title":"Telemetry - Phoenix.LiveView.Logger","doc":"The following `Phoenix.LiveView` and `Phoenix.LiveComponent` events are logged:\n\n  - `[:phoenix, :live_view, :mount, :start]`\n  - `[:phoenix, :live_view, :mount, :stop]`\n  - `[:phoenix, :live_view, :handle_params, :start]`\n  - `[:phoenix, :live_view, :handle_params, :stop]`\n  - `[:phoenix, :live_view, :handle_event, :start]`\n  - `[:phoenix, :live_view, :handle_event, :stop]`\n  - `[:phoenix, :live_component, :handle_event, :start]`\n  - `[:phoenix, :live_component, :handle_event, :stop]`\n\nSee the [Telemetry](./guides/server/telemetry.md) guide for more information.","ref":"Phoenix.LiveView.Logger.html#module-telemetry"},{"type":"module","title":"Parameter filtering - Phoenix.LiveView.Logger","doc":"If enabled, `Phoenix.LiveView.Logger` will filter parameters based on the configuration of `Phoenix.Logger`.","ref":"Phoenix.LiveView.Logger.html#module-parameter-filtering"},{"type":"module","title":"Phoenix.LiveView.Socket","doc":"The LiveView socket for Phoenix Endpoints.\n\nThis is typically mounted directly in your endpoint.\n\n    socket \"/live\", Phoenix.LiveView.Socket,\n      websocket: [connect_info: [session: @session_options]]\n\nTo share an underlying transport connection between regular\nPhoenix channels and LiveView processes, `use Phoenix.LiveView.Socket`\nfrom your own `MyAppWeb.UserSocket` module.\n\nNext, declare your `channel` definitions and optional `connect/3`, and\n`id/1` callbacks to handle your channel specific needs, then mount\nyour own socket in your endpoint:\n\n    socket \"/live\", MyAppWeb.UserSocket,\n      websocket: [connect_info: [session: @session_options]]\n\nIf you require session options to be set at runtime, you can use\nan MFA tuple. The function it designates must return a keyword list.\n\n    socket \"/live\", MyAppWeb.UserSocket,\n      websocket: [connect_info: [session: {__MODULE__, :runtime_opts, []}]]\n\n    # ...\n\n    def runtime_opts() do\n      Keyword.put(@session_options, :domain, host())\n    end","ref":"Phoenix.LiveView.Socket.html"},{"type":"type","title":"Phoenix.LiveView.Socket.assigns/0","doc":"The data in a LiveView as stored in the socket.","ref":"Phoenix.LiveView.Socket.html#t:assigns/0"},{"type":"opaque","title":"Phoenix.LiveView.Socket.assigns_not_in_socket/0","doc":"Struct returned when `assigns` is not in the socket.","ref":"Phoenix.LiveView.Socket.html#t:assigns_not_in_socket/0"},{"type":"type","title":"Phoenix.LiveView.Socket.fingerprints/0","doc":"","ref":"Phoenix.LiveView.Socket.html#t:fingerprints/0"},{"type":"type","title":"Phoenix.LiveView.Socket.t/0","doc":"","ref":"Phoenix.LiveView.Socket.html#t:t/0"},{"type":"module","title":"Phoenix.LiveViewTest.Element","doc":"The struct returned by `Phoenix.LiveViewTest.element/3`.\n\nThe following public fields represent the element:\n\n  * `selector` - The query selector\n  * `text_filter` - The text to further filter the element\n\nSee the `Phoenix.LiveViewTest` documentation for usage.","ref":"Phoenix.LiveViewTest.Element.html"},{"type":"module","title":"Phoenix.LiveViewTest.Upload","doc":"The struct returned by `Phoenix.LiveViewTest.file_input/4`.\n\nThe following public fields represent the element:\n\n  * `selector` - The query selector\n  * `entries` - The list of selected file entries\n\nSee the `Phoenix.LiveViewTest` documentation for usage.","ref":"Phoenix.LiveViewTest.Upload.html"},{"type":"module","title":"Phoenix.LiveViewTest.View","doc":"The struct for testing LiveViews.\n\nThe following public fields represent the LiveView:\n\n  * `id` - The DOM id of the LiveView\n  * `module` - The module of the running LiveView\n  * `pid` - The Pid of the running LiveView\n  * `endpoint` - The endpoint for the LiveView\n  * `target` - The target to scope events to\n\nSee the `Phoenix.LiveViewTest` documentation for usage.","ref":"Phoenix.LiveViewTest.View.html"},{"type":"module","title":"Phoenix.LiveView.UploadConfig","doc":"The struct representing an upload.","ref":"Phoenix.LiveView.UploadConfig.html"},{"type":"type","title":"Phoenix.LiveView.UploadConfig.t/0","doc":"","ref":"Phoenix.LiveView.UploadConfig.html#t:t/0"},{"type":"module","title":"Phoenix.LiveView.UploadEntry","doc":"The struct representing an upload entry.","ref":"Phoenix.LiveView.UploadEntry.html"},{"type":"type","title":"Phoenix.LiveView.UploadEntry.t/0","doc":"","ref":"Phoenix.LiveView.UploadEntry.html#t:t/0"},{"type":"behaviour","title":"Phoenix.LiveView.UploadWriter","doc":"Provide a behavior for writing uploaded chunks to a final destination.\n\nBy default, uploads are written to a temporary file on the server and\nconsumed by the LiveView by reading the temporary file or copying it to\ndurable location. Some usecases require custom handling of the uploaded\nchunks, such as streaming a user's upload to another server. In these cases,\nwe don't want the chunks to be written to disk since we only need to forward\nthem on.\n\n**Note**: Upload writers run inside the channel uploader process, so\nany blocking work will block the channel  errors will crash the channel process.\n\nCustom implementations of `Phoenix.LiveView.UploadWriter` can be passed to\n`allow_upload/3`. To initialize the writer with options, define a 3-arity function\nthat returns a tuple of `{writer, writer_opts}`. For example imagine\nan upload writer that logs the chunk sizes and tracks the total bytes sent by the\nclient:\n\n    socket\n    |> allow_upload(:avatar,\n      accept: :any,\n      writer: fn _name, _entry, _socket -> {EchoWriter, level: :debug} end\n    )\n\nAnd such an `EchoWriter` could look like this:\n\n    defmodule EchoWriter do\n      @behaviour Phoenix.LiveView.UploadWriter\n\n      require Logger\n\n      @impl true\n      def init(opts) do\n        {:ok, %{total: 0, level: Keyword.fetch!(opts, :level)}}\n      end\n\n      @impl true\n      def meta(state), do: %{level: state.level}\n\n      @impl true\n      def write_chunk(data, state) do\n        size = byte_size(data)\n        Logger.log(state.level, \"received chunk of #{size} bytes\")\n        {:ok, %{state | total: state.total + size}}\n      end\n\n      @impl true\n      def close(state, reason) do\n        Logger.log(state.level, \"closing upload after #{state.total} bytes}, #{inspect(reason)}\")\n        {:ok, state}\n      end\n    end\n\nWhen the LiveView consumes the uploaded entry, it will receive the `%{level: ...}`\nreturned from the meta callback. This allows the writer to keep state as it handles\nchunks to be later relayed to the LiveView when consumed.","ref":"Phoenix.LiveView.UploadWriter.html"},{"type":"behaviour","title":"Close reasons - Phoenix.LiveView.UploadWriter","doc":"The `close/2` callback is called when the upload is complete or cancelled. The following\nvalues can be passed:\n\n  * `:done` - The client sent all expected chunks and the upload is awaiting consumption\n  * `:cancel` - The upload was canceled, either by the server or the client navigating away.\n  * `{:error, reason}` - The upload was canceled due to an error returned from `write_chunk/2`.\n    For example, if If `write_chunk/2` returns `{:error, :enoent, state}`, the upload will be cancelled\n    and `close/2` will be called with the reason `{:error, :enoent}`.","ref":"Phoenix.LiveView.UploadWriter.html#module-close-reasons"},{"type":"callback","title":"Phoenix.LiveView.UploadWriter.close/2","doc":"","ref":"Phoenix.LiveView.UploadWriter.html#c:close/2"},{"type":"callback","title":"Phoenix.LiveView.UploadWriter.init/1","doc":"","ref":"Phoenix.LiveView.UploadWriter.html#c:init/1"},{"type":"callback","title":"Phoenix.LiveView.UploadWriter.meta/1","doc":"","ref":"Phoenix.LiveView.UploadWriter.html#c:meta/1"},{"type":"callback","title":"Phoenix.LiveView.UploadWriter.write_chunk/2","doc":"","ref":"Phoenix.LiveView.UploadWriter.html#c:write_chunk/2"},{"type":"module","title":"Phoenix.LiveComponent.CID","doc":"The struct representing an internal unique reference to the component instance,\navailable as the `@myself` assign in live components.\n\nRead more about the uses of `@myself` in the `Phoenix.LiveComponent` docs.","ref":"Phoenix.LiveComponent.CID.html"},{"type":"module","title":"Phoenix.LiveView.Component","doc":"The struct returned by components in .heex templates.\n\nThis component is never meant to be output directly\ninto the template. It should always be handled by\nthe diffing algorithm.","ref":"Phoenix.LiveView.Component.html"},{"type":"type","title":"Phoenix.LiveView.Component.t/0","doc":"","ref":"Phoenix.LiveView.Component.html#t:t/0"},{"type":"module","title":"Phoenix.LiveView.Comprehension","doc":"The struct returned by for-comprehensions in .heex templates.\n\nSee a description about its fields and use cases\nin `Phoenix.LiveView.Engine` docs.","ref":"Phoenix.LiveView.Comprehension.html"},{"type":"type","title":"Phoenix.LiveView.Comprehension.t/0","doc":"","ref":"Phoenix.LiveView.Comprehension.html#t:t/0"},{"type":"module","title":"Phoenix.LiveView.Engine","doc":"An `EEx` template engine that tracks changes.\n\nThis is often used by `Phoenix.LiveView.TagEngine` which also adds\nHTML validation. In the documentation below, we will explain how it\nworks internally. For user-facing documentation, see `Phoenix.LiveView`.","ref":"Phoenix.LiveView.Engine.html"},{"type":"module","title":"Phoenix.LiveView.Rendered - Phoenix.LiveView.Engine","doc":"Whenever you render a live template, it returns a\n`Phoenix.LiveView.Rendered` structure. This structure has\nthree fields: `:static`, `:dynamic` and `:fingerprint`.\n\nThe `:static` field is a list of literal strings. This\nallows the Elixir compiler to optimize this list and avoid\nallocating its strings on every render.\n\nThe `:dynamic` field contains a function that takes a boolean argument\n(see \"Tracking changes\" below), and returns a list of dynamic content.\nEach element in the list is either one of:\n\n  1. iodata - which is the dynamic content\n  2. nil - the dynamic content did not change\n  3. another `Phoenix.LiveView.Rendered` struct, see \"Nesting and fingerprinting\" below\n  4. a `Phoenix.LiveView.Comprehension` struct, see \"Comprehensions\" below\n  5. a `Phoenix.LiveView.Component` struct, see \"Component\" below\n\nWhen you render a live template, you can convert the\nrendered structure to iodata by alternating the static\nand dynamic fields, always starting with a static entry\nfollowed by a dynamic entry. The last entry will always\nbe static too. So the following structure:\n\n    %Phoenix.LiveView.Rendered{\n      static: [\"foo\", \"bar\", \"baz\"],\n      dynamic: fn track_changes? -> [\"left\", \"right\"] end\n    }\n\nResults in the following content to be sent over the wire\nas iodata:\n\n    [\"foo\", \"left\", \"bar\", \"right\", \"baz\"]\n\nThis is also what calling `Phoenix.HTML.Safe.to_iodata/1`\nwith a `Phoenix.LiveView.Rendered` structure returns.\n\nOf course, the benefit of live templates is exactly\nthat you do not need to send both static and dynamic\nsegments every time. So let's talk about tracking changes.","ref":"Phoenix.LiveView.Engine.html#module-phoenix-liveview-rendered"},{"type":"module","title":"Tracking changes - Phoenix.LiveView.Engine","doc":"By default, a live template does not track changes.\nChange tracking can be enabled by including a changed\nmap in the assigns with the key `__changed__` and passing\n`true` to the dynamic parts. The map should contain\nthe name of any changed field as key and the boolean\ntrue as value. If a field is not listed in `__changed__`,\nthen it is always considered unchanged.\n\nIf a field is unchanged and live believes a dynamic\nexpression no longer needs to be computed, its value\nin the `dynamic` list will be `nil`. This information\ncan be leveraged to avoid sending data to the client.","ref":"Phoenix.LiveView.Engine.html#module-tracking-changes"},{"type":"module","title":"Nesting and fingerprinting - Phoenix.LiveView.Engine","doc":"`Phoenix.LiveView` also tracks changes across live\ntemplates. Therefore, if your view has this:\n\n    <%= render \"form.html\", assigns %>\n\nPhoenix will be able to track what is static and dynamic\nacross templates, as well as what changed. A rendered\nnested `live` template will appear in the `dynamic`\nlist as another `Phoenix.LiveView.Rendered` structure,\nwhich must be handled recursively.\n\nHowever, because the rendering of live templates can\nbe dynamic in itself, it is important to distinguish\nwhich live template was rendered. For example,\nimagine this code:\n\n    <%= if something?, do: render(\"one.html\", assigns), else: render(\"other.html\", assigns) %>\n\nTo solve this, all `Phoenix.LiveView.Rendered` structs\nalso contain a fingerprint field that uniquely identifies\nit. If the fingerprints are equal, you have the same\ntemplate, and therefore it is possible to only transmit\nits changes.","ref":"Phoenix.LiveView.Engine.html#module-nesting-and-fingerprinting"},{"type":"module","title":"Comprehensions - Phoenix.LiveView.Engine","doc":"Another optimization done by live templates is to\ntrack comprehensions. If your code has this:\n\n    <%= for point <- @points do %>\n      x: <%= point.x %>\n      y: <%= point.y %>\n    <% end %>\n\nInstead of rendering all points with both static and\ndynamic parts, it returns a `Phoenix.LiveView.Comprehension`\nstruct with the static parts, that are shared across all\npoints, and a list of dynamics to be interpolated inside\nthe static parts. If `@points` is a list with `%{x: 1, y: 2}`\nand `%{x: 3, y: 4}`, the above expression would return:\n\n    %Phoenix.LiveView.Comprehension{\n      static: [\"\\n  x: \", \"\\n  y: \", \"\\n\"],\n      dynamics: [\n        [\"1\", \"2\"],\n        [\"3\", \"4\"]\n      ]\n    }\n\nThis allows live templates to drastically optimize\nthe data sent by comprehensions, as the static parts\nare emitted only once, regardless of the number of items.\n\nThe list of dynamics is always a list of iodatas or components,\nas we don't perform change tracking inside the comprehensions\nthemselves. Similarly, comprehensions do not have fingerprints\nbecause they are only optimized at the root, so conditional\nevaluation, as the one seen in rendering, is not possible.\nThe only possible outcome for a dynamic field that returns a\ncomprehension is `nil`.","ref":"Phoenix.LiveView.Engine.html#module-comprehensions"},{"type":"module","title":"Components - Phoenix.LiveView.Engine","doc":"Live also supports stateful components defined with\n`Phoenix.LiveComponent`. Since they are stateful, they are always\nhandled lazily by the diff algorithm.","ref":"Phoenix.LiveView.Engine.html#module-components"},{"type":"module","title":"Phoenix.LiveView.HTMLEngine","doc":"The HTMLEngine that powers `.heex` templates and the `~H` sigil.\n\nIt works by adding a HTML parsing and validation layer on top\nof `Phoenix.LiveView.TagEngine`.","ref":"Phoenix.LiveView.HTMLEngine.html"},{"type":"module","title":"Phoenix.LiveView.Rendered","doc":"The struct returned by .heex templates.\n\nSee a description about its fields and use cases\nin `Phoenix.LiveView.Engine` docs.","ref":"Phoenix.LiveView.Rendered.html"},{"type":"type","title":"Phoenix.LiveView.Rendered.t/0","doc":"","ref":"Phoenix.LiveView.Rendered.html#t:t/0"},{"type":"behaviour","title":"Phoenix.LiveView.TagEngine","doc":"An EEx engine that understands tags.\n\nThis cannot be directly used by Phoenix applications.\nInstead, it is the building block by engines such as\n`Phoenix.LiveView.HTMLEngine`.\n\nIt is typically invoked like this:\n\n    EEx.compile_string(source,\n      engine: Phoenix.LiveView.TagEngine,\n      line: 1,\n      file: path,\n      caller: __CALLER__,\n      source: source,\n      tag_handler: FooBarEngine\n    )\n\nWhere `:tag_handler` implements the behaviour defined by this module.","ref":"Phoenix.LiveView.TagEngine.html"},{"type":"callback","title":"Phoenix.LiveView.TagEngine.classify_type/1","doc":"Classify the tag type from the given binary.\n\nThis must return a tuple containing the type of the tag and the name of tag.\nFor instance, for LiveView which uses HTML as default tag handler this would\nreturn `{:tag, 'div'}` in case the given binary is identified as HTML tag.\n\nYou can also return {:error, \"reason\"} so that the compiler will display this\nerror.","ref":"Phoenix.LiveView.TagEngine.html#c:classify_type/1"},{"type":"function","title":"Phoenix.LiveView.TagEngine.component/3","doc":"Renders a component defined by the given function.\n\nThis function is rarely invoked directly by users. Instead, it is used by `~H`\nand other engine implementations to render `Phoenix.Component`s. For example,\nthe following:\n\n     \n\nIs the same as:\n\n    <%= component(\n          &MyApp.Weather.city/1,\n          [name: \"Kraków\"],\n          {__ENV__.module, __ENV__.function, __ENV__.file, __ENV__.line}\n        ) %>","ref":"Phoenix.LiveView.TagEngine.html#component/3"},{"type":"macro","title":"Phoenix.LiveView.TagEngine.inner_block/2","doc":"Define a inner block, generally used by slots.\n\nThis macro is mostly used by custom HTML engines that provide\na `slot` implementation and rarely called directly. The\n`name` must be the assign name the slot/block will be stored\nunder.\n\nIf you're using HEEx templates, you should use its higher\nlevel `<:slot>` notation instead. See `Phoenix.Component`\nfor more information.","ref":"Phoenix.LiveView.TagEngine.html#inner_block/2"},{"type":"callback","title":"Phoenix.LiveView.TagEngine.void?/1","doc":"Returns if the given binary is either void or not.\n\nThat's mainly useful for HTML tags and used internally by the compiler. You\ncan just implement as `def void?(_), do: false` if you want to ignore this.","ref":"Phoenix.LiveView.TagEngine.html#c:void?/1"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.20.4 (2024-02-01) - Changelog","doc":"#","ref":"changelog.html#0-20-4-2024-02-01"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix phx-remove on sticky LiveViews\n  * Fix phx-disabled-with not restoring button disabled state on empty diff acknowledgement\n  * Fix stream reset not ordering items correctly\n  * Send `{:shutdown, :cancel}` to `handle_async/3` on `cancel_async`\n  * Prevent events from child LiveViews from bubbling up to root LiveView when child is not mounted yet\n  * Fix `phx-mounted` being called twice for stream items\n  * Never move existing stream items if they already exist (use stream_delete and then stream_insert instead)\n  * Fix live component rendering breaking when the server adds a component back that was pruned by the client (#3026)\n  * Allow redirect from upload progress callback\n  * Fix nested components getting skipped when resetting a stream\n  * Fix nested components getting skipped in LiveComponents\n  * Fix stream limits not being applied correctly when bulk inserting\n  * Fix click-away being called incorrectly on form submits\n  * Fix inconsistencies between LiveViewTest and browser stream implementations\n  * Fix phx-feedback-for being reapplied when there are multiple inputs with the same name\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `JS.toggle_class`\n  * Add `JS.toggle_attribute`\n  * Force update select options when the options changed from the server while a select has focus\n  * Introduce `phx-feedback-group` for handling feedback for composite input groups\n  * Add `validate_attrs` to slots\n  * Support `phx-viewport` bindings in scrollable containers\n  * Perform client redirect when trying to live nav from dead client to avoid extra round trip\n  * Allow regular buttons with name/value attributes to send form events and adjusted dynamic form documentation to reflect this\n  * Allow form attribute on `live_file_input`","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.20.3 (2024-01-02) - Changelog","doc":"#","ref":"changelog.html#0-20-3-2024-01-02"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix phx-viewport bindings failing to fire after navigation\n  * Preserve order of appended items in stream in `LiveViewTest`\n  * Fix order of items on client when resetting a stream to existing set of items\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support `JS.push` from dead views","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.20.2 (2023-12-18) - Changelog","doc":"#","ref":"changelog.html#0-20-2-2023-12-18"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix javascript error when submitting a form that has in progress uploads\n  * Fix JS command `:target` failing to work when used as phx-submit or phx-change with a selector-based target\n  * Fix `JS.focus()` failing to focus negative tabindex\n  * Fix `LiveViewTest` failing to remove items after stream reset\n  * Fix `phx-window-blur` and `phx-window-focus` events not firing\n  * Fix SVG anchor links throwing errors when clicked\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Speed up DOM patching performance 3-30x 🔥\n  * Support `handle_async` Lifecycle callback\n  * Extend visibility checks for phx-click-away to better handle whether an element is visible in the viewport or not\n  * Allow `JS.patch` and `JS.navigate` to be tested with `render_click`\n  * Support `:supervisor` option to `assign_async` and `start_async`","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.20.1 (2023-10-09) - Changelog","doc":"#","ref":"changelog.html#0-20-1-2023-10-09"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix error with live uploads `auto_upload: true` when a file fails to preflight\n  * Fix error with live uploads where an early exit can cause a map key error\n  * Fix match error on live navigation when reconnecting from client\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support new `meta()` method on File/Blob sublcasses on JavaScript client to allow the client to pass arbitrary metadata when using `upload/uploadTo` from hook. The `%UploadEntry{}`'s new `client_meta` field is populated from this information.\n  * Improve void tagging and error messages","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.20.0 (2023-09-22) - Changelog","doc":"#","ref":"changelog.html#0-20-0-2023-09-22"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Deprecate the `~L` sigil in favor of `~H`\n  * Deprecate `preload/1` in LiveComponent in favor of `update_many/1`\n  * Deprecate `live_component/2-3` in favor of `<.live_component />`\n  * Deprecate `live_patch` in favor of `<.link patch={...} />`\n  * Deprecate `live_redirect` in favor of `<.link navigate={...} />`\n  * Deprecate `live_title_tag` in favor of `<.live_title />`\n\n#","ref":"changelog.html#deprecations"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* Remove previously deprecated `render_block/2` in favor of `render_slot/2`\n  * Remove previously deprecated `live_img_preview/2` in favor of `<.live_img_preview />`\n  * Remove previously deprecated `live_file_input/2` in favor of `<.live_file_input />`\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix uploads with `auto_upload: true` failing to propagate errors when any individual entry is invalid\n  * Fix uploads with `auto_upload: true` failing to auto upload valid entries errors when any individual entry is invalid\n  * Fix error on form recovery with `auto_upload: true`\n  * Fix issue on form recovery where hidden inputs would be selected by mistake\n  * Fix form recovery when phx-change is a JS command\n  * Fix stream reset on nested live components with nested streams.\n  * Fix window location resetting to null when using nested LiveView on connection error\n  * Fix anchors within contenteditable causing LiveSocket disconnects\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add heex debug annotations via `config :phoenix_live_view, debug_heex_annotations: true`, which provides special HTML comments that wrap around rendered components to help you identify where markup in your HTML document is rendered within your function component tree\n  * Add `assign_async`, `start_async`, `<.async_result>` and, `AsyncResult` for declaratively handling async operations in a LiveView or LiveComponent.\n  * Supporting passing `@myself` for `Phoenix.LiveView.send_update/3`\n  * Support change tracking on Access.get\n  * Allow overriding `id` of `<.live_img_preview>`","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.19.5 (2023-07-19) - Changelog","doc":"#","ref":"changelog.html#0-19-5-2023-07-19"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* The `close/1` callback of `Phoenix.LiveView.UploadWriter` is now `close/2` with the close reason passed as the second argument.\n  * The `write_chunk` callback of `Phoenix.LiveView.UploadWriter` must now return the updated\n    writer state when an error occurs. Instead of `{:error, reason}`, return `{:error, reason, new_state}`.\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Pass close reason to `Phoenix.LiveView.UploadWriter` close.\n  * Dispatch `phx:navigate` window events when LiveView changes the history state\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Call `Phoenix.LiveView.UploadWriter` close callback when LiveView goes down or connection is lost\n  * Fix JS.patch to a Phoenix router scope with `:host` causing errors\n  * Fix immediate navigation after patch not updating URL\n  * Fix stream reset on nested streams inside live components causing nested stream children to be removed","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.19.4 (2023-07-10) - Changelog","doc":"#","ref":"changelog.html#0-19-4-2023-07-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Introduce `Phoenix.LiveView.UploadWriter`","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.19.3 (2023-06-21) - Changelog","doc":"#","ref":"changelog.html#0-19-3-2023-06-21"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix `push_event` inside component update not being sent in some cases\n  * Bring back accidentally deprecated `upload_errors/1`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.19.2 (2023-06-12) - Changelog","doc":"#","ref":"changelog.html#0-19-2-2023-06-12"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix issue when ` ` is used","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.19.1 (2023-06-06) - Changelog","doc":"#","ref":"changelog.html#0-19-1-2023-06-06"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Allow `accept` attribute on `<.live_file_input>` to override default values\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix issue causing anchor clicks to disconnect LV when they were already handled via `preventDefault()` by other scripts","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.19.0 (2023-05-29) - Changelog","doc":"#","ref":"changelog.html#0-19-0-2023-05-29"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* Drop support for passing an id to the `phx-feedback-for` attribute. An input name must be passed instead.\n  * Remove previously deprecated `let` attribute. Use `:let` instead\n  * Remove previously deprecated `<%= live_img_preview(entry) %>`. Use `<.live_img_preview entry={entry} />` instead\n  * Remove previously deprecated `<%= live_file_input(upload) %>`. Use `<.live_file_input upload={upload} />` instead\n  * Remove previously deprecated `<%= live_component(Component) %>`. Use `<.live_component module={Component} id=\\\"hello\\\" />` instead\n  * Don't convert underscores to dashes automatically when rendering HTML attributes. Use dashes or underscores where appropriate instead.\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support stream resets with bulk insert operations\n  * Support ordered inputs within `inputs_for`, to pair with Ecto's new `sort_param` and `drop_param` casting\n  * Send form phx-value's on form events\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Deprecate passing `:dom_id` to `stream/4` in favor of `stream_configure/3`\n  * Deprecate `render_block/2` in favor of `render_slot/2`\n  * Deprecate `<%= live_img_preview(entry, opts) %>`. Use `<.live_img_preview entry={entry} {opts} />`\n  * Deprecate `<%= live_file_input(upload, opts) %>`. Use `<.live_file_input upload={upload} {opts} />`\n  * Deprecate stateless LiveComponent in favor of function components or in favor of `<.live_component id={...} />` (note the `id` is required)\n\n#","ref":"changelog.html#deprecations"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix LiveView disconnects when clicking a `download` link\n  * Fix stream deletes not being sent on nested for comprehensions\n  * Fix `phx-disconnected` bindings not firing on mount failures\n  * Support form recovery on forms with only hidden inputs","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.18 (2023-03-16) - Changelog","doc":"#","ref":"changelog.html#0-18-18-2023-03-16"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Allow `:live_action` to be assigned in a component\n  * Only filter internal function component attributes in `assigns_to_attributes`\n  * Only include submitter with name\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `JS.exec` command for executing commands defined on other element attributes","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.18.17 (2023-03-09) - Changelog","doc":"#","ref":"changelog.html#0-18-17-2023-03-09"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix callbacks like `handle_info` failing to be invoked in development after a code change with the Phoenix code reloader\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support [`submitter`](https://developer.mozilla.org/en-US/docs/Web/API/SubmitEvent/submitter) on form submit events.\n  * Avoid compile-time dependency for `attr` when referencing structs\n  * Validate reserved assigns. Attempting to assign `:uploads`, `:streams`, `:live_action`, `:socket`, `:myself` will now raise in `LiveView` and `LiveComponent`","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.18.16 (2023-02-23) - Changelog","doc":"#","ref":"changelog.html#0-18-16-2023-02-23"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support streams in Live Components\n  * Optimize plug error translation when a Plug.Exception is raised over connected LiveView\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix formatter issues when there are multiple HTML comments","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.15 (2023-02-16) - Changelog","doc":"#","ref":"changelog.html#0-18-15-2023-02-16"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `JS.transition` applying incorrect classes\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Reset phx-feedback-for errors on `type=\"reset\"` inputs and buttons","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.18.14 (2023-02-14) - Changelog","doc":"#","ref":"changelog.html#0-18-14-2023-02-14"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix LiveViewTest failing to find main live view","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.13 (2023-02-10) - Changelog","doc":"#","ref":"changelog.html#0-18-13-2023-02-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Improve error message when failing to use Phoenix.Component","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.18.12 (2023-02-10) - Changelog","doc":"#","ref":"changelog.html#0-18-12-2023-02-10"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Introduce streams for efficiently handling large collections\n  * Allow replies from `:handle_event` lifecycle hooks\n  * Add `<.inputs_for>` component to `Phoenix.Component`\n  * Support replies on lifecycle `:handle_event` hooks\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix change tracking when re-assigning a defaulted attribute to same default value\n  * Fix upload drag and drop failing to worka after using file select dialog\n  * Fix form recovery when form's first input is phx-change","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.11 (2023-01-19) - Changelog","doc":"#","ref":"changelog.html#0-18-11-2023-01-19"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix socket unloading connection for forms that have defaulted prevented","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.10 (2023-01-18) - Changelog","doc":"#","ref":"changelog.html#0-18-10-2023-01-18"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix svg tags with href incorrectly unloading socket on click\n  * Fix form submits with `target=\"_blank\"` incorrectly unloading socket on submit","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.9 (2023-01-17) - Changelog","doc":"#","ref":"changelog.html#0-18-9-2023-01-17"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix regular form submits failing to be dispatched","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.8 (2023-01-16) - Changelog","doc":"#","ref":"changelog.html#0-18-8-2023-01-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Restore scroll position on back when previous navigation was live patch\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix live layout not being applied until connected render","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.7 (2023-01-13) - Changelog","doc":"#","ref":"changelog.html#0-18-7-2023-01-13"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix live layout not being applied when passed to `:live_session` during disconnect render\n  * Fix external anchor clicks and links with hashes incorrectly unloading socket","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.6 (2023-01-09) - Changelog","doc":"#","ref":"changelog.html#0-18-6-2023-01-09"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix external anchor click unloading on external click","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.5 (2023-01-09) - Changelog","doc":"#","ref":"changelog.html#0-18-5-2023-01-09"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix external anchor click unloading socket","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.4 (2023-01-05) - Changelog","doc":"#","ref":"changelog.html#0-18-4-2023-01-05"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support string upload name to support dynamically generated `allow_upload`'s\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix nested LiveView race condition on live patch causing nested child to skip updates in some cases\n  * Fix browser history showing incorrect title when using live navigation with `@page_title`\n  * Fix undefined _target param when using `JS.push` for form changes\n  * Fix `phx-no-feedback` missing from inputs added after a form submit\n  * Fix `phx-disconnected` events firing when navigating away or submitting external forms","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.3 (2022-10-26) - Changelog","doc":"#","ref":"changelog.html#0-18-3-2022-10-26"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `embed_templates` to `Phoenix.Component` for embedding template files as function components\n  * Raise on global slot attributes\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix bug on slots when passing multiple slot entries with mix if/for syntax","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.18.2 (2022-10-04) - Changelog","doc":"#","ref":"changelog.html#0-18-2-2022-10-04"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix match error when defining `:values` before `:default`\n  * Allow tuples in external redirects\n  * Fix race condition on dispatching click away when enter is pressed\n  * Fix formatter breaking inline blocks when surrounded by text without whitespace\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `intersperse` component for rendering a separator between an enumerable","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.18.1 (2022-09-28) - Changelog","doc":"#","ref":"changelog.html#0-18-1-2022-09-28"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix phx-loading class being applied to dead views\n  * Fix `<.live_img_preview />` causing invalid attribute errors on uploads\n  * Do not fire phx events when element is disabled\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support `:include` option to extend global attributes on a case-by-case basis\n  * Warn when accessing a variable binding defined outside of `~H`","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.18.0 (2022-09-20) - Changelog","doc":"LiveView v0.18 includes a major new feature in the form of declarative assigns with new `attr`\nand `slot` APIs for specifying which attributes a function component supports, the type,\nand default values. Attributes and slots are compile-time verified and emit warnings (requires Elixir v1.14.0+).\n\nv0.18 includes a number of new function components which replace their EEx expression\ncounterparts `<%= ... %>`. For example, `live_redirect`, `live_patch`, and Phoenix.HTML's\n`link` have been replaced by a unified `Phoenix.Component.link/1` function component:\n\n    <.link href=\"https://myapp.com\">my app</.link>\n    <.link navigate={@path}>remount</.link>\n    <.link patch={@path}>patch</.link>\n\nThose new components live in the `Phoenix.Component` module. `Phoenix.LiveView.Helpers`\nitself has been soft deprecated and all relevant functionality has been migrated.\nYou must `import Phoenix.Component` where you previously imported `Phoenix.LiveView.Helpers`\nwhen upgrading. You may also need to `import Phoenix.Component` where you also imported `Phoenix.LiveView` and some of its functions have been moved to `Phoenix.Component`.\n\nAdditionally, the special `let` attribute on function components have been deprecated by\na `:let` usage.\n\n#","ref":"changelog.html#0-18-0-2022-09-20"},{"type":"extras","title":"Deprecations - Changelog","doc":"- `live_redirect` - deprecate in favor of new `<.link navigate={..}>` component of `Phoenix.Component`\n  - `live_patch` - deprecate in favor of new `<.link patch={..}>` component of `Phoenix.Component`\n  - `push_redirect` - deprecate in favor of new `push_navigate` function on `Phoenix.LiveView`\n\n#","ref":"changelog.html#deprecations"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Component] Add declarative assigns with compile-time verifications and warnings via `attr`/`slot`\n  - [Component] Add new attrs `:let` and `:for`, and `:if` with HTML tag, function component, and slot support. We still support `let` but the formatter will convert it to `:let` and soon it will be deprecated.\n  - [Component] Add `dynamic_tag` function component\n  - [Component] Add `link` function component\n  - [Component] Add `focus_wrap` function component to wrap focus around content like modals and dialogs for accessibility\n  - [Logger] Add new LiveView logger with telemetry instrumentation for lifecycle events\n  - [JS] Add new JS commands for `focus`, `focus_first`, `push_focus`, and `pop_focus` for accessibility\n  - [Socket] Support sharing `Phoenix.LiveView.Socket` with regular channels via `use Phoenix.LiveView.Socket`\n  - Add `_live_referer` connect param for handling `push_navigate` referal URL\n  - Add new `phx-connected` and `phx-disconnected` bindings for reacting to lifecycle changes\n  - Add dead view support for JS commands\n  - Add dead view support for hooks\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix external upload issue where listeners are not cleaned up when an external failure happens on the client\n  - Do not debounce `phx-blur`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.12 (2022-09-20) - Changelog","doc":"#","ref":"changelog.html#0-17-12-2022-09-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add support for upcoming Phoenix 1.7 flash interface","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.17.11 (2022-07-11) - Changelog","doc":"#","ref":"changelog.html#0-17-11-2022-07-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `replaceTransport` to LiveSocket\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Cancel debounced events from firing after a live navigation event\n  - Fix hash anchor failing to scroll to anchor element on live navigation\n  - Do not debounce `phx-blur` events","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.10 (2022-05-25) - Changelog","doc":"#","ref":"changelog.html#0-17-10-2022-05-25"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- [Formatter] Preserve single quote delimiter on attrs\n  - [Formatter] Do not format inline elements surrounded by texts without whitespaces\n  - [Formatter] Keep text and eex along when there isn't a whitespace\n  - [Formatter] Fix intentional line breaks after eex expressions\n  - [Formatter] Handle self close tags as inline\n  - [Formatter] Do not format inline elements without whitespaces among them\n  - [Formatter] Do not format when attr contenteditable is present\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Formatter] Introduce special attr phx-no-format to skip formatting","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.17.9 (2022-04-07) - Changelog","doc":"#","ref":"changelog.html#0-17-9-2022-04-07"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix sticky LiveViews failing to be patched during live navigation\n  - Do not raise on dynamic `phx-update` value","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.8 (2022-04-06) - Changelog","doc":"#","ref":"changelog.html#0-17-8-2022-04-06"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add HEEx formatter\n  - Support `phx-change` on individual inputs\n  - Dispatch `MouseEvent` on client\n  - Add `:bubbles` option to `JS.dispatch` to control event bubbling\n  - Expose underlying `liveSocket` instance on hooks\n  - Enable client debug by default on localhost\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix hook and sticky LiveView issues caused by back-to-back live redirects from mount\n  - Fix hook destroyed callback failing to be invoked for children of phx-remove in some cases\n  - Do not failsafe reload the page on push timeout if disconnected\n  - Do not bubble navigation click events to regular phx-click's\n  - No longer generate `csrf_token` for forms without action, reducing the payload during phx-change/phx-submit events","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.7 (2022-02-07) - Changelog","doc":"#","ref":"changelog.html#0-17-7-2022-02-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Optimize nested for comprehension diffs\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix error when `live_redirect` links are clicked when not connected in certain cases","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.6 (2022-01-18) - Changelog","doc":"#","ref":"changelog.html#0-17-6-2022-01-18"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `JS.set_attribute` and `JS.remove_attribute`\n  - Add `sticky: true` option to `live_render` to maintain a nested child on across live redirects\n  - Dispatch `phx:show-start`, `phx:show-end`, `phx:hide-start` and `phx:hide-end` on `JS.show|hide|toggle`\n  - Add `get_connect_info/2` that also works on disconnected render\n  - Add `LiveSocket` constructor options for configuration failsafe behavior via new `maxReloads`, `reloadJitterMin`, `reloadJitterMax`, `failsafeJitter` options\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Show form errors after submit even when no changes occur on server\n  - Fix `phx-disable-with` failing to disable elements outside of forms\n  - Fix phx ref tracking leaving elements in awaiting state when targeting an external LiveView\n  - Fix diff on response failing to await for active transitions in certain cases\n  - Fix `phx-click-away` not respecting `phx-target`\n  - Fix \"disconnect\" broadcast failing to failsafe refresh the page\n  - Fix `JS.push` with `:target` failing to send to correct component in certain cases\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog","doc":"- Deprecate `Phoenix.LiveView.get_connect_info/1` in favor of `get_connect_info/2`\n   - Deprecate `Phoenix.LiveViewTest.put_connect_info/2` in favor of calling the relevant functions in `Plug.Conn`\n   - Deprecate returning \"raw\" values from upload callbacks on `Phoenix.LiveView.consume_uploaded_entry/3` and `Phoenix.LiveView.consume_uploaded_entries/3`. The callback must return either `{:ok, value}` or `{:postpone, value}`. Returning any other value will emit a warning.","ref":"changelog.html#deprecations"},{"type":"extras","title":"0.17.5 (2021-11-02) - Changelog","doc":"#","ref":"changelog.html#0-17-5-2021-11-02"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Do not trigger `phx-click-away` if element is not visible\n  - Fix `phx-remove` failing to tear down nested live children","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.4 (2021-11-01) - Changelog","doc":"#","ref":"changelog.html#0-17-4-2021-11-01"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix variable scoping issues causing various content block or duplication rendering bugs","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.3 (2021-10-28) - Changelog","doc":"#","ref":"changelog.html#0-17-3-2021-10-28"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support 3-tuple for JS class transitions to support staged animations where a transition class is applied with a starting and ending class\n  - Allow JS commands to be executed on DOM nodes outside of the LiveView container\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Optimization - Changelog","doc":"- Avoid duplicate statics inside comprehension. In previous versions, comprehensions were able to avoid duplication only in the content of their root. Now we recursively traverse all comprehension nodes and send the static only once for the whole comprehension. This should massively reduce the cost of sending comprehensions over the wire\n\n#","ref":"changelog.html#optimization"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix HTML engine bug causing expressions to be duplicated or not rendered correctly\n  - Fix HTML engine bug causing slots to not be re-rendered when they should have\n  - Fix form recovery being sent to wrong target","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.2 (2021-10-22) - Changelog","doc":"#","ref":"changelog.html#0-17-2-2021-10-22"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix HTML engine bug causing attribute expressions to be incorrectly evaluated in certain cases\n  - Fix show/hide/toggle custom display not being restored\n  - Fix default `to` target for `JS.show|hide|dispatch`\n  - Fix form input targeting","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.1 (2021-10-21) - Changelog","doc":"#","ref":"changelog.html#0-17-1-2021-10-21"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix SVG element support for `phx` binding interactions","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.17.0 (2021-10-21) - Changelog","doc":"#","ref":"changelog.html#0-17-0-2021-10-21"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"#### `on_mount` changes\n\nThe hook API introduced in LiveView 0.16 has been improved based on feedback.\nLiveView 0.17 removes the custom module-function callbacks for the\n`Phoenix.LiveView.on_mount/1` macro and the `:on_mount` option for\n`Phoenix.LiveView.Router.live_session/3` in favor of supporting a custom\nargument. For clarity, the module function to be invoked during the mount\nlifecycle stage will always be named `on_mount/4`.\n\nFor example, if you had invoked `on_mount/1` like so:\n\n```elixir\non_mount MyAppWeb.MyHook\non_mount {MyAppWeb.MyHook, :assign_current_user}\n```\n\nand defined your callbacks as:\n\n```elixir\n# my_hook.ex\n\ndef mount(_params, _session, _socket) do\nend\n\ndef assign_current_user(_params, _session, _socket) do\nend\n```\n\nChange the callback to:\n\n```elixir\n# my_hook.ex\n\ndef on_mount(:default, _params, _session, _socket) do\nend\n\ndef on_mount(:assign_current_user, _params, _session, _socket) do\nend\n```\n\nWhen given only a module name, the first argument to `on_mount/4` will be the\natom `:default`.\n\n##","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"LEEx templates in stateful LiveComponents - Changelog","doc":"Stateful LiveComponents (where an `:id` is given) must now return HEEx templates\n(`~H` sigil or `.heex` extension). LEEx templates (`~L` sigil or `.leex` extension)\nare no longer supported. This addresses bugs and allows stateful components\nto be rendered more efficiently client-side.\n\n#### `phx-disconnected` class has been replaced with `phx-loading`\n\nDue to a bug in the newly released Safari 15, the previously used `.phx-disconnected` class has been replaced by a new `.phx-loading` class. The reason for the change is `phx.new` included a `.phx-disconnected` rule in the generated `app.css` which triggers the Safari bug. Renaming the class avoids applying the erroneous rule for existing applications. Folks can upgrade by simply renaming their `.phx-disconnected` rules to `.phx-loading`.\n\n#### `phx-capture-click` has been deprecated in favor of `phx-click-away`\n\nThe new `phx-click-away` binding replaces `phx-capture-click` and is much more versatile because it can detect \"click focus\" being lost on containers.\n\n##","ref":"changelog.html#leex-templates-in-stateful-livecomponents"},{"type":"extras","title":"Removal of previously deprecated functionality - Changelog","doc":"Some functionality that was previously deprecated has been removed:\n\n  - Implicit assigns in `live_component` do-blocks is no longer supported\n  - Passing a `@socket` to `live_component` will now raise if possible\n\n#","ref":"changelog.html#removal-of-previously-deprecated-functionality"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow slots in function components: they are marked as `<:slot_name>` and can be rendered with `<%= render_slot @slot_name %>`\n  - Add `JS` command for executing JavaScript utility operations on the client with an extended push API\n  - Optimize string attributes:\n    - If the attribute is a string interpolation, such as ` `, only the interpolation part is marked as dynamic\n    - If the attribute can be empty, such as \"class\" and \"style\", keep the attribute name as static\n  - Add a function component for rendering `Phoenix.LiveComponent`. Instead of `<%= live_component FormComponent, id: \"form\" %>`, you must now do: `<.live_component module={FormComponent} id=\"form\" />`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix LiveViews with form recovery failing to properly mount following a reconnect when preceded by a live redirect\n  - Fix stale session causing full redirect fallback when issuing a `push_redirect` from mount\n  - Add workaround for Safari bug causing ` ` tags with srcset and video with autoplay to fail to render\n  - Support EEx interpolation inside HTML comments in HEEx templates\n  - Support HTML tags inside script tags (as in regular HTML)\n  - Raise if using quotes in attribute names\n  - Include the filename in error messages when it is not possible to parse interpolated attributes\n  - Make sure the test client always sends the full URL on `live_patch`/`live_redirect`. This mirrors the behaviour of the JavaScript client\n  - Do not reload flash from session on `live_redirect`s\n  - Fix select drop-down flashes in Chrome when the DOM is patched during focus\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog","doc":"- `<%= live_component MyModule, id: @user.id, user: @user %>` is deprecated in favor of `<.live_component module={MyModule} id={@user.id} user={@user} />`. Notice the new API requires using HEEx templates. This change allows us to further improve LiveComponent and bring new features such as slots to them.\n  - `render_block/2` in deprecated in favor of `render_slot/2`","ref":"changelog.html#deprecations"},{"type":"extras","title":"0.16.4 (2021-09-22) - Changelog","doc":"#","ref":"changelog.html#0-16-4-2021-09-22"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Improve HEEx error messages\n  - Relax HTML tag validation to support mixed case tags\n  - Support self closing HTML tags\n  - Remove requirement for `handle_params` to be defined for lifecycle hooks\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix pushes failing to include channel `join_ref` on messages","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.16.3 (2021-09-03) - Changelog","doc":"#","ref":"changelog.html#0-16-3-2021-09-03"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `on_mount` hooks calling view mount before redirecting when the hook issues a halt redirect.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.16.2 (2021-09-03) - Changelog","doc":"#","ref":"changelog.html#0-16-2-2021-09-03"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Improve error messages on tokenization\n  - Improve error message if `@inner_block` is missing\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `phx-change` form recovery event being sent to wrong component on reconnect when component order changes","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.16.1 (2021-08-26) - Changelog","doc":"#","ref":"changelog.html#0-16-1-2021-08-26"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Relax `phoenix_html` dependency requirement\n  - Allow testing functional components by passing a function reference\n    to `Phoenix.LiveViewTest.render_component/3`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Do not generate CSRF tokens for non-POST forms\n  - Do not add compile-time dependencies on `on_mount` declarations","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.16.0 (2021-08-10) - Changelog","doc":"#","ref":"changelog.html#0-16-0-2021-08-10"},{"type":"extras","title":"Security Considerations Upgrading from 0.15 - Changelog","doc":"LiveView v0.16 optimizes live redirects by supporting navigation purely\nover the existing WebSocket connection. This is accomplished by the new\n`live_session/3` feature of `Phoenix.LiveView.Router`. The\n[security guide](/guides/server/security-model.md) has always stressed\nthe following:\n\n> ... As we have seen, LiveView begins its life-cycle as a regular HTTP\n> request. Then a stateful connection is established. Both the HTTP\n> request and the stateful connection receives the client data via\n> parameters and session. This means that any session validation must\n> happen both in the HTTP request (plug pipeline) and the stateful\n> connection (LiveView mount) ...\n\nThese guidelines continue to be valid, but it is now essential that the\nstateful connection enforces authentication and session validation within\nthe LiveView mount lifecycle because **a `live_redirect` from the client\nwill not go through the plug pipeline** as a hard-refresh or initial HTTP\nrender would. This means authentication, authorization, etc that may be\ndone in the `Plug.Conn` pipeline must also be performed within the\nLiveView mount lifecycle.\n\nLive sessions allow you to support a shared security model by allowing\n`live_redirect`s to only be issued between routes defined under the same\nlive session name. If a client attempts to live redirect to a different\nlive session, it will be refused and a graceful client-side redirect will\ntrigger a regular HTTP request to the attempted URL.\n\nSee the `Phoenix.LiveView.Router.live_session/3` docs for more information\nand example usage.\n\n#","ref":"changelog.html#security-considerations-upgrading-from-0-15"},{"type":"extras","title":"New HTML Engine - Changelog","doc":"LiveView v0.16 introduces HEEx (HTML + EEx) templates and the concept of function\ncomponents via `Phoenix.Component`. The new HEEx templates validate the markup in\nthe template while also providing smarter change tracking as well as syntax\nconveniences to make it easier to build composable components.\n\nA function component is any function that receives a map of assigns and returns\na `~H` template:\n\n```elixir\ndefmodule MyComponent do\n  use Phoenix.Component\n\n  def btn(assigns) do\n    ~H\"\"\"\n     <%= @text %> \n    \"\"\"\n  end\nend\n```\n\nThis component can now be used as in your HEEx templates as:\n\n     \n\nThe introduction of HEEx and function components brings a series of deprecation\nwarnings, some introduced in this release and others which will be added in the\nfuture. Note HEEx templates require Elixir v1.12+.\n\n#","ref":"changelog.html#new-html-engine"},{"type":"extras","title":"Upgrading and deprecations - Changelog","doc":"The main deprecation in this release is that the `~L` sigil and the `.leex` extension\nare now soft-deprecated. The docs have been updated to discourage them and using them\nwill emit warnings in future releases. We recommend using the `~H` sigil and the `.heex`\nextension for all future templates in your application. You should also plan to migrate\nthe old templates accordingly using the recommendations below.\n\nMigrating from `LEEx` to `HEEx` is relatively straightforward. There are two main differences.\nFirst of all, HEEx does not allow interpolation inside tags. So instead of:\n\n```elixir\n \n  ...\n \n```\n\nOne should use the HEEx syntax:\n\n```elixir\n \n  ...\n \n```\n\nThe other difference is in regards to `form_for`. Some templates may do the following:\n\n```elixir\n~L\"\"\"\n<%= f = form_for @changeset, \"#\" %>\n  <%= input f, :foo %>\n \n\"\"\"\n```\n\nHowever, when converted to `~H`, it is not valid HTML: there is a ` ` tag but\nits opening is hidden inside the Elixir code. On LiveView v0.16, there is a function\ncomponent named `form`:\n\n```elixir\n~H\"\"\"\n<.form :let={f} for={@changeset}>\n  <%= input f, :foo %>\n</.form>\n\"\"\"\n```\n\nWe understand migrating all templates from `~L` to `~H` can be a daunting task.\nTherefore we plan to support `~L` in LiveViews for a long time. However, we can't\ndo the same for stateful LiveComponents, as some important client-side features and\noptimizations will depend on the `~H` sigil. Therefore **our recommendation is to\nreplace `~L` by `~H` first in live components**, particularly stateful live components.\n\nFurthermore, stateless `live_component` (i.e. live components without an `:id`)\nwill be deprecated in favor of the new function components. Our plan is to support\nthem for a reasonable period of time, but you should avoid creating new ones in\nyour application.\n\n#","ref":"changelog.html#upgrading-and-deprecations"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"LiveView 0.16 removes the `:layout` and `:container` options from\n`Phoenix.LiveView.Routing.live/4` in favor of the `:root_layout`\nand `:container` options on `Phoenix.Router.live_session/3`.\n\nFor instance, if you have the following in LiveView 0.15 and prior:\n\n```elixir\nlive \"/path\", MyAppWeb.PageLive, layout: {MyAppWeb.LayoutView, \"custom_layout.html\"}\n```\n\nChange it to:\n\n```elixir\nlive_session :session_name, root_layout: {MyAppWeb.LayoutView, \"custom_layout.html\"} do\n  live \"/path\", MyAppWeb.PageLive\nend\n```\n\nOn the client, the `phoenix_live_view` package no longer provides a default export for `LiveSocket`.\n\nIf you have the following in your JavaScript entrypoint (typically located at `assets/js/app.js`):\n\n```js\nimport LiveSocket from \"phoenix_live_view\"\n```\n\nChange it to:\n\n```js\nimport { LiveSocket } from \"phoenix_live_view\"\n```\n\nAdditionally on the client, the root LiveView element no longer exposes the\nLiveView module name, therefore the `phx-view` attribute is never set.\nSimilarly, the `viewName` property of client hooks has been removed.\n\nCodebases calling a custom function `component/3` should rename it or specify its module to avoid a conflict,\nas LiveView introduces a macro with that name and it is special cased by the underlying engine.\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Introduce HEEx templates\n  - Introduce `Phoenix.Component`\n  - Introduce `Phoenix.Router.live_session/3` for optimized live redirects\n  - Introduce `on_mount` and `attach_hook` hooks which provide a mechanism to tap into key stages of the LiveView lifecycle\n  - Add upload methods to client-side hooks\n  - [Helpers] Optimize `live_img_preview` rendering\n  - [Helpers] Introduce `form` function component which wraps `Phoenix.HTML.form_for`\n  - [LiveViewTest] Add `with_target` for scoping components directly\n  - [LiveViewTest] Add `refute_redirected`\n  - [LiveViewTest] Support multiple `phx-target` values to mirror JS client\n  - [LiveViewTest] Add `follow_trigger_action`\n  - [JavaScript Client] Add `sessionStorage` option `LiveSocket` constructor to support client storage overrides\n  - [JavaScript Client] Do not failsafe reload the page in the background when a tab is unable to connect if the page is not visible\n\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Make sure components are loaded on `render_component` to ensure all relevant callbacks are invoked\n  - Fix `Phoenix.LiveViewTest.page_title` returning `nil` in some cases\n  - Fix buttons being re-enabled when explicitly set to disabled on server\n  - Fix live patch failing to update URL when live patch link is patched again via `handle_params` within the same callback lifecycle\n  - Fix `phx-no-feedback` class not applied when page is live-patched\n  - Fix `DOMException, querySelector, not a valid selector` when performing DOM lookups on non-standard IDs\n  - Fix select dropdown flashing close/opened when assigns are updated on Chrome/macOS\n  - Fix error with multiple `live_file_input` in one form\n  - Fix race condition in `showError` causing null `querySelector`\n  - Fix statics not resolving correctly across recursive diffs\n  - Fix no function clause matching in `Phoenix.LiveView.Diff.many_to_iodata`\n  - Fix upload input not being cleared after files are uploaded via a component\n  - Fix channel crash when uploading during reconnect\n  - Fix duplicate progress events being sent for large uploads\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog","doc":"- Implicit assigns when passing a `do-end` block to `live_component` is deprecated\n  - The `~L` sigil and the `.leex` extension are now soft-deprecated in favor of `~H` and `.heex`\n  - Stateless live components (a `live_component` call without an `:id`) are deprecated in favor of the new function component feature","ref":"changelog.html#deprecations"},{"type":"extras","title":"0.15.7 (2021-05-24) - Changelog","doc":"#","ref":"changelog.html#0-15-7-2021-05-24"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix broken webpack build throwing missing morphdom dependency","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.15.6 (2021-05-24) - Changelog","doc":"#","ref":"changelog.html#0-15-6-2021-05-24"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix live patch failing to update URL when live patch link is patched again from `handle_params`\n  - Fix regression in `LiveViewTest.render_upload/3` when using channel uploads and progress callback\n  - Fix component uploads not being cleaned up on remove\n  - Fix `KeyError` on LiveView reconnect when an active upload was previously in progress\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support function components via `component/3`\n  - Optimize progress events to send less messages for larger file sizes\n  - Allow session and local storage client overrides\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"- Deprecate `@socket/socket` argument on `live_component/3` call","ref":"changelog.html#deprecations"},{"type":"extras","title":"0.15.5 (2021-04-20) - Changelog","doc":"#","ref":"changelog.html#0-15-5-2021-04-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `upload_errors/1` for returning top-level upload errors\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `consume_uploaded_entry/3` with external uploads causing inconsistent entries state\n  - Fix `push_event` losing events when a single diff produces multiple events from different components\n  - Fix deep merging of component tree sharing","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.15.4 (2021-01-26) - Changelog","doc":"#","ref":"changelog.html#0-15-4-2021-01-26"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix nested `live_render`'s causing remound of child LiveView even when ID does not change\n  - Do not attempt push hook events unless connected\n  - Fix preflighted refs causing `auto_upload: true` to fail to submit form\n  - Replace single upload entry when `max_entries` is 1 instead of accumulating multiple file selections\n  - Fix `static_path` in `open_browser` failing to load stylesheets","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.15.3 (2021-01-02) - Changelog","doc":"#","ref":"changelog.html#0-15-3-2021-01-02"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `push_redirect` back causing timeout on the client","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.15.2 (2021-01-01) - Changelog","doc":"#","ref":"changelog.html#0-15-2-2021-01-01"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- Remove `beforeDestroy` from `phx-hook` callbacks\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix form recovery failing to send input on first connection failure\n  - Fix hooks not getting remounted after LiveView reconnect\n  - Fix hooks `reconnected` callback being fired with no prior disconnect","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.15.1 (2020-12-20) - Changelog","doc":"#","ref":"changelog.html#0-15-1-2020-12-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Ensure all click events bubble for mobile Safari\n  - Run `consume_uploaded_entries` in LiveView caller process\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix hooks not getting remounted after LiveView recovery\n  - Fix bug causing reload with jitter on timeout from previously closed channel\n  - Fix component child nodes being lost when component patch goes from single root node to multiple child siblings\n  - Fix `phx-capture-click` triggering on mouseup during text selection\n  - Fix LiveView `push_event`'s not clearing up in components\n  - Fix ` ` being patched by LiveView while focused","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.15.0 (2020-11-20) - Changelog","doc":"#","ref":"changelog.html#0-15-0-2020-11-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add live uploads support for file progress, interactive file selection, and direct to cloud support\n  - Implement `Phoenix.LiveViewTest.open_browser/2` that opens up a browser with the LiveView page\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- Remove `@inner_content` in components and introduce `render_block` for rendering component `@inner_block`\n  - Remove `@live_module` in socket templates in favor of `@socket.view`\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Make sure URLs are decoded after they are split\n  - Do not recover forms without inputs\n  - Fix race condition when components are removed and then immediately re-added before the client can notify their CIDs have been destroyed\n  - Do not render LiveView if only events/replies have been added to the socket\n  - Properly merge different components when sharing component subtrees on initial render\n  - Allow variables inside do-blocks to be tainted\n  - Fix `push_redirect` from mount hanging on the client and causing a fallback to full page reload when following a clicked `live_redirect` on the client","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.8 (2020-10-30) - Changelog","doc":"#","ref":"changelog.html#0-14-8-2020-10-30"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix compatibility with latest Plug","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.7 (2020-09-25) - Changelog","doc":"#","ref":"changelog.html#0-14-7-2020-09-25"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `redirect(socket, external: ...)` when returned from an event\n  - Properly follow location hashes on live patch/redirect\n  - Fix failure in `Phoenix.LiveViewTest` when `phx-update` has non-HTML nodes as children\n  - Fix `phx_trigger_action` submitting the form before the DOM updates are complete","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.6 (2020-09-21) - Changelog","doc":"#","ref":"changelog.html#0-14-6-2020-09-21"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix race condition on `phx-trigger-action` causing reconnects before server form submit","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.5 (2020-09-20) - Changelog","doc":"#","ref":"changelog.html#0-14-5-2020-09-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Optimize DOM prepend and append operations\n  - Add `Phoenix.LiveView.send_update_after/3`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix scroll position when using back/forward with `live_redirect`'s\n  - Handle recursive components when generating diffs\n  - Support hard redirects on mount\n  - Properly track nested components on deletion on `Phoenix.LiveViewTest`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.4 (2020-07-30) - Changelog","doc":"#","ref":"changelog.html#0-14-4-2020-07-30"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix hidden inputs throwing selection range error","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.3 (2020-07-24) - Changelog","doc":"#","ref":"changelog.html#0-14-3-2020-07-24"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support `render_layout` with LiveEEx\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix focused inputs being overwritten by latent patch\n  - Fix LiveView error when `\"_target\"` input name contains array\n  - Fix change tracking when passing a do-block to components","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.2 (2020-07-21) - Changelog","doc":"#","ref":"changelog.html#0-14-2-2020-07-21"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix Map of assigns together with `@inner_content` causing `no function clause matching in Keyword.put/3` error\n  - Fix `LiveViewTest` failing to patch children properly for append/prepend based phx-update's\n  - Fix argument error when providing `:as` option to a `live` route\n  - Fix page becoming unresponsive when the server crashes while handling a live patch\n  - Fix empty diff causing pending data-ref based updates, such as classes and `phx-disable-with` content to not be updated\n  - Fix bug where throttling keydown events would eat key presses\n  - Fix ` `'s failing to be disabled on form submit\n  - Fix text node DOM memory leak when using `phx-update` append/prepend\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow `:router` to be given to `render_component`\n  - Display file on compile warning for `~L`\n  - Log error on client when using a hook without a DOM ID\n  - Optimize `phx-update` append/prepend based DOM updates","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.14.1 (2020-07-09) - Changelog","doc":"#","ref":"changelog.html#0-14-1-2020-07-09"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix nested `live_render`'s failing to be torn down when removed from the DOM in certain cases\n  - Fix LEEx issue for nested conditions failing to be re-evaluated","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.14.0 (2020-07-07) - Changelog","doc":"#","ref":"changelog.html#0-14-0-2020-07-07"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix IE11 issue where `document.activeElement` creates a null reference\n  - Fix setup and teardown of root views when explicitly calling `liveSocket.disconnect()` followed by `liveSocket.connect()`\n  - Fix `error_tag` failing to be displayed for non-text based inputs such as selects and checkboxes as the `phx-no-feedback` class was always applied\n  - Fix `phx-error` class being applied on `live_redirect`\n  - Properly handle Elixir's special variables, such as `__MODULE__`\n  - No longer set disconnected class during patch\n  - Track flash keys to fix back-to-back flashes from being discarded\n  - Properly handle empty component diffs in the client for cases where the component has already been removed on the server\n  - Make sure components in nested live views do not conflict\n  - Fix `phx-static` not being sent from the client for child views\n  - Do not fail when trying to delete a view that was already deleted\n  - Ensure `beforeDestroy` is called on hooks in children of a removed element\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow the whole component static subtree to be shared when the component already exists on the client\n  - Add telemetry events to `mount`, `handle_params`, and `handle_event`\n  - Add `push_event` for pushing events and data from the server to the client\n  - Add client `handleEvent` hook method for receiving events pushed from the server\n  - Add ability to receive a reply to a `pushEvent` from the server via `{:reply, map, socket}`\n  - Use event listener for popstate to avoid conflicting with user-defined popstate handlers\n  - Log error on client when rendering a component with no direct DOM children\n  - Make `assigns.myself` a struct to catch mistakes\n  - Log if component doesn't exist on `send_update`, raise if module is unavailable","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.13.3 (2020-06-04) - Changelog","doc":"#","ref":"changelog.html#0-13-3-2020-06-04"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix duplicate debounced events from being triggered on blur with timed debounce\n  - Fix client error when `live_redirect`ed route results in a redirect to a non-live route on the server\n  - Fix DOM siblings being removed when a rootless component is updated\n  - Fix debounced input failing to send last change when blurred via Tab, Meta, or other non-printable keys\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `dom` option to `LiveSocket` with `onBeforeElUpdated` callback for external client library support of broad DOM operations","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.13.2 (2020-05-27) - Changelog","doc":"#","ref":"changelog.html#0-13-2-2020-05-27"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix a bug where swapping a root template with components would cause the LiveView to crash","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.13.1 (2020-05-26) - Changelog","doc":"#","ref":"changelog.html#0-13-1-2020-05-26"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix forced page refresh when `push_redirect` from a `live_redirect`\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Optimize component diffs to avoid sending empty diffs\n  - Optimize components to share static values\n  - [LiveViewTest] Automatically synchronize before render events","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.13.0 (2020-05-21) - Changelog","doc":"#","ref":"changelog.html#0-13-0-2020-05-21"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- No longer send event metadata by default. Metadata is now opt-in and user defined at the `LiveSocket` level.\n  To maintain backwards compatibility with pre-0.13 behaviour, you can provide the following metadata option:\n\n  ```\n  let liveSocket = new LiveSocket(\"/live\", Socket, {\n    params: {_csrf_token: csrfToken},\n    metadata: {\n      click: (e, el) => {\n        return {\n          altKey: e.altKey,\n          shiftKey: e.shiftKey,\n          ctrlKey: e.ctrlKey,\n          metaKey: e.metaKey,\n          x: e.x || e.clientX,\n          y: e.y || e.clientY,\n          pageX: e.pageX,\n          pageY: e.pageY,\n          screenX: e.screenX,\n          screenY: e.screenY,\n          offsetX: e.offsetX,\n          offsetY: e.offsetY,\n          detail: e.detail || 1,\n        }\n      },\n      keydown: (e, el) => {\n        return {\n          altGraphKey: e.altGraphKey,\n          altKey: e.altKey,\n          code: e.code,\n          ctrlKey: e.ctrlKey,\n          key: e.key,\n          keyIdentifier: e.keyIdentifier,\n          keyLocation: e.keyLocation,\n          location: e.location,\n          metaKey: e.metaKey,\n          repeat: e.repeat,\n          shiftKey: e.shiftKey\n        }\n      }\n    }\n  })\n  ```\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix error caused by Chrome sending a keydown event on native UI autocomplete without a `key`\n  - Fix server error when a live navigation request issues a redirect\n  - Fix double window bindings when explicit calls to LiveSocket connect/disconnect/connect\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Phoenix.LiveView.get_connect_info/1`\n  - Add `Phoenix.LiveViewTest.put_connect_info/2` and `Phoenix.LiveViewTest.put_connect_params/2`\n  - Add support for tracking static asset changes on the page across cold server deploys\n  - Add support for passing a `@myself` target to a hook's `pushEventTo` target\n  - Add configurable metadata for events with new `metadata` LiveSocket option\n  - Add `\"_mounts\"` key in connect params which specifies the number of times a LiveView has mounted","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.12.1 (2020-04-19) - Changelog","doc":"#","ref":"changelog.html#0-12-1-2020-04-19"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix component `innerHTML` being discarded when a sibling DOM element appears above it, in cases where the component lacks a DOM ID\n  - Fix Firefox reconnecting briefly during hard redirects\n  - Fix `phx-disable-with` and other pending attributes failing to be restored when an empty patch is returned by server\n  - Ensure LiveView module is loaded before mount to prevent first application request logging errors if the very first request is to a connected LiveView","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.12.0 (2020-04-16) - Changelog","doc":"This version of LiveView comes with an overhaul of the testing module, more closely integrating your LiveView template with your LiveView events. For example, in previous versions, you could write this test:\n\n  ```elixir\n  render_click(live_view, \"increment_by\", %{by: 1})\n  ```\n\nHowever, there is no guarantee that there is any element on the page with a `phx-click=\"increment\"` attribute and `phx-value-by` set to 1. With LiveView 0.12.0, you can now write:\n\n  ```elixir\n  live_view\n  |> element(\"#term .buttons a\", \"Increment\")\n  |> render_click()\n  ```\n\nThe new implementation will check there is a button at `#term .buttons a`, with \"Increment\" as text, validate that it has a `phx-click` attribute and automatically submit to it with all relevant `phx-value` entries. This brings us closer to integration/acceptance test frameworks without any of the overhead and complexities of running a headless browser.\n\n#","ref":"changelog.html#0-12-0-2020-04-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `assert_patch/3` and `assert_patched/2` for asserting on patches\n  - Add `follow_redirect/3` to automatically follow redirects from `render_*` events\n  - Add `phx-trigger-action` form annotation to trigger an HTTP form submit on next DOM patch\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix `phx-target` `@myself` targeting a sibling LiveView component with the same component ID\n  - Fix `phx:page-loading-stop` firing before the DOM patch has been performed\n  - Fix `phx-update=\"prepend\"` failing to properly patch the DOM when the same ID is updated back to back\n  - Fix redirects on mount failing to copy flash\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- `phx-error-for` has been removed in favor of `phx-feedback-for`. `phx-feedback-for` will set a `phx-no-feedback` class whenever feedback has to be hidden\n  - `Phoenix.LiveViewTest.children/1` has been renamed to `Phoenix.LiveViewTest.live_children/1`\n  - `Phoenix.LiveViewTest.find_child/2` has been renamed to `Phoenix.LiveViewTest.find_live_child/2`\n  - `Phoenix.LiveViewTest.assert_redirect/3` no longer matches on the flash, instead it returns the flash\n  - `Phoenix.LiveViewTest.assert_redirect/3` no longer matches on the patch redirects, use `assert_patch/3` instead\n  - `Phoenix.LiveViewTest.assert_remove/3` has been removed. If the LiveView crashes, it will cause the test to crash too\n  - Passing a path with DOM IDs to `render_*` test functions is deprecated. Furthermore, they now require a `phx-target=\"<%= @id %>\"` on the given DOM ID:\n\n    ```heex\n     \n      ...\n     \n    ```\n\n    ```elixir\n    html = render_submit([view, \"#component-id\"], event, value)\n    ```\n\n    In any case, this API is deprecated and you should migrate to the new element based API.","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"0.11.1 (2020-04-08) - Changelog","doc":"#","ref":"changelog.html#0-11-1-2020-04-08"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix readonly states failing to be undone after an empty diff\n  - Fix dynamically added child failing to be joined by the client\n  - Fix teardown bug causing stale client sessions to attempt a rejoin on reconnect\n  - Fix orphaned prepend/append content across joins\n  - Track `unless` in LiveEEx engine\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- `render_event`/`render_click` and friends now expect a DOM ID selector to be given when working with components. For example, instead of `render_click([live, \"user-13\"])`, you should write `render_click([live, \"#user-13\"])`, mirroring the `phx-target` API.\n  - Accessing the socket assigns directly `@socket.assigns[...]` in a template will now raise the exception `Phoenix.LiveView.Socket.AssignsNotInSocket`. The socket assigns are available directly inside the template as LiveEEx `assigns`, such as `@foo` and `@bar`. Any assign access should be done using the assigns in the template where proper change tracking takes place.\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Trigger debounced events immediately on input blur\n  - Support `defaults` option on `LiveSocket` constructor to configure default `phx-debounce` and `phx-throttle` values, allowing ` `\n  - Add `detail` key to click event metadata for detecting double/triple clicks","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.11.0 (2020-04-06) - Changelog","doc":"#","ref":"changelog.html#0-11-0-2020-04-06"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- Remove `socket.assigns` during render to avoid change tracking bugs. If you were previously relying on passing `@socket` to functions then referencing socket assigns, pass the explicit assign instead to your functions from the template.\n  - Removed `assets/css/live_view.css`. If you want to show a progress bar then in `app.css`, replace\n\n\n    ```diff\n    - @import \"../../../../deps/phoenix_live_view/assets/css/live_view.css\";\n    + @import \"../node_modules/nprogress/nprogress.css\";\n    ```\n\n    and add `nprogress` to `assets/package.json`. Full details in the [Progress animation guide](https://hexdocs.pm/phoenix_live_view/0.11.0/installation.html#progress-animation)\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix client issue with greater than two levels of LiveView nesting\n  - Fix bug causing entire LiveView to be re-rendering with only a component changed\n  - Fix issue where rejoins would not trigger `phx:page-loading-stop`\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support deep change tracking so `@foo.bar` only executes and diffs when bar changes\n  - Add `@myself` assign, to allow components to target themselves instead of relying on a DOM ID, for example: `phx-target=\"<%= @myself %>\"`\n  - Optimize various client rendering scenarios for faster DOM patching\n  of components and append/prepended content\n  - Add `enableProfiling()` and `disableProfiling()` to `LiveSocket` for client performance profiling to aid the development process\n  - Allow LiveViews to be rendered inside LiveComponents\n  - Add support for clearing flash inside components","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.10.0 (2020-03-18) - Changelog","doc":"#","ref":"changelog.html#0-10-0-2020-03-18"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- Rename socket assign `@live_view_module` to `@live_module`\n  - Rename socket assign `@live_view_action` to `@live_action`\n  - LiveView no longer uses the default app layout and `put_live_layout` is no longer supported. Instead, use `put_root_layout`. Note, however, that the layout given to `put_root_layout` must use `@inner_content` instead of `<%= render(@view_module, @view_template, assigns) %>` and that the root layout will also be used by regular views. Check the [Live Layouts](https://hexdocs.pm/phoenix_live_view/0.10.0/Phoenix.LiveView.html#module-live-layouts) section of the docs.\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Fix loading states causing nested LiveViews to be removed during live navigation\n  - Only trigger `phx-update=\"ignore\"` hook if data attributes have changed\n  - Fix LiveEEx fingerprint bug causing no diff to be sent in certain cases\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support collocated templates where an `.html.leex` template of the same basename of the LiveView will be automatically used for `render/1`\n  - Add `live_title_tag/2` helper for automatic prefix/suffix on `@page_title` updates","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.9.0 (2020-03-08) - Changelog","doc":"#","ref":"changelog.html#0-9-0-2020-03-08"},{"type":"extras","title":"Bug fixes - Changelog","doc":"- Do not set ignored inputs and buttons as readonly\n  - Only decode paths in URIs\n  - Only destroy main descendents when replacing main\n  - Fix sibling component patches when siblings at same root DOM tree\n  - Do not pick the layout from `:use` on child LiveViews\n  - Respect when the layout is set to `false` in the router and on mount\n  - Fix sibling component patch when component siblings lack a container\n  - Make flash optional (i.e. LiveView will still work if you don't `fetch_flash` before)\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Raise if `:flash` is given as an assign\n  - Support user-defined metadata in router\n  - Allow the router to be accessed as `socket.router`\n  - Allow `MFArgs` as the `:session` option in the `live` router macro\n  - Trigger page loading event when main LV errors\n  - Automatically clear the flash on live navigation examples - only the newly assigned flash is persisted","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.8.1 (2020-02-27) - Changelog","doc":"#","ref":"changelog.html#0-8-1-2020-02-27"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support `phx-disable-with` on live redirect and live patch links\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix focus issue on date and time inputs\n  - Fix LiveViews failing to mount when page restored from back/forward cache following a `redirect` on the server\n  - Fix IE coercing `undefined` to string when issuing pushState\n  - Fix IE error when focused element is null\n  - Fix client error when using components and live navigation where a dynamic template is rendered\n  - Fix latent component diff causing error when component removed from DOM before patch arrives\n  - Fix race condition where a component event received on the server for a component already removed by the server raised a match error","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.8.0 (2020-02-22) - Changelog","doc":"#","ref":"changelog.html#0-8-0-2020-02-22"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- Remove `Phoenix.LiveView.Flash` in favor of `:fetch_live_flash` imported by `Phoenix.LiveView.Router`\n  - Live layout must now access the child contents with `@inner_content` instead of invoking the LiveView directly\n  - Returning `:stop` tuples from LiveView `mount` or `handle_[params|call|cast|info|event]` is no longer supported. LiveViews are stopped when issuing a `redirect` or `push_redirect`\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `put_live_layout` plug to put the root layout used for live routes\n  - Allow `redirect` and `push_redirect` from mount\n  - Use acknowledgement tracking to avoid patching inputs until the server has processed the form event\n  - Add css loading states to all phx bound elements with event specific css classes\n  - Dispatch `phx:page-loading-start` and `phx:page-loading-stop` on window for live navigation, initial page loads, and form submits, for user controlled page loading integration\n  - Allow any phx bound element to specify `phx-page-loading` to dispatch loading events above when the event is pushed\n  - Add client side latency simulator with new `enableLatencySim(milliseconds)` and `disableLatencySim()`\n  - Add `enableDebug()` and `disableDebug()` to `LiveSocket` for ondemand browser debugging from the web console\n  - Do not connect LiveSocket WebSocket or bind client events unless a LiveView is found on the page\n  - Add `transport_pid/1` to return the websocket transport pid when the socket is connected\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix issue where a failed mount from a `live_redirect` would reload the current URL instead of the attempted new URL","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.7.1 (2020-02-13) - Changelog","doc":"#","ref":"changelog.html#0-7-1-2020-02-13"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix checkbox bug failing to send `phx-change` event to the server in certain cases\n  - Fix checkbox bug failing to maintain checked state when a checkbox is programmatically updated by the server\n  - Fix select bug in Firefox causing the highlighted index to jump when a patch is applied during hover state","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.7.0 (2020-02-12) - Changelog","doc":"#","ref":"changelog.html#0-7-0-2020-02-12"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- `live_redirect` was removed in favor of `push_patch` (for updating the URL and params of the current LiveView) and `push_redirect` (for updating the URL to another LiveView)\n  - `live_link` was removed in favor of  `live_patch` (for updating the URL and params of the current LiveView) and `live_redirect` (for updating the URL to another LiveView)\n  - `Phoenix.LiveViewTest.assert_redirect` no longer accepts an anonymous function in favor of executing the code\n  prior to asserting the redirects, just like `assert_receive`.\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Support `@live_view_action` in LiveViews to simplify tracking of URL state\n  - Recovery form input data automatically on disconnects or crash recovery\n  - Add `phx-auto-recover` form binding for specialized recovery\n  - Scroll to top of page while respecting anchor hash targets on `live_patch` and `live_redirect`\n  - Add `phx-capture-click` to use event capturing to bind a click event as it propagates inwards from the target\n  - Revamp flash support so it works between static views, live views, and components\n  - Add `phx-key` binding to scope `phx-window-keydown` and `phx-window-keyup` events\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Send `phx-value-*` on key events\n  - Trigger `updated` hook callbacks on `phx-update=\"ignore\"` container when the container's attributes have changed\n  - Fix nested `phx-update=\"append\"` raising ArgumentError in LiveViewTest\n  - Fix updates not being applied in rare cases where an leex template is wrapped in an if expression","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.6.0 (2020-01-22) - Changelog","doc":"#","ref":"changelog.html#0-6-0-2020-01-22"},{"type":"extras","title":"Deprecations - Changelog","doc":"- LiveView `mount/2` has been deprecated in favor of `mount/3`. The params are now passed as the first argument to `mount/3`, followed by the session and socket.\n\n#","ref":"changelog.html#deprecations"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- The socket session now accepts only string keys\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Allow window beforeunload to be cancelled without losing websocket connection\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix handle_params not decoding URL path parameters properly\n  - Fix LiveViewTest error when routing at root path\n  - Fix URI encoded params failing to be decoded in `handle_params`\n  - Fix component target failing to locate correct component when the target is on an input tag","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.5.2 (2020-01-17) - Changelog","doc":"#","ref":"changelog.html#0-5-2-2020-01-17"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix optimization bug causing some DOM nodes to be removed on updates","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.5.1 (2020-01-15) - Changelog","doc":"#","ref":"changelog.html#0-5-1-2020-01-15"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix phx-change bug causing phx-target to not be used","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.5.0 (2020-01-15) - Changelog","doc":"LiveView now makes the connection session automatically available in LiveViews. However, to do so, you need to configure your endpoint accordingly, **otherwise LiveView will fail to connect**.\n\nThe steps are:\n\n  1) Find `plug Plug.Session, ...` in your endpoint.ex and move the options `...` to a module attribute:\n\n      ```elixir\n      @session_options [\n        ...\n      ]\n      ```\n\n  2) Change the `plug Plug.Session` to use said attribute:\n\n      ```elixir\n      plug Plug.Session, @session_options\n      ```\n\n  3) Also pass the `@session_options` to your LiveView socket:\n\n      ```elixir\n      socket \"/live\", Phoenix.LiveView.Socket,\n        websocket: [connect_info: [session: @session_options]]\n      ```\n\n  4) You should define the CSRF meta tag inside   in your layout, before `app.js` is included:\n\n      ```heex\n       \n       \"> \n      ```\n\n  5) Then in your app.js:\n\n      ```\n      let csrfToken = document.querySelector(\"meta[name='csrf-token']\").getAttribute(\"content\");\n      let liveSocket = new LiveSocket(\"/live\", Socket, {params: {_csrf_token: csrfToken}});\n      ```\n\nAlso note that **the session from now on will have string keys**. LiveView will warn if atom keys are used.\n\n#","ref":"changelog.html#0-5-0-2020-01-15"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Respect new tab behavior in `live_link`\n  - Add `beforeUpdate` and `beforeDestroy` JS hooks\n  - Make all assigns defined on the socket mount available on the layout on first render\n  - Provide support for live layouts with new `:layout` option\n  - Detect duplicate IDs on the front-end when DEBUG mode is enabled\n  - Automatically forward the session to LiveView\n  - Support \"live_socket_id\" session key for identifying (and disconnecting) LiveView sockets\n  - Add support for `hibernate_after` on LiveView processes\n  - Support redirecting to full URLs on `live_redirect` and `redirect`\n  - Add `offsetX` and `offsetY` to click event metadata\n  - Allow `live_link` and `live_redirect` to exist anywhere in the page and it will always target the main LiveView (the one defined at the router)\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- `phx-target=\"window\"` has been removed in favor of `phx-window-keydown`, `phx-window-focus`, etc, and the `phx-target` binding has been repurposed for targeting LiveView and LiveComponent events from the client\n  - `Phoenix.LiveView` no longer defined `live_render` and `live_link`. These functions have been moved to `Phoenix.LiveView.Helpers` which can now be fully imported in your views. In other words, replace `import Phoenix.LiveView, only: [live_render: ..., live_link: ...]` by `import Phoenix.LiveView.Helpers`","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"0.4.1 (2019-11-07) - Changelog","doc":"#","ref":"changelog.html#0-4-1-2019-11-07"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix bug causing blurred inputs","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.4.0 (2019-11-07) - Changelog","doc":"#","ref":"changelog.html#0-4-0-2019-11-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `Phoenix.LiveComponent` to compartmentalize state, markup, and events in LiveView\n  - Handle outdated clients by refreshing the page with jitter when a valid, but outdated session is detected\n  - Only dispatch live link clicks to router LiveView\n  - Refresh the page for graceful error recovery on failed mount when the socket is in a connected state\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix `phx-hook` destroyed callback failing to be called in certain cases\n  - Fix back/forward bug causing LiveView to fail to remount","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.3.1 (2019-09-23) - Changelog","doc":"#","ref":"changelog.html#0-3-1-2019-09-23"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- `live_isolated` in tests no longer requires a router and a pipeline (it now expects only 3 arguments)\n  - Raise if `handle_params` is used on a non-router LiveView\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [LiveViewTest] Fix function clause errors caused by HTML comments","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.3.0 (2019-09-19) - Changelog","doc":"#","ref":"changelog.html#0-3-0-2019-09-19"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add `phx-debounce` and `phx-throttle` bindings to rate limit events\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- IE11 support now requires two additional polyfills, `mdn-polyfills/CustomEvent` and `mdn-polyfills/String.prototype.startsWith`\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix IE11 support caused by unsupported `getAttributeNames` lookup\n  - Fix Floki dependency compilation warnings","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.2.1 (2019-09-17) - Changelog","doc":"#","ref":"changelog.html#0-2-1-2019-09-17"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [LiveView.Router] Fix module concat failing to build correct layout module when using custom namespace\n  - [LiveViewTest] Fix `phx-update` append/prepend containers not building proper DOM content\n  - [LiveViewTest] Fix `phx-update` append/prepend containers not updating existing child containers with matching IDs","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.2.0 (2019-09-12) - Changelog","doc":"#","ref":"changelog.html#0-2-0-2019-09-12"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [LiveView] Add new `:container` option to `use Phoenix.LiveView`\n  - [LiveViewTest] Add `live_isolated` test helper for testing LiveViews which are not routable\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- Replace `configure_temporary_assigns/2` with 3-tuple mount return, supporting a `:temporary_assigns` key\n  - Do not allow `redirect`/`live_redirect` on mount nor in child live views\n  - All `phx-update` containers now require a unique ID\n  - `LiveSocket` JavaScript constructor now requires explicit dependency injection of Phoenix Socket constructor. For example:\n\n```\nimport {Socket} from \"phoenix\"\nimport LiveSocket from \"phoenix_live_view\"\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {...})\n```\n\n#","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix `phx-update=append/prepend` failing to join new nested live views or wire up new phx-hooks\n  - Fix number input handling causing some browsers to reset form fields on invalid inputs\n  - Fix multi-select decoding causing server error\n  - Fix multi-select change tracking failing to submit an event when a value is deselected\n  - Fix live redirect loop triggered under certain scenarios\n  - Fix params failing to update on re-mounts after live_redirect\n  - Fix blur event metadata being sent with type of `\"focus\"`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.1.2 (2019-08-28) - Changelog","doc":"#","ref":"changelog.html#0-1-2-2019-08-28"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- `phx-value` has no effect, use `phx-value-*` instead\n  - The `:path_params` key in session has no effect (use `handle_params` in `LiveView` instead)","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"0.1.1 (2019-08-27) - Changelog","doc":"#","ref":"changelog.html#0-1-1-2019-08-27"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Use optimized `insertAdjacentHTML` for faster append/prepend and proper css animation handling\n  - Allow for replacing previously appended/prepended elements by replacing duplicate IDs during append/prepend instead of adding new DOM nodes\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix duplicate append/prepend updates when parent content is updated\n  - Fix JS hooks not being applied for appending and prepended content","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.1.0 (2019-08-25) - Changelog","doc":"#","ref":"changelog.html#0-1-0-2019-08-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"- The LiveView `handle_in/3` callback now receives a map of metadata about the client event\n  - For `phx-change` events, `handle_in/3` now receives a `\"_target\"` param representing the keyspace of the form input name which triggered the change\n  - Multiple values may be provided for any phx binding by using the `phx-value-` prefix, such as `phx-value-myval1`, `phx-value-myval2`, etc\n  - Add control over the DOM patching via `phx-update`, which can be set to `\"replace\"`, `\"append\"`, `\"prepend\"` or `\"ignore\"`\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"- `phx-ignore` was renamed to `phx-update=\"ignore\"`","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Welcome","doc":"# Welcome\n\nWelcome to Phoenix LiveView documentation. Phoenix LiveView enables\nrich, real-time user experiences with server-rendered HTML. A general\noverview of LiveView and its benefits is [available in our README](https://github.com/phoenixframework/phoenix_live_view).","ref":"welcome.html"},{"type":"extras","title":"What is a LiveView? - Welcome","doc":"LiveViews are processes that receives events, updates its state,\nand render updates to a page as diffs.\n\nThe LiveView programming model is declarative: instead of saying\n\"once event X happens, change Y on the page\", events in LiveView\nare regular messages which may cause changes to its state. Once\nthe state changes, LiveView will re-render the relevant parts of\nits HTML template and push it to the browser, which updates itself\nin the most efficient manner.\n\nLiveView state is nothing more than functional and immutable\nElixir data structures. The events are either internal application messages\n(usually emitted by `Phoenix.PubSub`) or sent by the client/browser.\n\nLiveView is first rendered statically as part of regular\nHTTP requests, which provides quick times for \"First Meaningful\nPaint\", in addition to helping search and indexing engines.\nThen a persistent connection is established between client and\nserver. This allows LiveView applications to react faster to user\nevents as there is less work to be done and less data to be sent\ncompared to stateless requests that have to authenticate, decode, load,\nand encode data on every request.","ref":"welcome.html#what-is-a-liveview"},{"type":"extras","title":"Example - Welcome","doc":"LiveView is included by default in Phoenix applications.\nTherefore, to use LiveView, you must have already installed Phoenix\nand created your first application. If you haven't done so,\ncheck [Phoenix' installation guide](https://hexdocs.pm/phoenix/installation.html)\nto get started.\n\nThe behaviour of a LiveView is outlined by a module which implements\na series of functions as callbacks. Let's see an example. Write the\nfile below to `lib/my_app_web/live/thermostat_live.ex`:\n\n```elixir\ndefmodule MyAppWeb.ThermostatLive do\n  # In Phoenix v1.6+ apps, the line is typically: use MyAppWeb, :live_view\n  use Phoenix.LiveView\n\n  def render(assigns) do\n    ~H\"\"\"\n    Current temperature: <%= @temperature %>°F\n     + \n    \"\"\"\n  end\n\n  def mount(_params, _session, socket) do\n    temperature = 70 # Let's assume a fixed temperature for now\n    {:ok, assign(socket, :temperature, temperature)}\n  end\n\n  def handle_event(\"inc_temperature\", _params, socket) do\n    {:noreply, update(socket, :temperature, &(&1 + 1))}\n  end\nend\n```\n\nThe module above defines three functions (they are callbacks\nrequired by LiveView). The first one is `render/1`,\nwhich receives the socket `assigns` and is responsible for returning\nthe content to be rendered on the page. We use the `~H` sigil to define\na HEEx template, which stands for HTML+EEx. They are an extension of\nElixir's builtin EEx templates, with support for HTML validation, syntax-based\ncomponents, smart change tracking, and more. You can learn more about\nthe template syntax in `Phoenix.Component.sigil_H/2` (note\n`Phoenix.Component` is automatically imported when you use `Phoenix.LiveView`).\n\nThe data used on rendering comes from the `mount` callback. The\n`mount` callback is invoked when the LiveView starts. In it, you\ncan access the request parameters, read information stored in the\nsession (typically information which identifies who is the current\nuser), and a socket. The socket is where we keep all state, including\nassigns. `mount` proceeds to assign a default temperature to the socket.\nBecause Elixir data structures are immutable, LiveView APIs often\nreceive the socket and return an updated socket. Then we return\n`{:ok, socket}` to signal that we were able to mount the LiveView\nsuccessfully. After `mount`, LiveView will render the page with the\nvalues from `assigns` and sent it to the client.\n\nIf you look at the HTML rendered, you will notice there is a button\nwith a `phx-click` attribute. When the button is clicked, a\n`\"inc_temperature\"` event is sent to the server, which is matched and\nhandled by the `handle_event` callback. This callback updates the socket\nand returns `{:noreply, socket}` with the updated socket.\n`handle_*` callbacks in LiveView (and in Elixir in general) are\ninvoked based on some action, in this case, the user clicking a button.\nThe `{:noreply, socket}` return means there is no additional replies\nsent to the browser, only that a new version of the page is rendered.\nLiveView then computes diffs and sends them to client.\n\nNow we are ready to render our LiveView. You can serve the LiveView\ndirectly from your router:\n\n```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n  import Phoenix.LiveView.Router\n\n  scope \"/\", MyAppWeb do\n    live \"/thermostat\", ThermostatLive\n  end\nend\n```\n\nOnce the LiveView is rendered, a regular HTML response is sent. In your\napp.js file, you should find the following:\n\n```javascript\nimport {Socket} from \"phoenix\"\nimport {LiveSocket} from \"phoenix_live_view\"\n\nlet csrfToken = document.querySelector(\"meta[name='csrf-token']\").getAttribute(\"content\")\nlet liveSocket = new LiveSocket(\"/live\", Socket, {params: {_csrf_token: csrfToken}})\nliveSocket.connect()\n```\n\nNow the JavaScript client will connect over WebSockets and `mount/3` will be invoked\ninside a spawned LiveView process.","ref":"welcome.html#example"},{"type":"extras","title":"Parameters and session - Welcome","doc":"The mount callback receives three arguments: the request parameters, the session, and the socket.\n\nThe parameters can be used to read information from the URL. For example, assuming you have a `Thermostat` module defined somewhere that can read this information based on the house name, you could write this:\n\n```elixir\ndef mount(%{\"house\" => house}, _session, socket) do\n  temperature = Thermostat.get_house_reading(house)\n  {:ok, assign(socket, :temperature, temperature)}\nend\n```\n\nAnd then in your router:\n\n```elixir\nlive \"/thermostat/:house\", ThermostatLive\n```\n\nThe session retrieves information from a signed (or encrypted) cookie. This is where you can store authentication information, such as `current_user_id`:\n\n```elixir\ndef mount(_params, %{\"current_user_id\" => user_id}, socket) do\n  temperature = Thermostat.get_user_reading(user_id)\n  {:ok, assign(socket, :temperature, temperature)}\nend\n```\n\n> Phoenix comes with built-in authentication generators. See `mix phx.gen.auth`.\n\nMost times, in practice, you will use both:\n\n```elixir\ndef mount(%{\"house\" => house}, %{\"current_user_id\" => user_id}, socket) do\n  temperature = Thermostat.get_house_reading(user_id, house)\n  {:ok, assign(socket, :temperature, temperature)}\nend\n```\n\nIn other words, you want to read the information about a given house, as long as the user has access to it.","ref":"welcome.html#parameters-and-session"},{"type":"extras","title":"Bindings - Welcome","doc":"Phoenix supports DOM element bindings for client-server interaction. For\nexample, to react to a click on a button, you would render the element:\n\n     + \n\nThen on the server, all LiveView bindings are handled with the `handle_event/3`\ncallback, for example:\n\n    def handle_event(\"inc_temperature\", _value, socket) do\n      {:noreply, update(socket, :temperature, &(&1 + 1))}\n    end\n\nTo update UI state, for example, to open and close dropdowns, switch tabs,\netc, LiveView also supports JS commands (`Phoenix.LiveView.JS`), which\nexecute directly on the client without reaching the server. To learn more,\nsee [our bindings page](bindings.md) for a complete list of all LiveView\nbindings as well as our [JavaScript interoperability guide](js-interop.md).\n\nLiveView has built-in support for forms, including uploads and association\nmanagement. See `Phoenix.Component.form/1` as a starting point and\n`Phoenix.Component.inputs_for/1` for working with associations.\nThe [Uploads](uploads.md) and [Form bindings](form-bindings.md) guides provide\nmore information about advanced features.","ref":"welcome.html#bindings"},{"type":"extras","title":"Navigation - Welcome","doc":"LiveView provides functionality to allow page navigation using the\n[browser's pushState API](https://developer.mozilla.org/en-US/docs/Web/API/History_API).\nWith live navigation, the page is updated without a full page reload.\n\nYou can either *patch* the current LiveView, updating its URL, or\n*navigate* to a new LiveView. You can learn more about them in the\n[Live Navigation](live-navigation.md) guide.","ref":"welcome.html#navigation"},{"type":"extras","title":"Generators - Welcome","doc":"Phoenix v1.6 and later includes code generators for LiveView. If you want to see\nan example of how to structure your application, from the database all the way up\nto LiveViews, run the following:\n\n    mix phx.gen.live Blog Post posts title:string body:text\n\nFor more information, run `mix help phx.gen.live`.\n\nFor authentication, with built-in LiveView support, run `mix phx.gen.auth Account User users`.","ref":"welcome.html#generators"},{"type":"extras","title":"Compartmentalize state, markup, and events in LiveView - Welcome","doc":"LiveView supports two extension mechanisms: function components, provided by\n`HEEx` templates, and stateful components.\n\nFunction components are any function that receives an assigns map, similar\nto `render(assigns)` in our LiveView, and returns a `~H` template. For example:\n\n    def weather_greeting(assigns) do\n      ~H\"\"\"\n       \n         Hello <%= @name %> \n         \n       \n      \"\"\"\n    end\n\nYou can learn more about function components in the `Phoenix.Component`\nmodule. At the end of the day, they are a useful mechanism to reuse markup\nin your LiveViews.\n\nHowever, sometimes you need to compartmentalize or reuse more than markup.\nPerhaps you want to move part of the state or part of the events in your\nLiveView to a separate module. For these cases, LiveView provides\n`Phoenix.LiveComponent`, which are rendered using\n[`live_component/1`](`Phoenix.Component.live_component/1`):\n\n    <.live_component module={UserComponent} id={user.id} user={user} />\n\nComponents have their own `mount/3` and `handle_event/3` callbacks, as\nwell as their own state with change tracking support. Components are also\nlightweight as they \"run\" in the same process as the parent `LiveView`.\nHowever, this means an error in a component would cause the whole view to\nfail to render. See `Phoenix.LiveComponent` for a complete rundown on components.\n\nFinally, if you want complete isolation between parts of a LiveView, you can\nalways render a LiveView inside another LiveView by calling\n[`live_render/3`](`Phoenix.Component.live_render/3`). This child LiveView\nruns in a separate process than the parent, with its own callbacks. If a child\nLiveView crashes, it won't affect the parent. If the parent crashes, all children\nare terminated.\n\nWhen rendering a child LiveView, the `:id` option is required to uniquely\nidentify the child. A child LiveView will only ever be rendered and mounted\na single time, provided its ID remains unchanged. To force a child to re-mount\nwith new session data, a new ID must be provided.\n\nGiven that a LiveView runs on its own process, it is an excellent tool for creating\ncompletely isolated UI elements, but it is a slightly expensive abstraction if\nall you want is to compartmentalize markup or events (or both).\n\nTo sum it up:\n\n  * use `Phoenix.Component` to compartmentalize/reuse markup\n  * use `Phoenix.LiveComponent` to compartmentalize state, markup, and events\n  * use nested `Phoenix.LiveView` to compartmentalize state, markup, events, and error isolation","ref":"welcome.html#compartmentalize-state-markup-and-events-in-liveview"},{"type":"extras","title":"Guides - Welcome","doc":"This documentation is split into two categories. We have the API\nreference for all LiveView modules, that's where you will learn\nmore about `Phoenix.Component`, `Phoenix.LiveView`, and so on.\n\nLiveView also has many guides to help you on your journey,\nsplit on server-side and client-side:\n\n#","ref":"welcome.html#guides"},{"type":"extras","title":"Server-side - Welcome","doc":"These guides focus on server-side functionality:\n\n* [Assigns and HEEx templates](assigns-eex.md)\n* [Error and exception handling](error-handling.md)\n* [Live Layouts](live-layouts.md)\n* [Live Navigation](live-navigation.md)\n* [Security considerations of the LiveView model](security-model.md)\n* [Telemetry](telemetry.md)\n* [Uploads](uploads.md)\n* [Using Gettext for internationalization](using-gettext.md)\n\n#","ref":"welcome.html#server-side"},{"type":"extras","title":"Client-side - Welcome","doc":"These guides focus on LiveView bindings and client-side integration:\n\n* [Bindings](bindings.md)\n* [Form bindings](form-bindings.md)\n* [DOM patching and temporary assigns](dom-patching.md)\n* [JavaScript interoperability](js-interop.md)\n* [Uploads (External)](uploads-external.md)","ref":"welcome.html#client-side"},{"type":"extras","title":"Assigns and HEEx templates","doc":"# Assigns and HEEx templates\n\nAll of the data in a LiveView is stored in the socket, which is a server \nside struct called `Phoenix.LiveView.Socket`. Your own data is stored\nunder the `assigns` key of said struct. The server data is never shared\nwith the client beyond what your template renders.\n\nPhoenix template language is called HEEx (HTML+EEx). EEx is Embedded \nElixir, an Elixir string template engine. Those templates\nare either files with the `.heex` extension or they are created\ndirectly in source files via the `~H` sigil. You can learn more about\nthe HEEx syntax by checking the docs for [the `~H` sigil](`Phoenix.Component.sigil_H/2`).\n\nThe `Phoenix.Component.assign/2` and `Phoenix.Component.assign/3`\nfunctions help store those values. Those values can be accessed\nin the LiveView as `socket.assigns.name` but they are accessed\ninside HEEx templates as `@name`.\n\nIn this section, we are going to cover how LiveView minimizes\nthe payload over the wire by understanding the interplay between\nassigns and templates.","ref":"assigns-eex.html"},{"type":"extras","title":"Change tracking - Assigns and HEEx templates","doc":"When you first render a `.heex` template, it will send all of the\nstatic and dynamic parts of the template to the client. Imagine the\nfollowing template:\n\n```heex\n <%= expand_title(@title) %> \n```\n\nIt has two static parts, ` ` and ` ` and one dynamic part\nmade of `expand_title(@title)`. Further rendering of this template\nwon't resend the static parts and it will only resend the dynamic\npart if it changes.\n\nThe tracking of changes is done via assigns. If the `@title` assign\nchanges, then LiveView will execute `expand_title(@title)` and send\nthe new content. If `@title` is the same, nothing is executed and\nnothing is sent.\n\nChange tracking also works when accessing map/struct fields.\nTake this template:\n\n```heex\n \n  <%= @user.name %>\n \n```\n\nIf the `@user.name` changes but `@user.id` doesn't, then LiveView\nwill re-render only `@user.name` and it will not execute or resend `@user.id`\nat all.\n\nThe change tracking also works when rendering other templates as\nlong as they are also `.heex` templates:\n\n```heex\n<%= render \"child_template.html\", assigns %>\n```\n\nOr when using function components:\n\n```heex\n<.show_name name={@user.name} />\n```\n\nThe assign tracking feature also implies that you MUST avoid performing\ndirect operations in the template. For example, if you perform a database\nquery in your template:\n\n```heex\n<%= for user <- Repo.all(User) do %>\n  <%= user.name %>\n<% end %>\n```\n\nThen Phoenix will never re-render the section above, even if the number of\nusers in the database changes. Instead, you need to store the users as\nassigns in your LiveView before it renders the template:\n\n    assign(socket, :users, Repo.all(User))\n\nGenerally speaking, **data loading should never happen inside the template**,\nregardless if you are using LiveView or not. The difference is that LiveView\nenforces this best practice.","ref":"assigns-eex.html#change-tracking"},{"type":"extras","title":"Pitfalls - Assigns and HEEx templates","doc":"There are some common pitfalls to keep in mind when using the `~H` sigil\nor `.heex` templates inside LiveViews.\n\nDue to the scope of variables, LiveView has to disable change tracking\nwhenever variables are used in the template, with the exception of\nvariables introduced by Elixir block constructs such as `case`,\n`for`, `if`, and others. Therefore, you **must avoid** code like\nthis in your HEEx templates:\n\n```heex\n<% some_var = @x + @y %>\n<%= some_var %>\n```\n\nInstead, use a function:\n\n```heex\n<%= sum(@x, @y) %>\n```\n\nSimilarly, **do not** define variables at the top of your `render` function\nfor LiveViews or LiveComponents:\n\n    def render(assigns) do\n      sum = assigns.x + assigns.y\n\n      ~H\"\"\"\n      <%= sum %>\n      \"\"\"\n    end\n\nInstead explicitly precompute the assign outside of render:\n\n    assign(socket, sum: socket.assigns.x + socket.assigns.y)\n\nUnlike LiveView's `render/1` callback, a function component can\nmodify the assigns it receives via the `assign/2`, `assign/3`,\n`assign_new/3`, and `update/3` functions. Therefore, you can assign\nthe computed values before declaring your template:\n\n    attr :x, :integer, required: true\n    attr :y, :integer, required: true\n    def sum_component(assigns) do\n      assigns = assign(assigns, sum: assigns.x + assigns.y)\n\n      ~H\"\"\"\n      <%= @sum %>\n      \"\"\"\n    end\n\nGenerally speaking, avoid accessing variables inside `HEEx` templates, as code that\naccess variables is always executed on every render. This also applies to the\n`assigns` variable. The exception are variables introduced by Elixir's block\nconstructs. For example, accessing the `post` variable defined by the comprehension\nbelow works as expected:\n\n```heex\n<%= for post <- @posts do %>\n  ...\n<% end %>\n```\n\nWhen talking about variables, it is also worth discussing the `assigns`\nspecial variable. Every time you use the `~H` sigil, you must define an\n`assigns` variable, which is also available on every `.heex` template.\n\nSometimes you might want to pass all assigns from one function component to\nanother. For example, imagine you have a complex `card` component with \nheader, content and footer section. You might refactor your component\ninto three smaller components internally:\n\n```elixir\ndef card(assigns) do\n  ~H\"\"\"\n   \n    <.card_header {assigns} />\n    <.card_body {assigns} />\n    <.card_footer {assigns} />\n   \n  \"\"\"\nend\n\ndefp card_header(assigns) do\n  ...\nend\n\ndefp card_body(assigns) do\n  ...\nend\n\ndefp card_footer(assigns) do\n  ...\nend\n```\n\nBecause of the way function components handle attributes, the above code will\nnot perform change tracking and it will always re-render all three components\non every change.\n\nGenerally, you should avoid passing all assigns and instead be explicit about\nwhich assigns the child components need:\n\n```elixir\ndef card(assigns) do\n  ~H\"\"\"\n   \n    <.card_header title={@title} class={@title_class} />\n    <.card_body>\n      <%= render_slot(@inner_block) %>\n    </.card_body>\n    <.card_footer on_close={@on_close} />\n   \n  \"\"\"\nend\n```\n\nIf you really need to pass all assigns you should instead use the regular\nfunction call syntax:\n\n```elixir\ndef card(assigns) do\n  ~H\"\"\"\n   \n    <%= card_header(assigns) %>\n    <%= card_body(assigns) %>\n    <%= card_footer(assigns) %>\n   \n  \"\"\"\nend\n```\n\nThis ensures that the change tracking information from the parent component\nis passed to each child component, only re-rendering what is necessary.\nHowever, generally speaking, it is best to avoid passing `assigns` altogether\nand instead let LiveView figure out the best way to track changes.\n\nTo sum up:\n\n  1. Avoid defining local variables inside HEEx templates, except within Elixir's constructs\n\n  2. Avoid passing or accessing the `assigns` variable inside HEEx templates","ref":"assigns-eex.html#pitfalls"},{"type":"extras","title":"Deployments","doc":"# Deployments\n\nOne of the questions that arise from LiveView stateful model is what considerations are necessary when deploying a new version of LiveView.\n\nFirst off, whenever LiveView disconnects, it will automatically attempt to reconnect to the server using exponential back-off. This means it will try immediately, then wait 2s and try again, then 5s and so on. If you are deploying, this typically means the next reconnection will immediately succeed and your load balancer will automatically redirect to the new servers.\n\nHowever, your LiveView _may_ still have state that will be lost in this transition. How to deal with it? The good news is that there are a series of practices you can follow that will not only help with deployments but it will improve your application in general.\n\n1. Keep state in the query parameters when appropriate. For example, if your application has tabs and the user clicked a tab, instead of using `phx-click` and `c:Phoenix.LiveView.handle_event/3` to manage it, you should implement it using `<.link patch={...}>` passing the tab name as parameter. You will then receive the new tab name `c:Phoenix.LiveView.handle_params/3` which will set the relevant assign to choose which tab to display. You can even define specific URLs for each tab in your application router. By doing this, you will reduce the amount of server state, make tab navigation sharable via links, improving search engine indexing, and more.\n\n2. Consider storing other relevant state in the database. For example, if you are building a chat app and you want to store which messages have been read, you can store so in the database. Once the page is loaded, you retrieve the index of the last read message. This makes the application more robust, allow data to be synchronized across devices, etc.\n\n3. If your application uses forms (which is most likely true), keep in mind that Phoenix perform automatic form recovery: in case of disconnections, Phoenix will collect the form data and resubmit it on reconnection. This mechanism works out of the box for most forms but you may want to customize it or test it for your most complex forms. See the relevant section [in the \"Form bindings\" document](../client/form-bindings.md) to learn more.\n\nThe idea is that: if you follow the practices above, most of your state is already handled within your app and therefore deployments should not bring additional concerns. Not only that, it will bring overall benefits to your app such as indexing, link sharing, device sharing, and so on.\n\nIf you really have complex state that cannot be immediately handled, then you may need to resort to special strategies. This may be persisting \"old\" state to Redis/S3/Database and loading the new state on the new connections. Or you may take special care when migrating connections (for example, if you are building a game, you may want to wait for on-going sessions to finish before turning down the old server while routing new sessions to the new ones). Such cases will depend on your requirements (and they would likely exist regardless of which application stack you are using).","ref":"deployments.html"},{"type":"extras","title":"Error and exception handling","doc":"# Error and exception handling\n\nAs with any other Elixir code, exceptions may happen during the LiveView\nlife-cycle. In this section we will describe how LiveView reacts to errors\nat different stages.","ref":"error-handling.html"},{"type":"extras","title":"Expected scenarios - Error and exception handling","doc":"In this section, we will talk about error cases that you expect to happen\nwithin your application. For example, a user filling in a form with invalid\ndata is expected. In a LiveView, we typically handle those cases by storing\na change in the LiveView state, which causes the LiveView to be re-rendered\nwith the error message.\n\nWe may also use `flash` messages for this. For example, imagine you have a\npage to manage all \"Team members\" in an organization. However, if there is\nonly one member left in the organization, they should not be allowed to\nleave. You may want to handle this by using flash messages:\n\n    if MyApp.Org.leave(socket.assigns.current_org, member) do\n      {:noreply, socket}\n    else\n      {:noreply, put_flash(socket, :error, \"last member cannot leave organization\")}\n    end\n\nHowever, one may argue that, if the last member of an organization cannot\nleave it, it may be better to not even show the \"Leave\" button in the UI\nwhen the organization has only one member.\n\nGiven the button does not appear in the UI, triggering the \"leave\" when\nthe organization has now only one member is an unexpected scenario. This\nmeans we can probably rewrite the code above to:\n\n    true = MyApp.Org.leave(socket.assigns.current_org, member)\n    {:noreply, socket}\n\nIf `leave` returns false by any chance, it will just raise. Or you can\neven provide a `leave!` function that raises a specific exception:\n\n    MyApp.Org.leave!(socket.assigns.current_org, member)\n    {:noreply, socket}\n\nHowever, what will happen with a LiveView in case of exceptions?\nLet's talk about unexpected scenarios.","ref":"error-handling.html#expected-scenarios"},{"type":"extras","title":"Unexpected scenarios - Error and exception handling","doc":"Elixir developers tend to write assertive code. This means that, if we\nexpect `leave` to always return true, we can explicitly match on its\nresult, as we did above:\n\n    true = MyApp.Org.leave(socket.assigns.current_org, member)\n    {:noreply, socket}\n\nIf `leave` fails and returns `false`, an exception is raised. It is common\nfor Elixir developers to use exceptions for unexpected scenarios in their\nPhoenix applications.\n\nFor example, if you are building an application where a user may belong to\none or more organizations, when accessing the organization page, you may want to\ncheck that the user has access to it like this:\n\n    organizations_query = Ecto.assoc(socket.assigns.current_user, :organizations)\n    Repo.get!(organizations_query, params[\"org_id\"])\n\nThe code above builds a query that returns all organizations that belongs to\nthe current user and then validates that the given \"org_id\" belongs to the\nuser. If there is no such \"org_id\" or if the user has no access to it, an\n`Ecto.NoResultsError` exception is raised.\n\nDuring a regular controller request, this exception will be converted to a\n404 exception and rendered as a custom error page, as\n[detailed here](https://hexdocs.pm/phoenix/custom_error_pages.html).\nTo understand how a LiveView reacts to exceptions, we need to consider two\nscenarios: exceptions on mount and during any event.","ref":"error-handling.html#unexpected-scenarios"},{"type":"extras","title":"Common scenarios - Error and exception handling","doc":"Sometimes, it is desirable to have error handling in multiple LiveViews. In\nthis case, it is possible to check for an `@error` variable in the templates\nand conditionally render if it is set, or to redirect to a separate error page.\nHowever another option is to replace the rendering function using\n`Phoenix.LiveView.render_with/2` within a callback or hook such as\n`Phoenix.LiveView.on_mount/1`. This allows common error rendering without\nrequiring a redirect or a check in every template.\n\nFor example:\n\n    def on_mount(:check_errors, params, _session, socket) when is_map_key(params, \"invalid\") do\n      {:cont, Phoenix.LiveView.render_with(socket, &MyApp.Components.param_error/1)}\n    end\n\n    def on_mount(:check_errors, _params, _session, socket) do\n      {:cont, socket}\n    end","ref":"error-handling.html#common-scenarios"},{"type":"extras","title":"Exceptions on mount - Error and exception handling","doc":"Given the code on mount runs both on the initial disconnected render and the\nconnected render, an exception on mount will trigger the following events:\n\nExceptions during disconnected render:\n\n  1. An exception on mount is caught and converted to an exception page\n    by Phoenix error views - pretty much like the way it works with controllers\n\nExceptions during connected render:\n\n  1. An exception on mount will crash the LiveView process - which will be logged\n  2. Once the client has noticed the crash during `mount`, it will fully reload the page\n  3. Reloading the page will start a disconnected render, that will cause `mount`\n    to be invoked again and most likely raise the same exception. Except this time\n    it will be caught and converted to an exception page by Phoenix error views\n\nIn other words, LiveView will reload the page in case of errors, making it\nfail as if LiveView was not involved in the rendering in the first place.","ref":"error-handling.html#exceptions-on-mount"},{"type":"extras","title":"Exceptions on events (`handle_info`, `handle_event`, etc) - Error and exception handling","doc":"If the error happens during an event, the LiveView process will crash. The client\nwill notice the error and remount the LiveView - without reloading the page. This\nis enough to update the page and show the user the latest information.\n\nFor example, let's say two users try to leave the organization at the same time.\nIn this case, both of them see the \"Leave\" button, but our `leave` function call\nwill succeed only for one of them:\n\n    true = MyApp.Org.leave(socket.assigns.current_org, member)\n    {:noreply, socket}\n\nWhen the exception raises, the client will remount the LiveView. Once you remount,\nyour code will now notice that there is only one user in the organization and\ntherefore no longer show the \"Leave\" button. In other words, by remounting,\nwe often update the state of the page, allowing exceptions to be automatically\nhandled.\n\nNote that the choice between conditionally checking on the result of the `leave`\nfunction with an `if`, or simply asserting it returns `true`, is completely\nup to you. If the likelihood of everyone leaving the organization at the same\ntime is low, then you may as well treat it as an unexpected scenario. Although\nother developers will be more comfortable by explicitly handling those cases.\nIn both scenarios, LiveView has you covered.","ref":"error-handling.html#exceptions-on-events-handle_info-handle_event-etc"},{"type":"extras","title":"Live layouts","doc":"# Live layouts\n\nFrom Phoenix v1.7, your application is made of two layouts:\n\n  * the root layout - this is a layout used by both LiveView and\n    regular views. This layout typically contains the ` `\n    definition alongside the head and body tags. Any content defined\n    in the root layout will remain the same, even as you live navigate\n    across LiveViews. The root layout is typically declared on the\n    router with `put_root_layout` and defined as \"root.html.heex\"\n    in your layouts folder\n\n  * the app layout - this is the default application layout which\n    is rendered on both regular HTTP requests and LiveViews.\n    It defaults to \"app.html.heex\"\n\nOverall, those layouts are found in `components/layouts` and are\nembedded within `MyAppWeb.Layouts`.\n\nAll layouts must call `<%= @inner_content %>` to inject the\ncontent rendered by the layout.","ref":"live-layouts.html"},{"type":"extras","title":"Root layout - Live layouts","doc":"The \"root\" layout is rendered only on the initial request and\ntherefore it has access to the `@conn` assign. The root layout\nis typically defined in your router:\n\n    plug :put_root_layout, html: {MyAppWeb.LayoutView, :root}\n\nThe root layout can also be set via the `:root_layout` option\nin your router via `Phoenix.LiveView.Router.live_session/2`.","ref":"live-layouts.html#root-layout"},{"type":"extras","title":"Application layout - Live layouts","doc":"The \"app.html.heex\" layout is rendered with either `@conn` or\n`@socket`. Both Controllers and LiveViews explicitly define\nthe default layouts they will use. See the `def controller`\nand `def live_view` definitions in your `MyAppWeb` to learn how\nit is included.\n\nFor LiveViews, the default layout can be overridden in two different\nways for flexibility:\n\n  1. The `:layout` option in `Phoenix.LiveView.Router.live_session/2`,\n     when set, will override the `:layout` option given via\n     `use Phoenix.LiveView`\n\n  2. The `:layout` option returned on mount, via `{:ok, socket, layout: ...}`\n     will override any previously set layout option\n\nThe LiveView itself will be rendered inside the layout wrapped by\nthe `:container` tag.","ref":"live-layouts.html#application-layout"},{"type":"extras","title":"Updating document title - Live layouts","doc":"Because the root layout from the Plug pipeline is rendered outside of\nLiveView, the contents cannot be dynamically changed. The one exception\nis the ` ` of the HTML document. Phoenix LiveView special cases\nthe `@page_title` assign to allow dynamically updating the title of the\npage, which is useful when using live navigation, or annotating the browser\ntab with a notification. For example, to update the user's notification\ncount in the browser's title bar, first set the `page_title` assign on\nmount:\n\n    def mount(_params, _session, socket) do\n      socket = assign(socket, page_title: \"Latest Posts\")\n      {:ok, socket}\n    end\n\nThen access `@page_title` in the root layout:\n\n```heex\n <%= @page_title %> \n```\n\nYou can also use the `Phoenix.Component.live_title/1` component to support\nadding automatic prefix and suffix to the page title when rendered and\non subsequent updates:\n\n```heex\n \n  <%= assigns[:page_title] || \"Welcome\" %>\n \n```\n\nAlthough the root layout is not updated by LiveView, by simply assigning\nto `page_title`, LiveView knows you want the title to be updated:\n\n    def handle_info({:new_messages, count}, socket) do\n      {:noreply, assign(socket, page_title: \"Latest Posts (#{count} new)\")}\n    end\n\n*Note*: If you find yourself needing to dynamically patch other parts of the\nbase layout, such as injecting new scripts or styles into the ` ` during\nlive navigation, *then a regular, non-live, page navigation should be used\ninstead*. Assigning the `@page_title` updates the `document.title` directly,\nand therefore cannot be used to update any other part of the base layout.","ref":"live-layouts.html#updating-document-title"},{"type":"extras","title":"Live navigation","doc":"# Live navigation\n\nLiveView provides functionality to allow page navigation using the\n[browser's pushState API](https://developer.mozilla.org/en-US/docs/Web/API/History_API).\nWith live navigation, the page is updated without a full page reload.\n\nYou can trigger live navigation in two ways:\n\n  * From the client - this is done by passing either `patch={url}` or `navigate={url}`\n    to the `Phoenix.Component.link/1` component.\n\n  * From the server - this is done by `Phoenix.LiveView.push_patch/2` or `Phoenix.LiveView.push_navigate/2`.\n\nFor example, instead of writing the following in a template:\n\n```heex\n<.link href={~p\"/pages/#{@page + 1}\"}>Next</.link>\n```\n\nYou would write:\n\n```heex\n<.link patch={~p\"/pages/#{@page + 1}\"}>Next</.link>\n```\n\nOr in a LiveView:\n\n```elixir\n{:noreply, push_patch(socket, to: ~p\"/pages/#{@page + 1}\")}\n```\n\nThe \"patch\" operations must be used when you want to navigate to the\ncurrent LiveView, simply updating the URL and the current parameters,\nwithout mounting a new LiveView. When patch is used, the\n[`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`) callback is\ninvoked and the minimal set of changes are sent to the client.\nSee the next section for more information.\n\nThe \"navigate\" operations must be used when you want to dismount the\ncurrent LiveView and mount a new one. You can only \"navigate\" between\nLiveViews in the same session. While redirecting, a `phx-loading` class\nis added to the LiveView, which can be used to indicate to the user a\nnew page is being loaded.\n\nIf you attempt to patch to another LiveView or navigate across live sessions,\na full page reload is triggered. This means your application continues to work,\nin case your application structure changes and that's not reflected in the navigation.\n\nHere is a quick breakdown:\n\n  * `<.link href={...}>` and [`redirect/2`](`Phoenix.Controller.redirect/2`)\n    are HTTP-based, work everywhere, and perform full page reloads\n\n  * `<.link navigate={...}>` and [`push_navigate/2`](`Phoenix.LiveView.push_navigate/2`)\n    work across LiveViews in the same session. They mount a new LiveView\n    while keeping the current layout\n\n  * `<.link patch={...}>` and [`push_patch/2`](`Phoenix.LiveView.push_patch/2`)\n    updates the current LiveView and sends only the minimal diff while also\n    maintaining the scroll position\n\n## `handle_params/3`\n\nThe [`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`) callback is invoked\nafter [`mount/3`](`c:Phoenix.LiveView.mount/3`) and before the initial render.\nIt is also invoked every time `<.link patch={...}>`\nor [`push_patch/2`](`Phoenix.LiveView.push_patch/2`) are used.\nIt receives the request parameters as first argument, the url as second,\nand the socket as third.\n\nFor example, imagine you have a `UserTable` LiveView to show all users in\nthe system and you define it in the router as:\n\n    live \"/users\", UserTable\n\nNow to add live sorting, you could do:\n\n```heex\n<.link patch={path(~p\"/users\", sort_by: \"name\")}>Sort by name</.link>\n```\n\nWhen clicked, since we are navigating to the current LiveView,\n[`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`) will be invoked.\nRemember you should never trust the received params, so you must use the callback to\nvalidate the user input and change the state accordingly:\n\n    def handle_params(params, _uri, socket) do\n      socket =\n        case params[\"sort_by\"] do\n          sort_by when sort_by in ~w(name company) -> assign(socket, sort_by: sort_by)\n          _ -> socket\n        end\n\n      {:noreply, load_users(socket)}\n    end\n\nNote we returned `{:noreply, socket}`, where `:noreply` means no\nadditional information is sent to the client. As with other `handle_*`\ncallbacks, changes to the state inside\n[`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`) will trigger\na new server render.\n\nNote the parameters given to [`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`)\nare the same as the ones given to [`mount/3`](`c:Phoenix.LiveView.mount/3`).\nSo how do you decide which callback to use to load data?\nGenerally speaking, data should always be loaded on [`mount/3`](`c:Phoenix.LiveView.mount/3`),\nsince [`mount/3`](`c:Phoenix.LiveView.mount/3`) is invoked once per LiveView life-cycle.\nOnly the params you expect to be changed via\n`<.link patch={...}>` or\n[`push_patch/2`](`Phoenix.LiveView.push_patch/2`) must be loaded on\n[`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`).\n\nFor example, imagine you have a blog. The URL for a single post is:\n\"/blog/posts/:post_id\". In the post page, you have comments and they are paginated.\nYou use `<.link patch={...}>` to update the shown\ncomments every time the user paginates, updating the URL to \"/blog/posts/:post_id?page=X\".\nIn this example, you will access `\"post_id\"` on [`mount/3`](`c:Phoenix.LiveView.mount/3`) and\nthe page of comments on [`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`).","ref":"live-navigation.html"},{"type":"extras","title":"Replace page address - Live navigation","doc":"LiveView also allows the current browser URL to be replaced. This is useful when you\nwant certain events to change the URL but without polluting the browser's history.\nThis can be done by passing the `<.link replace>` option to any of the navigation helpers.","ref":"live-navigation.html#replace-page-address"},{"type":"extras","title":"Multiple LiveViews in the same page - Live navigation","doc":"LiveView allows you to have multiple LiveViews in the same page by calling\n`Phoenix.Component.live_render/3` in your templates. However, only\nthe LiveViews defined directly in your router can use the \"Live Navigation\"\nfunctionality described here. This is important because LiveViews work\nclosely with your router, guaranteeing you can only navigate to known\nroutes.","ref":"live-navigation.html#multiple-liveviews-in-the-same-page"},{"type":"extras","title":"Security considerations","doc":"# Security considerations\n\nLiveView begins its life-cycle as a regular HTTP request. Then a stateful\nconnection is established. Both the HTTP request and the stateful connection\nreceive the client data via parameters and session.\n\nThis means that any session validation must happen both in the HTTP request\n(plug pipeline) and the stateful connection (LiveView mount).","ref":"security-model.html"},{"type":"extras","title":"Authentication vs authorization - Security considerations","doc":"When speaking about security, there are two terms commonly used:\nauthentication and authorization. Authentication is about identifying\na user. Authorization is about telling if a user has access to a certain\nresource or feature in the system.\n\nIn a regular web application, once a user is authenticated, for example by\nentering their email and password, or by using a third-party service such as\nGoogle, Twitter, or Facebook, a token identifying the user is stored in the\nsession, which is a cookie (a key-value pair) stored in the user's browser.\n\nEvery time there is a request, we read the value from the session, and, if\nvalid, we fetch the user stored in the session from the database. The session\nis automatically validated by Phoenix and tools like `mix phx.gen.auth` can\ngenerate the building blocks of an authentication system for you.\n\nOnce the user is authenticated, they may perform many actions on the page,\nand some of those actions require specific permissions. This is called\nauthorization and the specific rules often change per application.\n\nIn a regular web application, we perform authentication and authorization\nchecks on every request. Given LiveViews start as a regular HTTP request,\nthey share the authentication logic with regular requests through plugs.\nOnce the user is authenticated, we typically validate the sessions on\nthe `mount` callback. Authorization rules generally happen on `mount`\n(for instance, is the user allowed to see this page?) and also on\n`handle_event` (is the user allowed to delete this item?).","ref":"security-model.html#authentication-vs-authorization"},{"type":"extras","title":"Mounting considerations - Security considerations","doc":"The [`mount/3`](`c:Phoenix.LiveView.mount/3`) callback is invoked both on\nthe initial HTTP mount and when LiveView is connected. Therefore, any\nauthorization performed during mount will cover all scenarios.\n\nOnce the user is authorized and stored in the session, the logic to fetch the user and further authorize its account needs to happen inside LiveView. For example, if you have the following plugs:\n\n    plug :ensure_user_authenticated\n    plug :ensure_user_confirmed\n\nThen the [`mount/3`](`c:Phoenix.LiveView.mount/3`) callback of your LiveView\nshould execute those same verifications:\n\n    def mount(_params, %{\"user_id\" => user_id} = _session, socket) do\n      socket = assign(socket, current_user: Accounts.get_user!(user_id))\n\n      socket =\n        if socket.assigns.current_user.confirmed_at do\n          socket\n        else\n          redirect(socket, to: \"/login\")\n        end\n\n      {:ok, socket}\n    end\n\nBeginning with v0.17, LiveView includes the `on_mount` (`Phoenix.LiveView.on_mount/1`) hook,\nwhich allows you to encapsulate this logic and execute it on every mount,\nas you would with plug:\n\n    defmodule MyAppWeb.UserLiveAuth do\n      import Phoenix.Component\n      import Phoenix.LiveView\n      alias MyAppWeb.Accounts # from `mix phx.gen.auth`\n\n      def on_mount(:default, _params, %{\"user_token\" => user_token} = _session, socket) do\n        socket =\n          assign_new(socket, :current_user, fn ->\n            Accounts.get_user_by_session_token(user_token)\n          end)\n\n        if socket.assigns.current_user.confirmed_at do\n          {:cont, socket}\n        else\n          {:halt, redirect(socket, to: \"/login\")}\n        end\n      end\n    end\n\nWe use [`assign_new/3`](`Phoenix.Component.assign_new/3`). This is a\nconvenience to avoid fetching the `current_user` multiple times across\nLiveViews.\n\nNow we can use the hook whenever relevant. One option is to specify\nthe hook in your router under `live_session`:\n\n    live_session :default, on_mount: MyAppWeb.UserLiveAuth do\n      # Your routes\n    end\n\nAlternatively, you can either specify the hook directly in the LiveView:\n\n    defmodule MyAppWeb.PageLive do\n      use MyAppWeb, :live_view\n      on_mount MyAppWeb.UserLiveAuth\n\n      ...\n    end\n\nIf you prefer, you can add the hook to `def live_view` under `MyAppWeb`,\nto run it on all LiveViews by default:\n\n    def live_view do\n      quote do\n        use Phoenix.LiveView,\n          layout: {MyAppWeb.LayoutView, :live}\n\n        on_mount MyAppWeb.UserLiveAuth\n        unquote(html_helpers())\n      end\n    end","ref":"security-model.html#mounting-considerations"},{"type":"extras","title":"Events considerations - Security considerations","doc":"Every time the user performs an action on your system, you should verify if the user\nis authorized to do so, regardless if you are using LiveViews or not. For example,\nimagine a user can see all projects in a web application, but they may not have\npermission to delete any of them. At the UI level, you handle this accordingly\nby not showing the delete button in the projects listing, but a savvy user can\ndirectly talk to the server and request a deletion anyway. For this reason, **you\nmust always verify permissions on the server**.\n\nIn LiveView, most actions are handled by the `handle_event` callback. Therefore,\nyou typically authorize the user within those callbacks. In the scenario just\ndescribed, one might implement this:\n\n    on_mount MyAppWeb.UserLiveAuth\n\n    def mount(_params, _session, socket) do\n      {:ok, load_projects(socket)}\n    end\n\n    def handle_event(\"delete_project\", %{\"project_id\" => project_id}, socket) do\n      Project.delete!(socket.assigns.current_user, project_id)\n      {:noreply, update(socket, :projects, &Enum.reject(&1, fn p -> p.id == project_id end)}\n    end\n\n    defp load_projects(socket) do\n      projects = Project.all_projects(socket.assigns.current_user)\n      assign(socket, projects: projects)\n    end\n\nFirst, we used `on_mount` to authenticate the user based on the data stored in\nthe session. Then we load all projects based on the authenticated user. Now,\nwhenever there is a request to delete a project, we still pass the current user\nas argument to the `Project` context, so it verifies if the user is allowed to\ndelete it or not. In case it cannot delete, it is fine to just raise an exception.\nAfter all, users are not meant to trigger this code path anyway (unless they are\nfiddling with something they are not supposed to!).","ref":"security-model.html#events-considerations"},{"type":"extras","title":"Disconnecting all instances of a live user - Security considerations","doc":"So far, the security model between LiveView and regular web applications have\nbeen remarkably similar. After all, we must always authenticate and authorize\nevery user. The main difference between them happens on logout or when revoking\naccess.\n\nBecause LiveView is a permanent connection between client and server, if a user\nis logged out, or removed from the system, this change won't reflect on the\nLiveView part unless the user reloads the page.\n\nLuckily, it is possible to address this by setting a `live_socket_id` in the\nsession. For example, when logging in a user, you could do:\n\n    conn\n    |> put_session(:current_user_id, user.id)\n    |> put_session(:live_socket_id, \"users_socket:#{user.id}\")\n\nNow all LiveView sockets will be identified and listen to the given `live_socket_id`.\nYou can then disconnect all live users identified by said ID by broadcasting on\nthe topic:\n\n    MyAppWeb.Endpoint.broadcast(\"users_socket:#{user.id}\", \"disconnect\", %{})\n\n> Note: If you use `mix phx.gen.auth` to generate your authentication system,\n> lines to that effect are already present in the generated code. The generated\n> code uses a `user_token` instead of referring to the `user_id`.\n\nOnce a LiveView is disconnected, the client will attempt to reestablish\nthe connection and re-execute the [`mount/3`](`c:Phoenix.LiveView.mount/3`)\ncallback. In this case, if the user is no longer logged in or it no longer has\naccess to the current resource, `mount/3` will fail and the user will be\nredirected.\n\nThis is the same mechanism provided by `Phoenix.Channel`s. Therefore, if\nyour application uses both channels and LiveViews, you can use the same\ntechnique to disconnect any stateful connection.\n\n## `live_session` and `live_redirect`\n\nLiveView supports live redirect, which allows users to navigate between\npages over the LiveView connection. Whenever there is a `live_redirect`,\na new LiveView will be mounted, skipping the regular HTTP requests and\nwithout going through the plug pipeline.\n\nHowever, if you want to draw stronger boundaries between parts of your\napplication, you can also use `Phoenix.LiveView.Router.live_session/2`\nto group your live routes. This can be handy because you can only\n`live_redirect` between LiveViews in the same `live_session`.\n\nFor example, imagine you need to authenticate two distinct types of users.\nYour regular users login via email and password, and you have an admin\ndashboard that uses http auth. You can specify different `live_session`s\nfor each authentication flow:\n\n    live_session :default do\n      scope \"/\" do\n        pipe_through [:authenticate_user]\n        get ...\n        live ...\n      end\n    end\n\n    live_session :admin do\n      scope \"/admin\" do\n        pipe_through [:http_auth_admin]\n        get ...\n        live ...\n      end\n    end\n\nNow every time you try to navigate to an admin panel, and out of it,\na regular page navigation will happen and a brand new live connection\nwill be established.\n\nOnce again, it is worth remembering that LiveViews require their own\nsecurity checks, so we use `pipe_through` above to protect the regular\nroutes (get, post, etc.) and the LiveViews should run their own checks\nusing `on_mount` hooks.\n\n`live_session` can also be used to enforce each LiveView group has\na different root layout, since layouts are not updated between live\nredirects:\n\n    live_session :default, root_layout: {LayoutView, :root} do\n      ...\n    end\n\n    live_session :admin, root_layout: {LayoutView, :admin} do\n      ...\n    end\n\nFinally, you can even combine `live_session` with `on_mount`. Instead\nof declaring `on_mount` on every LiveView, you can declare it at the\nrouter level and it will enforce it on all LiveViews under it:\n\n    live_session :default, on_mount: MyAppWeb.UserLiveAuth do\n      scope \"/\" do\n        pipe_through [:authenticate_user]\n        live ...\n      end\n    end\n\n    live_session :admin, on_mount: MyAppWeb.AdminLiveAuth do\n      scope \"/admin\" do\n        pipe_through [:authenticate_admin]\n        live ...\n      end\n    end\n\nEach live route under the `:default` `live_session` will invoke\nthe `MyAppWeb.UserLiveAuth` hook on mount. This module was defined\nearlier in this guide. We will also pipe regular web requests through\n`:authenticate_user`, which must execute the same checks as\n`MyAppWeb.UserLiveAuth`, but tailored to plug.\n\nSimilarly, the `:admin` `live_session` has its own authentication\nflow, powered by `MyAppWeb.AdminLiveAuth`. It also defines a plug\nequivalent named `:authenticate_admin`, which will be used by any\nregular request. If there are no regular web requests defined under\na live session, then the `pipe_through` checks are not necessary.\n\nDeclaring the `on_mount` on `live_session` is exactly the same as\ndeclaring it in each LiveView. It will be executed every time a\nLiveView is mounted, even after `live_redirect`s.","ref":"security-model.html#disconnecting-all-instances-of-a-live-user"},{"type":"extras","title":"Summing up - Security considerations","doc":"The important concepts to keep in mind are:\n\n  * If you have both LiveViews and regular web requests, then any\n    authorization logic in your plugs must be replicated in your LiveView.\n    Code that executes on the `mount` callback always runs on both live\n    and regular web requests to that LiveView\n\n  * All actions (events) must also be explicitly authorized by\n    checking permissions. Those permissions are often domain/business\n    specific, and typically happen in your context modules. This is\n    also a requirement for regular requests and responses\n\n  * `live_session` can be used to draw boundaries between groups of\n    LiveViews. While you could use `live_session` to draw lines between\n    different authorization rules, doing so would lead to frequent page\n    reloads. For this reason, we typically use `live_session` to enforce\n    different *authentication* requirements or whenever you need to\n    change root layouts","ref":"security-model.html#summing-up"},{"type":"extras","title":"Telemetry","doc":"# Telemetry\n\nLiveView currently exposes the following [`telemetry`](https://hexdocs.pm/telemetry) events:\n\n  * `[:phoenix, :live_view, :mount, :start]` - Dispatched by a `Phoenix.LiveView`\n    immediately before [`mount/3`](`c:Phoenix.LiveView.mount/3`) is invoked.\n\n    * Measurement:\n\n          %{system_time: System.monotonic_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            params: unsigned_params | :not_mounted_at_router,\n            session: map,\n            uri: String.t() | nil\n          }\n\n  * `[:phoenix, :live_view, :mount, :stop]` - Dispatched by a `Phoenix.LiveView`\n    when the [`mount/3`](`c:Phoenix.LiveView.mount/3`) callback completes successfully.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            params: unsigned_params | :not_mounted_at_router,\n            session: map,\n            uri: String.t() | nil\n          }\n\n  * `[:phoenix, :live_view, :mount, :exception]` - Dispatched by a `Phoenix.LiveView`\n    when an exception is raised in the [`mount/3`](`c:Phoenix.LiveView.mount/3`) callback.\n\n    * Measurement: `%{duration: native_time}`\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            kind: atom,\n            reason: term,\n            params: unsigned_params | :not_mounted_at_router,\n            session: map,\n            uri: String.t() | nil\n          }\n\n  * `[:phoenix, :live_view, :handle_params, :start]` - Dispatched by a `Phoenix.LiveView`\n    immediately before [`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`) is invoked.\n\n    * Measurement:\n\n          %{system_time: System.monotonic_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            params: unsigned_params,\n            uri: String.t()\n          }\n\n  * `[:phoenix, :live_view, :handle_params, :stop]` - Dispatched by a `Phoenix.LiveView`\n    when the [`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`) callback completes successfully.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            params: unsigned_params,\n            uri: String.t()\n          }\n\n  * `[:phoenix, :live_view, :handle_params, :exception]` - Dispatched by a `Phoenix.LiveView`\n    when an exception is raised in the [`handle_params/3`](`c:Phoenix.LiveView.handle_params/3`) callback.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            kind: atom,\n            reason: term,\n            params: unsigned_params,\n            uri: String.t()\n          }\n\n  * `[:phoenix, :live_view, :handle_event, :start]` - Dispatched by a `Phoenix.LiveView`\n    immediately before [`handle_event/3`](`c:Phoenix.LiveView.handle_event/3`) is invoked.\n\n    * Measurement:\n\n          %{system_time: System.monotonic_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            event: String.t(),\n            params: unsigned_params\n          }\n\n  * `[:phoenix, :live_view, :handle_event, :stop]` - Dispatched by a `Phoenix.LiveView`\n    when the [`handle_event/3`](`c:Phoenix.LiveView.handle_event/3`) callback completes successfully.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            event: String.t(),\n            params: unsigned_params\n          }\n\n  * `[:phoenix, :live_view, :handle_event, :exception]` - Dispatched by a `Phoenix.LiveView`\n    when an exception is raised in the [`handle_event/3`](`c:Phoenix.LiveView.handle_event/3`) callback.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            kind: atom,\n            reason: term,\n            event: String.t(),\n            params: unsigned_params\n          }\n\n  * `[:phoenix, :live_view, :render, :start]` - Dispatched by a `Phoenix.LiveView`\n    immediately before [`render/1`](`c:Phoenix.LiveComponent.render/1`) is invoked.\n\n    * Measurement:\n\n          %{system_time: System.monotonic_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            force?: boolean,\n            changed?: boolean\n          }\n\n  * `[:phoenix, :live_view, :render, :stop]` - Dispatched by a `Phoenix.LiveView`\n    when the [`render/1`](`c:Phoenix.LiveView.render/1`) callback completes successfully.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            force?: boolean,\n            changed?: boolean\n          }\n\n  * `[:phoenix, :live_view, :render, :exception]` - Dispatched by a `Phoenix.LiveView`\n    when an exception is raised in the [`render/1`](`c:Phoenix.LiveView.render/1`) callback.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            kind: atom,\n            reason: term,\n            force?: boolean,\n            changed?: boolean\n          }\n\n  * `[:phoenix, :live_component, :update, :start]` - Dispatched by a `Phoenix.LiveComponent`\n    immediately before [`update/2`](`c:Phoenix.LiveComponent.update/2`) or a\n    [`update_many/1`](`c:Phoenix.LiveComponent.update_many/1`) is invoked.\n\n    In the case of[`update/2`](`c:Phoenix.LiveComponent.update/2`) it might dispatch one event\n    for multiple calls.\n\n    * Measurement:\n\n          %{system_time: System.monotonic_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            component: atom,\n            assigns_sockets: [{map(), Phoenix.LiveView.Socket.t}]\n          }\n\n  * `[:phoenix, :live_component, :update, :stop]` - Dispatched by a `Phoenix.LiveComponent`\n    when the [`update/2`](`c:Phoenix.LiveComponent.update/2`) or a\n    [`update_many/1`](`c:Phoenix.LiveComponent.update_many/1`) callback completes successfully.\n\n    In the case of[`update/2`](`c:Phoenix.LiveComponent.update/2`) it might dispatch one event\n    for multiple calls. The `sockets` metadata contain the updated sockets.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            component: atom,\n            assigns_sockets: [{map(), Phoenix.LiveView.Socket.t}],\n            sockets: [Phoenix.LiveView.Socket.t]\n          }\n\n  * `[:phoenix, :live_component, :update, :exception]` - Dispatched by a `Phoenix.LiveComponent`\n    when an exception is raised in the [`update/2`](`c:Phoenix.LiveComponent.update/2`) or a\n    [`update_many/1`](`c:Phoenix.LiveComponent.update_many/1`) callback.\n\n    In the case of[`update/2`](`c:Phoenix.LiveComponent.update/2`) it might dispatch one event\n    for multiple calls.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            kind: atom,\n            reason: term,\n            component: atom,\n            assigns_sockets: [{map(), Phoenix.LiveView.Socket.t}]\n          }\n\n  * `[:phoenix, :live_component, :handle_event, :start]` - Dispatched by a `Phoenix.LiveComponent`\n    immediately before [`handle_event/3`](`c:Phoenix.LiveComponent.handle_event/3`) is invoked.\n\n    * Measurement:\n\n          %{system_time: System.monotonic_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            component: atom,\n            event: String.t(),\n            params: unsigned_params\n          }\n\n  * `[:phoenix, :live_component, :handle_event, :stop]` - Dispatched by a `Phoenix.LiveComponent`\n    when the [`handle_event/3`](`c:Phoenix.LiveComponent.handle_event/3`) callback completes successfully.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            component: atom,\n            event: String.t(),\n            params: unsigned_params\n          }\n\n  * `[:phoenix, :live_component, :handle_event, :exception]` - Dispatched by a `Phoenix.LiveComponent`\n    when an exception is raised in the [`handle_event/3`](`c:Phoenix.LiveComponent.handle_event/3`) callback.\n\n    * Measurement:\n\n          %{duration: native_time}\n\n    * Metadata:\n\n          %{\n            socket: Phoenix.LiveView.Socket.t,\n            kind: atom,\n            reason: term,\n            component: atom,\n            event: String.t(),\n            params: unsigned_params\n          }","ref":"telemetry.html"},{"type":"extras","title":"Uploads","doc":"# Uploads\n\nLiveView supports interactive file uploads with progress for\nboth direct to server uploads as well as direct-to-cloud\n[external uploads](uploads-external.html) on the client.","ref":"uploads.html"},{"type":"extras","title":"Built-in Features - Uploads","doc":"* Accept specification - Define accepted file types, max\n    number of entries, max file size, etc. When the client\n    selects file(s), the file metadata is automatically\n    validated against the specification. See\n    `Phoenix.LiveView.allow_upload/3`.\n\n  * Reactive entries - Uploads are populated in an\n    `@uploads` assign in the socket. Entries automatically\n    respond to progress, errors, cancellation, etc.\n\n  * Drag and drop - Use the `phx-drop-target` attribute to\n    enable. See `Phoenix.Component.live_file_input/1`.","ref":"uploads.html#built-in-features"},{"type":"extras","title":"Allow uploads - Uploads","doc":"You enable an upload, typically on mount, via [`allow_upload/3`]:\n\n```elixir\n@impl Phoenix.LiveView\ndef mount(_params, _session, socket) do\n  {:ok,\n   socket\n   |> assign(:uploaded_files, [])\n   |> allow_upload(:avatar, accept: ~w(.jpg .jpeg), max_entries: 2)}\nend\n```\n\nThat's it for now! We will come back to the LiveView to\nimplement some form- and upload-related callbacks later, but\nmost of the functionality around uploads takes place in the\ntemplate.","ref":"uploads.html#allow-uploads"},{"type":"extras","title":"Render reactive elements - Uploads","doc":"Use the `Phoenix.Component.live_file_input/1` component\nto render a file input for the upload:\n\n```heex\n<%!-- lib/my_app_web/live/upload_live.html.heex --%>\n\n \n  <.live_file_input upload={@uploads.avatar} />\n   Upload \n \n```\n\n> **Important:** You must bind `phx-submit` and `phx-change` on the form.\n\nNote that while [`live_file_input/1`]\nallows you to set additional attributes on the file input,\nmany attributes such as `id`, `accept`, and `multiple` will\nbe set automatically based on the [`allow_upload/3`] spec.\n\nReactive updates to the template will occur as the end-user\ninteracts with the file input.\n\n#","ref":"uploads.html#render-reactive-elements"},{"type":"extras","title":"Upload entries - Uploads","doc":"Uploads are populated in an `@uploads` assign in the socket.\nEach allowed upload contains a _list_ of entries,\nirrespective of the `:max_entries` value in the\n[`allow_upload/3`] spec. These entry structs contain all the\ninformation about an upload, including progress, client file\ninfo, errors, etc.\n\nLet's look at an annotated example:\n\n```heex\n<%!-- lib/my_app_web/live/upload_live.html.heex --%>\n\n<%!-- use phx-drop-target with the upload ref to enable file drag and drop --%>\n \n\n<%!-- render each avatar entry --%>\n<%= for entry <- @uploads.avatar.entries do %>\n   \n\n     \n      <.live_img_preview entry={entry} />\n       <%= entry.client_name %> \n     \n\n    <%!-- entry.progress will update automatically for in-flight entries --%>\n      <%= entry.progress %>%  \n\n    <%!-- a regular click event whose handler will invoke Phoenix.LiveView.cancel_upload/3 --%>\n     &times; \n\n    <%!-- Phoenix.Component.upload_errors/2 returns a list of error atoms --%>\n    <%= for err <- upload_errors(@uploads.avatar, entry) do %>\n       <%= error_to_string(err) %> \n    <% end %>\n\n   \n<% end %>\n\n<%!-- Phoenix.Component.upload_errors/1 returns a list of error atoms --%>\n<%= for err <- upload_errors(@uploads.avatar) do %>\n   <%= error_to_string(err) %> \n<% end %>\n\n \n```\n\nThe `section` element in the example acts as the\n`phx-drop-target` for the `:avatar` upload. Users can interact\nwith the file input or they can drop files over the element\nto add new entries.\n\nUpload entries are created when a file is added to the form\ninput and each will exist until it has been consumed,\nfollowing a successfully completed upload.\n\n#","ref":"uploads.html#upload-entries"},{"type":"extras","title":"Entry validation - Uploads","doc":"Validation occurs automatically based on any conditions\nthat were specified in [`allow_upload/3`] however, as\nmentioned previously you are required to bind `phx-change`\non the form in order for the validation to be performed.\nTherefore you must implement at least a minimal callback:\n\n```elixir\n@impl Phoenix.LiveView\ndef handle_event(\"validate\", _params, socket) do\n  {:noreply, socket}\nend\n```\n\nEntries for files that do not match the [`allow_upload/3`]\nspec will contain errors. Use\n`Phoenix.Component.upload_errors/2` and your own\nhelper function to render a friendly error message:\n\n```elixir\ndef error_to_string(:too_large), do: \"Too large\"\ndef error_to_string(:not_accepted), do: \"You have selected an unacceptable file type\"\n```\n\nFor error messages that affect all entries, use\n`Phoenix.Component.upload_errors/1`, and your own\nhelper function to render a friendly error message:\n\n```elixir\ndef error_to_string(:too_many_files), do: \"You have selected too many files\"\n```\n\n#","ref":"uploads.html#entry-validation"},{"type":"extras","title":"Cancel an entry - Uploads","doc":"Upload entries may also be canceled, either programmatically\nor as a result of a user action. For instance, to handle the\nclick event in the template above, you could do the following:\n\n```elixir\n@impl Phoenix.LiveView\ndef handle_event(\"cancel-upload\", %{\"ref\" => ref}, socket) do\n  {:noreply, cancel_upload(socket, :avatar, ref)}\nend\n```","ref":"uploads.html#cancel-an-entry"},{"type":"extras","title":"Consume uploaded entries - Uploads","doc":"When the end-user submits a form containing a [`live_file_input/1`],\nthe JavaScript client first uploads the file(s) before\ninvoking the callback for the form's `phx-submit` event.\n\nWithin the callback for the `phx-submit` event, you invoke\nthe `Phoenix.LiveView.consume_uploaded_entries/3` function\nto process the completed uploads, persisting the relevant\nupload data alongside the form data:\n\n```elixir\n@impl Phoenix.LiveView\ndef handle_event(\"save\", _params, socket) do\n  uploaded_files =\n    consume_uploaded_entries(socket, :avatar, fn %{path: path}, _entry ->\n      dest = Path.join(Application.app_dir(:my_app, \"priv/static/uploads\"), Path.basename(path))\n      # You will need to create `priv/static/uploads` for `File.cp!/2` to work.\n      File.cp!(path, dest)\n      {:ok, ~p\"/uploads/#{Path.basename(dest)}\"}\n    end)\n\n  {:noreply, update(socket, :uploaded_files, &(&1 ++ uploaded_files))}\nend\n```\n\n> **Note**: While client metadata cannot be trusted, max file size validations\n> are enforced as each chunk is received when performing direct to server uploads.\n\nThis example writes the file directly to disk, under the `priv` folder.\nIn order to access your upload, for example in an ` ` tag, you need\nto add the `uploads` directory to `static_paths/0`.  In a vanilla Phoenix\nproject, this is found in `lib/my_app_web.ex`.\n\nAnother thing to be aware of is that in development, changes to\n`priv/static/uploads` will be picked up by `live_reload`.  This means that as\nsoon as your upload succeeds, your app will be reloaded in the browser.  This\ncan be temporarily disabled by setting `code_reloader: false` in `config/dev.exs`.\n\nBesides the above, this approach also has limitations in production. If you are\nrunning multiple instances of your application, the uploaded file will be store\nonly in one of the instances. Any request routed to the other machine will\nultimately fail.\n\nFor these reasons, it is best if uploads are stored elsewhere, such as the\ndatabase (depending on the size and contents) or a separate storage service.\nFor more information on implementing client-side, direct-to-cloud uploads,\nsee the [External Uploads guide](uploads-external.md).","ref":"uploads.html#consume-uploaded-entries"},{"type":"extras","title":"Appendix A: UploadLive - Uploads","doc":"A complete example of the LiveView from this guide:\n\n```elixir\n# lib/my_app_web/live/upload_live.ex\ndefmodule MyAppWeb.UploadLive do\n  use MyAppWeb, :live_view\n\n  @impl Phoenix.LiveView\n  def mount(_params, _session, socket) do\n    {:ok,\n    socket\n    |> assign(:uploaded_files, [])\n    |> allow_upload(:avatar, accept: ~w(.jpg .jpeg), max_entries: 2)}\n  end\n\n  @impl Phoenix.LiveView\n  def handle_event(\"validate\", _params, socket) do\n    {:noreply, socket}\n  end\n\n  @impl Phoenix.LiveView\n  def handle_event(\"cancel-upload\", %{\"ref\" => ref}, socket) do\n    {:noreply, cancel_upload(socket, :avatar, ref)}\n  end\n\n  @impl Phoenix.LiveView\n  def handle_event(\"save\", _params, socket) do\n    uploaded_files =\n      consume_uploaded_entries(socket, :avatar, fn %{path: path}, _entry ->\n        dest = Path.join([:code.priv_dir(:my_app), \"static\", \"uploads\", Path.basename(path)])\n        # You will need to create `priv/static/uploads` for `File.cp!/2` to work.\n        File.cp!(path, dest)\n        {:ok, ~p\"/uploads/#{Path.basename(dest)}\"}\n      end)\n\n    {:noreply, update(socket, :uploaded_files, &(&1 ++ uploaded_files))}\n  end\n\n  defp error_to_string(:too_large), do: \"Too large\"\n  defp error_to_string(:too_many_files), do: \"You have selected too many files\"\n  defp error_to_string(:not_accepted), do: \"You have selected an unacceptable file type\"\nend\n```\n\nTo access your uploads via your app, make sure to add `uploads` to\n`MyAppWeb.static_paths/0`.\n\n[`allow_upload/3`]: `Phoenix.LiveView.allow_upload/3`\n[`live_file_input/1`]: `Phoenix.Component.live_file_input/1`","ref":"uploads.html#appendix-a-uploadlive"},{"type":"extras","title":"Gettext for internationalization","doc":"# Gettext for internationalization\n\nFor internationalization with [gettext](https://hexdocs.pm/gettext/Gettext.html),\nyou must call `Gettext.put_locale/2` on the LiveView mount callback to instruct\nthe LiveView which locale should be used for rendering the page.\n\nHowever, one question that has to be answered is how to retrieve the locale in\nthe first place. There are many approaches to solve this problem:\n\n1. The locale could be stored in the URL as a parameter\n2. The locale could be stored in the session\n3. The locale could be stored in the database\n\nWe will briefly cover these approaches to provide some direction.","ref":"using-gettext.html"},{"type":"extras","title":"Locale from parameters - Gettext for internationalization","doc":"You can say all URLs have a locale parameter. In your router:\n\n    scope \"/:locale\" do\n      live ...\n      get ...\n    end\n\nAccessing a page without a locale should automatically redirect\nto a URL with locale (the best locale could be fetched from\nHTTP headers, which is outside of the scope of this guide).\n\nThen, assuming all URLs have a locale, you can set the Gettext\nlocale accordingly:\n\n    def mount(%{\"locale\" => locale}, _session, socket) do\n      Gettext.put_locale(MyApp.Gettext, locale)\n      {:ok, socket}\n    end\n\n\nYou can also use the [`on_mount`](`Phoenix.LiveView.on_mount/1`) hook to\nautomatically restore the locale for every LiveView in your application:\n\n    defmodule MyAppWeb.RestoreLocale do\n      def on_mount(:default, %{\"locale\" => locale}, _session, socket) do\n        Gettext.put_locale(MyApp.Gettext, locale)\n        {:cont, socket}\n      end\n\n      # catch-all case\n      def on_mount(:default, _params, _session, socket), do: {:cont, socket}\n    end\n\nThen, add this hook to `def live_view` under `MyAppWeb`, to run it on all\nLiveViews by default:\n\n    def live_view do\n      quote do\n        use Phoenix.LiveView,\n          layout: {MyAppWeb.LayoutView, :live}\n\n        on_mount MyAppWeb.RestoreLocale\n        unquote(view_helpers())\n      end\n    end\n\nNote that, because the Gettext locale is not stored in the assigns, if you\nwant to change the locale, you must use `<.link navigate={...} />`, instead\nof simply patching the page.","ref":"using-gettext.html#locale-from-parameters"},{"type":"extras","title":"Locale from session - Gettext for internationalization","doc":"You may also store the locale in the Plug session. For example, in a controller\nyou might do:\n\n    def put_user_session(conn, current_user) do\n      Gettext.put_locale(MyApp.Gettext, current_user.locale)\n\n      conn\n      |> put_session(:user_id, current_user.id)\n      |> put_session(:locale, current_user.locale)\n    end\n\nand then restore the locale from session within your LiveView mount:\n\n    def mount(_params, %{\"locale\" => locale}, socket) do\n      Gettext.put_locale(MyApp.Gettext, locale)\n      {:ok, socket}\n    end\n\nYou can also encapsulate this in a hook, as done in the previous section.\n\nHowever, if the locale is stored in the session, you can only change it\nby using regular controller requests. Therefore you should always use\n`<.link to={...} />` to point to a controller that change the session\naccordingly, reloading any LiveView.","ref":"using-gettext.html#locale-from-session"},{"type":"extras","title":"Locale from database - Gettext for internationalization","doc":"You may also allow users to store their locale configuration in the database.\nThen, on `mount/3`, you can retrieve the user id from the session and load\nthe locale:\n\n    def mount(_params, %{\"user_id\" => user_id}, socket) do\n      user = Users.get_user!(user_id)\n      Gettext.put_locale(MyApp.Gettext, user.locale)\n      {:ok, socket}\n    end\n\nIn practice, you may end-up mixing more than one approach listed here.\nFor example, reading from the database is great once the user is logged in\nbut, before that happens, you may need to store the locale in the session\nor in the URL.\n\nSimilarly, you can keep the locale in the URL, but change the URL accordingly\nto the user preferred locale once they sign in. Hopefully this guide gives\nsome suggestions on how to move forward and explore the best approach for your\napplication.","ref":"using-gettext.html#locale-from-database"},{"type":"extras","title":"Bindings","doc":"# Bindings\n\nPhoenix supports DOM element bindings for client-server interaction. For\nexample, to react to a click on a button, you would render the element:\n\n     + \n\nThen on the server, all LiveView bindings are handled with the `handle_event`\ncallback, for example:\n\n    def handle_event(\"inc_temperature\", _value, socket) do\n      {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id)\n      {:noreply, assign(socket, :temperature, new_temp)}\n    end\n\n| Binding                | Attributes |\n|------------------------|------------|\n| [Params](#click-events) | `phx-value-*` |\n| [Click Events](#click-events) | `phx-click`, `phx-click-away` |\n| [Form Events](form-bindings.md) | `phx-change`, `phx-submit`, `phx-feedback-for`, `phx-feedback-group`, `phx-disable-with`, `phx-trigger-action`, `phx-auto-recover` |\n| [Focus Events](#focus-and-blur-events) | `phx-blur`, `phx-focus`, `phx-window-blur`, `phx-window-focus` |\n| [Key Events](#key-events) | `phx-keydown`, `phx-keyup`, `phx-window-keydown`, `phx-window-keyup`, `phx-key` |\n| [Scroll Events](#scroll-events-and-infinite-stream-pagination) | `phx-viewport-top`, `phx-viewport-bottom` |\n| [DOM Patching](dom-patching.md) | `phx-mounted`, `phx-update`, `phx-remove` |\n| [JS Interop](js-interop.md#client-hooks) | `phx-hook` |\n| [Lifecycle Events](#lifecycle-events) | `phx-mounted`, `phx-disconnected`, `phx-connected` |\n| [Rate Limiting](#rate-limiting-events-with-debounce-and-throttle) | `phx-debounce`, `phx-throttle` |\n| [Static tracking](`Phoenix.LiveView.static_changed?/1`) | `phx-track-static` |","ref":"bindings.html"},{"type":"extras","title":"Click Events - Bindings","doc":"The `phx-click` binding is used to send click events to the server.\nWhen any client event, such as a `phx-click` click is pushed, the value\nsent to the server will be chosen with the following priority:\n\n  * The `:value` specified in `Phoenix.LiveView.JS.push/3`, such as:\n\n    ```heex\n     \n    ```\n\n  * Any number of optional `phx-value-` prefixed attributes, such as:\n\n         \n\n    will send the following map of params to the server:\n\n        def handle_event(\"inc\", %{\"myvar1\" => \"val1\", \"myvar2\" => \"val2\"}, socket) do\n\n    If the `phx-value-` prefix is used, the server payload will also contain a `\"value\"`\n    if the element's value attribute exists.\n\n  * The payload will also include any additional user defined metadata of the client event.\n    For example, the following `LiveSocket` client option would send the coordinates and\n    `altKey` information for all clicks:\n\n        let liveSocket = new LiveSocket(\"/live\", Socket, {\n          params: {_csrf_token: csrfToken},\n          metadata: {\n            click: (e, el) => {\n              return {\n                altKey: e.altKey,\n                clientX: e.clientX,\n                clientY: e.clientY\n              }\n            }\n          }\n        })\n\nThe `phx-click-away` event is fired when a click event happens outside of the element.\nThis is useful for hiding toggled containers like drop-downs.","ref":"bindings.html#click-events"},{"type":"extras","title":"Focus and Blur Events - Bindings","doc":"Focus and blur events may be bound to DOM elements that emit\nsuch events, using the `phx-blur`, and `phx-focus` bindings, for example:\n\n```heex\n \n```\n\nTo detect when the page itself has received focus or blur,\n`phx-window-focus` and `phx-window-blur` may be specified. These window\nlevel events may also be necessary if the element in consideration\n(most often a `div` with no tabindex) cannot receive focus. Like other\nbindings, `phx-value-*` can be provided on the bound element, and those\nvalues will be sent as part of the payload. For example:\n\n```heex\n \n  ...\n \n```","ref":"bindings.html#focus-and-blur-events"},{"type":"extras","title":"Key Events - Bindings","doc":"The `onkeydown`, and `onkeyup` events are supported via the `phx-keydown`,\nand `phx-keyup` bindings. Each binding supports a `phx-key` attribute, which triggers\nthe event for the specific key press. If no `phx-key` is provided, the event is triggered\nfor any key press. When pushed, the value sent to the server will contain the `\"key\"`\nthat was pressed, plus any user-defined metadata. For example, pressing the\nEscape key looks like this:\n\n    %{\"key\" => \"Escape\"}\n\nTo capture additional user-defined metadata, the `metadata` option for keydown events\nmay be provided to the `LiveSocket` constructor. For example:\n\n    let liveSocket = new LiveSocket(\"/live\", Socket, {\n      params: {_csrf_token: csrfToken},\n      metadata: {\n        keydown: (e, el) => {\n          return {\n            key: e.key,\n            metaKey: e.metaKey,\n            repeat: e.repeat\n          }\n        }\n      }\n    })\n\nTo determine which key has been pressed you should use `key` value. The\navailable options can be found on\n[MDN](https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key/Key_Values)\nor via the [Key Event Viewer](https://w3c.github.io/uievents/tools/key-event-viewer.html).\n\n*Note*: `phx-keyup` and `phx-keydown` are not supported on inputs.\nInstead use form bindings, such as `phx-change`, `phx-submit`, etc.\n\n*Note*: it is possible for certain browser features like autofill to trigger key events\nwith no `\"key\"` field present in the value map sent to the server. For this reason, we\nrecommend always having a fallback catch-all event handler for LiveView key bindings.\nBy default, the bound element will be the event listener, but a\nwindow-level binding may be provided via `phx-window-keydown` or `phx-window-keyup`,\nfor example:\n\n    def render(assigns) do\n      ~H\"\"\"\n       \n        Current temperature: <%= @temperature %>\n       \n      \"\"\"\n    end\n\n    def handle_event(\"update_temp\", %{\"key\" => \"ArrowUp\"}, socket) do\n      {:ok, new_temp} = Thermostat.inc_temperature(socket.assigns.id)\n      {:noreply, assign(socket, :temperature, new_temp)}\n    end\n\n    def handle_event(\"update_temp\", %{\"key\" => \"ArrowDown\"}, socket) do\n      {:ok, new_temp} = Thermostat.dec_temperature(socket.assigns.id)\n      {:noreply, assign(socket, :temperature, new_temp)}\n    end\n\n    def handle_event(\"update_temp\", _, socket) do\n      {:noreply, socket}\n    end","ref":"bindings.html#key-events"},{"type":"extras","title":"Scroll Events and Infinite Stream pagination - Bindings","doc":"The `phx-viewport-top` and `phx-viewport-bottom` bindings allow you to detect when a container's\nfirst child reaches the top of the viewport, or the last child reaches the bottom of the viewport.\nThis is useful for infinite scrolling where you want to send paging events for the next results set or previous results set as the user is scrolling up and down and reaches the top or bottom of the viewport.\n\nGenerally, applications will add padding above and below a container when performing infinite scrolling to allow smooth scrolling as results are loaded. Combined with `Phoenix.LiveView.stream/3`, the `phx-viewport-top` and `phx-viewport-bottom` allow for infinite virtualized list that only keeps a small set of actual elements in the DOM. For example:\n\n    def mount(_, _, socket) do\n      {:ok,\n       socket\n       |> assign(page: 1, per_page: 20)\n       |> paginate_posts(1)}\n    end\n\n    defp paginate_posts(socket, new_page) when new_page >= 1 do\n      %{per_page: per_page, page: cur_page} = socket.assigns\n      posts = Blog.list_posts(offset: (new_page - 1) * per_page, limit: per_page)\n\n      {posts, at, limit} =\n        if new_page >= cur_page do\n          {posts, -1, per_page * 3 * -1}\n        else\n          {Enum.reverse(posts), 0, per_page * 3}\n        end\n\n      case posts do\n        [] ->\n          assign(socket, end_of_timeline?: at == -1)\n\n        [_ | _] = posts ->\n          socket\n          |> assign(end_of_timeline?: false)\n          |> assign(:page, new_page)\n          |> stream(:posts, posts, at: at, limit: limit)\n      end\n    end\n\nOur `paginate_posts` function fetches a page of posts, and determines if the user is paging to a previous page or next page. Based on the direction of paging, the stream is either prepended to, or appended to with `at` of `0` or `-1` respectively. We also set the `limit` of the stream to three times the `per_page` to allow enough posts in the UI to appear as an infinite list, but small enough to maintain UI performance. We also set an `@end_of_timeline?` assign to track whether the user is at the end of results or not. Finally, we update the `@page` assign and posts stream. We can then wire up our container to support the viewport events:\n\n```heex\n  1 && \"prev-page\"}\n  phx-viewport-bottom={!@end_of_timeline? && \"next-page\"}\n  phx-page-loading\n  class={[\n    if(@end_of_timeline?, do: \"pb-10\", else: \"pb-[calc(200vh)]\"),\n    if(@page == 1, do: \"pt-10\", else: \"pt-[calc(200vh)]\")\n  ]}\n>\n   \n    <.post_card post={post} />\n   \n \n \n  🎉 You made it to the beginning of time 🎉\n \n```\n\nThere's not much here, but that's the point! This little snippet of UI is driving a fully virtualized list with bidirectional infinite scrolling. We use the `phx-viewport-top` binding to send the `\"prev-page\"` event to the LiveView, but only if the user is beyond the first page. It doesn't make sense to load negative page results, so we remove the binding entirely in those cases. Next, we wire up `phx-viewport-bottom` to send the `\"next-page\"` event, but only if we've yet to reach the end of the timeline. Finally, we conditionally apply some css classes which sets a large top and bottom padding to twice the viewport height based on the current pagination for smooth scrolling.\n\nTo complete our solution, we only need to handle the `\"prev-page\"` and `\"next-page\"` events in the LiveView:\n\n    def handle_event(\"next-page\", _, socket) do\n      {:noreply, paginate_posts(socket, socket.assigns.page + 1)}\n    end\n\n    def handle_event(\"prev-page\", %{\"_overran\" => true}, socket) do\n      {:noreply, paginate_posts(socket, 1)}\n    end\n\n    def handle_event(\"prev-page\", _, socket) do\n      if socket.assigns.page > 1 do\n        {:noreply, paginate_posts(socket, socket.assigns.page - 1)}\n      else\n        {:noreply, socket}\n      end\n    end\n\nThis code simply calls the `paginate_posts` function we defined as our first step, using the current or next page to drive the results. Notice that we match on a special `\"_overran\" => true` parameter in our `\"prev-page\"` event. The viewport events send this parameter when the user has \"overran\" the viewport top or bottom. Imagine the case where the user is scrolling back up through many pages of results, but grabs the scrollbar and returns immediately to the top of the page. This means our ` ` container was overrun by the top of the viewport, and we need to reset the the UI to page the first page.","ref":"bindings.html#scroll-events-and-infinite-stream-pagination"},{"type":"extras","title":"Rate limiting events with Debounce and Throttle - Bindings","doc":"All events can be rate-limited on the client by using the\n`phx-debounce` and `phx-throttle` bindings, with the exception of the `phx-blur`\nbinding, which is fired immediately.\n\nRate limited and debounced events have the following behavior:\n\n  * `phx-debounce` - Accepts either an integer timeout value (in milliseconds),\n    or `\"blur\"`. When an integer is provided, emitting the event is delayed by\n    the specified milliseconds. When `\"blur\"` is provided, emitting the event is\n    delayed until the field is blurred by the user. When the value is omitted\n    a default of 300ms is used. Debouncing is typically used for input elements.\n\n  * `phx-throttle` - Accepts an integer timeout value to throttle the event in milliseconds.\n    Unlike debounce, throttle will immediately emit the event, then rate limit it at once\n    per provided timeout. When the value is omitted a default of 300ms is used.\n    Throttling is typically used to rate limit clicks, mouse and keyboard actions.\n\nFor example, to avoid validating an email until the field is blurred, while validating\nthe username at most every 2 seconds after a user changes the field:\n\n```heex\n \n   \n   \n \n```\n\nAnd to rate limit a volume up click to once every second:\n\n     + \n\nLikewise, you may throttle held-down keydown:\n\n```heex\n \n  ...\n \n```\n\nUnless held-down keys are required, a better approach is generally to use\n`phx-keyup` bindings which only trigger on key up, thereby being self-limiting.\nHowever, `phx-keydown` is useful for games and other use cases where a constant\npress on a key is desired. In such cases, throttle should always be used.\n\n#","ref":"bindings.html#rate-limiting-events-with-debounce-and-throttle"},{"type":"extras","title":"Debounce and Throttle special behavior - Bindings","doc":"The following specialized behavior is performed for forms and keydown bindings:\n\n  * When a `phx-submit`, or a `phx-change` for a different input is triggered,\n    any current debounce or throttle timers are reset for existing inputs.\n\n  * A `phx-keydown` binding is only throttled for key repeats. Unique keypresses\n    back-to-back will dispatch the pressed key events.","ref":"bindings.html#debounce-and-throttle-special-behavior"},{"type":"extras","title":"JS Commands - Bindings","doc":"LiveView bindings support a JavaScript command interface via the `Phoenix.LiveView.JS` module, which allows you to specify utility operations that execute on the client when firing `phx-` binding events, such as `phx-click`, `phx-change`, etc. Commands compose together to allow you to push events, add classes to elements, transition elements in and out, and more.\nSee the `Phoenix.LiveView.JS` documentation for full usage.\n\nFor a small example of what's possible, imagine you want to show and hide a modal on the page without needing to make the round trip to the server to render the content:\n\n```heex\n \n  My Modal\n \n\n \n  show modal\n \n\n \n  hide modal\n \n\n \n  toggle modal\n \n```\n\nOr if your UI library relies on classes to perform the showing or hiding:\n\n```heex\n \n  My Modal\n \n\n \n  show modal\n \n\n \n  hide modal\n \n```\n\nCommands compose together. For example, you can push an event to the server and\nimmediately hide the modal on the client:\n\n```heex\n \n  My Modal\n \n\n  JS.remove_class(\"show\", to: \"#modal\", transition: \"fade-out\")}>\n  hide modal\n \n```\n\nIt is also useful to extract commands into their own functions:\n\n```elixir\nalias Phoenix.LiveView.JS\n\ndef hide_modal(js \\\\ %JS{}, selector) do\n  js\n  |> JS.push(\"modal-closed\")\n  |> JS.remove_class(\"show\", to: selector, transition: \"fade-out\")\nend\n```\n\n```heex\n hide modal \n```\n\nThe `Phoenix.LiveView.JS.push/3` command is particularly powerful in allowing you to customize the event being pushed to the server. For example, imagine you start with a familiar `phx-click` which pushes a message to the server when clicked:\n\n```heex\n click \n```\n\nNow imagine you want to customize what happens when the `\"clicked\"` event is pushed, such as which component should be targeted, which element should receive css loading state classes, etc. This can be accomplished with options on the JS push command. For example:\n\n```heex\n click \n```\n\nSee `Phoenix.LiveView.JS.push/3` for all supported options.","ref":"bindings.html#js-commands"},{"type":"extras","title":"Lifecycle Events - Bindings","doc":"LiveView supports the `phx-mounted`, `phx-connected`, and `phx-disconnected` events to react to\ndifferent lifecycle events with JS commands.\n\nTo execute commands when an element first appears on the page, you can leverage `phx-mounted`,\nsuch as to animate a notice into view:\n\n```heex\n \n  Welcome back!\n \n```\n\nIf `phx-mounted` is used on the initial page render, it will be invoked only after the initial WebSocket connection is established.\n\nTo manage the connection lifecycle, you can combine `phx-disconnected` and `phx-connected` to show an element when the LiveView has lost its connection, and hide it when the connection recovers:\n\n```heex\n \n  Attempting to reconnect...\n \n```\n\n#","ref":"bindings.html#lifecycle-events"},{"type":"extras","title":"LiveView vs static view - Bindings","doc":"`phx-connected` and `phx-disconnected` are only executed when operating\ninside a LiveView container. For static templates, they will have no effect.\n\nFor LiveView, the `phx-mounted` binding is executed as soon as the LiveView is\nmounted with a connection. When using `phx-mounted` in static views, it is executed\nas soon as the DOM is ready.","ref":"bindings.html#liveview-vs-static-view"},{"type":"extras","title":"LiveView Specific Events - Bindings","doc":"The `lv:` event prefix supports LiveView specific features that are handled\nby LiveView without calling the user's `handle_event/3` callbacks. Today,\nthe following events are supported:\n\n  - `lv:clear-flash` – clears the flash when sent to the server. If a\n    `phx-value-key` is provided, the specific key will be removed from the flash.\n\nFor example:\n\n```heex\n \n  <%= live_flash(@flash, :info) %>\n \n```","ref":"bindings.html#liveview-specific-events"},{"type":"extras","title":"Loading states and errors - Bindings","doc":"All `phx-` event bindings apply their own css classes when pushed. For example\nthe following markup:\n\n```heex\n ... \n```\n\nOn click, would receive the `phx-click-loading` class, and on keydown would receive\nthe `phx-keydown-loading` class. The css loading classes are maintained until an\nacknowledgement is received on the client for the pushed event.\n\nIn the case of forms, when a `phx-change` is sent to the server, the input element\nwhich emitted the change receives the `phx-change-loading` class, along with the\nparent form tag. The following events receive css loading classes:\n\n  - `phx-click` - `phx-click-loading`\n  - `phx-change` - `phx-change-loading`\n  - `phx-submit` - `phx-submit-loading`\n  - `phx-focus` - `phx-focus-loading`\n  - `phx-blur` - `phx-blur-loading`\n  - `phx-window-keydown` - `phx-keydown-loading`\n  - `phx-window-keyup` - `phx-keyup-loading`\n\nAdditionally, the following classes are applied to the LiveView's parent\ncontainer:\n\n  - `\"phx-connected\"` - applied when the view has connected to the server\n  - `\"phx-loading\"` - applied when the view is not connected to the server\n  - `\"phx-error\"` - applied when an error occurs on the server. Note, this\n    class will be applied in conjunction with `\"phx-loading\"` if connection\n    to the server is lost.\n\nFor navigation related loading states (both automatic and manual), see `phx-page-loading` as described in\n[JavaScript interoperability: Live navigation events](js-interop.html#live-navigation-events).","ref":"bindings.html#loading-states-and-errors"},{"type":"extras","title":"DOM patching & temporary assigns","doc":"# DOM patching & temporary assigns\n\nA container can be marked with `phx-update`, allowing the DOM patch\noperations to avoid updating or removing portions of the LiveView, or to append\nor prepend the updates rather than replacing the existing contents. This\nis useful for client-side interop with existing libraries that do their\nown DOM operations. The following `phx-update` values are supported:\n\n  * `replace` - the default operation. Replaces the element with the contents\n  * `stream` - supports stream operations. Streams are used to manage large\n    collections in the UI without having to store the collection on the server\n  * `ignore` - ignores updates to the DOM regardless of new content changes\n\nWhen using `phx-update`, a unique DOM ID must always be set in the\ncontainer. If using \"stream\", a DOM ID must also be set\nfor each child. When inserting stream elements containing an\nID already present in the container, LiveView will replace the existing\nelement with the new content. See `Phoenix.LiveView.stream/3` for more\ninformation.\n\nThe \"ignore\" behaviour is frequently used when you need to integrate\nwith another JS library. Updates from the server to the element's content and attributes are ignored, *except for data attributes*. Changes, additions, and removals from the server to data attributes are merged with the ignored element which can be used to pass data to the JS handler.\n\nTo react to elements being mounted to the DOM, the `phx-mounted` binding\ncan be used. For example, to animate an element on mount:\n\n     \n\nIf `phx-mounted` is used on the initial page render, it will be invoked only\nafter the initial WebSocket connection is established.\n\nTo react to elements being removed from the DOM, the `phx-remove` binding\nmay be specified, which can contain a `Phoenix.LiveView.JS` command to execute.\n\n*Note*: The `phx-remove` command is only executed for the removed parent element.\nIt does not cascade to children.","ref":"dom-patching.html"},{"type":"extras","title":"Temporary assigns - DOM patching & temporary assigns","doc":"By default, all LiveView assigns are stateful, which enables change\ntracking and stateful interactions. In some cases, it's useful to mark\nassigns as temporary, meaning they will be reset to a default value after\neach update. This allows otherwise large but infrequently updated values\nto be discarded after the client has been patched.\n\nImagine you want to implement a chat application with LiveView. You\ncould render each message like this:\n\n```heex\n<%= for message <- @messages do %>\n    <%= message.username %>:  <%= message.text %> \n<% end %>\n```\n\nEvery time there is a new message, you would append it to the `@messages`\nassign and re-render all messages.\n\nAs you may suspect, keeping the whole chat conversation in memory\nand resending it on every update would be too expensive, even with\nLiveView smart change tracking. By using temporary assigns and phx-update,\nwe don't need to keep any messages in memory, and send messages to be\nappended to the UI only when there are new ones.\n\nTo do so, the first step is to mark which assigns are temporary and\nwhat values they should be reset to on mount:\n\n    def mount(_params, _session, socket) do\n      socket = assign(socket, :messages, load_last_20_messages())\n      {:ok, socket, temporary_assigns: [messages: []]}\n    end\n\nOn mount we also load the initial number of messages we want to\nsend. After the initial render, the initial batch of messages will\nbe reset back to an empty list.\n\nNow, whenever there are one or more new messages, we will assign\nonly the new messages to `@messages`:\n\n    socket = assign(socket, :messages, new_messages)\n\nIn the template, we want to wrap all of the messages in a container\nand tag this content with `phx-update`. Remember, we must add an ID\nto the container as well as to each child:\n\n```heex\n \n  <%= for message <- @messages do %>\n     \n       <%= message.username %>:  <%= message.text %>\n     \n  <% end %>\n \n```\n\nWhen the client receives new messages, it now knows to append to the\nold content rather than replace it.\n\nYou can also update the direction of messages. Suppose there is an edit to a message\nthat is being sent to your LiveView like this:\n\n    def handle_info({:update_message, message}, socket) do\n      {:noreply, update(socket, :messages, fn messages -> [message | messages] end)}\n    end\n\nYou can add it to the list like you do with new messages. LiveView is aware that this\nmessage was rendered on the client, even though the message itself is discarded on the\nserver after it is rendered.\n\nLiveView uses DOM ids to check if a message is rendered before or not. If an id is\nrendered before, the DOM element is updated rather than appending or prepending a new node.\nAlso, the order of elements is not changed. You can use it to show edited messages, show likes, or\nanything that would require an update to a rendered message.","ref":"dom-patching.html#temporary-assigns"},{"type":"extras","title":"Form bindings","doc":"# Form bindings","ref":"form-bindings.html"},{"type":"extras","title":"Form Events - Form bindings","doc":"To handle form changes and submissions, use the `phx-change` and `phx-submit`\nevents. In general, it is preferred to handle input changes at the form level,\nwhere all form fields are passed to the LiveView's callback given any\nsingle input change. For example, to handle real-time form validation and\nsaving, your form would use both `phx-change` and `phx-submit` bindings.\nLet's get started with an example:\n\n```heex\n<.form for={@form} phx-change=\"validate\" phx-submit=\"save\">\n  <.input type=\"text\" field={@form[:username]} />\n  <.input type=\"email\" field={@form[:email]} />\n   Save \n</.form>\n```\n\n`.form` is the function component defined in `Phoenix.Component.form/1`,\nwe recommend reading its documentation for more details on how it works\nand all supported options. `.form` expects a `@form` assign, which can\nbe created from a changeset or user parameters via `Phoenix.Component.to_form/1`.\n\n`input/1` is a function component for rendering inputs, most often\ndefined in your own application, often encapsulating labelling,\nerror handling, and more. Here is a simple version to get started with:\n\n    attr :field, Phoenix.HTML.FormField\n    attr :rest, :global, include: ~w(type)\n    def input(assigns) do\n      ~H\"\"\"\n       \n      \"\"\"\n    end\n\n> #","ref":"form-bindings.html#form-events"},{"type":"extras","title":"The `CoreComponents` module {: .info} - Form bindings","doc":">\n> If your application was generated with Phoenix v1.7, then `mix phx.new`\n> automatically imports many ready-to-use function components, such as\n> `.input` component with built-in features and styles.\n\nWith the form rendered, your LiveView picks up the events in `handle_event`\ncallbacks, to validate and attempt to save the parameter accordingly:\n\n    def render(assigns) ...\n\n    def mount(_params, _session, socket) do\n      {:ok, assign(socket, form: to_form(Accounts.change_user(%User{})))}\n    end\n\n    def handle_event(\"validate\", %{\"user\" => params}, socket) do\n      form =\n        %User{}\n        |> Accounts.change_user(params)\n        |> Map.put(:action, :insert)\n        |> to_form()\n\n      {:noreply, assign(socket, form: form)}\n    end\n\n    def handle_event(\"save\", %{\"user\" => user_params}, socket) do\n      case Accounts.create_user(user_params) do\n        {:ok, user} ->\n          {:noreply,\n           socket\n           |> put_flash(:info, \"user created\")\n           |> redirect(to: ~p\"/users/#{user}\")}\n\n        {:error, %Ecto.Changeset{} = changeset} ->\n          {:noreply, assign(socket, form: to_form(changeset))}\n      end\n    end\n\nThe validate callback simply updates the changeset based on all form input\nvalues, then convert the changeset to a form and assign it to the socket.\nIf the form changes, such as generating new errors, [`render/1`](`c:Phoenix.LiveView.render/1`)\nis invoked and the form is re-rendered.\n\nLikewise for `phx-submit` bindings, the same callback is invoked and\npersistence is attempted. On success, a `:noreply` tuple is returned and the\nsocket is annotated for redirect with `Phoenix.LiveView.redirect/2` to\nthe new user page, otherwise the socket assigns are updated with the errored\nchangeset to be re-rendered for the client.\n\nYou may wish for an individual input to use its own change event or to target\na different component. This can be accomplished by annotating the input itself\nwith `phx-change`, for example:\n\n```\n<.form for={@form} phx-change=\"validate\" phx-submit=\"save\">\n  ...\n  <.input field={@form[:email]}  phx-change=\"email_changed\" phx-target={@myself} />\n</.form>\n```\n\nThen your LiveView or LiveComponent would handle the event:\n\n```elixir\ndef handle_event(\"email_changed\", %{\"user\" => %{\"email\" => email}}, socket) do\n  ...\nend\n```\n\n_Note_: only the individual input is sent as params for an input marked with `phx-change`.","ref":"form-bindings.html#the-corecomponents-module-info"},{"type":"extras","title":"Error Feedback - Form bindings","doc":"For proper error feedback on form updates, the error tags must specify which\ninput they belong to. This is accomplished with the `phx-feedback-for` and `phx-feedback-group` attributes.\n\nThe `phx-feedback-for` annotation specifies the name (or id, for backwards compatibility) of the input it belongs to. Failing to add the `phx-feedback-for` attribute will result in displaying error messages for form fields that the user has not changed yet (e.g. required\nfields further down on the page).\n\nFor example, your `MyAppWeb.CoreComponents` may use this function:\n\n    def input(assigns) do\n      ~H\"\"\"\n       \n         \n        <.error :for={msg <- @errors}><%= msg %></.error>\n       \n      \"\"\"\n    end\n\n    def error(assigns) do\n      ~H\"\"\"\n       \n         \n        <%= render_slot(@inner_block) %>\n       \n      \"\"\"\n    end\n\nNow, any DOM container with the `phx-feedback-for` attribute will receive a\n`phx-no-feedback` class in cases where the form fields has yet to receive\nuser input/focus. Using new CSS rules or tailwindcss variants allows you\nerrors to be shown, hidden, and styled as feedback changes.\n\nLikewise, the `phx-feedback-group` attribute behaves similarly to `phx-feedback-for`, but allows error tags and inputs to specify a composite name for a group of inputs.","ref":"form-bindings.html#error-feedback"},{"type":"extras","title":"Number inputs - Form bindings","doc":"Number inputs are a special case in LiveView forms. On programmatic updates,\nsome browsers will clear invalid inputs. So LiveView will not send change events\nfrom the client when an input is invalid, instead allowing the browser's native\nvalidation UI to drive user interaction. Once the input becomes valid, change and\nsubmit events will be sent normally.\n\n```heex\n \n```\n\nThis is known to have a plethora of problems including accessibility, large numbers\nare converted to exponential notation, and scrolling can accidentally increase or\ndecrease the number.\n\nOne alternative is the `inputmode` attribute, which may serve your application's needs\nand users much better. According to [Can I Use?](https://caniuse.com/#search=inputmode),\nthe following is supported by 86% of the global market (as of Sep 2021):\n\n```heex\n \n```","ref":"form-bindings.html#number-inputs"},{"type":"extras","title":"Password inputs - Form bindings","doc":"Password inputs are also special cased in `Phoenix.HTML`. For security reasons,\npassword field values are not reused when rendering a password input tag. This\nrequires explicitly setting the `:value` in your markup, for example:\n\n```heex\n<.input field={f[:password]} value={input_value(f[:password].value)} />\n<.input field={f[:password_confirmation]} value={input_value(f[:password_confirmation].value)} />\n```","ref":"form-bindings.html#password-inputs"},{"type":"extras","title":"Nested inputs - Form bindings","doc":"Nested inputs are handled using `.inputs_for` function component. By default\nit will add the necessary hidden input fields for tracking ids of Ecto associations.\n\n```heex\n<.inputs_for :let={fp} field={f[:friends]}>\n  <.input field={fp[:name]} type=\"text\">\n</.inputs_for>\n```","ref":"form-bindings.html#nested-inputs"},{"type":"extras","title":"File inputs - Form bindings","doc":"LiveView forms support [reactive file inputs](uploads.md),\nincluding drag and drop support via the `phx-drop-target`\nattribute:\n\n```heex\n \n  ...\n  <.live_file_input upload={@uploads.avatar} />\n \n```\n\nSee `Phoenix.Component.live_file_input/1` for more.","ref":"form-bindings.html#file-inputs"},{"type":"extras","title":"Submitting the form action over HTTP - Form bindings","doc":"The `phx-trigger-action` attribute can be added to a form to trigger a standard\nform submit on DOM patch to the URL specified in the form's standard `action`\nattribute. This is useful to perform pre-final validation of a LiveView form\nsubmit before posting to a controller route for operations that require\nPlug session mutation. For example, in your LiveView template you can\nannotate the `phx-trigger-action` with a boolean assign:\n\n```heex\n<.form :let={f} for={@changeset}\n  action={Routes.reset_password_path(@socket, :create)}\n  phx-submit=\"save\"\n  phx-trigger-action={@trigger_submit}>\n```\n\nThen in your LiveView, you can toggle the assign to trigger the form with the current\nfields on next render:\n\n    def handle_event(\"save\", params, socket) do\n      case validate_change_password(socket.assigns.user, params) do\n        {:ok, changeset} ->\n          {:noreply, assign(socket, changeset: changeset, trigger_submit: true)}\n\n        {:error, changeset} ->\n          {:noreply, assign(socket, changeset: changeset)}\n      end\n    end\n\nOnce `phx-trigger-action` is true, LiveView disconnects and then submits the form.","ref":"form-bindings.html#submitting-the-form-action-over-http"},{"type":"extras","title":"Recovery following crashes or disconnects - Form bindings","doc":"By default, all forms marked with `phx-change` and having `id`\nattribute will recover input values automatically after the user has\nreconnected or the LiveView has remounted after a crash. This is\nachieved by the client triggering the same `phx-change` to the server\nas soon as the mount has been completed.\n\n**Note:** if you want to see form recovery working in development, please\nmake sure to disable live reloading in development by commenting out the\nLiveReload plug in your `endpoint.ex` file or by setting `code_reloader: false`\nin your `config/dev.exs`. Otherwise live reloading may cause the current page\nto be reloaded whenever you restart the server, which will discard all form\nstate.\n\nFor most use cases, this is all you need and form recovery will happen\nwithout consideration. In some cases, where forms are built step-by-step in a\nstateful fashion, it may require extra recovery handling on the server outside\nof your existing `phx-change` callback code. To enable specialized recovery,\nprovide a `phx-auto-recover` binding on the form to specify a different event\nto trigger for recovery, which will receive the form params as usual. For example,\nimagine a LiveView wizard form where the form is stateful and built based on what\nstep the user is on and by prior selections:\n\n     \n\nOn the server, the `\"validate_wizard_step\"` event is only concerned with the\ncurrent client form data, but the server maintains the entire state of the wizard.\nTo recover in this scenario, you can specify a recovery event, such as `\"recover_wizard\"`\nabove, which would wire up to the following server callbacks in your LiveView:\n\n    def handle_event(\"validate_wizard_step\", params, socket) do\n      # regular validations for current step\n      {:noreply, socket}\n    end\n\n    def handle_event(\"recover_wizard\", params, socket) do\n      # rebuild state based on client input data up to the current step\n      {:noreply, socket}\n    end\n\nTo forgo automatic form recovery, set `phx-auto-recover=\"ignore\"`.","ref":"form-bindings.html#recovery-following-crashes-or-disconnects"},{"type":"extras","title":"Resetting Forms - Form bindings","doc":"To reset a LiveView form, you can use the standard `type=\"reset\"` on a\nform button or input. When clicked, the form inputs will be reset to their\noriginal values, and Phoenix will hide errors for `phx-feedback-for` elements.\nAfter the form is reset, a `phx-change` event is emitted with the `_target` param\ncontaining the reset `name`. For example, the following element:\n\n     \n      ...\n       Reset \n     \n\nCan be handled on the server differently from your regular change function:\n\n    def handle_event(\"changed\", %{\"_target\" => [\"reset\"]} = params, socket) do\n      # handle form reset\n    end\n\n    def handle_event(\"changed\", params, socket) do\n      # handle regular form change\n    end","ref":"form-bindings.html#resetting-forms"},{"type":"extras","title":"JavaScript client specifics - Form bindings","doc":"The JavaScript client is always the source of truth for current input values.\nFor any given input with focus, LiveView will never overwrite the input's current\nvalue, even if it deviates from the server's rendered updates. This works well\nfor updates where major side effects are not expected, such as form validation\nerrors, or additive UX around the user's input values as they fill out a form.\n\nFor these use cases, the `phx-change` input does not concern itself with disabling\ninput editing while an event to the server is in flight. When a `phx-change` event\nis sent to the server, the input tag and parent form tag receive the\n`phx-change-loading` CSS class, then the payload is pushed to the server with a\n`\"_target\"` param in the root payload containing the keyspace of the input name\nwhich triggered the change event.\n\nFor example, if the following input triggered a change event:\n\n```heex\n \n```\n\nThe server's `handle_event/3` would receive a payload:\n\n    %{\"_target\" => [\"user\", \"username\"], \"user\" => %{\"username\" => \"Name\"}}\n\nThe `phx-submit` event is used for form submissions where major side effects\ntypically happen, such as rendering new containers, calling an external\nservice, or redirecting to a new page.\n\nOn submission of a form bound with a `phx-submit` event:\n\n1. The form's inputs are set to `readonly`\n2. Any submit button on the form is disabled\n3. The form receives the `\"phx-submit-loading\"` class\n\nOn completion of server processing of the `phx-submit` event:\n\n1. The submitted form is reactivated and loses the `\"phx-submit-loading\"` class\n2. The last input with focus is restored (unless another input has received focus)\n3. Updates are patched to the DOM as usual\n\nTo handle latent events, the ` ` tag of a form can be annotated with\n`phx-disable-with`, which swaps the element's `innerText` with the provided\nvalue during event submission. For example, the following code would change\nthe \"Save\" button to \"Saving...\", and restore it to \"Save\" on acknowledgment:\n\n```heex\n Save \n```\n\nYou may also take advantage of LiveView's CSS loading state classes to\nswap out your form content while the form is submitting. For example,\nwith the following rules in your `app.css`:\n\n    .while-submitting { display: none; }\n    .inputs { display: block; }\n\n    .phx-submit-loading .while-submitting { display: block; }\n    .phx-submit-loading .inputs { display: none; }\n\nYou can show and hide content with the following markup:\n\n```heex\n \n   Please wait while we save our content... \n   \n     \n   \n \n```\n\nAdditionally, we strongly recommend including a unique HTML \"id\" attribute on the form.\nWhen DOM siblings change, elements without an ID will be replaced rather than moved,\nwhich can cause issues such as form fields losing focus.","ref":"form-bindings.html#javascript-client-specifics"},{"type":"extras","title":"Triggering `phx-` form events with JavaScript - Form bindings","doc":"Often it is desirable to trigger an event on a DOM element without explicit\nuser interaction on the element. For example, a custom form element such as a\ndate picker or custom select input which utilizes a hidden input element to\nstore the selected state.\n\nIn these cases, the event functions on the DOM API can be used, for example\nto trigger a `phx-change` event:\n\n```\ndocument.getElementById(\"my-select\").dispatchEvent(\n  new Event(\"input\", {bubbles: true})\n)\n```\n\nWhen using a client hook, `this.el` can be used to determine the element as\noutlined in the \"Client hooks\" documentation.\n\nIt is also possible to trigger a `phx-submit` using a \"submit\" event:\n\n```\ndocument.getElementById(\"my-form\").dispatchEvent(\n  new Event(\"submit\", {bubbles: true, cancelable: true})\n)\n```","ref":"form-bindings.html#triggering-phx-form-events-with-javascript"},{"type":"extras","title":"JavaScript interoperability","doc":"# JavaScript interoperability\n\nTo enable LiveView client/server interaction, we instantiate a LiveSocket. For example:\n\n```javascript\nimport {Socket} from \"phoenix\"\nimport {LiveSocket} from \"phoenix_live_view\"\n\nlet csrfToken = document.querySelector(\"meta[name='csrf-token']\").getAttribute(\"content\")\nlet liveSocket = new LiveSocket(\"/live\", Socket, {params: {_csrf_token: csrfToken}})\nliveSocket.connect()\n```\n\nAll options are passed directly to the `Phoenix.Socket` constructor,\nexcept for the following LiveView specific options:\n\n  * `bindingPrefix` - the prefix to use for phoenix bindings. Defaults `\"phx-\"`\n  * `params` - the `connect_params` to pass to the view's mount callback. May be\n    a literal object or closure returning an object. When a closure is provided,\n    the function receives the view's element.\n  * `hooks` – a reference to a user-defined hooks namespace, containing client\n    callbacks for server/client interop. See the [Client hooks](#client-hooks-via-phx-hook)\n    section below for details.\n  * `uploaders` – a reference to a user-defined uploaders namespace, containing\n    client callbacks for client-side direct-to-cloud uploads. See the\n    [External Uploads guide](uploads-external.md) for details.","ref":"js-interop.html"},{"type":"extras","title":"Debugging Client Events - JavaScript interoperability","doc":"To aid debugging on the client when troubleshooting issues, the `enableDebug()`\nand `disableDebug()` functions are exposed on the `LiveSocket` JavaScript instance.\nCalling `enableDebug()` turns on debug logging which includes LiveView life-cycle and\npayload events as they come and go from client to server. In practice, you can expose\nyour instance on `window` for quick access in the browser's web console, for example:\n\n```\n// app.js\nlet liveSocket = new LiveSocket(...)\nliveSocket.connect()\nwindow.liveSocket = liveSocket\n\n// in the browser's web console\n>> liveSocket.enableDebug()\n```\n\nThe debug state uses the browser's built-in `sessionStorage`, so it will remain in effect\nfor as long as your browser session lasts.","ref":"js-interop.html#debugging-client-events"},{"type":"extras","title":"Simulating Latency - JavaScript interoperability","doc":"Proper handling of latency is critical for good UX. LiveView's CSS loading states allow\nthe client to provide user feedback while awaiting a server response. In development,\nnear zero latency on localhost does not allow latency to be easily represented or tested,\nso LiveView includes a latency simulator with the JavaScript client to ensure your\napplication provides a pleasant experience. Like the `enableDebug()` function above,\nthe `LiveSocket` instance includes `enableLatencySim(milliseconds)` and `disableLatencySim()`\nfunctions which apply throughout the current browser session. The `enableLatencySim` function\naccepts an integer in milliseconds for the round-trip-time to the server. For example:\n\n```javascript\n// app.js\nlet liveSocket = new LiveSocket(...)\nliveSocket.connect()\nwindow.liveSocket = liveSocket\n\n// in the browser's web console\n>> liveSocket.enableLatencySim(1000)\n[Log] latency simulator enabled for the duration of this browser session.\n      Call disableLatencySim() to disable\n```","ref":"js-interop.html#simulating-latency"},{"type":"extras","title":"Event listeners - JavaScript interoperability","doc":"LiveView emits several events to the browsers and allows developers to submit\ntheir own events too.\n\n#","ref":"js-interop.html#event-listeners"},{"type":"extras","title":"Live navigation events - JavaScript interoperability","doc":"For live page navigation via `<.link navigate={...}>` and `<.link patch={...}>`,\ntheir server-side equivalents `push_redirect` and `push_patch`, as well as form\nsubmits via `phx-submit`, the JavaScript events `\"phx:page-loading-start\"` and\n`\"phx:page-loading-stop\"` are dispatched on window. Additionally, any `phx-`\nevent may dispatch page loading events by annotating the DOM element with\n`phx-page-loading`. This is useful for showing main page loading status, for example:\n\n```\n// app.js\nimport topbar from \"topbar\"\nwindow.addEventListener(\"phx:page-loading-start\", info => topbar.show())\nwindow.addEventListener(\"phx:page-loading-stop\", info => topbar.hide())\n```\n\nWithin the callback, `info.detail` will be an object that contains a `kind`\nkey, with a value that depends on the triggering event:\n\n  - `\"redirect\"` - the event was triggered by a redirect\n  - `\"patch\"` - the event was triggered by a patch\n  - `\"initial\"` - the event was triggered by initial page load\n  - `\"element\"` - the event was triggered by a `phx-` bound element, such as `phx-click`\n\nFor all kinds of page loading events, all but `\"element\"` will receive an additional `to`\nkey in the info metadata pointing to the href associated with the page load.\n\nIn the case of an `\"element\"` page loading event, the info will contain a\n`\"target\"` key containing the DOM element which triggered the page loading\nstate.\n\nA lower level `phx:navigate` event is also triggered any time the browser's URL bar\nis programmatically changed by Phoenix or the user navigation forward or back. The\n`info.detail` will contain the following information:\n\n  - `\"href\"` - the location the URL bar was navigated to.\n  - `\"patch\"` - the boolean flag indicating this was a patch navigation.\n  - `\"pop\"` - the boolean flag indication this was a navigation via `popstate`\n    from a user navigation forward or back in history.\n\n#","ref":"js-interop.html#live-navigation-events"},{"type":"extras","title":"Handling server-pushed events - JavaScript interoperability","doc":"When the server uses `Phoenix.LiveView.push_event/3`, the event name\nwill be dispatched in the browser with the `phx:` prefix. For example,\nimagine the following template where you want to highlight an existing\nelement from the server to draw the user's attention:\n\n```heex\n \n  <%= item.title %>\n \n```\n\nNext, the server can issue a highlight using the standard `push_event`:\n\n```elixir\ndef handle_info({:item_updated, item}, socket) do\n  {:noreply, push_event(socket, \"highlight\", %{id: \"item-#{item.id}\"})}\nend\n```\n\nFinally, a window event listener can listen for the event and conditionally\nexecute the highlight command if the element matches:\n\n```javascript\nlet liveSocket = new LiveSocket(...)\nwindow.addEventListener(\"phx:highlight\", (e) => {\n  let el = document.getElementById(e.detail.id)\n  if(el) {\n    // logic for highlighting\n  }\n})\n```\n\nIf you desire, you can also integrate this functionality with Phoenix'\nJS commands, executing JS commands for the given element whenever highlight\nis triggered. First, update the element to embed the JS command into a data\nattribute:\n\n```heex\n \n  <%= item.title %>\n \n```\n\nNow, in the event listener, use `LiveSocket.execJS` to trigger all JS\ncommands in the new attribute:\n\n```javascript\nlet liveSocket = new LiveSocket(...)\nwindow.addEventListener(\"phx:highlight\", (e) => {\n  document.querySelectorAll(`[data-highlight]`).forEach(el => {\n    if(el.id == e.detail.id){\n      liveSocket.execJS(el, el.getAttribute(\"data-highlight\"))\n    }\n  })\n})\n```","ref":"js-interop.html#handling-server-pushed-events"},{"type":"extras","title":"Client hooks via `phx-hook` - JavaScript interoperability","doc":"To handle custom client-side JavaScript when an element is added, updated,\nor removed by the server, a hook object may be provided via `phx-hook`.\n`phx-hook` must point to an object with the following life-cycle callbacks:\n\n  * `mounted` - the element has been added to the DOM and its server\n    LiveView has finished mounting\n  * `beforeUpdate` - the element is about to be updated in the DOM.\n    *Note*: any call here must be synchronous as the operation cannot\n    be deferred or cancelled.\n  * `updated` - the element has been updated in the DOM by the server\n  * `destroyed` - the element has been removed from the page, either\n    by a parent update, or by the parent being removed entirely\n  * `disconnected` - the element's parent LiveView has disconnected from the server\n  * `reconnected` - the element's parent LiveView has reconnected to the server\n\n*Note:* When using hooks outside the context of a LiveView, `mounted` is the only\ncallback invoked, and only those elements on the page at DOM ready will be tracked.\nFor dynamic tracking of the DOM as elements are added, removed, and updated, a LiveView\nshould be used.\n\nThe above life-cycle callbacks have in-scope access to the following attributes:\n\n  * `el` - attribute referencing the bound DOM node\n  * `liveSocket` - the reference to the underlying `LiveSocket` instance\n  * `pushEvent(event, payload, (reply, ref) => ...)` - method to push an event from the client to the LiveView server\n  * `pushEventTo(selectorOrTarget, event, payload, (reply, ref) => ...)` - method to push targeted events from the client\n    to LiveViews and LiveComponents. It sends the event to the LiveComponent or LiveView the `selectorOrTarget` is\n    defined in, where its value can be either a query selector or an actual DOM element. If the query selector returns\n    more than one element it will send the event to all of them, even if all the elements are in the same LiveComponent\n    or LiveView.\n  * `handleEvent(event, (payload) => ...)` - method to handle an event pushed from the server\n  * `upload(name, files)` - method to inject a list of file-like objects into an uploader.\n  * `uploadTo(selectorOrTarget, name, files)` - method to inject a list of file-like objects into an uploader.\n    The hook will send the files to the uploader with `name` defined by [`allow_upload/3`](`Phoenix.LiveView.allow_upload/3`)\n    on the server-side. Dispatching new uploads triggers an input change event which will be sent to the\n    LiveComponent or LiveView the `selectorOrTarget` is defined in, where its value can be either a query selector or an\n    actual DOM element. If the query selector returns more than one live file input, an error will be logged.\n\nFor example, the markup for a controlled input for phone-number formatting could be written\nlike this:\n\n```heex\n \n```\n\nThen a hook callback object could be defined and passed to the socket:\n\n```javascript\nlet Hooks = {}\nHooks.PhoneNumber = {\n  mounted() {\n    this.el.addEventListener(\"input\", e => {\n      let match = this.el.value.replace(/\\D/g, \"\").match(/^(\\d{3})(\\d{3})(\\d{4})$/)\n      if(match) {\n        this.el.value = `${match[1]}-${match[2]}-${match[3]}`\n      }\n    })\n  }\n}\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {hooks: Hooks, ...})\n...\n```\n\n*Note*: when using `phx-hook`, a unique DOM ID must always be set.\n\nFor integration with client-side libraries which require a broader access to full\nDOM management, the `LiveSocket` constructor accepts a `dom` option with an\n`onBeforeElUpdated` callback. The `fromEl` and `toEl` DOM nodes are passed to the\nfunction just before the DOM patch operations occurs in LiveView. This allows external\nlibraries to (re)initialize DOM elements or copy attributes as necessary as LiveView\nperforms its own patch operations. The update operation cannot be cancelled or deferred,\nand the return value is ignored.\n\nFor example, the following option could be used to guarantee that some attributes set on the client-side are kept intact:\n\n```javascript\nonBeforeElUpdated(from, to){\n  for (const attr of from.attributes){\n    if (attr.name.startsWith(\"data-js-\")){\n      to.setAttribute(attr.name, attr.value);\n    }\n  }\n}\n```\n\nIn the example above, all attributes starting with `data-js-` won't be replaced when the DOM is patched by LiveView.\n\n#","ref":"js-interop.html#client-hooks-via-phx-hook"},{"type":"extras","title":"Client-server communication - JavaScript interoperability","doc":"A hook can push events to the LiveView by using the `pushEvent` function and receive a\nreply from the server via a `{:reply, map, socket}` return value. The reply payload will be\npassed to the optional `pushEvent` response callback.\n\nCommunication with the hook from the server can be done by reading data attributes on the\nhook element or by using `Phoenix.LiveView.push_event/3` on the server and `handleEvent` on the client.\n\nFor example, to implement infinite scrolling, one can pass the current page using data attributes:\n\n```heex\n \n```\n\nAnd then in the client:\n\n```javascript\nHooks.InfiniteScroll = {\n  page() { return this.el.dataset.page },\n  mounted(){\n    this.pending = this.page()\n    window.addEventListener(\"scroll\", e => {\n      if(this.pending == this.page() && scrollAt() > 90){\n        this.pending = this.page() + 1\n        this.pushEvent(\"load-more\", {})\n      }\n    })\n  },\n  updated(){ this.pending = this.page() }\n}\n```\n\nHowever, the data attribute approach is not a good approach if you need to frequently push data to the client. To push out-of-band events to the client, for example to render charting points, one could do:\n\n     \n    {:noreply, push_event(socket, \"points\", %{points: new_points})}\n\nAnd then on the client:\n\n```javascript\nHooks.Chart = {\n  mounted(){\n    this.handleEvent(\"points\", ({points}) => MyChartLib.addPoints(points))\n  }\n}\n```\n\n*Note*: remember events pushed from the server via `push_event` are global and will be dispatched\nto all active hooks on the client who are handling that event.\n\n*Note*: In case a LiveView pushes events and renders content, `handleEvent` callbacks are invoked after the page is updated. Therefore, if the LiveView redirects at the same time it pushes events, callbacks won't be invoked on the old page's elements. Callbacks would be invoked on the redirected page's newly mounted hook elements.","ref":"js-interop.html#client-server-communication"},{"type":"extras","title":"External Uploads","doc":"# External Uploads\n\n> This guide continues from the configuration started in the\n> server [Uploads guide](uploads.html).\n\nUploads to external cloud providers, such as Amazon S3,\nGoogle Cloud, etc., can be achieved by using the\n`:external` option in [`allow_upload/3`](`Phoenix.LiveView.allow_upload/3`).\n\nYou provide a 2-arity function to allow the server to\ngenerate metadata for each upload entry, which is passed to\na user-specified JavaScript function on the client.\n\nTypically when your function is invoked, you will generate a\npre-signed URL, specific to your cloud storage provider, that\nwill provide temporary access for the end-user to upload data\ndirectly to your cloud storage.","ref":"uploads-external.html"},{"type":"extras","title":"Chunked HTTP Uploads - External Uploads","doc":"For any service that supports large file\nuploads via chunked HTTP requests with `Content-Range`\nheaders, you can use the UpChunk JS library by Mux to do all\nthe hard work of uploading the file. For small file uploads\nor to get started quickly, consider [uploading directly to S3](#direct-to-s3)\ninstead.\n\nYou only need to wire the UpChunk instance to the LiveView\nUploadEntry callbacks, and LiveView will take care of the rest.\n\nInstall [UpChunk](https://github.com/muxinc/upchunk) by\nsaving [its contents](https://unpkg.com/@mux/upchunk@2)\nto `assets/vendor/upchunk.js` or by installing it with `npm`:\n\n```shell\n$ npm install --prefix assets --save @mux/upchunk\n```\n\nConfigure your uploader on `c:Phoenix.LiveView.mount/3`:\n\n    def mount(_params, _session, socket) do\n      {:ok,\n       socket\n       |> assign(:uploaded_files, [])\n       |> allow_upload(:avatar, accept: :any, max_entries: 3, external: &presign_upload/2)}\n    end\n\nSupply the `:external` option to\n`Phoenix.LiveView.allow_upload/3`. It requires a 2-arity\nfunction that generates a signed URL where the client will\npush the bytes for the upload entry. This function must\nreturn either `{:ok, meta, socket}` or `{:error, meta, socket}`,\nwhere `meta` must me a map.\n\nFor example, if you were using a context that provided a\n[`start_session`](https://developers.google.com/youtube/v3/guides/using_resumable_upload_protocol##Start_Resumable_Session)\nfunction, you might write something like this:\n\n    defp presign_upload(entry, socket) do\n      {:ok, %{\"Location\" => link}} =\n        SomeTube.start_session(%{\n          \"uploadType\" => \"resumable\",\n          \"x-upload-content-length\" => entry.client_size\n        })\n\n      {:ok, %{uploader: \"UpChunk\", entrypoint: link}, socket}\n    end\n\nFinally, on the client-side, we use UpChunk to create an\nupload from the temporary URL generated on the server and\nattach listeners for its events to the entry's callbacks:\n\n```js\nimport * as UpChunk from \"@mux/upchunk\"\n\nlet Uploaders = {}\n\nUploaders.UpChunk = function(entries, onViewError){\n  entries.forEach(entry => {\n    // create the upload session with UpChunk\n    let { file, meta: { entrypoint } } = entry\n    let upload = UpChunk.createUpload({ endpoint: entrypoint, file })\n\n    // stop uploading in the event of a view error\n    onViewError(() => upload.pause())\n\n    // upload error triggers LiveView error\n    upload.on(\"error\", (e) => entry.error(e.detail.message))\n\n    // notify progress events to LiveView\n    upload.on(\"progress\", (e) => {\n      if(e.detail < 100){ entry.progress(e.detail) }\n    })\n\n    // success completes the UploadEntry\n    upload.on(\"success\", () => entry.progress(100))\n  })\n}\n\n// Don't forget to assign Uploaders to the liveSocket\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  uploaders: Uploaders,\n  params: {_csrf_token: csrfToken}\n})\n```","ref":"uploads-external.html#chunked-http-uploads"},{"type":"extras","title":"Direct to S3 - External Uploads","doc":"The largest object that can be uploaded to S3 in a single PUT is 5 GB\naccording to [S3 FAQ](https://aws.amazon.com/s3/faqs/). For larger file\nuploads, consider using chunking as shown above.\n\nThis guide assumes an existing S3 bucket is set up with the correct CORS configuration\nwhich allows uploading directly to the bucket.\n\nAn example CORS config is:\n\n```js\n[\n    {\n        \"AllowedHeaders\": [ \"*\" ],\n        \"AllowedMethods\": [ \"PUT\", \"POST\" ],\n        \"AllowedOrigins\": [ \"*\" ],\n        \"ExposeHeaders\": []\n    }\n]\n```\n\nYou may put your domain in the \"allowedOrigins\" instead. More information on configuring CORS for\nS3 buckets is [available on AWS](https://docs.aws.amazon.com/AmazonS3/latest/userguide/ManageCorsUsing.html).\n\nIn order to enforce all of your file constraints when uploading to S3,\nit is necessary to perform a multipart form POST with your file data.\nYou should have the following S3 information ready before proceeding:\n\n1. aws_access_key_id\n2. aws_secret_access_key\n3. bucket_name\n4. region\n\nWe will first implement the LiveView portion:\n\n```elixir\ndef mount(_params, _session, socket) do\n  {:ok,\n    socket\n    |> assign(:uploaded_files, [])\n    |> allow_upload(:avatar, accept: :any, max_entries: 3, external: &presign_upload/2)}\nend\n\ndefp presign_upload(entry, socket) do\n  uploads = socket.assigns.uploads\n  bucket = \"phx-upload-example\"\n  key = \"public/#{entry.client_name}\"\n\n  config = %{\n    region: \"us-east-1\",\n    access_key_id: System.fetch_env!(\"AWS_ACCESS_KEY_ID\"),\n    secret_access_key: System.fetch_env!(\"AWS_SECRET_ACCESS_KEY\")\n  }\n\n  {:ok, fields} =\n    SimpleS3Upload.sign_form_upload(config, bucket,\n      key: key,\n      content_type: entry.client_type,\n      max_file_size: uploads[entry.upload_config].max_file_size,\n      expires_in: :timer.hours(1)\n    )\n\n  meta = %{uploader: \"S3\", key: key, url: \"http://#{bucket}.s3-#{config.region}.amazonaws.com\", fields: fields}\n  {:ok, meta, socket}\nend\n```\n\nHere, we implemented a `presign_upload/2` function, which we passed as a\ncaptured anonymous function to `:external`. It generates a pre-signed URL\nfor the upload and returns our `:ok` result, with a payload of metadata\nfor the client, along with our unchanged socket. \n\nNext, we add a missing module `SimpleS3Upload` to generate pre-signed URLs\nfor S3. Create a file called `simple_s3_upload.ex`. Get the file's content\nfrom this zero-dependency module called [`SimpleS3Upload`](https://gist.github.com/chrismccord/37862f1f8b1f5148644b75d20d1cb073)\nwritten by Chris McCord.\n\n> Tip: if you encounter errors with the `:crypto` module or with S3 blocking ACLs, \n> please read the comments in the gist above for solutions.\n\nNext, we add our JavaScript client-side uploader. The metadata *must* contain the\n`:uploader` key, specifying the name of the JavaScript client-side uploader.\nIn this case, it's `\"S3\"`, as shown above.\n\nAdd a new file `uploaders.js` in the following directory `assets/js/` next to `app.js`.\nThe content for this `S3` client uploader:\n\n```js\nlet Uploaders = {}\n\nUploaders.S3 = function(entries, onViewError){\n  entries.forEach(entry => {\n    let formData = new FormData()\n    let {url, fields} = entry.meta\n    Object.entries(fields).forEach(([key, val]) => formData.append(key, val))\n    formData.append(\"file\", entry.file)\n    let xhr = new XMLHttpRequest()\n    onViewError(() => xhr.abort())\n    xhr.onload = () => xhr.status === 204 ? entry.progress(100) : entry.error()\n    xhr.onerror = () => entry.error()\n    xhr.upload.addEventListener(\"progress\", (event) => {\n      if(event.lengthComputable){\n        let percent = Math.round((event.loaded / event.total) * 100)\n        if(percent < 100){ entry.progress(percent) }\n      }\n    })\n\n    xhr.open(\"POST\", url, true)\n    xhr.send(formData)\n  })\n}\n\nexport default Uploaders;\n```\n\nWe define an `Uploaders.S3` function, which receives our entries. It then\nperforms an AJAX request for each entry, using the `entry.progress()` and\n`entry.error()` functions to report upload events back to the LiveView.\nThe name of the uploader must match the one we return on the `:uploader`\nmetadata in LiveView.\n\nFinally, head over to `app.js` and add the `uploaders: Uploaders` key to\nthe `LiveSocket` constructor to tell phoenix where to find the uploaders returned \nwithin the external metadata.\n\n```js\n// for uploading to S3\nimport Uploaders from \"./uploaders\"\n\nlet liveSocket = new LiveSocket(\"/live\",\n   Socket, {\n     params: {_csrf_token: csrfToken},\n     uploaders: Uploaders\n  }\n)\n```\n\nNow \"S3\" returned from the server will match the one in the client.\nTo debug client-side javascript when trying to upload, you can inspect your\nbrowser and look at the console or networks tab to view the error logs.\n\n#","ref":"uploads-external.html#direct-to-s3"},{"type":"extras","title":"Direct to S3-Compatible - External Uploads","doc":"> This section assumes that you installed and configured [ExAws](https://hexdocs.pm/ex_aws/readme.html)\n> and [ExAws.S3](https://hexdocs.pm/ex_aws_s3/ExAws.S3.html) correctly in your project and can execute\n> the examples in the page without errors.\n\nMost S3 compatible platforms like Cloudflare R2 don't support `POST` when\nuploading files so we need to use `PUT` with a signed URL instead of the\nsigned `POST`and send the file straight to the service, to do so we need to\nchange the `presign_url/2` function and the `Uploaders.S3` that does the upload.\n\nThe new `presign_upload/2`:\n\n```elixir\ndef presign_upload(entry, socket) do\n  config = ExAws.Config.new(:s3)\n  bucket = \"bucket\"\n  key = \"public/#{entry.client_name}\"\n\n  {:ok, url} =\n    ExAws.S3.presigned_url(config, :put, bucket, key,\n      expires_in: 3600,\n      query_params: [\"Content-Type\": entry.client_type]\n    )\n   {:ok, %{uploader: \"S3\", key: key, url: url}, socket}\nend\n```\n\nThe new `Uploaders.S3`:\n\n```js\nUploaders.S3 = function (entries, onViewError) {\n  entries.forEach(entry => {\n    let xhr = new XMLHttpRequest()\n    onViewError(() => xhr.abort())\n    xhr.onload = () => xhr.status === 200 ? entry.progress(100) : entry.error()\n    xhr.onerror = () => entry.error()\n\n    xhr.upload.addEventListener(\"progress\", (event) => {\n      if(event.lengthComputable){\n        let percent = Math.round((event.loaded / event.total) * 100)\n        if(percent < 100){ entry.progress(percent) }\n      }\n    })\n\n    let url = entry.meta.url\n    xhr.open(\"PUT\", url, true)\n    xhr.send(entry.file)\n  })\n}\n```","ref":"uploads-external.html#direct-to-s3-compatible"}],"content_type":"text/markdown"}